@@ -1,22 +1,22 @@
 /* BGP-4 dump routine
-   Copyright (C) 1999 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1999 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -36,6 +36,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "bgpd/bgp_route.h"
 #include "bgpd/bgp_attr.h"
 #include "bgpd/bgp_dump.h"
+#include "bgpd/bgp_errors.h"
+#include "bgpd/bgp_packet.h"
 
 enum bgp_dump_type {
 	BGP_DUMP_ALL,
@@ -104,22 +106,23 @@ static FILE *bgp_dump_open_file(struct bgp_dump *bgp_dump)
 {
 	int ret;
 	time_t clock;
-	struct tm *tm;
+	struct tm tm;
 	char fullpath[MAXPATHLEN];
 	char realpath[MAXPATHLEN];
 	mode_t oldumask;
 
 	time(&clock);
-	tm = localtime(&clock);
+	localtime_r(&clock, &tm);
 
 	if (bgp_dump->filename[0] != DIRECTORY_SEP) {
-		sprintf(fullpath, "%s/%s", vty_get_cwd(), bgp_dump->filename);
-		ret = strftime(realpath, MAXPATHLEN, fullpath, tm);
+		snprintf(fullpath, sizeof(fullpath), "%s/%s", vty_get_cwd(),
+			 bgp_dump->filename);
+		ret = strftime(realpath, MAXPATHLEN, fullpath, &tm);
 	} else
-		ret = strftime(realpath, MAXPATHLEN, bgp_dump->filename, tm);
+		ret = strftime(realpath, MAXPATHLEN, bgp_dump->filename, &tm);
 
 	if (ret == 0) {
-		zlog_warn("bgp_dump_open_file: strftime error");
+		flog_warn(EC_BGP_DUMP, "bgp_dump_open_file: strftime error");
 		return NULL;
 	}
 
@@ -131,7 +134,7 @@ static FILE *bgp_dump_open_file(struct bgp_dump *bgp_dump)
 	bgp_dump->fp = fopen(realpath, "w");
 
 	if (bgp_dump->fp == NULL) {
-		zlog_warn("bgp_dump_open_file: %s: %s", realpath,
+		flog_warn(EC_BGP_DUMP, "bgp_dump_open_file: %s: %s", realpath,
 			  strerror(errno));
 		umask(oldumask);
 		return NULL;
@@ -145,7 +148,7 @@ static int bgp_dump_interval_add(struct bgp_dump *bgp_dump, int interval)
 {
 	int secs_into_day;
 	time_t t;
-	struct tm *tm;
+	struct tm tm;
 
 	if (interval > 0) {
 		/* Periodic dump every interval seconds */
@@ -156,19 +159,21 @@ static int bgp_dump_interval_add(struct bgp_dump *bgp_dump, int interval)
 			 * midnight
 			 */
 			(void)time(&t);
-			tm = localtime(&t);
-			secs_into_day = tm->tm_sec + 60 * tm->tm_min
-					+ 60 * 60 * tm->tm_hour;
+			localtime_r(&t, &tm);
+			secs_into_day = tm.tm_sec + 60 * tm.tm_min
+					+ 60 * 60 * tm.tm_hour;
 			interval = interval
 				   - secs_into_day % interval; /* always > 0 */
 		}
-		bgp_dump->t_interval = thread_add_timer(
-			bm->master, bgp_dump_interval_func, bgp_dump, interval);
+		bgp_dump->t_interval = NULL;
+		thread_add_timer(bm->master, bgp_dump_interval_func, bgp_dump,
+				 interval, &bgp_dump->t_interval);
 	} else {
 		/* One-off dump: execute immediately, don't affect any scheduled
 		 * dumps */
-		bgp_dump->t_interval = thread_add_event(
-			bm->master, bgp_dump_interval_func, bgp_dump, 0);
+		bgp_dump->t_interval = NULL;
+		thread_add_event(bm->master, bgp_dump_interval_func, bgp_dump,
+				 0, &bgp_dump->t_interval);
 	}
 
 	return 0;
@@ -230,9 +235,9 @@ static void bgp_dump_routes_index_table(struct bgp *bgp)
 	stream_put_in_addr(obuf, &bgp->router_id);
 
 	/* View name */
-	if (bgp->name) {
-		stream_putw(obuf, strlen(bgp->name));
-		stream_put(obuf, bgp->name, strlen(bgp->name));
+	if (bgp->name_pretty) {
+		stream_putw(obuf, strlen(bgp->name_pretty));
+		stream_put(obuf, bgp->name_pretty, strlen(bgp->name_pretty));
 	} else {
 		stream_putw(obuf, 0);
 	}
@@ -275,7 +280,7 @@ static void bgp_dump_routes_index_table(struct bgp *bgp)
 		if (sockunion_family(&peer->su) == AF_INET) {
 			stream_put_in_addr(obuf, &peer->su.sin.sin_addr);
 		} else if (sockunion_family(&peer->su) == AF_INET6) {
-			stream_write(obuf, (u_char *)&peer->su.sin6.sin6_addr,
+			stream_write(obuf, (uint8_t *)&peer->su.sin6.sin6_addr,
 				     IPV6_MAX_BYTELEN);
 		}
 
@@ -296,13 +301,14 @@ static void bgp_dump_routes_index_table(struct bgp *bgp)
 }
 
 
-static struct bgp_info *bgp_dump_route_node_record(int afi, struct bgp_node *rn,
-						   struct bgp_info *info,
-						   unsigned int seq)
+static struct bgp_path_info *
+bgp_dump_route_node_record(int afi, struct bgp_node *rn,
+			   struct bgp_path_info *path, unsigned int seq)
 {
 	struct stream *obuf;
 	size_t sizep;
 	size_t endp;
+	const struct prefix *p = bgp_node_get_prefix(rn);
 
 	obuf = bgp_dump_obuf;
 	stream_reset(obuf);
@@ -321,19 +327,19 @@ static struct bgp_info *bgp_dump_route_node_record(int afi, struct bgp_node *rn,
 	stream_putl(obuf, seq);
 
 	/* Prefix length */
-	stream_putc(obuf, rn->p.prefixlen);
+	stream_putc(obuf, p->prefixlen);
 
 	/* Prefix */
 	if (afi == AFI_IP) {
 		/* We'll dump only the useful bits (those not 0), but have to
 		 * align on 8 bits */
-		stream_write(obuf, (u_char *)&rn->p.u.prefix4,
-			     (rn->p.prefixlen + 7) / 8);
+		stream_write(obuf, (uint8_t *)&p->u.prefix4,
+			     (p->prefixlen + 7) / 8);
 	} else if (afi == AFI_IP6) {
 		/* We'll dump only the useful bits (those not 0), but have to
 		 * align on 8 bits */
-		stream_write(obuf, (u_char *)&rn->p.u.prefix6,
-			     (rn->p.prefixlen + 7) / 8);
+		stream_write(obuf, (uint8_t *)&p->u.prefix6,
+			     (p->prefixlen + 7) / 8);
 	}
 
 	/* Save where we are now, so we can overwride the entry count later */
@@ -346,18 +352,18 @@ static struct bgp_info *bgp_dump_route_node_record(int afi, struct bgp_node *rn,
 	stream_putw(obuf, 0);
 
 	endp = stream_get_endp(obuf);
-	for (; info; info = info->next) {
+	for (; path; path = path->next) {
 		size_t cur_endp;
 
 		/* Peer index */
-		stream_putw(obuf, info->peer->table_dump_index);
+		stream_putw(obuf, path->peer->table_dump_index);
 
 		/* Originated */
-		stream_putl(obuf, time(NULL) - (bgp_clock() - info->uptime));
+		stream_putl(obuf, time(NULL) - (bgp_clock() - path->uptime));
 
 		/* Dump attribute. */
 		/* Skip prefix & AFI/SAFI for MP_NLRI */
-		bgp_dump_routes_attr(obuf, info->attr, &rn->p);
+		bgp_dump_routes_attr(obuf, path->attr, p);
 
 		cur_endp = stream_get_endp(obuf);
 		if (cur_endp > BGP_MAX_PACKET_SIZE + BGP_DUMP_MSG_HEADER
@@ -376,7 +382,7 @@ static struct bgp_info *bgp_dump_route_node_record(int afi, struct bgp_node *rn,
 	bgp_dump_set_size(obuf, MSG_TABLE_DUMP_V2);
 	fwrite(STREAM_DATA(obuf), stream_get_endp(obuf), 1, bgp_dump_routes.fp);
 
-	return info;
+	return path;
 }
 
 
@@ -384,7 +390,7 @@ static struct bgp_info *bgp_dump_route_node_record(int afi, struct bgp_node *rn,
 static unsigned int bgp_dump_routes_func(int afi, int first_run,
 					 unsigned int seq)
 {
-	struct bgp_info *info;
+	struct bgp_path_info *path;
 	struct bgp_node *rn;
 	struct bgp *bgp;
 	struct bgp_table *table;
@@ -407,9 +413,9 @@ static unsigned int bgp_dump_routes_func(int afi, int first_run,
 	table = bgp->rib[afi][SAFI_UNICAST];
 
 	for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn)) {
-		info = rn->info;
-		while (info) {
-			info = bgp_dump_route_node_record(afi, rn, info, seq);
+		path = bgp_node_get_bgp_path_info(rn);
+		while (path) {
+			path = bgp_dump_route_node_record(afi, rn, path, seq);
 			seq++;
 		}
 	}
@@ -463,7 +469,7 @@ static void bgp_dump_common(struct stream *obuf, struct peer *peer,
 	}
 
 	if (peer->su.sa.sa_family == AF_INET) {
-		stream_putw(obuf, peer->ifindex);
+		stream_putw(obuf, peer->ifp ? peer->ifp->ifindex : 0);
 		stream_putw(obuf, AFI_IP);
 
 		stream_put(obuf, &peer->su.sin.sin_addr, IPV4_MAX_BYTELEN);
@@ -475,7 +481,7 @@ static void bgp_dump_common(struct stream *obuf, struct peer *peer,
 			stream_put(obuf, empty, IPV4_MAX_BYTELEN);
 	} else if (peer->su.sa.sa_family == AF_INET6) {
 		/* Interface Index and Address family. */
-		stream_putw(obuf, peer->ifindex);
+		stream_putw(obuf, peer->ifp ? peer->ifp->ifindex : 0);
 		stream_putw(obuf, AFI_IP6);
 
 		/* Source IP Address and Destination IP Address. */
@@ -490,13 +496,13 @@ static void bgp_dump_common(struct stream *obuf, struct peer *peer,
 }
 
 /* Dump BGP status change. */
-void bgp_dump_state(struct peer *peer, int status_old, int status_new)
+int bgp_dump_state(struct peer *peer)
 {
 	struct stream *obuf;
 
 	/* If dump file pointer is disabled return immediately. */
 	if (bgp_dump_all.fp == NULL)
-		return;
+		return 0;
 
 	/* Make dump stream. */
 	obuf = bgp_dump_obuf;
@@ -506,8 +512,8 @@ void bgp_dump_state(struct peer *peer, int status_old, int status_new)
 			bgp_dump_all.type);
 	bgp_dump_common(obuf, peer, 1); /* force this in as4speak*/
 
-	stream_putw(obuf, status_old);
-	stream_putw(obuf, status_new);
+	stream_putw(obuf, peer->ostatus);
+	stream_putw(obuf, peer->status);
 
 	/* Set length. */
 	bgp_dump_set_size(obuf, MSG_PROTOCOL_BGP4MP);
@@ -515,6 +521,7 @@ void bgp_dump_state(struct peer *peer, int status_old, int status_new)
 	/* Write to the stream. */
 	fwrite(STREAM_DATA(obuf), stream_get_endp(obuf), 1, bgp_dump_all.fp);
 	fflush(bgp_dump_all.fp);
+	return 0;
 }
 
 static void bgp_dump_packet_func(struct bgp_dump *bgp_dump, struct peer *peer,
@@ -552,7 +559,8 @@ static void bgp_dump_packet_func(struct bgp_dump *bgp_dump, struct peer *peer,
 }
 
 /* Called from bgp_packet.c when BGP packet is received. */
-void bgp_dump_packet(struct peer *peer, int type, struct stream *packet)
+static int bgp_dump_packet(struct peer *peer, uint8_t type, bgp_size_t size,
+		struct stream *packet)
 {
 	/* bgp_dump_all. */
 	bgp_dump_packet_func(&bgp_dump_all, peer, packet);
@@ -560,6 +568,7 @@ void bgp_dump_packet(struct peer *peer, int type, struct stream *packet)
 	/* bgp_dump_updates. */
 	if (type == BGP_MSG_UPDATE)
 		bgp_dump_packet_func(&bgp_dump_updates, peer, packet);
+	return 0;
 }
 
 static unsigned int bgp_dump_parse_time(const char *str)
@@ -578,7 +587,7 @@ static unsigned int bgp_dump_parse_time(const char *str)
 	len = strlen(str);
 
 	for (i = 0; i < len; i++) {
-		if (isdigit((int)str[i])) {
+		if (isdigit((unsigned char)str[i])) {
 			time *= 10;
 			time += str[i] - '0';
 		} else if (str[i] == 'H' || str[i] == 'h') {
@@ -594,7 +603,7 @@ static unsigned int bgp_dump_parse_time(const char *str)
 				return 0;
 			total += time * 60;
 			time = 0;
-			seen_h = 1;
+			seen_m = 1;
 		} else
 			return 0;
 	}
@@ -628,9 +637,8 @@ static int bgp_dump_set(struct vty *vty, struct bgp_dump *bgp_dump,
 		/* Check interval string. */
 		interval = bgp_dump_parse_time(interval_str);
 		if (interval == 0) {
-			vty_out(vty, "Malformed interval string%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+			vty_out(vty, "Malformed interval string\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 
 		/* Setting interval string */
@@ -661,10 +669,7 @@ static int bgp_dump_set(struct vty *vty, struct bgp_dump *bgp_dump,
 static int bgp_dump_unset(struct bgp_dump *bgp_dump)
 {
 	/* Removing file name. */
-	if (bgp_dump->filename) {
-		XFREE(MTYPE_BGP_DUMP_STR, bgp_dump->filename);
-		bgp_dump->filename = NULL;
-	}
+	XFREE(MTYPE_BGP_DUMP_STR, bgp_dump->filename);
 
 	/* Closing file. */
 	if (bgp_dump->fp) {
@@ -681,10 +686,7 @@ static int bgp_dump_unset(struct bgp_dump *bgp_dump)
 	bgp_dump->interval = 0;
 
 	/* Removing interval string. */
-	if (bgp_dump->interval_str) {
-		XFREE(MTYPE_BGP_DUMP_STR, bgp_dump->interval_str);
-		bgp_dump->interval_str = NULL;
-	}
+	XFREE(MTYPE_BGP_DUMP_STR, bgp_dump->interval_str);
 
 	return CMD_SUCCESS;
 }
@@ -709,7 +711,7 @@ DEFUN (dump_bgp_all,
 	const struct bgp_dump_type_map *map = NULL;
 
 	for (map = bgp_dump_type_map; map->str; map++)
-		if (strcmp(argv[idx_dump_routes]->arg, map->str) == 0)
+		if (strmatch(argv[idx_dump_routes]->text, map->str))
 			bgp_dump_type = map->type;
 
 	switch (bgp_dump_type) {
@@ -755,7 +757,7 @@ DEFUN (no_dump_bgp_all,
 	struct bgp_dump *bgp_dump_struct = NULL;
 
 	for (map = bgp_dump_type_map; map->str; map++)
-		if (strcmp(argv[idx_dump_routes]->arg, map->str) == 0)
+		if (strmatch(argv[idx_dump_routes]->text, map->str))
 			bgp_dump_type = map->type;
 
 	switch (bgp_dump_type) {
@@ -776,34 +778,14 @@ DEFUN (no_dump_bgp_all,
 	return bgp_dump_unset(bgp_dump_struct);
 }
 
+static int config_write_bgp_dump(struct vty *vty);
 /* BGP node structure. */
-static struct cmd_node bgp_dump_node = {DUMP_NODE, "", 1};
-
-#if 0
-char *
-config_time2str (unsigned int interval)
-{
-  static char buf[BUFSIZ];
-
-  buf[0] = '\0';
-
-  if (interval / 3600)
-    {
-      sprintf (buf, "%dh", interval / 3600);
-      interval %= 3600;
-    }
-  if (interval / 60)
-    {
-      sprintf (buf + strlen (buf), "%dm", interval /60);
-      interval %= 60;
-    }
-  if (interval)
-    {
-      sprintf (buf + strlen (buf), "%d", interval);
-    }
-  return buf;
-}
-#endif
+static struct cmd_node bgp_dump_node = {
+	.name = "dump",
+	.node = DUMP_NODE,
+	.prompt = "",
+	.config_write = config_write_bgp_dump,
+};
 
 static int config_write_bgp_dump(struct vty *vty)
 {
@@ -813,12 +795,12 @@ static int config_write_bgp_dump(struct vty *vty)
 			type_str = "all-et";
 
 		if (bgp_dump_all.interval_str)
-			vty_out(vty, "dump bgp %s %s %s%s", type_str,
+			vty_out(vty, "dump bgp %s %s %s\n", type_str,
 				bgp_dump_all.filename,
-				bgp_dump_all.interval_str, VTY_NEWLINE);
+				bgp_dump_all.interval_str);
 		else
-			vty_out(vty, "dump bgp %s %s%s", type_str,
-				bgp_dump_all.filename, VTY_NEWLINE);
+			vty_out(vty, "dump bgp %s %s\n", type_str,
+				bgp_dump_all.filename);
 	}
 	if (bgp_dump_updates.filename) {
 		const char *type_str = "updates";
@@ -826,21 +808,21 @@ static int config_write_bgp_dump(struct vty *vty)
 			type_str = "updates-et";
 
 		if (bgp_dump_updates.interval_str)
-			vty_out(vty, "dump bgp %s %s %s%s", type_str,
+			vty_out(vty, "dump bgp %s %s %s\n", type_str,
 				bgp_dump_updates.filename,
-				bgp_dump_updates.interval_str, VTY_NEWLINE);
+				bgp_dump_updates.interval_str);
 		else
-			vty_out(vty, "dump bgp %s %s%s", type_str,
-				bgp_dump_updates.filename, VTY_NEWLINE);
+			vty_out(vty, "dump bgp %s %s\n", type_str,
+				bgp_dump_updates.filename);
 	}
 	if (bgp_dump_routes.filename) {
 		if (bgp_dump_routes.interval_str)
-			vty_out(vty, "dump bgp routes-mrt %s %s%s",
+			vty_out(vty, "dump bgp routes-mrt %s %s\n",
 				bgp_dump_routes.filename,
-				bgp_dump_routes.interval_str, VTY_NEWLINE);
+				bgp_dump_routes.interval_str);
 		else
-			vty_out(vty, "dump bgp routes-mrt %s%s",
-				bgp_dump_routes.filename, VTY_NEWLINE);
+			vty_out(vty, "dump bgp routes-mrt %s\n",
+				bgp_dump_routes.filename);
 	}
 	return 0;
 }
@@ -856,10 +838,13 @@ void bgp_dump_init(void)
 		stream_new((BGP_MAX_PACKET_SIZE << 1) + BGP_DUMP_MSG_HEADER
 			   + BGP_DUMP_HEADER_SIZE);
 
-	install_node(&bgp_dump_node, config_write_bgp_dump);
+	install_node(&bgp_dump_node);
 
 	install_element(CONFIG_NODE, &dump_bgp_all_cmd);
 	install_element(CONFIG_NODE, &no_dump_bgp_all_cmd);
+
+	hook_register(bgp_packet_dump, bgp_dump_packet);
+	hook_register(peer_status_changed, bgp_dump_state);
 }
 
 void bgp_dump_finish(void)
@@ -870,4 +855,6 @@ void bgp_dump_finish(void)
 
 	stream_free(bgp_dump_obuf);
 	bgp_dump_obuf = NULL;
+	hook_unregister(bgp_packet_dump, bgp_dump_packet);
+	hook_unregister(peer_status_changed, bgp_dump_state);
 }
