@@ -27,7 +27,6 @@
 #include "thread.h"
 #include <lib/version.h>
 #include "memory.h"
-#include "memory_vty.h"
 #include "prefix.h"
 #include "log.h"
 #include "privs.h"
@@ -54,11 +53,13 @@
 #include "bgpd/bgp_regex.h"
 #include "bgpd/bgp_clist.h"
 #include "bgpd/bgp_debug.h"
+#include "bgpd/bgp_errors.h"
 #include "bgpd/bgp_filter.h"
 #include "bgpd/bgp_zebra.h"
 #include "bgpd/bgp_packet.h"
 #include "bgpd/bgp_keepalives.h"
 #include "bgpd/bgp_network.h"
+#include "bgpd/bgp_errors.h"
 
 #ifdef ENABLE_BGP_VNC
 #include "bgpd/rfapi/rfapi_backend.h"
@@ -68,11 +69,12 @@
 static const struct option longopts[] = {
 	{"bgp_port", required_argument, NULL, 'p'},
 	{"listenon", required_argument, NULL, 'l'},
-	{"retain", no_argument, NULL, 'r'},
 	{"no_kernel", no_argument, NULL, 'n'},
 	{"skip_runas", no_argument, NULL, 'S'},
 	{"ecmp", required_argument, NULL, 'e'},
 	{"int_num", required_argument, NULL, 'I'},
+	{"no_zebra", no_argument, NULL, 'Z'},
+	{"socket_size", required_argument, NULL, 's'},
 	{0}};
 
 /* signal definitions */
@@ -102,9 +104,6 @@ static struct quagga_signal_t bgp_signals[] = {
 	},
 };
 
-/* Route retain mode flag. */
-static int retain_mode = 0;
-
 /* privileges */
 static zebra_capabilities_t _caps_p[] = {ZCAP_BIND, ZCAP_NET_RAW,
 					 ZCAP_NET_ADMIN, ZCAP_SYS_ADMIN};
@@ -135,7 +134,7 @@ void sighup(void)
 	zlog_info("bgpd restarting!");
 
 	/* Reload config file. */
-	vty_read_config(bgpd_di.config_file, config_default);
+	vty_read_config(NULL, bgpd_di.config_file, config_default);
 
 	/* Try to return to normal operation. */
 }
@@ -144,9 +143,10 @@ void sighup(void)
 __attribute__((__noreturn__)) void sigint(void)
 {
 	zlog_notice("Terminating on signal");
+	assert(bm->terminating == false);
+	bm->terminating = true;	/* global flag that shutting down */
 
-	if (!retain_mode)
-		bgp_terminate();
+	bgp_terminate();
 
 	bgp_exit(0);
 
@@ -168,7 +168,7 @@ void sigusr1(void)
 */
 static __attribute__((__noreturn__)) void bgp_exit(int status)
 {
-	struct bgp *bgp;
+	struct bgp *bgp, *bgp_default, *bgp_evpn;
 	struct listnode *node, *nnode;
 
 	/* it only makes sense for this to be called on a clean exit */
@@ -180,9 +180,19 @@ static __attribute__((__noreturn__)) void bgp_exit(int status)
 
 	bgp_close();
 
+	bgp_default = bgp_get_default();
+	bgp_evpn = bgp_get_evpn();
+
 	/* reverse bgp_master_init */
-	for (ALL_LIST_ELEMENTS(bm->bgp, node, nnode, bgp))
+	for (ALL_LIST_ELEMENTS(bm->bgp, node, nnode, bgp)) {
+		if (bgp_default == bgp || bgp_evpn == bgp)
+			continue;
 		bgp_delete(bgp);
+	}
+	if (bgp_evpn && bgp_evpn != bgp_default)
+		bgp_delete(bgp_evpn);
+	if (bgp_default)
+		bgp_delete(bgp_default);
 
 	/* reverse bgp_dump_init */
 	bgp_dump_finish();
@@ -218,13 +228,16 @@ static __attribute__((__noreturn__)) void bgp_exit(int status)
 	community_list_terminate(bgp_clist);
 
 	bgp_vrf_terminate();
-#if ENABLE_BGP_VNC
+#ifdef ENABLE_BGP_VNC
 	vnc_zebra_destroy();
 #endif
 	bgp_zebra_destroy();
 
 	bf_free(bm->rd_idspace);
-	list_delete_and_null(&bm->bgp);
+	list_delete(&bm->bgp);
+
+	bgp_lp_finish();
+
 	memset(bm, 0, sizeof(*bm));
 
 	frr_fini();
@@ -256,18 +269,39 @@ static int bgp_vrf_enable(struct vrf *vrf)
 		zlog_debug("VRF enable add %s id %u", vrf->name, vrf->vrf_id);
 
 	bgp = bgp_lookup_by_name(vrf->name);
-	if (bgp) {
+	if (bgp && bgp->vrf_id != vrf->vrf_id) {
+		if (bgp->name && strmatch(vrf->name, VRF_DEFAULT_NAME)) {
+			XFREE(MTYPE_BGP, bgp->name);
+			XFREE(MTYPE_BGP, bgp->name_pretty);
+			bgp->name_pretty = XSTRDUP(MTYPE_BGP, "VRF default");
+			bgp->inst_type = BGP_INSTANCE_TYPE_DEFAULT;
+#ifdef ENABLE_BGP_VNC
+			if (!bgp->rfapi) {
+				bgp->rfapi = bgp_rfapi_new(bgp);
+				assert(bgp->rfapi);
+				assert(bgp->rfapi_cfg);
+			}
+#endif /* ENABLE_BGP_VNC */
+		}
 		old_vrf_id = bgp->vrf_id;
 		/* We have instance configured, link to VRF and make it "up". */
 		bgp_vrf_link(bgp, vrf);
 
 		bgp_handle_socket(bgp, vrf, old_vrf_id, true);
-		/* Update any redistribute vrf bitmaps if the vrf_id changed */
+		/* Update any redistribution if vrf_id changed */
 		if (old_vrf_id != bgp->vrf_id)
-			bgp_update_redist_vrf_bitmaps(bgp, old_vrf_id);
+			bgp_redistribute_redo(bgp);
 		bgp_instance_up(bgp);
 		vpn_leak_zebra_vrf_label_update(bgp, AFI_IP);
 		vpn_leak_zebra_vrf_label_update(bgp, AFI_IP6);
+		vpn_leak_postchange(BGP_VPN_POLICY_DIR_TOVPN, AFI_IP,
+				    bgp_get_default(), bgp);
+		vpn_leak_postchange(BGP_VPN_POLICY_DIR_FROMVPN, AFI_IP,
+				    bgp_get_default(), bgp);
+		vpn_leak_postchange(BGP_VPN_POLICY_DIR_TOVPN, AFI_IP6,
+				    bgp_get_default(), bgp);
+		vpn_leak_postchange(BGP_VPN_POLICY_DIR_FROMVPN, AFI_IP6,
+				    bgp_get_default(), bgp);
 	}
 
 	return 0;
@@ -289,15 +323,23 @@ static int bgp_vrf_disable(struct vrf *vrf)
 
 		vpn_leak_zebra_vrf_label_withdraw(bgp, AFI_IP);
 		vpn_leak_zebra_vrf_label_withdraw(bgp, AFI_IP6);
+		vpn_leak_prechange(BGP_VPN_POLICY_DIR_TOVPN, AFI_IP,
+				   bgp_get_default(), bgp);
+		vpn_leak_prechange(BGP_VPN_POLICY_DIR_FROMVPN, AFI_IP,
+				   bgp_get_default(), bgp);
+		vpn_leak_prechange(BGP_VPN_POLICY_DIR_TOVPN, AFI_IP6,
+				   bgp_get_default(), bgp);
+		vpn_leak_prechange(BGP_VPN_POLICY_DIR_FROMVPN, AFI_IP6,
+				   bgp_get_default(), bgp);
 
 		old_vrf_id = bgp->vrf_id;
 		bgp_handle_socket(bgp, vrf, VRF_UNKNOWN, false);
 		/* We have instance configured, unlink from VRF and make it
 		 * "down". */
 		bgp_vrf_unlink(bgp, vrf);
-		/* Update any redistribute vrf bitmaps if the vrf_id changed */
+		/* Delete any redistribute vrf bitmaps if the vrf_id changed */
 		if (old_vrf_id != bgp->vrf_id)
-			bgp_update_redist_vrf_bitmaps(bgp, old_vrf_id);
+			bgp_unset_redist_vrf_bitmaps(bgp, old_vrf_id);
 		bgp_instance_down(bgp);
 	}
 
@@ -307,7 +349,8 @@ static int bgp_vrf_disable(struct vrf *vrf)
 
 static void bgp_vrf_init(void)
 {
-	vrf_init(bgp_vrf_new, bgp_vrf_enable, bgp_vrf_disable, bgp_vrf_delete);
+	vrf_init(bgp_vrf_new, bgp_vrf_enable, bgp_vrf_disable,
+		 bgp_vrf_delete, bgp_vrf_enable);
 }
 
 static void bgp_vrf_terminate(void)
@@ -315,13 +358,22 @@ static void bgp_vrf_terminate(void)
 	vrf_terminate();
 }
 
+static const struct frr_yang_module_info *const bgpd_yang_modules[] = {
+	&frr_interface_info,
+	&frr_route_map_info,
+	&frr_vrf_info,
+};
+
 FRR_DAEMON_INFO(bgpd, BGP, .vty_port = BGP_VTY_PORT,
 
 		.proghelp = "Implementation of the BGP routing protocol.",
 
 		.signals = bgp_signals, .n_signals = array_size(bgp_signals),
 
-		.privs = &bgpd_privs, )
+		.privs = &bgpd_privs, .yang_modules = bgpd_yang_modules,
+		.n_yang_modules = array_size(bgpd_yang_modules), )
+
+#define DEPRECATED_OPTIONS ""
 
 /* Main routine of bgpd. Treatment of argument and start bgp finite
    state machine is handled at here. */
@@ -333,24 +385,34 @@ int main(int argc, char **argv)
 	int bgp_port = BGP_PORT_DEFAULT;
 	char *bgp_address = NULL;
 	int no_fib_flag = 0;
+	int no_zebra_flag = 0;
 	int skip_runas = 0;
 	int instance = 0;
+	int buffer_size = BGP_SOCKET_SNDBUF_SIZE;
 
 	frr_preinit(&bgpd_di, argc, argv);
 	frr_opt_add(
-		"p:l:rSne:I:", longopts,
+		"p:l:SnZe:I:s:" DEPRECATED_OPTIONS, longopts,
 		"  -p, --bgp_port     Set BGP listen port number (0 means do not listen).\n"
 		"  -l, --listenon     Listen on specified address (implies -n)\n"
-		"  -r, --retain       When program terminates, retain added route by bgpd.\n"
 		"  -n, --no_kernel    Do not install route to kernel.\n"
+		"  -Z, --no_zebra     Do not communicate with Zebra.\n"
 		"  -S, --skip_runas   Skip capabilities checks, and changing user and group IDs.\n"
 		"  -e, --ecmp         Specify ECMP to use.\n"
-		"  -I, --int_num      Set instance number (label-manager)\n");
+		"  -I, --int_num      Set instance number (label-manager)\n"
+		"  -s, --socket_size  Set BGP peer socket send buffer size\n");
 
 	/* Command line argument treatment. */
 	while (1) {
 		opt = frr_getopt(argc, argv, 0);
 
+		if (opt && opt < 128 && strchr(DEPRECATED_OPTIONS, opt)) {
+			fprintf(stderr,
+				"The -%c option no longer exists.\nPlease refer to the manual.\n",
+				opt);
+			continue;
+		}
+
 		if (opt == EOF)
 			break;
 
@@ -364,19 +426,21 @@ int main(int argc, char **argv)
 			else
 				bgp_port = tmp_port;
 			break;
-		case 'e':
-			multipath_num = atoi(optarg);
-			if (multipath_num > MULTIPATH_NUM
-			    || multipath_num <= 0) {
-				zlog_err(
-					"Multipath Number specified must be less than %d and greater than 0",
+		case 'e': {
+			unsigned long int parsed_multipath =
+				strtoul(optarg, NULL, 10);
+			if (parsed_multipath == 0
+			    || parsed_multipath > MULTIPATH_NUM
+			    || parsed_multipath > UINT_MAX) {
+				flog_err(
+					EC_BGP_MULTIPATH,
+					"Multipath Number specified must be less than %u and greater than 0",
 					MULTIPATH_NUM);
 				return 1;
 			}
+			multipath_num = parsed_multipath;
 			break;
-		case 'r':
-			retain_mode = 1;
-			break;
+		}
 		case 'l':
 			bgp_address = optarg;
 		/* listenon implies -n */
@@ -384,6 +448,9 @@ int main(int argc, char **argv)
 		case 'n':
 			no_fib_flag = 1;
 			break;
+		case 'Z':
+			no_zebra_flag = 1;
+			break;
 		case 'S':
 			skip_runas = 1;
 			break;
@@ -393,6 +460,9 @@ int main(int argc, char **argv)
 				zlog_err("Instance %i out of range (0..%u)",
 					 instance, (unsigned short)-1);
 			break;
+		case 's':
+			buffer_size = atoi(optarg);
+			break;
 		default:
 			frr_help_exit(1);
 			break;
@@ -402,14 +472,16 @@ int main(int argc, char **argv)
 		memset(&bgpd_privs, 0, sizeof(bgpd_privs));
 
 	/* BGP master init. */
-	bgp_master_init(frr_init());
+	bgp_master_init(frr_init(), buffer_size);
 	bm->port = bgp_port;
 	if (bgp_port == 0)
 		bgp_option_set(BGP_OPT_NO_LISTEN);
 	bm->address = bgp_address;
-	if (no_fib_flag)
+	if (no_fib_flag || no_zebra_flag)
 		bgp_option_set(BGP_OPT_NO_FIB);
-
+	if (no_zebra_flag)
+		bgp_option_set(BGP_OPT_NO_ZEBRA);
+	bgp_error_init();
 	/* Initializations. */
 	bgp_vrf_init();
 
@@ -421,9 +493,10 @@ int main(int argc, char **argv)
 
 	frr_config_fork();
 	/* must be called after fork() */
+	bgp_gr_apply_running_config();
 	bgp_pthreads_run();
 	frr_run(bm->master);
 
 	/* Not reached. */
-	return (0);
+	return 0;
 }
