@@ -111,6 +111,9 @@ DECLARE_MTYPE(LCOMMUNITY)
 DECLARE_MTYPE(LCOMMUNITY_STR)
 DECLARE_MTYPE(LCOMMUNITY_VAL)
 
+DECLARE_MTYPE(BGP_EVPN_ES)
+DECLARE_MTYPE(BGP_EVPN_ES_VTEP)
+
 DECLARE_MTYPE(BGP_EVPN)
 DECLARE_MTYPE(BGP_EVPN_IMPORT_RT)
 DECLARE_MTYPE(BGP_EVPN_VRF_IMPORT_RT)
@@ -123,4 +126,7 @@ DECLARE_MTYPE(BGP_FLOWSPEC_COMPILED)
 DECLARE_MTYPE(BGP_FLOWSPEC_NAME)
 DECLARE_MTYPE(BGP_FLOWSPEC_INDEX)
 
+DECLARE_MTYPE(BGP_SRV6_L3VPN)
+DECLARE_MTYPE(BGP_SRV6_VPN)
+
 #endif /* _QUAGGA_BGP_MEMORY_H */
