@@ -44,12 +44,12 @@
 #include "bgpd/bgp_damp.h"
 #include "bgpd/bgp_fsm.h"
 #include "bgpd/bgp_vty.h"
-#include "zebra/rib.h"
-#include "zebra/zserv.h" /* For ZEBRA_SERV_PATH. */
+
+DEFINE_MTYPE_STATIC(BGPD, MARTIAN_STRING, "BGP Martian Address Intf String");
 
 char *bnc_str(struct bgp_nexthop_cache *bnc, char *buf, int size)
 {
-	prefix2str(&(bnc->node->p), buf, size);
+	prefix2str(bgp_node_get_prefix(bnc->node), buf, size);
 	return buf;
 }
 
@@ -80,12 +80,21 @@ static void bgp_nexthop_cache_reset(struct bgp_table *table)
 	struct bgp_node *rn;
 	struct bgp_nexthop_cache *bnc;
 
-	for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn))
-		if ((bnc = rn->info) != NULL) {
-			bnc_free(bnc);
-			rn->info = NULL;
-			bgp_unlock_node(rn);
+	for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn)) {
+		bnc = bgp_node_get_bgp_nexthop_info(rn);
+		if (!bnc)
+			continue;
+
+		while (!LIST_EMPTY(&(bnc->paths))) {
+			struct bgp_path_info *path = LIST_FIRST(&(bnc->paths));
+
+			path_nh_map(path, bnc, false);
 		}
+
+		bnc_free(bnc);
+		bgp_node_set_bgp_nexthop_info(rn, NULL);
+		bgp_unlock_node(rn);
+	}
 }
 
 static void *bgp_tip_hash_alloc(void *p)
@@ -105,14 +114,14 @@ static void bgp_tip_hash_free(void *addr)
 	XFREE(MTYPE_TIP_ADDR, addr);
 }
 
-static unsigned int bgp_tip_hash_key_make(void *p)
+static unsigned int bgp_tip_hash_key_make(const void *p)
 {
 	const struct tip_addr *addr = p;
 
 	return jhash_1word(addr->addr.s_addr, 0);
 }
 
-static int bgp_tip_hash_cmp(const void *p1, const void *p2)
+static bool bgp_tip_hash_cmp(const void *p1, const void *p2)
 {
 	const struct tip_addr *addr1 = p1;
 	const struct tip_addr *addr2 = p2;
@@ -169,43 +178,88 @@ void bgp_tip_del(struct bgp *bgp, struct in_addr *tip)
 	}
 }
 
+/* BGP own address structure */
+struct bgp_addr {
+	struct prefix p;
+	struct list *ifp_name_list;
+};
+
+static void show_address_entry(struct hash_bucket *bucket, void *args)
+{
+	struct vty *vty = (struct vty *)args;
+	struct bgp_addr *addr = (struct bgp_addr *)bucket->data;
+	char *name;
+	struct listnode *node;
+	char str[INET6_ADDRSTRLEN] = {0};
+
+	vty_out(vty, "addr: %s, count: %d : ",
+		inet_ntop(addr->p.family, &(addr->p.u.prefix),
+			  str, INET6_ADDRSTRLEN),
+		addr->ifp_name_list->count);
+
+	for (ALL_LIST_ELEMENTS_RO(addr->ifp_name_list, node, name)) {
+		vty_out(vty, " %s,", name);
+	}
+
+	vty_out(vty, "\n");
+}
+
+void bgp_nexthop_show_address_hash(struct vty *vty, struct bgp *bgp)
+{
+	hash_iterate(bgp->address_hash,
+		     (void (*)(struct hash_bucket *, void *))show_address_entry,
+		     vty);
+}
+
+static void bgp_address_hash_string_del(void *val)
+{
+	char *data = val;
+
+	XFREE(MTYPE_MARTIAN_STRING, data);
+}
+
 static void *bgp_address_hash_alloc(void *p)
 {
-	const struct in_addr *val = (const struct in_addr *)p;
-	struct bgp_addr *addr;
+	struct bgp_addr *copy_addr = p;
+	struct bgp_addr *addr = NULL;
 
 	addr = XMALLOC(MTYPE_BGP_ADDR, sizeof(struct bgp_addr));
-	addr->refcnt = 0;
-	addr->addr.s_addr = val->s_addr;
+	prefix_copy(&addr->p, &copy_addr->p);
+
+	addr->ifp_name_list = list_new();
+	addr->ifp_name_list->del = bgp_address_hash_string_del;
 
 	return addr;
 }
 
-static void bgp_address_hash_free(void *addr)
+static void bgp_address_hash_free(void *data)
 {
+	struct bgp_addr *addr = data;
+
+	list_delete(&addr->ifp_name_list);
 	XFREE(MTYPE_BGP_ADDR, addr);
 }
 
-static unsigned int bgp_address_hash_key_make(void *p)
+static unsigned int bgp_address_hash_key_make(const void *p)
 {
 	const struct bgp_addr *addr = p;
 
-	return jhash_1word(addr->addr.s_addr, 0);
+	return prefix_hash_key(&addr->p);
 }
 
-static int bgp_address_hash_cmp(const void *p1, const void *p2)
+static bool bgp_address_hash_cmp(const void *p1, const void *p2)
 {
 	const struct bgp_addr *addr1 = p1;
 	const struct bgp_addr *addr2 = p2;
 
-	return addr1->addr.s_addr == addr2->addr.s_addr;
+	return prefix_same(&addr1->p, &addr2->p);
 }
 
 void bgp_address_init(struct bgp *bgp)
 {
 	bgp->address_hash =
 		hash_create(bgp_address_hash_key_make, bgp_address_hash_cmp,
-			    "BGP Address Hash");
+				"BGP Connected Address Hash");
 }
 
 void bgp_address_destroy(struct bgp *bgp)
@@ -217,36 +271,66 @@ void bgp_address_destroy(struct bgp *bgp)
 	bgp->address_hash = NULL;
 }
 
-static void bgp_address_add(struct bgp *bgp, struct prefix *p)
+static void bgp_address_add(struct bgp *bgp, struct connected *ifc,
+			    struct prefix *p)
 {
 	struct bgp_addr tmp;
 	struct bgp_addr *addr;
+	struct listnode *node;
+	char *name;
 
-	tmp.addr = p->u.prefix4;
+	tmp.p = *p;
+
+	if (tmp.p.family == AF_INET)
+		tmp.p.prefixlen = IPV4_MAX_BITLEN;
+	else if (tmp.p.family == AF_INET6)
+		tmp.p.prefixlen = IPV6_MAX_BITLEN;
 
 	addr = hash_get(bgp->address_hash, &tmp, bgp_address_hash_alloc);
-	if (!addr)
-		return;
 
-	addr->refcnt++;
+	for (ALL_LIST_ELEMENTS_RO(addr->ifp_name_list, node, name)) {
+		if (strcmp(ifc->ifp->name, name) == 0)
+			break;
+	}
+	if (!node) {
+		name = XSTRDUP(MTYPE_MARTIAN_STRING, ifc->ifp->name);
+		listnode_add(addr->ifp_name_list, name);
+	}
 }
 
-static void bgp_address_del(struct bgp *bgp, struct prefix *p)
+static void bgp_address_del(struct bgp *bgp, struct connected *ifc,
+			    struct prefix *p)
 {
 	struct bgp_addr tmp;
 	struct bgp_addr *addr;
+	struct listnode *node;
+	char *name;
 
-	tmp.addr = p->u.prefix4;
+	tmp.p = *p;
+
+	if (tmp.p.family == AF_INET)
+		tmp.p.prefixlen = IPV4_MAX_BITLEN;
+	else if (tmp.p.family == AF_INET6)
+		tmp.p.prefixlen = IPV6_MAX_BITLEN;
 
 	addr = hash_lookup(bgp->address_hash, &tmp);
 	/* may have been deleted earlier by bgp_interface_down() */
 	if (addr == NULL)
 		return;
 
-	addr->refcnt--;
+	for (ALL_LIST_ELEMENTS_RO(addr->ifp_name_list, node, name)) {
+		if (strcmp(ifc->ifp->name, name) == 0)
+			break;
+	}
 
-	if (addr->refcnt == 0) {
+	if (node) {
+		list_delete_node(addr->ifp_name_list, node);
+		XFREE(MTYPE_MARTIAN_STRING, name);
+	}
+
+	if (addr->ifp_name_list->count == 0) {
 		hash_release(bgp->address_hash, addr);
+		list_delete(&addr->ifp_name_list);
 		XFREE(MTYPE_BGP_ADDR, addr);
 	}
 }
@@ -274,18 +358,18 @@ void bgp_connected_add(struct bgp *bgp, struct connected *ifc)
 		if (prefix_ipv4_any((struct prefix_ipv4 *)&p))
 			return;
 
-		bgp_address_add(bgp, addr);
+		bgp_address_add(bgp, ifc, addr);
 
 		rn = bgp_node_get(bgp->connected_table[AFI_IP],
 				  (struct prefix *)&p);
-		if (rn->info) {
-			bc = rn->info;
+		bc = bgp_node_get_bgp_connected_ref_info(rn);
+		if (bc)
 			bc->refcnt++;
-		} else {
+		else {
 			bc = XCALLOC(MTYPE_BGP_CONN,
 				     sizeof(struct bgp_connected_ref));
 			bc->refcnt = 1;
-			rn->info = bc;
+			bgp_node_set_bgp_connected_ref_info(rn, bc);
 		}
 
 		for (ALL_LIST_ELEMENTS(bgp->peer, node, nnode, peer)) {
@@ -308,16 +392,19 @@ void bgp_connected_add(struct bgp *bgp, struct connected *ifc)
 		if (IN6_IS_ADDR_LINKLOCAL(&p.u.prefix6))
 			return;
 
+		bgp_address_add(bgp, ifc, addr);
+
 		rn = bgp_node_get(bgp->connected_table[AFI_IP6],
 				  (struct prefix *)&p);
-		if (rn->info) {
-			bc = rn->info;
+
+		bc = bgp_node_get_bgp_connected_ref_info(rn);
+		if (bc)
 			bc->refcnt++;
-		} else {
+		else {
 			bc = XCALLOC(MTYPE_BGP_CONN,
 				     sizeof(struct bgp_connected_ref));
 			bc->refcnt = 1;
-			rn->info = bc;
+			bgp_node_set_bgp_connected_ref_info(rn, bc);
 		}
 	}
 }
@@ -326,77 +413,138 @@ void bgp_connected_delete(struct bgp *bgp, struct connected *ifc)
 {
 	struct prefix p;
 	struct prefix *addr;
-	struct bgp_node *rn;
+	struct bgp_node *rn = NULL;
 	struct bgp_connected_ref *bc;
 
 	addr = ifc->address;
 
 	p = *(CONNECTED_PREFIX(ifc));
+	apply_mask(&p);
 	if (addr->family == AF_INET) {
-		apply_mask_ipv4((struct prefix_ipv4 *)&p);
-
 		if (prefix_ipv4_any((struct prefix_ipv4 *)&p))
 			return;
 
-		bgp_address_del(bgp, addr);
+		bgp_address_del(bgp, ifc, addr);
 
 		rn = bgp_node_lookup(bgp->connected_table[AFI_IP], &p);
-		if (!rn)
-			return;
-
-		bc = rn->info;
-		bc->refcnt--;
-		if (bc->refcnt == 0) {
-			XFREE(MTYPE_BGP_CONN, bc);
-			rn->info = NULL;
-		}
-		bgp_unlock_node(rn);
-		bgp_unlock_node(rn);
 	} else if (addr->family == AF_INET6) {
-		apply_mask_ipv6((struct prefix_ipv6 *)&p);
-
 		if (IN6_IS_ADDR_UNSPECIFIED(&p.u.prefix6))
 			return;
 
 		if (IN6_IS_ADDR_LINKLOCAL(&p.u.prefix6))
 			return;
 
-		rn = bgp_node_lookup(bgp->connected_table[AFI_IP6],
-				     (struct prefix *)&p);
-		if (!rn)
-			return;
+		bgp_address_del(bgp, ifc, addr);
 
-		bc = rn->info;
-		bc->refcnt--;
-		if (bc->refcnt == 0) {
-			XFREE(MTYPE_BGP_CONN, bc);
-			rn->info = NULL;
-		}
-		bgp_unlock_node(rn);
-		bgp_unlock_node(rn);
+		rn = bgp_node_lookup(bgp->connected_table[AFI_IP6], &p);
+	}
+
+	if (!rn)
+		return;
+
+	bc = bgp_node_get_bgp_connected_ref_info(rn);
+	bc->refcnt--;
+	if (bc->refcnt == 0) {
+		XFREE(MTYPE_BGP_CONN, bc);
+		bgp_node_set_bgp_connected_ref_info(rn, NULL);
+	}
+	bgp_unlock_node(rn);
+	bgp_unlock_node(rn);
+}
+
+static void bgp_connected_cleanup(struct route_table *table,
+				  struct route_node *rn)
+{
+	struct bgp_connected_ref *bc;
+	struct bgp_node *bn = bgp_node_from_rnode(rn);
+
+	bc = bgp_node_get_bgp_connected_ref_info(bn);
+	if (!bc)
+		return;
+
+	bc->refcnt--;
+	if (bc->refcnt == 0) {
+		XFREE(MTYPE_BGP_CONN, bc);
+		bgp_node_set_bgp_connected_ref_info(bn, NULL);
 	}
 }
 
-int bgp_nexthop_self(struct bgp *bgp, struct in_addr nh_addr)
+bool bgp_nexthop_self(struct bgp *bgp, afi_t afi, uint8_t type,
+		      uint8_t sub_type, struct attr *attr, struct bgp_node *rn)
 {
-	struct bgp_addr tmp, *addr;
-	struct tip_addr tmp_tip, *tip;
+	uint8_t new_afi = afi == AFI_IP ? AF_INET : AF_INET6;
+	struct bgp_addr tmp_addr = {{0}}, *addr = NULL;
+	struct tip_addr tmp_tip, *tip = NULL;
+	const struct prefix *p = bgp_node_get_prefix(rn);
+	bool is_bgp_static_route =
+		((type == ZEBRA_ROUTE_BGP) && (sub_type == BGP_ROUTE_STATIC))
+			? true
+			: false;
 
-	tmp.addr = nh_addr;
+	if (!is_bgp_static_route)
+		new_afi = BGP_ATTR_NEXTHOP_AFI_IP6(attr) ? AF_INET6 : AF_INET;
 
-	addr = hash_lookup(bgp->address_hash, &tmp);
+	tmp_addr.p.family = new_afi;
+	switch (new_afi) {
+	case AF_INET:
+		if (is_bgp_static_route) {
+			tmp_addr.p.u.prefix4 = p->u.prefix4;
+			tmp_addr.p.prefixlen = p->prefixlen;
+		} else {
+			/* Here we need to find out which nexthop to be used*/
+			if (attr->flag & ATTR_FLAG_BIT(BGP_ATTR_NEXT_HOP)) {
+				tmp_addr.p.u.prefix4 = attr->nexthop;
+				tmp_addr.p.prefixlen = IPV4_MAX_BITLEN;
+			} else if ((attr->mp_nexthop_len)
+				   && ((attr->mp_nexthop_len
+					== BGP_ATTR_NHLEN_IPV4)
+				       || (attr->mp_nexthop_len
+					   == BGP_ATTR_NHLEN_VPNV4))) {
+				tmp_addr.p.u.prefix4 =
+					attr->mp_nexthop_global_in;
+				tmp_addr.p.prefixlen = IPV4_MAX_BITLEN;
+			} else
+				return false;
+		}
+		break;
+	case AF_INET6:
+		if (is_bgp_static_route) {
+			tmp_addr.p.u.prefix6 = p->u.prefix6;
+			tmp_addr.p.prefixlen = p->prefixlen;
+		} else {
+			tmp_addr.p.u.prefix6 = attr->mp_nexthop_global;
+			tmp_addr.p.prefixlen = IPV6_MAX_BITLEN;
+		}
+		break;
+	default:
+		break;
+	}
+
+	addr = hash_lookup(bgp->address_hash, &tmp_addr);
 	if (addr)
-		return 1;
+		return true;
 
-	tmp_tip.addr = nh_addr;
-	tip = hash_lookup(bgp->tip_hash, &tmp_tip);
-	if (tip)
-		return 1;
+	if (new_afi == AF_INET) {
+		memset(&tmp_tip, 0, sizeof(struct tip_addr));
+		tmp_tip.addr = attr->nexthop;
 
-	return 0;
+		if (attr->flag & ATTR_FLAG_BIT(BGP_ATTR_NEXT_HOP)) {
+			tmp_tip.addr = attr->nexthop;
+		} else if ((attr->mp_nexthop_len) &&
+			   ((attr->mp_nexthop_len == BGP_ATTR_NHLEN_IPV4)
+			    || (attr->mp_nexthop_len == BGP_ATTR_NHLEN_VPNV4))) {
+			tmp_tip.addr = attr->mp_nexthop_global_in;
+		}
+
+		tip = hash_lookup(bgp->tip_hash, &tmp_tip);
+		if (tip)
+			return true;
+	}
+
+	return false;
 }
 
-int bgp_multiaccess_check_v4(struct in_addr nexthop, struct peer *peer)
+bool bgp_multiaccess_check_v4(struct in_addr nexthop, struct peer *peer)
 {
 	struct bgp_node *rn1;
 	struct bgp_node *rn2;
@@ -409,7 +557,7 @@ int bgp_multiaccess_check_v4(struct in_addr nexthop, struct peer *peer)
 
 	rn1 = bgp_node_match(peer->bgp->connected_table[AFI_IP], &p);
 	if (!rn1)
-		return 0;
+		return false;
 
 	p.family = AF_INET;
 	p.prefixlen = IPV4_MAX_BITLEN;
@@ -418,24 +566,100 @@ int bgp_multiaccess_check_v4(struct in_addr nexthop, struct peer *peer)
 	rn2 = bgp_node_match(peer->bgp->connected_table[AFI_IP], &p);
 	if (!rn2) {
 		bgp_unlock_node(rn1);
-		return 0;
+		return false;
 	}
 
-	ret = (rn1 == rn2) ? 1 : 0;
+	ret = (rn1 == rn2);
 
 	bgp_unlock_node(rn1);
 	bgp_unlock_node(rn2);
 
-	return (ret);
+	return ret;
 }
 
-int bgp_subgrp_multiaccess_check_v4(struct in_addr nexthop,
-				    struct update_subgroup *subgrp)
+bool bgp_multiaccess_check_v6(struct in6_addr nexthop, struct peer *peer)
+{
+	struct bgp_node *rn1;
+	struct bgp_node *rn2;
+	struct prefix p;
+	int ret;
+
+	p.family = AF_INET6;
+	p.prefixlen = IPV6_MAX_BITLEN;
+	p.u.prefix6 = nexthop;
+
+	rn1 = bgp_node_match(peer->bgp->connected_table[AFI_IP6], &p);
+	if (!rn1)
+		return false;
+
+	p.family = AF_INET6;
+	p.prefixlen = IPV6_MAX_BITLEN;
+	p.u.prefix6 = peer->su.sin6.sin6_addr;
+
+	rn2 = bgp_node_match(peer->bgp->connected_table[AFI_IP6], &p);
+	if (!rn2) {
+		bgp_unlock_node(rn1);
+		return false;
+	}
+
+	ret = (rn1 == rn2);
+
+	bgp_unlock_node(rn1);
+	bgp_unlock_node(rn2);
+
+	return ret;
+}
+
+bool bgp_subgrp_multiaccess_check_v6(struct in6_addr nexthop,
+				     struct update_subgroup *subgrp,
+				     struct peer *exclude)
+{
+	struct bgp_node *rn1 = NULL, *rn2 = NULL;
+	struct peer_af *paf = NULL;
+	struct prefix p = {0}, np = {0};
+	struct bgp *bgp = NULL;
+
+	np.family = AF_INET6;
+	np.prefixlen = IPV6_MAX_BITLEN;
+	np.u.prefix6 = nexthop;
+
+	p.family = AF_INET;
+	p.prefixlen = IPV6_MAX_BITLEN;
+
+	bgp = SUBGRP_INST(subgrp);
+	rn1 = bgp_node_match(bgp->connected_table[AFI_IP6], &np);
+	if (!rn1)
+		return false;
+
+	SUBGRP_FOREACH_PEER (subgrp, paf) {
+		/* Skip peer we're told to exclude - e.g., source of route. */
+		if (paf->peer == exclude)
+			continue;
+
+		p.u.prefix6 = paf->peer->su.sin6.sin6_addr;
+		rn2 = bgp_node_match(bgp->connected_table[AFI_IP6], &p);
+		if (rn1 == rn2) {
+			bgp_unlock_node(rn1);
+			bgp_unlock_node(rn2);
+			return true;
+		}
+
+		if (rn2)
+			bgp_unlock_node(rn2);
+	}
+
+	bgp_unlock_node(rn1);
+	return false;
+}
+
+bool bgp_subgrp_multiaccess_check_v4(struct in_addr nexthop,
+				     struct update_subgroup *subgrp,
+				     struct peer *exclude)
 {
 	struct bgp_node *rn1, *rn2;
 	struct peer_af *paf;
 	struct prefix p, np;
-	struct bgp *bgp = NULL;
+	struct bgp *bgp;
 
 	np.family = AF_INET;
 	np.prefixlen = IPV4_MAX_BITLEN;
@@ -444,21 +668,23 @@ int bgp_subgrp_multiaccess_check_v4(struct in_addr nexthop,
 	p.family = AF_INET;
 	p.prefixlen = IPV4_MAX_BITLEN;
 
-	rn1 = rn2 = NULL;
-
 	bgp = SUBGRP_INST(subgrp);
 	rn1 = bgp_node_match(bgp->connected_table[AFI_IP], &np);
 	if (!rn1)
-		return 0;
+		return false;
 
 	SUBGRP_FOREACH_PEER (subgrp, paf) {
+		/* Skip peer we're told to exclude - e.g., source of route. */
+		if (paf->peer == exclude)
+			continue;
+
 		p.u.prefix4 = paf->peer->su.sin.sin_addr;
 
 		rn2 = bgp_node_match(bgp->connected_table[AFI_IP], &p);
 		if (rn1 == rn2) {
 			bgp_unlock_node(rn1);
 			bgp_unlock_node(rn2);
-			return 1;
+			return true;
 		}
 
 		if (rn2)
@@ -466,7 +692,7 @@ int bgp_subgrp_multiaccess_check_v4(struct in_addr nexthop,
 	}
 
 	bgp_unlock_node(rn1);
-	return 0;
+	return false;
 }
 
 static void bgp_show_nexthops_detail(struct vty *vty, struct bgp *bgp,
@@ -512,56 +738,75 @@ static void bgp_show_nexthops_detail(struct vty *vty, struct bgp *bgp,
 		}
 }
 
-static void bgp_show_nexthops(struct vty *vty, struct bgp *bgp, int detail)
+static void bgp_show_nexthops(struct vty *vty, struct bgp *bgp, int detail,
+			     bool import_table)
 {
 	struct bgp_node *rn;
 	struct bgp_nexthop_cache *bnc;
 	char buf[PREFIX2STR_BUFFER];
 	time_t tbuf;
 	afi_t afi;
+	struct bgp_table **table;
 
-	vty_out(vty, "Current BGP nexthop cache:\n");
+	if (import_table)
+		vty_out(vty, "Current BGP import check cache:\n");
+	else
+		vty_out(vty, "Current BGP nexthop cache:\n");
+	if (import_table)
+		table = bgp->import_check_table;
+	else
+		table = bgp->nexthop_cache_table;
 	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
-		if (!bgp->nexthop_cache_table[afi])
+		if (!table || !table[afi])
 			continue;
-
-		for (rn = bgp_table_top(bgp->nexthop_cache_table[afi]); rn;
+		for (rn = bgp_table_top(table[afi]); rn;
 		     rn = bgp_route_next(rn)) {
-			if ((bnc = rn->info) != NULL) {
-				if (CHECK_FLAG(bnc->flags, BGP_NEXTHOP_VALID)) {
-					vty_out(vty,
-						" %s valid [IGP metric %d], #paths %d\n",
-						inet_ntop(rn->p.family,
-							  &rn->p.u.prefix, buf,
-							  sizeof(buf)),
-						bnc->metric, bnc->path_count);
+			struct peer *peer;
+			const struct prefix *p = bgp_node_get_prefix(rn);
 
-					if (!detail)
-						continue;
+			bnc = bgp_node_get_bgp_nexthop_info(rn);
+			if (!bnc)
+				continue;
+			peer = (struct peer *)bnc->nht_info;
 
-					bgp_show_nexthops_detail(vty, bgp, bnc);
-
-				} else {
-					vty_out(vty, " %s invalid\n",
-						inet_ntop(rn->p.family,
-							  &rn->p.u.prefix, buf,
-							  sizeof(buf)));
-					if (CHECK_FLAG(bnc->flags,
-						       BGP_NEXTHOP_CONNECTED))
-						vty_out(vty,
-							"  Must be Connected\n");
-				}
-				tbuf = time(NULL)
-				       - (bgp_clock() - bnc->last_update);
-				vty_out(vty, "  Last update: %s", ctime(&tbuf));
+			if (CHECK_FLAG(bnc->flags, BGP_NEXTHOP_VALID)) {
+				vty_out(vty,
+					" %s valid [IGP metric %d], #paths %d",
+					inet_ntop(p->family, &p->u.prefix, buf,
+						  sizeof(buf)),
+					bnc->metric, bnc->path_count);
+				if (peer)
+					vty_out(vty, ", peer %s", peer->host);
 				vty_out(vty, "\n");
+
+				if (!detail)
+					continue;
+
+				bgp_show_nexthops_detail(vty, bgp, bnc);
+
+			} else {
+				vty_out(vty, " %s invalid",
+					inet_ntop(p->family, &p->u.prefix, buf,
+						  sizeof(buf)));
+				if (peer)
+					vty_out(vty, ", peer %s", peer->host);
+				vty_out(vty, "\n");
+				if (CHECK_FLAG(bnc->flags,
+					       BGP_NEXTHOP_CONNECTED))
+					vty_out(vty, "  Must be Connected\n");
+				if (!CHECK_FLAG(bnc->flags,
+						BGP_NEXTHOP_REGISTERED))
+					vty_out(vty, "  Is not Registered\n");
 			}
+			tbuf = time(NULL) - (bgp_clock() - bnc->last_update);
+			vty_out(vty, "  Last update: %s", ctime(&tbuf));
+			vty_out(vty, "\n");
 		}
 	}
 }
 
 static int show_ip_bgp_nexthop_table(struct vty *vty, const char *name,
-				     int detail)
+				     int detail, bool import_table)
 {
 	struct bgp *bgp;
 
@@ -574,7 +819,7 @@ static int show_ip_bgp_nexthop_table(struct vty *vty, const char *name,
 		return CMD_WARNING;
 	}
 
-	bgp_show_nexthops(vty, bgp, detail);
+	bgp_show_nexthops(vty, bgp, detail, import_table);
 
 	return CMD_SUCCESS;
 }
@@ -587,9 +832,9 @@ static void bgp_show_all_instances_nexthops_vty(struct vty *vty)
 	for (ALL_LIST_ELEMENTS(bm->bgp, node, nnode, bgp)) {
 		vty_out(vty, "\nInstance %s:\n",
 			(bgp->inst_type == BGP_INSTANCE_TYPE_DEFAULT)
-				? "Default"
+				? VRF_DEFAULT_NAME
 				: bgp->name);
-		bgp_show_nexthops(vty, bgp, 0);
+		bgp_show_nexthops(vty, bgp, 0, false);
 	}
 }
 
@@ -610,7 +855,28 @@ DEFUN (show_ip_bgp_nexthop,
 	    || argv_find(argv, argc, "vrf", &idx))
 		vrf = argv[++idx]->arg;
 	int detail = argv_find(argv, argc, "detail", &idx) ? 1 : 0;
-	return show_ip_bgp_nexthop_table(vty, vrf, detail);
+
+	return show_ip_bgp_nexthop_table(vty, vrf, detail, false);
+}
+
+DEFUN (show_ip_bgp_import_check,
+       show_ip_bgp_import_check_cmd,
+       "show [ip] bgp [<view|vrf> VIEWVRFNAME] import-check-table [detail]",
+       SHOW_STR
+       IP_STR
+       BGP_STR
+       BGP_INSTANCE_HELP_STR
+       "BGP import check table\n"
+       "Show detailed information\n")
+{
+	int idx = 0;
+	char *vrf = NULL;
+
+	if (argv_find(argv, argc, "view", &idx)
+	    || argv_find(argv, argc, "vrf", &idx))
+		vrf = argv[++idx]->arg;
+	int detail = argv_find(argv, argc, "detail", &idx) ? 1 : 0;
+	return show_ip_bgp_nexthop_table(vty, vrf, detail, true);
 }
 
 DEFUN (show_ip_bgp_instance_all_nexthop,
@@ -643,6 +909,7 @@ void bgp_scan_init(struct bgp *bgp)
 void bgp_scan_vty_init(void)
 {
 	install_element(VIEW_NODE, &show_ip_bgp_nexthop_cmd);
+	install_element(VIEW_NODE, &show_ip_bgp_import_check_cmd);
 	install_element(VIEW_NODE, &show_ip_bgp_instance_all_nexthop_cmd);
 }
 
@@ -656,6 +923,8 @@ void bgp_scan_finish(struct bgp *bgp)
 		bgp_table_unlock(bgp->nexthop_cache_table[afi]);
 		bgp->nexthop_cache_table[afi] = NULL;
 
+		bgp->connected_table[afi]->route_table->cleanup =
+			bgp_connected_cleanup;
 		bgp_table_unlock(bgp->connected_table[afi]);
 		bgp->connected_table[afi] = NULL;
 
