@@ -38,9 +38,12 @@
 #include "bgpd/bgp_attr.h"
 #include "bgpd/bgp_nexthop.h"
 #include "bgpd/bgp_debug.h"
+#include "bgpd/bgp_errors.h"
 #include "bgpd/bgp_nht.h"
 #include "bgpd/bgp_fsm.h"
 #include "bgpd/bgp_zebra.h"
+#include "bgpd/bgp_flowspec_util.h"
+#include "bgpd/bgp_evpn.h"
 
 extern struct zclient *zclient;
 
@@ -49,9 +52,7 @@ static void register_zebra_rnh(struct bgp_nexthop_cache *bnc,
 static void unregister_zebra_rnh(struct bgp_nexthop_cache *bnc,
 				 int is_bgp_static_route);
 static void evaluate_paths(struct bgp_nexthop_cache *bnc);
-static int make_prefix(int afi, struct bgp_info *ri, struct prefix *p);
-static void path_nh_map(struct bgp_info *path, struct bgp_nexthop_cache *bnc,
-			int keep);
+static int make_prefix(int afi, struct bgp_path_info *pi, struct prefix *p);
 
 static int bgp_isvalid_nexthop(struct bgp_nexthop_cache *bnc)
 {
@@ -65,52 +66,32 @@ static int bgp_isvalid_labeled_nexthop(struct bgp_nexthop_cache *bnc)
 		|| (bnc && CHECK_FLAG(bnc->flags, BGP_NEXTHOP_LABELED_VALID)));
 }
 
-int bgp_find_nexthop(struct bgp_info *path, int connected)
-{
-	struct bgp_nexthop_cache *bnc = path->nexthop;
-
-	if (!bnc)
-		return 0;
-
-	/*
-	 * We are cheating here.  Views have no associated underlying
-	 * ability to detect nexthops.  So when we have a view
-	 * just tell everyone the nexthop is valid
-	 */
-	if (path->peer && path->peer->bgp->inst_type == BGP_INSTANCE_TYPE_VIEW)
-		return 1;
-
-	if (connected && !(CHECK_FLAG(bnc->flags, BGP_NEXTHOP_CONNECTED)))
-		return 0;
-
-	return (bgp_isvalid_nexthop(bnc));
-}
-
 static void bgp_unlink_nexthop_check(struct bgp_nexthop_cache *bnc)
 {
 	if (LIST_EMPTY(&(bnc->paths)) && !bnc->nht_info) {
 		if (BGP_DEBUG(nht, NHT)) {
 			char buf[PREFIX2STR_BUFFER];
-			zlog_debug("bgp_unlink_nexthop: freeing bnc %s",
-				   bnc_str(bnc, buf, PREFIX2STR_BUFFER));
+			zlog_debug("bgp_unlink_nexthop: freeing bnc %s(%s)",
+				   bnc_str(bnc, buf, PREFIX2STR_BUFFER),
+				   bnc->bgp->name_pretty);
 		}
 		unregister_zebra_rnh(bnc,
 				     CHECK_FLAG(bnc->flags, BGP_STATIC_ROUTE));
-		bnc->node->info = NULL;
+		bgp_node_set_bgp_nexthop_info(bnc->node, NULL);
 		bgp_unlock_node(bnc->node);
 		bnc->node = NULL;
 		bnc_free(bnc);
 	}
 }
 
-void bgp_unlink_nexthop(struct bgp_info *path)
+void bgp_unlink_nexthop(struct bgp_path_info *path)
 {
 	struct bgp_nexthop_cache *bnc = path->nexthop;
 
 	if (!bnc)
 		return;
 
-	path_nh_map(path, NULL, 0);
+	path_nh_map(path, NULL, false);
 
 	bgp_unlink_nexthop_check(bnc);
 }
@@ -127,11 +108,10 @@ void bgp_unlink_nexthop_by_peer(struct peer *peer)
 
 	rn = bgp_node_get(peer->bgp->nexthop_cache_table[afi], &p);
 
-	if (!rn->info)
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc)
 		return;
 
-	bnc = rn->info;
-
 	/* cleanup the peer reference */
 	bnc->nht_info = NULL;
 
@@ -143,17 +123,18 @@ void bgp_unlink_nexthop_by_peer(struct peer *peer)
  * we need both the bgp_route and bgp_nexthop pointers.
  */
 int bgp_find_or_add_nexthop(struct bgp *bgp_route, struct bgp *bgp_nexthop,
-			    afi_t afi, struct bgp_info *ri,
+			    afi_t afi, struct bgp_path_info *pi,
 			    struct peer *peer, int connected)
 {
 	struct bgp_node *rn;
 	struct bgp_nexthop_cache *bnc;
 	struct prefix p;
 	int is_bgp_static_route = 0;
+	const struct prefix *bnc_p;
 
-	if (ri) {
-		is_bgp_static_route = ((ri->type == ZEBRA_ROUTE_BGP)
-				       && (ri->sub_type == BGP_ROUTE_STATIC))
+	if (pi) {
+		is_bgp_static_route = ((pi->type == ZEBRA_ROUTE_BGP)
+				       && (pi->sub_type == BGP_ROUTE_STATIC))
 					      ? 1
 					      : 0;
 
@@ -161,24 +142,19 @@ int bgp_find_or_add_nexthop(struct bgp *bgp_route, struct bgp *bgp_nexthop,
 		   to derive
 		   address-family from the next-hop. */
 		if (!is_bgp_static_route)
-			afi = BGP_ATTR_NEXTHOP_AFI_IP6(ri->attr) ? AFI_IP6
+			afi = BGP_ATTR_NEXTHOP_AFI_IP6(pi->attr) ? AFI_IP6
 								 : AFI_IP;
 
-		/* This will return TRUE if the global IPv6 NH is a link local
+		/* This will return true if the global IPv6 NH is a link local
 		 * addr */
-		if (make_prefix(afi, ri, &p) < 0)
+		if (make_prefix(afi, pi, &p) < 0)
 			return 1;
 	} else if (peer) {
-		/* Don't register link local NH */
-		if (afi == AFI_IP6
-		    && IN6_IS_ADDR_LINKLOCAL(&peer->su.sin6.sin6_addr))
-			return 1;
-
 		if (!sockunion2hostprefix(&peer->su, &p)) {
 			if (BGP_DEBUG(nht, NHT)) {
 				zlog_debug(
 					"%s: Attempting to register with unknown AFI %d (not %d or %d)",
-					__FUNCTION__, afi, AFI_IP, AFI_IP6);
+					__func__, afi, AFI_IP, AFI_IP6);
 			}
 			return 0;
 		}
@@ -190,32 +166,36 @@ int bgp_find_or_add_nexthop(struct bgp *bgp_route, struct bgp *bgp_nexthop,
 	else
 		rn = bgp_node_get(bgp_nexthop->nexthop_cache_table[afi], &p);
 
-	if (!rn->info) {
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc) {
 		bnc = bnc_new();
-		rn->info = bnc;
+		bgp_node_set_bgp_nexthop_info(rn, bnc);
 		bnc->node = rn;
 		bnc->bgp = bgp_nexthop;
 		bgp_lock_node(rn);
 		if (BGP_DEBUG(nht, NHT)) {
 			char buf[PREFIX2STR_BUFFER];
 
-			zlog_debug("Allocated bnc %s peer %p",
-				   bnc_str(bnc, buf, PREFIX2STR_BUFFER), peer);
+			zlog_debug("Allocated bnc %s(%s) peer %p",
+				   bnc_str(bnc, buf, PREFIX2STR_BUFFER),
+				   bnc->bgp->name_pretty, peer);
 		}
 	}
 
-	bnc = rn->info;
+	bnc_p = bgp_node_get_prefix(bnc->node);
+
 	bgp_unlock_node(rn);
 	if (is_bgp_static_route) {
 		SET_FLAG(bnc->flags, BGP_STATIC_ROUTE);
 
 		/* If we're toggling the type, re-register */
-		if ((bgp_flag_check(bgp_route, BGP_FLAG_IMPORT_CHECK))
+		if ((CHECK_FLAG(bgp_route->flags, BGP_FLAG_IMPORT_CHECK))
 		    && !CHECK_FLAG(bnc->flags, BGP_STATIC_ROUTE_EXACT_MATCH)) {
 			SET_FLAG(bnc->flags, BGP_STATIC_ROUTE_EXACT_MATCH);
 			UNSET_FLAG(bnc->flags, BGP_NEXTHOP_REGISTERED);
 			UNSET_FLAG(bnc->flags, BGP_NEXTHOP_VALID);
-		} else if ((!bgp_flag_check(bgp_route, BGP_FLAG_IMPORT_CHECK))
+		} else if ((!CHECK_FLAG(bgp_route->flags,
+					BGP_FLAG_IMPORT_CHECK))
 			   && CHECK_FLAG(bnc->flags,
 					 BGP_STATIC_ROUTE_EXACT_MATCH)) {
 			UNSET_FLAG(bnc->flags, BGP_STATIC_ROUTE_EXACT_MATCH);
@@ -247,23 +227,26 @@ int bgp_find_or_add_nexthop(struct bgp *bgp_route, struct bgp *bgp_nexthop,
 		UNSET_FLAG(bnc->flags, BGP_NEXTHOP_VALID);
 	}
 	if (bgp_route->inst_type == BGP_INSTANCE_TYPE_VIEW) {
-		bnc->flags |= BGP_NEXTHOP_REGISTERED;
-		bnc->flags |= BGP_NEXTHOP_VALID;
-	} else if (!CHECK_FLAG(bnc->flags, BGP_NEXTHOP_REGISTERED))
+		SET_FLAG(bnc->flags, BGP_NEXTHOP_REGISTERED);
+		SET_FLAG(bnc->flags, BGP_NEXTHOP_VALID);
+	} else if (!CHECK_FLAG(bnc->flags, BGP_NEXTHOP_REGISTERED)
+		   && !is_default_host_route(bnc_p))
 		register_zebra_rnh(bnc, is_bgp_static_route);
-	if (ri && ri->nexthop != bnc) {
+
+	if (pi && pi->nexthop != bnc) {
 		/* Unlink from existing nexthop cache, if any. This will also
 		 * free
 		 * the nexthop cache entry, if appropriate.
 		 */
-		bgp_unlink_nexthop(ri);
+		bgp_unlink_nexthop(pi);
 
-		path_nh_map(ri, bnc, 1); /* updates NHT ri list reference */
+		/* updates NHT pi list reference */
+		path_nh_map(pi, bnc, true);
 
 		if (CHECK_FLAG(bnc->flags, BGP_NEXTHOP_VALID) && bnc->metric)
-			(bgp_info_extra_get(ri))->igpmetric = bnc->metric;
-		else if (ri->extra)
-			ri->extra->igpmetric = 0;
+			(bgp_path_info_extra_get(pi))->igpmetric = bnc->metric;
+		else if (pi->extra)
+			pi->extra->igpmetric = 0;
 	} else if (peer)
 		bnc->nht_info = (void *)peer; /* NHT peer reference */
 
@@ -287,32 +270,36 @@ void bgp_delete_connected_nexthop(afi_t afi, struct peer *peer)
 	if (!peer)
 		return;
 
-	/* We don't register link local address for NHT */
-	if (afi == AFI_IP6 && IN6_IS_ADDR_LINKLOCAL(&peer->su.sin6.sin6_addr))
-		return;
-
 	if (!sockunion2hostprefix(&peer->su, &p))
 		return;
 
 	rn = bgp_node_lookup(
 		peer->bgp->nexthop_cache_table[family2afi(p.family)], &p);
-	if (!rn || !rn->info) {
+	if (!rn) {
 		if (BGP_DEBUG(nht, NHT))
-			zlog_debug("Cannot find connected NHT node for peer %s",
-				   peer->host);
-		if (rn)
-			bgp_unlock_node(rn);
+			zlog_debug(
+				"Cannot find connected NHT node for peer %s(%s)",
+				peer->host, peer->bgp->name_pretty);
 		return;
 	}
 
-	bnc = rn->info;
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc) {
+		if (BGP_DEBUG(nht, NHT))
+			zlog_debug(
+				"Cannot find connected NHT node for peer %s(%s) on route_node as expected",
+				peer->host, peer->bgp->name_pretty);
+		bgp_unlock_node(rn);
+		return;
+	}
 	bgp_unlock_node(rn);
 
 	if (bnc->nht_info != peer) {
 		if (BGP_DEBUG(nht, NHT))
 			zlog_debug(
-				"Connected NHT %p node for peer %s points to %p",
-				bnc, peer->host, bnc->nht_info);
+				"Connected NHT %p node for peer %s(%s) points to %p",
+				bnc, peer->host, bnc->bgp->name_pretty,
+				bnc->nht_info);
 		return;
 	}
 
@@ -320,10 +307,11 @@ void bgp_delete_connected_nexthop(afi_t afi, struct peer *peer)
 
 	if (LIST_EMPTY(&(bnc->paths))) {
 		if (BGP_DEBUG(nht, NHT))
-			zlog_debug("Freeing connected NHT node %p for peer %s",
-				   bnc, peer->host);
+			zlog_debug(
+				"Freeing connected NHT node %p for peer %s(%s)",
+				bnc, peer->host, bnc->bgp->name_pretty);
 		unregister_zebra_rnh(bnc, 0);
-		bnc->node->info = NULL;
+		bgp_node_set_bgp_nexthop_info(bnc->node, NULL);
 		bgp_unlock_node(bnc->node);
 		bnc_free(bnc);
 	}
@@ -343,7 +331,8 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 
 	bgp = bgp_lookup_by_vrf_id(vrf_id);
 	if (!bgp) {
-		zlog_err(
+		flog_err(
+			EC_BGP_NH_UPD,
 			"parse nexthop update: instance not found for vrf_id %u",
 			vrf_id);
 		return;
@@ -351,8 +340,8 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 
 	if (!zapi_nexthop_update_decode(zclient->ibuf, &nhr)) {
 		if (BGP_DEBUG(nht, NHT))
-			zlog_debug("%s: Failure to decode nexthop update",
-				   __PRETTY_FUNCTION__);
+			zlog_debug("%s[%s]: Failure to decode nexthop update",
+				   __func__, bgp->name_pretty);
 		return;
 	}
 
@@ -365,19 +354,30 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 			bgp->import_check_table[family2afi(nhr.prefix.family)],
 			&nhr.prefix);
 
-	if (!rn || !rn->info) {
+	if (!rn) {
 		if (BGP_DEBUG(nht, NHT)) {
 			char buf[PREFIX2STR_BUFFER];
 			prefix2str(&nhr.prefix, buf, sizeof(buf));
-			zlog_debug("parse nexthop update(%s): rn not found",
-				   buf);
+			zlog_debug("parse nexthop update(%s(%s)): rn not found",
+				   buf, bgp->name_pretty);
 		}
-		if (rn)
-			bgp_unlock_node(rn);
 		return;
 	}
 
-	bnc = rn->info;
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc) {
+		if (BGP_DEBUG(nht, NHT)) {
+			char buf[PREFIX2STR_BUFFER];
+
+			prefix2str(&nhr.prefix, buf, sizeof(buf));
+			zlog_debug(
+				"parse nexthop update(%s(%s)): bnc node info not found",
+				buf, bgp->name_pretty);
+		}
+		bgp_unlock_node(rn);
+		return;
+	}
+
 	bgp_unlock_node(rn);
 	bnc->last_update = bgp_clock();
 	bnc->change_flags = 0;
@@ -387,9 +387,10 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 		char buf[PREFIX2STR_BUFFER];
 		prefix2str(&nhr.prefix, buf, sizeof(buf));
 		zlog_debug(
-			"%u: Rcvd NH update %s - metric %d/%d #nhops %d/%d flags 0x%x",
-			vrf_id, buf, nhr.metric, bnc->metric, nhr.nexthop_num,
-			bnc->nexthop_num, bnc->flags);
+			"%s(%u): Rcvd NH update %s - metric %d/%d #nhops %d/%d flags 0x%x",
+			bnc->bgp->name_pretty, vrf_id, buf, nhr.metric,
+			bnc->metric, nhr.nexthop_num, bnc->nexthop_num,
+			bnc->flags);
 	}
 
 	if (nhr.metric != bnc->metric)
@@ -399,6 +400,8 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 		bnc->change_flags |= BGP_NEXTHOP_CHANGED;
 
 	if (nhr.nexthop_num) {
+		struct peer *peer = bnc->nht_info;
+
 		/* notify bgp fsm if nbr ip goes from invalid->valid */
 		if (!bnc->nexthop_num)
 			UNSET_FLAG(bnc->flags, BGP_NEXTHOP_PEER_NOTIFIED);
@@ -414,6 +417,26 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 
 			nexthop = nexthop_from_zapi_nexthop(&nhr.nexthops[i]);
 
+			/*
+			 * Turn on RA for the v6 nexthops
+			 * we receive from bgp.  This is to allow us
+			 * to work with v4 routing over v6 nexthops
+			 */
+			if (peer && !peer->ifp
+			    && CHECK_FLAG(peer->flags,
+					  PEER_FLAG_CAPABILITY_ENHE)
+			    && nhr.prefix.family == AF_INET6
+			    && nexthop->type != NEXTHOP_TYPE_BLACKHOLE) {
+				struct interface *ifp;
+
+				ifp = if_lookup_by_index(nexthop->ifindex,
+							 nexthop->vrf_id);
+				if (ifp)
+					zclient_send_interface_radv_req(
+						zclient, nexthop->vrf_id, ifp,
+						true,
+						BGP_UNNUM_DEFAULT_RA_INTERVAL);
+			}
 			/* There is at least one label-switched path */
 			if (nexthop->nh_label &&
 				nexthop->nh_label->num_labels) {
@@ -446,8 +469,7 @@ void bgp_parse_nexthop_update(int command, vrf_id_t vrf_id)
 				continue;
 
 			for (oldnh = bnc->nexthop; oldnh; oldnh = oldnh->next)
-				if (nexthop_same_no_recurse(oldnh, nexthop) &&
-				    nexthop_labels_match(oldnh, nexthop))
+				if (nexthop_same(oldnh, nexthop))
 					break;
 
 			if (!oldnh)
@@ -485,7 +507,7 @@ void bgp_cleanup_nexthops(struct bgp *bgp)
 
 		for (rn = bgp_table_top(bgp->nexthop_cache_table[afi]); rn;
 		     rn = bgp_route_next(rn)) {
-			bnc = rn->info;
+			bnc = bgp_node_get_bgp_nexthop_info(rn);
 			if (!bnc)
 				continue;
 
@@ -501,39 +523,53 @@ void bgp_cleanup_nexthops(struct bgp *bgp)
  * make_prefix - make a prefix structure from the path (essentially
  * path's node.
  */
-static int make_prefix(int afi, struct bgp_info *ri, struct prefix *p)
+static int make_prefix(int afi, struct bgp_path_info *pi, struct prefix *p)
 {
 
-	int is_bgp_static = ((ri->type == ZEBRA_ROUTE_BGP)
-			     && (ri->sub_type == BGP_ROUTE_STATIC))
+	int is_bgp_static = ((pi->type == ZEBRA_ROUTE_BGP)
+			     && (pi->sub_type == BGP_ROUTE_STATIC))
 				    ? 1
 				    : 0;
+	struct bgp_node *net = pi->net;
+	const struct prefix *p_orig = bgp_node_get_prefix(net);
 
+	if (p_orig->family == AF_FLOWSPEC) {
+		if (!pi->peer)
+			return -1;
+		return bgp_flowspec_get_first_nh(pi->peer->bgp,
+						 pi, p);
+	}
 	memset(p, 0, sizeof(struct prefix));
 	switch (afi) {
 	case AFI_IP:
 		p->family = AF_INET;
 		if (is_bgp_static) {
-			p->u.prefix4 = ri->net->p.u.prefix4;
-			p->prefixlen = ri->net->p.prefixlen;
+			p->u.prefix4 = p_orig->u.prefix4;
+			p->prefixlen = p_orig->prefixlen;
 		} else {
-			p->u.prefix4 = ri->attr->nexthop;
+			p->u.prefix4 = pi->attr->nexthop;
 			p->prefixlen = IPV4_MAX_BITLEN;
 		}
 		break;
 	case AFI_IP6:
-		/* We don't register link local NH */
-		if (ri->attr->mp_nexthop_len != BGP_ATTR_NHLEN_IPV6_GLOBAL
-		    || IN6_IS_ADDR_LINKLOCAL(&ri->attr->mp_nexthop_global))
-			return -1;
-
 		p->family = AF_INET6;
 
 		if (is_bgp_static) {
-			p->u.prefix6 = ri->net->p.u.prefix6;
-			p->prefixlen = ri->net->p.prefixlen;
+			p->u.prefix6 = p_orig->u.prefix6;
+			p->prefixlen = p_orig->prefixlen;
 		} else {
-			p->u.prefix6 = ri->attr->mp_nexthop_global;
+			/* If we receive MP_REACH nexthop with ::(LL)
+			 * or LL(LL), use LL address as nexthop cache.
+			 */
+			if (pi->attr->mp_nexthop_len
+				    == BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL
+			    && (IN6_IS_ADDR_UNSPECIFIED(
+					&pi->attr->mp_nexthop_global)
+				|| IN6_IS_ADDR_LINKLOCAL(
+					&pi->attr->mp_nexthop_global)))
+				p->u.prefix6 = pi->attr->mp_nexthop_local;
+			else
+				p->u.prefix6 = pi->attr->mp_nexthop_global;
 			p->prefixlen = IPV6_MAX_BITLEN;
 		}
 		break;
@@ -541,7 +577,7 @@ static int make_prefix(int afi, struct bgp_info *ri, struct prefix *p)
 		if (BGP_DEBUG(nht, NHT)) {
 			zlog_debug(
 				"%s: Attempting to make prefix with unknown AFI %d (not %d or %d)",
-				__FUNCTION__, afi, AFI_IP, AFI_IP6);
+				__func__, afi, AFI_IP, AFI_IP6);
 		}
 		break;
 	}
@@ -559,7 +595,7 @@ static int make_prefix(int afi, struct bgp_info *ri, struct prefix *p)
  */
 static void sendmsg_zebra_rnh(struct bgp_nexthop_cache *bnc, int command)
 {
-	struct prefix *p;
+	const struct prefix *p;
 	bool exact_match = false;
 	int ret;
 
@@ -567,10 +603,21 @@ static void sendmsg_zebra_rnh(struct bgp_nexthop_cache *bnc, int command)
 		return;
 
 	/* Don't try to register if Zebra doesn't know of this instance. */
-	if (!IS_BGP_INST_KNOWN_TO_ZEBRA(bnc->bgp))
+	if (!IS_BGP_INST_KNOWN_TO_ZEBRA(bnc->bgp)) {
+		if (BGP_DEBUG(zebra, ZEBRA))
+			zlog_debug(
+				"%s: No zebra instance to talk to, not installing NHT entry",
+				__func__);
 		return;
+	}
 
-	p = &(bnc->node->p);
+	if (!bgp_zebra_num_connects()) {
+		if (BGP_DEBUG(zebra, ZEBRA))
+			zlog_debug(
+				"%s: We have not connected yet, cannot send nexthops",
+				__func__);
+	}
+	p = bgp_node_get_prefix(bnc->node);
 	if ((command == ZEBRA_NEXTHOP_REGISTER
 	     || command == ZEBRA_IMPORT_ROUTE_REGISTER)
 	    && (CHECK_FLAG(bnc->flags, BGP_NEXTHOP_CONNECTED)
@@ -583,14 +630,15 @@ static void sendmsg_zebra_rnh(struct bgp_nexthop_cache *bnc, int command)
 		prefix2str(p, buf, PREFIX2STR_BUFFER);
 		zlog_debug("%s: sending cmd %s for %s (vrf %s)",
 			__func__, zserv_command_string(command), buf,
-			bnc->bgp->name);
+			bnc->bgp->name_pretty);
 	}
 
 	ret = zclient_send_rnh(zclient, command, p, exact_match,
 			       bnc->bgp->vrf_id);
 	/* TBD: handle the failure */
 	if (ret < 0)
-		zlog_warn("sendmsg_nexthop: zclient_send_message() failed");
+		flog_warn(EC_BGP_ZEBRA_SEND,
+			  "sendmsg_nexthop: zclient_send_message() failed");
 
 	if ((command == ZEBRA_NEXTHOP_REGISTER)
 	    || (command == ZEBRA_IMPORT_ROUTE_REGISTER))
@@ -651,19 +699,21 @@ static void unregister_zebra_rnh(struct bgp_nexthop_cache *bnc,
 static void evaluate_paths(struct bgp_nexthop_cache *bnc)
 {
 	struct bgp_node *rn;
-	struct bgp_info *path;
+	struct bgp_path_info *path;
 	int afi;
 	struct peer *peer = (struct peer *)bnc->nht_info;
 	struct bgp_table *table;
 	safi_t safi;
 	struct bgp *bgp_path;
+	const struct prefix *p;
 
 	if (BGP_DEBUG(nht, NHT)) {
 		char buf[PREFIX2STR_BUFFER];
 		bnc_str(bnc, buf, PREFIX2STR_BUFFER);
 		zlog_debug(
-			"NH update for %s - flags 0x%x chgflags 0x%x - evaluate paths",
-			buf, bnc->flags, bnc->change_flags);
+			"NH update for %s(%s) - flags 0x%x chgflags 0x%x - evaluate paths",
+			buf, bnc->bgp->name_pretty, bnc->flags,
+			bnc->change_flags);
 	}
 
 	LIST_FOREACH (path, &(bnc->paths), nh_thread) {
@@ -675,7 +725,8 @@ static void evaluate_paths(struct bgp_nexthop_cache *bnc)
 
 		rn = path->net;
 		assert(rn && bgp_node_table(rn));
-		afi = family2afi(rn->p.family);
+		p = bgp_node_get_prefix(rn);
+		afi = family2afi(p->family);
 		table = bgp_node_table(rn);
 		safi = table->safi;
 
@@ -705,52 +756,79 @@ static void evaluate_paths(struct bgp_nexthop_cache *bnc)
 			bnc_is_valid_nexthop =
 				bgp_isvalid_labeled_nexthop(bnc) ? 1 : 0;
 		} else {
-			bnc_is_valid_nexthop =
-				bgp_isvalid_nexthop(bnc) ? 1 : 0;
+			if (bgp_update_martian_nexthop(
+				    bnc->bgp, afi, safi, path->type,
+				    path->sub_type, path->attr, rn)) {
+				if (BGP_DEBUG(nht, NHT))
+					zlog_debug(
+						"%s: prefix %pRN (vrf %s), ignoring path due to martian or self-next-hop",
+						__func__, rn, bgp_path->name);
+			} else
+				bnc_is_valid_nexthop =
+					bgp_isvalid_nexthop(bnc) ? 1 : 0;
 		}
 
-		if (BGP_DEBUG(nht, NHT)) {
-			char buf[PREFIX_STRLEN];
+		if (BGP_DEBUG(nht, NHT))
+			zlog_debug("%s: prefix %pRN (vrf %s) %svalid", __func__,
+				   rn, bgp_path->name,
+				   (bnc_is_valid_nexthop ? "" : "not "));
 
-			prefix2str(&rn->p, buf, PREFIX_STRLEN);
-			zlog_debug("%s: prefix %s (vrf %s) %svalid",
-				__func__, buf, bgp_path->name,
-				(bnc_is_valid_nexthop ? "" : "not "));
-		}
-
-		if ((CHECK_FLAG(path->flags, BGP_INFO_VALID) ? 1 : 0)
+		if ((CHECK_FLAG(path->flags, BGP_PATH_VALID) ? 1 : 0)
 		    != bnc_is_valid_nexthop) {
-			if (CHECK_FLAG(path->flags, BGP_INFO_VALID)) {
-				bgp_aggregate_decrement(bgp_path, &rn->p,
-							path, afi, safi);
-				bgp_info_unset_flag(rn, path, BGP_INFO_VALID);
+			if (CHECK_FLAG(path->flags, BGP_PATH_VALID)) {
+				bgp_aggregate_decrement(bgp_path, p, path, afi,
+							safi);
+				bgp_path_info_unset_flag(rn, path,
+							 BGP_PATH_VALID);
 			} else {
-				bgp_info_set_flag(rn, path, BGP_INFO_VALID);
-				bgp_aggregate_increment(bgp_path, &rn->p,
-							path, afi, safi);
+				bgp_path_info_set_flag(rn, path,
+						       BGP_PATH_VALID);
+				bgp_aggregate_increment(bgp_path, p, path, afi,
+							safi);
 			}
 		}
 
 		/* Copy the metric to the path. Will be used for bestpath
 		 * computation */
 		if (bgp_isvalid_nexthop(bnc) && bnc->metric)
-			(bgp_info_extra_get(path))->igpmetric = bnc->metric;
+			(bgp_path_info_extra_get(path))->igpmetric =
+				bnc->metric;
 		else if (path->extra)
 			path->extra->igpmetric = 0;
 
 		if (CHECK_FLAG(bnc->change_flags, BGP_NEXTHOP_METRIC_CHANGED)
 		    || CHECK_FLAG(bnc->change_flags, BGP_NEXTHOP_CHANGED))
-			SET_FLAG(path->flags, BGP_INFO_IGP_CHANGED);
+			SET_FLAG(path->flags, BGP_PATH_IGP_CHANGED);
+
+		if (safi == SAFI_EVPN && bgp_evpn_is_prefix_nht_supported(p)) {
+			if (CHECK_FLAG(path->flags, BGP_PATH_VALID))
+				bgp_evpn_import_route(bgp_path, afi, safi, p,
+						      path);
+			else
+				bgp_evpn_unimport_route(bgp_path, afi, safi, p,
+							path);
+		}
 
 		bgp_process(bgp_path, rn, afi, safi);
 	}
 
-	if (peer && !CHECK_FLAG(bnc->flags, BGP_NEXTHOP_PEER_NOTIFIED)) {
-		if (BGP_DEBUG(nht, NHT))
-			zlog_debug("%s: Updating peer (%s) status with NHT",
-				   __FUNCTION__, peer->host);
-		bgp_fsm_nht_update(peer, bgp_isvalid_nexthop(bnc));
-		SET_FLAG(bnc->flags, BGP_NEXTHOP_PEER_NOTIFIED);
+	if (peer) {
+		int valid_nexthops = bgp_isvalid_nexthop(bnc);
+
+		if (valid_nexthops)
+			peer->last_reset = PEER_DOWN_WAITING_OPEN;
+		else
+			peer->last_reset = PEER_DOWN_WAITING_NHT;
+
+		if (!CHECK_FLAG(bnc->flags, BGP_NEXTHOP_PEER_NOTIFIED)) {
+			if (BGP_DEBUG(nht, NHT))
+				zlog_debug(
+					"%s: Updating peer (%s(%s)) status with NHT",
+					__func__, peer->host,
+					peer->bgp->name_pretty);
+			bgp_fsm_event_update(peer, valid_nexthops);
+			SET_FLAG(bnc->flags, BGP_NEXTHOP_PEER_NOTIFIED);
+		}
 	}
 
 	RESET_FLAG(bnc->change_flags);
@@ -764,8 +842,8 @@ static void evaluate_paths(struct bgp_nexthop_cache *bnc)
  *   make - if set, make the association. if unset, just break the existing
  *          association.
  */
-static void path_nh_map(struct bgp_info *path, struct bgp_nexthop_cache *bnc,
-			int make)
+void path_nh_map(struct bgp_path_info *path, struct bgp_nexthop_cache *bnc,
+		 bool make)
 {
 	if (path->nexthop) {
 		LIST_REMOVE(path, nh_thread);
@@ -778,3 +856,127 @@ static void path_nh_map(struct bgp_info *path, struct bgp_nexthop_cache *bnc,
 		path->nexthop->path_count++;
 	}
 }
+
+/*
+ * This function is called to register nexthops to zebra
+ * as that we may have tried to install the nexthops
+ * before we actually have a zebra connection
+ */
+void bgp_nht_register_nexthops(struct bgp *bgp)
+{
+	struct bgp_node *rn;
+	struct bgp_nexthop_cache *bnc;
+	afi_t afi;
+
+	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
+		if (!bgp->nexthop_cache_table[afi])
+			continue;
+
+		for (rn = bgp_table_top(bgp->nexthop_cache_table[afi]); rn;
+		     rn = bgp_route_next(rn)) {
+			bnc = bgp_node_get_bgp_nexthop_info(rn);
+
+			if (!bnc)
+				continue;
+
+			register_zebra_rnh(bnc, 0);
+		}
+	}
+}
+
+void bgp_nht_reg_enhe_cap_intfs(struct peer *peer)
+{
+	struct bgp *bgp;
+	struct bgp_node *rn;
+	struct bgp_nexthop_cache *bnc;
+	struct nexthop *nhop;
+	struct interface *ifp;
+	struct prefix p;
+
+	if (peer->ifp)
+		return;
+
+	bgp = peer->bgp;
+
+	if (!bgp->nexthop_cache_table[AFI_IP6])
+		return;
+
+	if (!sockunion2hostprefix(&peer->su, &p)) {
+		zlog_warn("%s: Unable to convert sockunion to prefix for %s",
+			  __func__, peer->host);
+		return;
+	}
+
+	if (p.family != AF_INET6)
+		return;
+	rn = bgp_node_lookup(bgp->nexthop_cache_table[AFI_IP6], &p);
+	if (!rn)
+		return;
+
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc)
+		return;
+
+	if (peer != bnc->nht_info)
+		return;
+
+	for (nhop = bnc->nexthop; nhop; nhop = nhop->next) {
+		ifp = if_lookup_by_index(nhop->ifindex, nhop->vrf_id);
+
+		if (!ifp)
+			continue;
+
+		zclient_send_interface_radv_req(zclient,
+						nhop->vrf_id,
+						ifp, true,
+						BGP_UNNUM_DEFAULT_RA_INTERVAL);
+	}
+}
+
+void bgp_nht_dereg_enhe_cap_intfs(struct peer *peer)
+{
+	struct bgp *bgp;
+	struct bgp_node *rn;
+	struct bgp_nexthop_cache *bnc;
+	struct nexthop *nhop;
+	struct interface *ifp;
+	struct prefix p;
+
+	if (peer->ifp)
+		return;
+
+	bgp = peer->bgp;
+
+	if (!bgp->nexthop_cache_table[AFI_IP6])
+		return;
+
+	if (!sockunion2hostprefix(&peer->su, &p)) {
+		zlog_warn("%s: Unable to convert sockunion to prefix for %s",
+			  __func__, peer->host);
+		return;
+	}
+
+	if (p.family != AF_INET6)
+		return;
+
+	rn = bgp_node_lookup(bgp->nexthop_cache_table[AFI_IP6], &p);
+	if (!rn)
+		return;
+
+	bnc = bgp_node_get_bgp_nexthop_info(rn);
+	if (!bnc)
+		return;
+
+	if (peer != bnc->nht_info)
+		return;
+
+	for (nhop = bnc->nexthop; nhop; nhop = nhop->next) {
+		ifp = if_lookup_by_index(nhop->ifindex, nhop->vrf_id);
+
+		if (!ifp)
+			continue;
+
+		zclient_send_interface_radv_req(zclient, nhop->vrf_id, ifp, 0,
+						0);
+	}
+}
