@@ -34,6 +34,7 @@
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_attr.h"
 #include "bgpd/bgp_debug.h"
+#include "bgpd/bgp_errors.h"
 #include "bgpd/bgp_fsm.h"
 #include "bgpd/bgp_packet.h"
 #include "bgpd/bgp_open.h"
@@ -48,11 +49,11 @@
 
    So there is many configurable point.  First of all we want set each
    peer whether we send capability negotiation to the peer or not.
-   Next, if we send capability to the peer we want to set my capabilty
+   Next, if we send capability to the peer we want to set my capability
    inforation at each peer. */
 
-void bgp_capability_vty_out(struct vty *vty, struct peer *peer,
-			    uint8_t use_json, json_object *json_neigh)
+void bgp_capability_vty_out(struct vty *vty, struct peer *peer, bool use_json,
+			    json_object *json_neigh)
 {
 	char *pnt;
 	char *end;
@@ -80,8 +81,9 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer,
 			afi_t afi;
 			safi_t safi;
 
-			bgp_map_afi_safi_iana2int(ntohs(mpc.afi), mpc.safi,
-						  &afi, &safi);
+			(void)bgp_map_afi_safi_iana2int(ntohs(mpc.afi),
+							mpc.safi, &afi, &safi);
+
 			if (use_json) {
 				switch (afi) {
 				case AFI_IP:
@@ -252,7 +254,8 @@ static int bgp_capability_mp(struct peer *peer, struct capability_header *hdr)
 
 	/* Verify length is 4 */
 	if (hdr->length != 4) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_LENGTH,
 			"MP Cap: Received invalid length %d, non-multiple of 4",
 			hdr->length);
 		return -1;
@@ -261,8 +264,9 @@ static int bgp_capability_mp(struct peer *peer, struct capability_header *hdr)
 	bgp_capability_mp_data(s, &mpc);
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s OPEN has MP_EXT CAP for afi/safi: %u/%u",
-			   peer->host, mpc.afi, mpc.safi);
+		zlog_debug("%s OPEN has MP_EXT CAP for afi/safi: %s/%s",
+			   peer->host, iana_afi2str(mpc.afi),
+			   iana_safi2str(mpc.safi));
 
 	/* Convert AFI, SAFI to internal values, check. */
 	if (bgp_map_afi_safi_iana2int(mpc.afi, mpc.safi, &afi, &safi))
@@ -322,8 +326,8 @@ static int bgp_capability_orf_entry(struct peer *peer,
 	pkt_safi = mpc.safi;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s ORF Cap entry for afi/safi: %u/%u", peer->host,
-			   mpc.afi, mpc.safi);
+		zlog_debug("%s ORF Cap entry for afi/safi: %s/%s", peer->host,
+			   iana_afi2str(mpc.afi), iana_safi2str(mpc.safi));
 
 	/* Convert AFI, SAFI to internal values, check. */
 	if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
@@ -404,11 +408,11 @@ static int bgp_capability_orf_entry(struct peer *peer,
 		if (bgp_debug_neighbor_events(peer))
 			zlog_debug(
 				"%s OPEN has %s ORF capability"
-				" as %s for afi/safi: %d/%d",
+				" as %s for afi/safi: %s/%s",
 				peer->host,
 				lookup_msg(orf_type_str, type, NULL),
-				lookup_msg(orf_mode_str, mode, NULL), pkt_afi,
-				pkt_safi);
+				lookup_msg(orf_mode_str, mode, NULL),
+				iana_afi2str(pkt_afi), iana_safi2str(pkt_safi));
 
 		if (hdr->code == CAPABILITY_CODE_ORF) {
 			sm_cap = PEER_CAP_ORF_PREFIX_SM_RCV;
@@ -447,7 +451,8 @@ static int bgp_capability_restart(struct peer *peer,
 
 	/* Verify length is a multiple of 4 */
 	if ((caphdr->length - 2) % 4) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_LENGTH,
 			"Restart Cap: Received invalid length %d, non-multiple of 4",
 			caphdr->length);
 		return -1;
@@ -457,6 +462,8 @@ static int bgp_capability_restart(struct peer *peer,
 	restart_flag_time = stream_getw(s);
 	if (CHECK_FLAG(restart_flag_time, RESTART_R_BIT))
 		SET_FLAG(peer->cap, PEER_CAP_RESTART_BIT_RCV);
+	else
+		UNSET_FLAG(peer->cap, PEER_CAP_RESTART_BIT_RCV);
 
 	UNSET_FLAG(restart_flag_time, 0xF000);
 	peer->v_gr_restart = restart_flag_time;
@@ -483,20 +490,22 @@ static int bgp_capability_restart(struct peer *peer,
 		if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
-					"%s Addr-family %d/%d(afi/safi) not supported."
+					"%s Addr-family %s/%s(afi/safi) not supported."
 					" Ignore the Graceful Restart capability for this AFI/SAFI",
-					peer->host, pkt_afi, pkt_safi);
+					peer->host, iana_afi2str(pkt_afi),
+					iana_safi2str(pkt_safi));
 		} else if (!peer->afc[afi][safi]) {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
-					"%s Addr-family %d/%d(afi/safi) not enabled."
+					"%s Addr-family %s/%s(afi/safi) not enabled."
 					" Ignore the Graceful Restart capability",
-					peer->host, pkt_afi, pkt_safi);
+					peer->host, iana_afi2str(pkt_afi),
+					iana_safi2str(pkt_safi));
 		} else {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
 					"%s Address family %s is%spreserved",
-					peer->host, afi_safi_print(afi, safi),
+					peer->host, get_afi_safi_str(afi, safi, false),
 					CHECK_FLAG(
 						peer->af_cap[afi][safi],
 						PEER_CAP_RESTART_AF_PRESERVE_RCV)
@@ -519,7 +528,8 @@ static as_t bgp_capability_as4(struct peer *peer, struct capability_header *hdr)
 	SET_FLAG(peer->cap, PEER_CAP_AS4_RCV);
 
 	if (hdr->length != CAPABILITY_CODE_AS4_LEN) {
-		zlog_err("%s AS4 capability has incorrect data length %d",
+		flog_err(EC_BGP_PKT_OPEN,
+			 "%s AS4 capability has incorrect data length %d",
 			 peer->host, hdr->length);
 		return 0;
 	}
@@ -543,7 +553,8 @@ static int bgp_capability_addpath(struct peer *peer,
 
 	/* Verify length is a multiple of 4 */
 	if (hdr->length % 4) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_LENGTH,
 			"Add Path: Received invalid length %d, non-multiple of 4",
 			hdr->length);
 		return -1;
@@ -558,8 +569,9 @@ static int bgp_capability_addpath(struct peer *peer,
 
 		if (bgp_debug_neighbor_events(peer))
 			zlog_debug(
-				"%s OPEN has AddPath CAP for afi/safi: %u/%u%s%s",
-				peer->host, pkt_afi, pkt_safi,
+				"%s OPEN has AddPath CAP for afi/safi: %s/%s%s%s",
+				peer->host, iana_afi2str(pkt_afi),
+				iana_safi2str(pkt_safi),
 				(send_receive & BGP_ADDPATH_RX) ? ", receive"
 								: "",
 				(send_receive & BGP_ADDPATH_TX) ? ", transmit"
@@ -569,16 +581,18 @@ static int bgp_capability_addpath(struct peer *peer,
 		if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
-					"%s Addr-family %d/%d(afi/safi) not supported."
+					"%s Addr-family %s/%s(afi/safi) not supported."
 					" Ignore the Addpath Attribute for this AFI/SAFI",
-					peer->host, pkt_afi, pkt_safi);
+					peer->host, iana_afi2str(pkt_afi),
+					iana_safi2str(pkt_safi));
 			continue;
 		} else if (!peer->afc[afi][safi]) {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
-					"%s Addr-family %d/%d(afi/safi) not enabled."
+					"%s Addr-family %s/%s(afi/safi) not enabled."
 					" Ignore the AddPath capability for this AFI/SAFI",
-					peer->host, pkt_afi, pkt_safi);
+					peer->host, iana_afi2str(pkt_afi),
+					iana_safi2str(pkt_safi));
 			continue;
 		}
 
@@ -601,7 +615,8 @@ static int bgp_capability_enhe(struct peer *peer, struct capability_header *hdr)
 
 	/* Verify length is a multiple of 4 */
 	if (hdr->length % 6) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_LENGTH,
 			"Extended NH: Received invalid length %d, non-multiple of 6",
 			hdr->length);
 		return -1;
@@ -617,16 +632,18 @@ static int bgp_capability_enhe(struct peer *peer, struct capability_header *hdr)
 
 		if (bgp_debug_neighbor_events(peer))
 			zlog_debug(
-				"%s Received with afi/safi/next-hop afi: %u/%u/%u",
-				peer->host, pkt_afi, pkt_safi, pkt_nh_afi);
+				"%s Received with afi/safi/next-hop afi: %s/%s/%u",
+				peer->host, iana_afi2str(pkt_afi),
+				iana_safi2str(pkt_safi), pkt_nh_afi);
 
 		/* Convert AFI, SAFI to internal values, check. */
 		if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
 			if (bgp_debug_neighbor_events(peer))
 				zlog_debug(
-					"%s Addr-family %d/%d(afi/safi) not supported."
+					"%s Addr-family %s/%s(afi/safi) not supported."
 					" Ignore the ENHE Attribute for this AFI/SAFI",
-					peer->host, pkt_afi, pkt_safi);
+					peer->host, iana_afi2str(pkt_afi),
+					iana_safi2str(pkt_safi));
 			continue;
 		}
 
@@ -641,12 +658,14 @@ static int bgp_capability_enhe(struct peer *peer, struct capability_header *hdr)
 		nh_afi = afi_iana2int(pkt_nh_afi);
 
 		if (afi != AFI_IP || nh_afi != AFI_IP6
-		    || !(safi == SAFI_UNICAST
+		    || !(safi == SAFI_UNICAST || safi == SAFI_MPLS_VPN
 			 || safi == SAFI_LABELED_UNICAST)) {
-			zlog_warn(
-				"%s Unexpected afi/safi/next-hop afi: %u/%u/%u "
+			flog_warn(
+				EC_BGP_CAPABILITY_INVALID_DATA,
+				"%s Unexpected afi/safi/next-hop afi: %s/%s/%u "
 				"in Extended Next-hop capability, ignoring",
-				peer->host, pkt_afi, pkt_safi, pkt_nh_afi);
+				peer->host, iana_afi2str(pkt_afi),
+				iana_safi2str(pkt_safi), pkt_nh_afi);
 			continue;
 		}
 
@@ -674,9 +693,10 @@ static int bgp_capability_hostname(struct peer *peer,
 
 	len = stream_getc(s);
 	if (stream_get_getp(s) + len > end) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_DATA,
 			"%s: Received malformed hostname capability from peer %s",
-			__FUNCTION__, peer->host);
+			__func__, peer->host);
 		return -1;
 	}
 
@@ -690,31 +710,26 @@ static int bgp_capability_hostname(struct peer *peer,
 	if (len) {
 		str[len] = '\0';
 
-		if (peer->hostname != NULL) {
-			XFREE(MTYPE_BGP_PEER_HOST, peer->hostname);
-			peer->hostname = NULL;
-		}
-
-		if (peer->domainname != NULL) {
-			XFREE(MTYPE_BGP_PEER_HOST, peer->domainname);
-			peer->domainname = NULL;
-		}
+		XFREE(MTYPE_BGP_PEER_HOST, peer->hostname);
+		XFREE(MTYPE_BGP_PEER_HOST, peer->domainname);
 
 		peer->hostname = XSTRDUP(MTYPE_BGP_PEER_HOST, str);
 	}
 
 	if (stream_get_getp(s) + 1 > end) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_DATA,
 			"%s: Received invalid domain name len (hostname capability) from peer %s",
-			__FUNCTION__, peer->host);
+			__func__, peer->host);
 		return -1;
 	}
 
 	len = stream_getc(s);
 	if (stream_get_getp(s) + len > end) {
-		zlog_warn(
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_DATA,
 			"%s: Received runt domain name (hostname capability) from peer %s",
-			__FUNCTION__, peer->host);
+			__func__, peer->host);
 		return -1;
 	}
 
@@ -727,6 +742,9 @@ static int bgp_capability_hostname(struct peer *peer,
 
 	if (len) {
 		str[len] = '\0';
+
+		XFREE(MTYPE_BGP_PEER_HOST, peer->domainname);
+
 		peer->domainname = XSTRDUP(MTYPE_BGP_PEER_HOST, str);
 	}
 
@@ -802,6 +820,7 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 	int ret;
 	struct stream *s = BGP_INPUT(peer);
 	size_t end = stream_get_getp(s) + length;
+	uint16_t restart_flag_time = 0;
 
 	assert(STREAM_READABLE(s) >= length);
 
@@ -949,10 +968,12 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 				   specific
 				   capabilities.  It seems reasonable for now...
 				   */
-				zlog_warn("%s Vendor specific capability %d",
+				flog_warn(EC_BGP_CAPABILITY_VENDOR,
+					  "%s Vendor specific capability %d",
 					  peer->host, caphdr.code);
 			} else {
-				zlog_warn(
+				flog_warn(
+					EC_BGP_CAPABILITY_UNKNOWN,
 					"%s unrecognized capability code: %d - ignored",
 					peer->host, caphdr.code);
 				memcpy(*error, sp, caphdr.length + 2);
@@ -967,7 +988,8 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		}
 		if (stream_get_getp(s) != (start + caphdr.length)) {
 			if (stream_get_getp(s) > (start + caphdr.length))
-				zlog_warn(
+				flog_warn(
+					EC_BGP_CAPABILITY_INVALID_LENGTH,
 					"%s Cap-parser for %s read past cap-length, %u!",
 					peer->host,
 					lookup_msg(capcode_str, caphdr.code,
@@ -975,6 +997,11 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 					caphdr.length);
 			stream_set_getp(s, start + caphdr.length);
 		}
+
+		if (!CHECK_FLAG(peer->cap, PEER_CAP_RESTART_RCV)) {
+			UNSET_FLAG(restart_flag_time, 0xF000);
+			peer->v_gr_restart = restart_flag_time;
+		}
 	}
 	return 0;
 }
@@ -986,15 +1013,15 @@ static int bgp_auth_parse(struct peer *peer, size_t length)
 	return -1;
 }
 
-static int strict_capability_same(struct peer *peer)
+static bool strict_capability_same(struct peer *peer)
 {
 	int i, j;
 
 	for (i = AFI_IP; i < AFI_MAX; i++)
 		for (j = SAFI_UNICAST; j < SAFI_MAX; j++)
 			if (peer->afc[i][j] != peer->afc_nego[i][j])
-				return 0;
-	return 1;
+				return false;
+	return true;
 }
 
 /* peek into option, stores ASN to *as4 if the AS4 capability was found.
@@ -1008,7 +1035,7 @@ as_t peek_for_as4_capability(struct peer *peer, uint8_t length)
 	as_t as4 = 0;
 
 	if (BGP_DEBUG(as4, AS4))
-		zlog_info(
+		zlog_debug(
 			"%s [AS4] rcv OPEN w/ OPTION parameter len: %u,"
 			" peeking for as4",
 			peer->host, length);
@@ -1052,7 +1079,7 @@ as_t peek_for_as4_capability(struct peer *peer, uint8_t length)
 
 				if (hdr.code == CAPABILITY_CODE_AS4) {
 					if (BGP_DEBUG(as4, AS4))
-						zlog_info(
+						zlog_debug(
 							"[AS4] found AS4 capability, about to parse");
 					as4 = bgp_capability_as4(peer, &hdr);
 
@@ -1183,10 +1210,10 @@ int bgp_open_option_parse(struct peer *peer, uint8_t length, int *mp_capability)
 		    && !peer->afc_nego[AFI_IP6][SAFI_ENCAP]
 		    && !peer->afc_nego[AFI_IP6][SAFI_FLOWSPEC]
 		    && !peer->afc_nego[AFI_L2VPN][SAFI_EVPN]) {
-			zlog_err(
-				"%s [Error] Configured AFI/SAFIs do not "
-				"overlap with received MP capabilities",
-				peer->host);
+			flog_err(EC_BGP_PKT_OPEN,
+				 "%s [Error] Configured AFI/SAFIs do not "
+				 "overlap with received MP capabilities",
+				 peer->host);
 
 			if (error != error_data)
 				bgp_notify_send_with_data(
@@ -1270,6 +1297,87 @@ static void bgp_open_capability_orf(struct stream *s, struct peer *peer,
 	stream_putc_at(s, capp, cap_len);
 }
 
+static void bgp_peer_send_gr_capability(struct stream *s, struct peer *peer,
+					unsigned long cp)
+{
+	int len;
+	iana_afi_t pkt_afi;
+	afi_t afi;
+	safi_t safi;
+	iana_safi_t pkt_safi;
+	uint32_t restart_time;
+	unsigned long capp = 0;
+	unsigned long rcapp = 0;
+
+	if (!CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART)
+	    && !CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART_HELPER))
+		return;
+
+	if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+		zlog_debug("[BGP_GR] Sending helper Capability for Peer :%s :",
+			   peer->host);
+
+	SET_FLAG(peer->cap, PEER_CAP_RESTART_ADV);
+	stream_putc(s, BGP_OPEN_OPT_CAP);
+	capp = stream_get_endp(s); /* Set Capability Len Pointer */
+	stream_putc(s, 0);	 /* Capability Length */
+	stream_putc(s, CAPABILITY_CODE_RESTART);
+	/* Set Restart Capability Len Pointer */
+	rcapp = stream_get_endp(s);
+	stream_putc(s, 0);
+	restart_time = peer->bgp->restart_time;
+	if (peer->bgp->t_startup) {
+		SET_FLAG(restart_time, RESTART_R_BIT);
+		SET_FLAG(peer->cap, PEER_CAP_RESTART_BIT_ADV);
+
+		if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+			zlog_debug("[BGP_GR] Sending R-Bit for Peer :%s :",
+				   peer->host);
+	}
+
+	stream_putw(s, restart_time);
+
+	/* Send address-family specific graceful-restart capability
+	 * only when GR config is present
+	 */
+	if (CHECK_FLAG(peer->flags, PEER_FLAG_GRACEFUL_RESTART)) {
+		if (CHECK_FLAG(peer->bgp->flags, BGP_FLAG_GR_PRESERVE_FWD)
+		    && BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+			zlog_debug("[BGP_GR] F bit Set");
+
+		FOREACH_AFI_SAFI (afi, safi) {
+			if (!peer->afc[afi][safi])
+				continue;
+
+			if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+				zlog_debug(
+					"[BGP_GR] Sending GR Capability for AFI :%d :, SAFI :%d:",
+					afi, safi);
+
+			/* Convert AFI, SAFI to values for
+			 * packet.
+			 */
+			bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
+						  &pkt_safi);
+			stream_putw(s, pkt_afi);
+			stream_putc(s, pkt_safi);
+			if (CHECK_FLAG(peer->bgp->flags,
+				       BGP_FLAG_GR_PRESERVE_FWD))
+				stream_putc(s, RESTART_F_BIT);
+			else
+				stream_putc(s, 0);
+		}
+	}
+
+	/* Total Graceful restart capability Len. */
+	len = stream_get_endp(s) - rcapp - 1;
+	stream_putc_at(s, rcapp, len);
+
+	/* Total Capability Len. */
+	len = stream_get_endp(s) - capp - 1;
+	stream_putc_at(s, capp, len);
+}
+
 /* Fill in capability open option to the packet. */
 void bgp_open_capability(struct stream *s, struct peer *peer)
 {
@@ -1280,7 +1388,6 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 	safi_t safi;
 	iana_safi_t pkt_safi;
 	as_t local_as;
-	uint32_t restart_time;
 	uint8_t afi_safi_count = 0;
 	int adv_addpath_tx = 0;
 
@@ -1317,9 +1424,8 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 			 */
 			if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_ENHE)
 			    && peer->su.sa.sa_family == AF_INET6
-			    && IN6_IS_ADDR_LINKLOCAL(&peer->su.sin6.sin6_addr)
 			    && afi == AFI_IP
-			    && (safi == SAFI_UNICAST
+			    && (safi == SAFI_UNICAST || safi == SAFI_MPLS_VPN
 				|| safi == SAFI_LABELED_UNICAST)) {
 				/* RFC 5549 Extended Next Hop Encoding
 				 */
@@ -1374,10 +1480,7 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 			/* Only advertise addpath TX if a feature that
 			 * will use it is
 			 * configured */
-			if (CHECK_FLAG(peer->af_flags[afi][safi],
-				       PEER_FLAG_ADDPATH_TX_ALL_PATHS)
-			    || CHECK_FLAG(peer->af_flags[afi][safi],
-					  PEER_FLAG_ADDPATH_TX_BESTPATH_PER_AS))
+			if (peer->addpath_type[afi][safi] != BGP_ADDPATH_NONE)
 				adv_addpath_tx = 1;
 		}
 	}
@@ -1477,50 +1580,7 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 				cmd_domainname_get());
 	}
 
-	/* Sending base graceful-restart capability irrespective of the config
-	 */
-	SET_FLAG(peer->cap, PEER_CAP_RESTART_ADV);
-	stream_putc(s, BGP_OPEN_OPT_CAP);
-	capp = stream_get_endp(s); /* Set Capability Len Pointer */
-	stream_putc(s, 0);	 /* Capability Length */
-	stream_putc(s, CAPABILITY_CODE_RESTART);
-	rcapp = stream_get_endp(s); /* Set Restart Capability Len Pointer */
-	stream_putc(s, 0);
-	restart_time = peer->bgp->restart_time;
-	if (peer->bgp->t_startup) {
-		SET_FLAG(restart_time, RESTART_R_BIT);
-		SET_FLAG(peer->cap, PEER_CAP_RESTART_BIT_ADV);
-	}
-	stream_putw(s, restart_time);
-
-	/* Send address-family specific graceful-restart capability only when GR
-	   config
-	   is present */
-	if (bgp_flag_check(peer->bgp, BGP_FLAG_GRACEFUL_RESTART)) {
-		FOREACH_AFI_SAFI (afi, safi) {
-			if (peer->afc[afi][safi]) {
-				/* Convert AFI, SAFI to values for
-				 * packet. */
-				bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
-							  &pkt_safi);
-				stream_putw(s, pkt_afi);
-				stream_putc(s, pkt_safi);
-				if (bgp_flag_check(peer->bgp,
-						   BGP_FLAG_GR_PRESERVE_FWD))
-					stream_putc(s, RESTART_F_BIT);
-				else
-					stream_putc(s, 0);
-			}
-		}
-	}
-
-	/* Total Graceful restart capability Len. */
-	len = stream_get_endp(s) - rcapp - 1;
-	stream_putc_at(s, rcapp, len);
-
-	/* Total Capability Len. */
-	len = stream_get_endp(s) - capp - 1;
-	stream_putc_at(s, capp, len);
+	bgp_peer_send_gr_capability(s, peer, cp);
 
 	/* Total Opt Parm Len. */
 	len = stream_get_endp(s) - cp - 1;
