@@ -24,6 +24,11 @@
 #include "mpls.h"
 #include "table.h"
 #include "queue.h"
+#include "linklist.h"
+#include "bgpd.h"
+#include "bgp_advertise.h"
+
+extern void bgp_delete_listnode(struct bgp_node *node);
 
 struct bgp_table {
 	/* table belongs to this instance */
@@ -39,6 +44,31 @@ struct bgp_table {
 	uint64_t version;
 };
 
+enum bgp_path_selection_reason {
+	bgp_path_selection_none,
+	bgp_path_selection_first,
+	bgp_path_selection_evpn_sticky_mac,
+	bgp_path_selection_evpn_seq,
+	bgp_path_selection_evpn_lower_ip,
+	bgp_path_selection_weight,
+	bgp_path_selection_local_pref,
+	bgp_path_selection_local_route,
+	bgp_path_selection_confed_as_path,
+	bgp_path_selection_as_path,
+	bgp_path_selection_origin,
+	bgp_path_selection_med,
+	bgp_path_selection_peer,
+	bgp_path_selection_confed,
+	bgp_path_selection_igp_metric,
+	bgp_path_selection_older,
+	bgp_path_selection_router_id,
+	bgp_path_selection_cluster_length,
+	bgp_path_selection_stale,
+	bgp_path_selection_local_configured,
+	bgp_path_selection_neighbor_ip,
+	bgp_path_selection_default,
+};
+
 struct bgp_node {
 	/*
 	 * CAUTION
@@ -50,7 +80,7 @@ struct bgp_node {
 	 */
 	ROUTE_NODE_FIELDS
 
-	struct bgp_adj_out *adj_out;
+	struct bgp_adj_out_rb adj_out;
 
 	struct bgp_adj_in *adj_in;
 
@@ -58,14 +88,21 @@ struct bgp_node {
 
 	STAILQ_ENTRY(bgp_node) pq;
 
+	uint64_t version;
+
 	mpls_label_t local_label;
 
-	uint64_t version;
 	uint8_t flags;
 #define BGP_NODE_PROCESS_SCHEDULED	(1 << 0)
 #define BGP_NODE_USER_CLEAR             (1 << 1)
 #define BGP_NODE_LABEL_CHANGED          (1 << 2)
 #define BGP_NODE_REGISTERED_FOR_LABEL   (1 << 3)
+#define BGP_NODE_SELECT_DEFER           (1 << 4)
+	/* list node pointer */
+	struct listnode *rt_node;
+	struct bgp_addpath_node_data tx_addpath;
+
+	enum bgp_path_selection_reason reason;
 };
 
 /*
@@ -111,7 +148,7 @@ static inline struct route_node *bgp_node_to_rnode(struct bgp_node *node)
  */
 static inline struct bgp_table *bgp_node_table(struct bgp_node *node)
 {
-	return bgp_node_to_rnode(node)->table->info;
+	return route_table_get_info(bgp_node_to_rnode(node)->table);
 }
 
 /*
@@ -129,6 +166,7 @@ static inline struct bgp_node *bgp_node_parent_nolock(struct bgp_node *node)
  */
 static inline void bgp_unlock_node(struct bgp_node *node)
 {
+	bgp_delete_listnode(node);
 	route_unlock_node(bgp_node_to_rnode(node));
 }
 
@@ -179,7 +217,7 @@ static inline struct bgp_node *bgp_route_next_until(struct bgp_node *node,
  * bgp_node_get
  */
 static inline struct bgp_node *bgp_node_get(struct bgp_table *const table,
-					    struct prefix *p)
+					    const struct prefix *p)
 {
 	return bgp_node_from_rnode(route_node_get(table->route_table, p));
 }
@@ -188,7 +226,7 @@ static inline struct bgp_node *bgp_node_get(struct bgp_table *const table,
  * bgp_node_lookup
  */
 static inline struct bgp_node *
-bgp_node_lookup(const struct bgp_table *const table, struct prefix *p)
+bgp_node_lookup(const struct bgp_table *const table, const struct prefix *p)
 {
 	return bgp_node_from_rnode(route_node_lookup(table->route_table, p));
 }
@@ -205,7 +243,7 @@ static inline struct bgp_node *bgp_lock_node(struct bgp_node *node)
  * bgp_node_match
  */
 static inline struct bgp_node *bgp_node_match(const struct bgp_table *table,
-					      struct prefix *p)
+					      const struct prefix *p)
 {
 	return bgp_node_from_rnode(route_node_match(table->route_table, p));
 }
@@ -239,7 +277,7 @@ static inline unsigned long bgp_table_count(const struct bgp_table *const table)
  * bgp_table_get_next
  */
 static inline struct bgp_node *bgp_table_get_next(const struct bgp_table *table,
-						  struct prefix *p)
+						  const struct prefix *p)
 {
 	return bgp_node_from_rnode(route_table_get_next(table->route_table, p));
 }
@@ -309,4 +347,109 @@ static inline uint64_t bgp_table_version(struct bgp_table *table)
 	return table->version;
 }
 
+void bgp_table_range_lookup(const struct bgp_table *table,
+			    const struct prefix *p,
+			    uint8_t maxlen, struct list *matches);
+
+
+static inline struct bgp_aggregate *
+bgp_node_get_bgp_aggregate_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void
+bgp_node_set_bgp_aggregate_info(struct bgp_node *node,
+				struct bgp_aggregate *aggregate)
+{
+	node->info = aggregate;
+}
+
+static inline struct bgp_distance *
+bgp_node_get_bgp_distance_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void bgp_node_set_bgp_distance_info(struct bgp_node *node,
+						  struct bgp_distance *distance)
+{
+	node->info = distance;
+}
+
+static inline struct bgp_static *
+bgp_node_get_bgp_static_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void bgp_node_set_bgp_static_info(struct bgp_node *node,
+						struct bgp_static *bgp_static)
+{
+	node->info = bgp_static;
+}
+
+static inline struct bgp_connected_ref *
+bgp_node_get_bgp_connected_ref_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void
+bgp_node_set_bgp_connected_ref_info(struct bgp_node *node,
+				    struct bgp_connected_ref *bc)
+{
+	node->info = bc;
+}
+
+static inline struct bgp_nexthop_cache *
+bgp_node_get_bgp_nexthop_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void bgp_node_set_bgp_nexthop_info(struct bgp_node *node,
+					     struct bgp_nexthop_cache *bnc)
+{
+	node->info = bnc;
+}
+
+static inline struct bgp_path_info *
+bgp_node_get_bgp_path_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void bgp_node_set_bgp_path_info(struct bgp_node *node,
+					      struct bgp_path_info *bi)
+{
+	node->info = bi;
+}
+
+static inline struct bgp_table *
+bgp_node_get_bgp_table_info(struct bgp_node *node)
+{
+	return node->info;
+}
+
+static inline void bgp_node_set_bgp_table_info(struct bgp_node *node,
+					       struct bgp_table *table)
+{
+	node->info = table;
+}
+
+static inline bool bgp_node_has_bgp_path_info_data(struct bgp_node *node)
+{
+	return !!node->info;
+}
+
+static inline const struct prefix *bgp_node_get_prefix(struct bgp_node *node)
+{
+	return &node->p;
+}
+
+#ifdef _FRR_ATTRIBUTE_PRINTFRR
+#pragma FRR printfrr_ext "%pRN"  (struct bgp_node *)
+#endif
+
 #endif /* _QUAGGA_BGP_TABLE_H */
