@@ -19,10 +19,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _QUAGGA_BGP_UPDGRP_H
@@ -30,13 +29,34 @@
 
 #include "bgp_advertise.h"
 
-#define BGP_DEFAULT_SUBGROUP_COALESCE_TIME 200
+/*
+ * The following three heuristic constants determine how long advertisement to
+ * a subgroup will be delayed after it is created. The intent is to allow
+ * transient changes in peer state (primarily session establishment) to settle,
+ * so that more peers can be grouped together and benefit from sharing
+ * advertisement computations with the subgroup.
+ *
+ * These values have a very large impact on initial convergence time; any
+ * changes should be accompanied by careful performance testing at all scales.
+ *
+ * The coalesce time 'C' for a new subgroup within a particular BGP instance
+ * 'B' with total number of known peers 'P', established or not, is computed as
+ * follows:
+ *
+ * C = MIN(BGP_MAX_SUBGROUP_COALESCE_TIME,
+ *         BGP_DEFAULT_SUBGROUP_COALESCE_TIME +
+ *         (P*BGP_PEER_ADJUST_SUBGROUP_COALESCE_TIME))
+ */
+#define BGP_DEFAULT_SUBGROUP_COALESCE_TIME 1000
+#define BGP_MAX_SUBGROUP_COALESCE_TIME 10000
+#define BGP_PEER_ADJUST_SUBGROUP_COALESCE_TIME 50
 
 #define PEER_UPDGRP_FLAGS                                                      \
 	(PEER_FLAG_LOCAL_AS_NO_PREPEND | PEER_FLAG_LOCAL_AS_REPLACE_AS)
 
 #define PEER_UPDGRP_AF_FLAGS                                                   \
 	(PEER_FLAG_SEND_COMMUNITY | PEER_FLAG_SEND_EXT_COMMUNITY               \
+	 | PEER_FLAG_SEND_LARGE_COMMUNITY                                      \
 	 | PEER_FLAG_DEFAULT_ORIGINATE | PEER_FLAG_REFLECTOR_CLIENT            \
 	 | PEER_FLAG_RSERVER_CLIENT | PEER_FLAG_NEXTHOP_SELF                   \
 	 | PEER_FLAG_NEXTHOP_UNCHANGED | PEER_FLAG_FORCE_NEXTHOP_SELF          \
@@ -45,8 +65,7 @@
 	 | PEER_FLAG_REMOVE_PRIVATE_AS_ALL                                     \
 	 | PEER_FLAG_REMOVE_PRIVATE_AS_REPLACE                                 \
 	 | PEER_FLAG_REMOVE_PRIVATE_AS_ALL_REPLACE                             \
-	 | PEER_FLAG_ADDPATH_TX_ALL_PATHS                                      \
-	 | PEER_FLAG_ADDPATH_TX_BESTPATH_PER_AS | PEER_FLAG_AS_OVERRIDE)
+	 | PEER_FLAG_AS_OVERRIDE)
 
 #define PEER_UPDGRP_CAP_FLAGS (PEER_CAP_AS4_RCV)
 
@@ -58,7 +77,7 @@
 typedef enum { BGP_ATTR_VEC_NH = 0, BGP_ATTR_VEC_MAX } bpacket_attr_vec_type;
 
 typedef struct {
-	u_int32_t flags;
+	uint32_t flags;
 	unsigned long offset;
 } bpacket_attr_vec;
 
@@ -120,19 +139,19 @@ struct update_group {
 	uint64_t id;
 	time_t uptime;
 
-	u_int32_t join_events;
-	u_int32_t prune_events;
-	u_int32_t merge_events;
-	u_int32_t updgrp_switch_events;
-	u_int32_t peer_refreshes_combined;
-	u_int32_t adj_count;
-	u_int32_t split_events;
-	u_int32_t merge_checks_triggered;
+	uint32_t join_events;
+	uint32_t prune_events;
+	uint32_t merge_events;
+	uint32_t updgrp_switch_events;
+	uint32_t peer_refreshes_combined;
+	uint32_t adj_count;
+	uint32_t split_events;
+	uint32_t merge_checks_triggered;
 
-	u_int32_t subgrps_created;
-	u_int32_t subgrps_deleted;
+	uint32_t subgrps_created;
+	uint32_t subgrps_deleted;
 
-	u_int32_t num_dbg_en_peers;
+	uint32_t num_dbg_en_peers;
 };
 
 /*
@@ -180,7 +199,7 @@ struct update_subgroup {
 	struct stream *work;
 
 	/* We use a separate stream to encode MP_REACH_NLRI for efficient
-	 * NLRI packing. peer->work stores all the other attributes. The
+	 * NLRI packing. peer->obuf_work stores all the other attributes. The
 	 * actual packet is then constructed by concatenating the two.
 	 */
 	struct stream *scratch;
@@ -189,13 +208,13 @@ struct update_subgroup {
 	struct bgp_synchronize *sync;
 
 	/* send prefix count */
-	unsigned long scount;
+	uint32_t scount;
 
 	/* announcement attribute hash */
 	struct hash *hash;
 
 	struct thread *t_coalesce;
-	u_int32_t v_coalesce;
+	uint32_t v_coalesce;
 
 	struct thread *t_merge_check;
 
@@ -217,25 +236,25 @@ struct update_subgroup {
 		uint64_t subgroup_id;
 	} split_from;
 
-	u_int32_t join_events;
-	u_int32_t prune_events;
+	uint32_t join_events;
+	uint32_t prune_events;
 
 	/*
 	 * This is bumped up when another subgroup merges into this one.
 	 */
-	u_int32_t merge_events;
-	u_int32_t updgrp_switch_events;
-	u_int32_t peer_refreshes_combined;
-	u_int32_t adj_count;
-	u_int32_t split_events;
-	u_int32_t merge_checks_triggered;
+	uint32_t merge_events;
+	uint32_t updgrp_switch_events;
+	uint32_t peer_refreshes_combined;
+	uint32_t adj_count;
+	uint32_t split_events;
+	uint32_t merge_checks_triggered;
 
 	uint64_t id;
 
-	u_int16_t sflags;
+	uint16_t sflags;
 
 	/* Subgroup flags, see below  */
-	u_int16_t flags;
+	uint16_t flags;
 };
 
 /*
@@ -268,14 +287,13 @@ struct update_subgroup {
  */
 #define SUBGRP_DECR_STAT(subgrp, stat) SUBGRP_INCR_STAT_BY(subgrp, stat, -1)
 
-
 typedef int (*updgrp_walkcb)(struct update_group *updgrp, void *ctx);
 
 /* really a private structure */
 struct updwalk_context {
 	struct vty *vty;
 	struct bgp_node *rn;
-	struct bgp_info *ri;
+	struct bgp_path_info *pi;
 	uint64_t updgrp_id;
 	uint64_t subgrp_id;
 	bgp_policy_type_e policy_type;
@@ -284,7 +302,7 @@ struct updwalk_context {
 	int policy_route_update;
 	updgrp_walkcb cb;
 	void *context;
-	u_int8_t flags;
+	uint8_t flags;
 
 #define UPDWALK_FLAGS_ADVQUEUE   (1 << 0)
 #define UPDWALK_FLAGS_ADVERTISED (1 << 1)
@@ -322,23 +340,23 @@ struct updwalk_context {
  * Walk all subgroups in an update group.
  */
 #define UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)                                  \
-	LIST_FOREACH(subgrp, &((updgrp)->subgrps), updgrp_train)
+	LIST_FOREACH (subgrp, &((updgrp)->subgrps), updgrp_train)
 
 #define UPDGRP_FOREACH_SUBGRP_SAFE(updgrp, subgrp, tmp_subgrp)                 \
-	LIST_FOREACH_SAFE(subgrp, &((updgrp)->subgrps), updgrp_train,          \
-			  tmp_subgrp)
+	LIST_FOREACH_SAFE (subgrp, &((updgrp)->subgrps), updgrp_train,         \
+			   tmp_subgrp)
 
 #define SUBGRP_FOREACH_PEER(subgrp, paf)                                       \
-	LIST_FOREACH(paf, &(subgrp->peers), subgrp_train)
+	LIST_FOREACH (paf, &(subgrp->peers), subgrp_train)
 
 #define SUBGRP_FOREACH_PEER_SAFE(subgrp, paf, temp_paf)                        \
-	LIST_FOREACH_SAFE(paf, &(subgrp->peers), subgrp_train, temp_paf)
+	LIST_FOREACH_SAFE (paf, &(subgrp->peers), subgrp_train, temp_paf)
 
 #define SUBGRP_FOREACH_ADJ(subgrp, adj)                                        \
-	TAILQ_FOREACH(adj, &(subgrp->adjq), subgrp_adj_train)
+	TAILQ_FOREACH (adj, &(subgrp->adjq), subgrp_adj_train)
 
 #define SUBGRP_FOREACH_ADJ_SAFE(subgrp, adj, adj_temp)                         \
-	TAILQ_FOREACH_SAFE(adj, &(subgrp->adjq), subgrp_adj_train, adj_temp)
+	TAILQ_FOREACH_SAFE (adj, &(subgrp->adjq), subgrp_adj_train, adj_temp)
 
 /* Prototypes.  */
 /* bgp_updgrp.c */
@@ -355,9 +373,9 @@ extern void update_subgroup_remove_peer(struct update_subgroup *,
 					struct peer_af *);
 extern struct bgp_table *update_subgroup_rib(struct update_subgroup *);
 extern void update_subgroup_split_peer(struct peer_af *, struct update_group *);
-extern int update_subgroup_check_merge(struct update_subgroup *, const char *);
-extern int update_subgroup_trigger_merge_check(struct update_subgroup *,
-					       int force);
+extern bool update_subgroup_check_merge(struct update_subgroup *, const char *);
+extern bool update_subgroup_trigger_merge_check(struct update_subgroup *,
+						int force);
 extern void update_group_policy_update(struct bgp *bgp, bgp_policy_type_e ptype,
 				       const char *pname, int route_update,
 				       int start_event);
@@ -386,13 +404,13 @@ extern struct bpacket *bpacket_queue_first(struct bpacket_queue *q);
 struct bpacket *bpacket_queue_last(struct bpacket_queue *q);
 unsigned int bpacket_queue_length(struct bpacket_queue *q);
 unsigned int bpacket_queue_hwm_length(struct bpacket_queue *q);
-int bpacket_queue_is_full(struct bgp *bgp, struct bpacket_queue *q);
+bool bpacket_queue_is_full(struct bgp *bgp, struct bpacket_queue *q);
 extern void bpacket_queue_advance_peer(struct peer_af *paf);
 extern void bpacket_queue_remove_peer(struct peer_af *paf);
 extern void bpacket_add_peer(struct bpacket *pkt, struct peer_af *paf);
 unsigned int bpacket_queue_virtual_length(struct peer_af *paf);
 extern void bpacket_queue_show_vty(struct bpacket_queue *q, struct vty *vty);
-int subgroup_packets_to_build(struct update_subgroup *subgrp);
+bool subgroup_packets_to_build(struct update_subgroup *subgrp);
 extern struct bpacket *subgroup_update_packet(struct update_subgroup *s);
 extern struct bpacket *subgroup_withdraw_packet(struct update_subgroup *s);
 extern struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
@@ -424,23 +442,24 @@ extern void subgroup_announce_all(struct update_subgroup *subgrp);
 extern void subgroup_default_originate(struct update_subgroup *subgrp,
 				       int withdraw);
 extern void group_announce_route(struct bgp *bgp, afi_t afi, safi_t safi,
-				 struct bgp_node *rn, struct bgp_info *ri);
+				 struct bgp_node *rn, struct bgp_path_info *pi);
 extern void subgroup_clear_table(struct update_subgroup *subgrp);
 extern void update_group_announce(struct bgp *bgp);
 extern void update_group_announce_rrclients(struct bgp *bgp);
 extern void peer_af_announce_route(struct peer_af *paf, int combine);
 extern struct bgp_adj_out *bgp_adj_out_alloc(struct update_subgroup *subgrp,
 					     struct bgp_node *rn,
-					     u_int32_t addpath_tx_id);
+					     uint32_t addpath_tx_id);
 extern void bgp_adj_out_remove_subgroup(struct bgp_node *rn,
 					struct bgp_adj_out *adj,
 					struct update_subgroup *subgrp);
 extern void bgp_adj_out_set_subgroup(struct bgp_node *rn,
 				     struct update_subgroup *subgrp,
-				     struct attr *attr, struct bgp_info *binfo);
+				     struct attr *attr,
+				     struct bgp_path_info *path);
 extern void bgp_adj_out_unset_subgroup(struct bgp_node *rn,
 				       struct update_subgroup *subgrp,
-				       char withdraw, u_int32_t addpath_tx_id);
+				       char withdraw, uint32_t addpath_tx_id);
 void subgroup_announce_table(struct update_subgroup *subgrp,
 			     struct bgp_table *table);
 extern void subgroup_trigger_write(struct update_subgroup *subgrp);
@@ -450,8 +469,6 @@ extern int update_group_clear_update_dbg(struct update_group *updgrp,
 
 extern void update_bgp_group_free(struct bgp *bgp);
 extern int bgp_addpath_encode_tx(struct peer *peer, afi_t afi, safi_t safi);
-extern int bgp_addpath_tx_path(struct peer *peer, afi_t afi, safi_t safi,
-			       struct bgp_info *ri);
 
 /*
  * Inline functions
@@ -574,9 +591,9 @@ static inline void bgp_announce_peer(struct peer *peer)
  */
 static inline int advertise_list_is_empty(struct update_subgroup *subgrp)
 {
-	if (!BGP_ADV_FIFO_EMPTY(&subgrp->sync->update)
-	    || !BGP_ADV_FIFO_EMPTY(&subgrp->sync->withdraw)
-	    || !BGP_ADV_FIFO_EMPTY(&subgrp->sync->withdraw_low)) {
+	if (bgp_adv_fifo_count(&subgrp->sync->update)
+	    || bgp_adv_fifo_count(&subgrp->sync->withdraw)
+	    || bgp_adv_fifo_count(&subgrp->sync->withdraw_low)) {
 		return 0;
 	}
 
