@@ -32,7 +32,6 @@
 #include "if.h"
 #include "stream.h"
 
-#include "isisd/dict.h"
 #include "isisd/isis_constants.h"
 #include "isisd/isis_common.h"
 #include "isisd/isis_flags.h"
@@ -48,6 +47,8 @@
 #include "isisd/isis_events.h"
 #include "isisd/isis_mt.h"
 #include "isisd/isis_tlvs.h"
+#include "isisd/fabricd.h"
+#include "isisd/isis_nb.h"
 
 extern struct isis *isis;
 
@@ -92,6 +93,7 @@ struct isis_adjacency *isis_new_adj(const uint8_t *id, const uint8_t *snpa,
 				.last_dis_change = time(NULL);
 		}
 	}
+	adj->adj_sids = list_new();
 
 	return adj;
 }
@@ -121,6 +123,46 @@ struct isis_adjacency *isis_adj_lookup_snpa(const uint8_t *ssnpa,
 	return NULL;
 }
 
+bool isis_adj_exists(const struct isis_area *area, int level,
+		     const uint8_t *sysid)
+{
+	struct isis_circuit *circuit;
+	struct listnode *node;
+
+	for (ALL_LIST_ELEMENTS_RO(area->circuit_list, node, circuit)) {
+		struct isis_adjacency *adj;
+		struct listnode *anode;
+		struct list *adjdb;
+
+		switch (circuit->circ_type) {
+		case CIRCUIT_T_BROADCAST:
+			adjdb = circuit->u.bc.adjdb[level - 1];
+			if (!adjdb)
+				continue;
+
+			for (ALL_LIST_ELEMENTS_RO(adjdb, anode, adj)) {
+				if (!memcmp(adj->sysid, sysid, ISIS_SYS_ID_LEN))
+					return true;
+			}
+			break;
+		case CIRCUIT_T_P2P:
+			adj = circuit->u.p2p.neighbor;
+			if (!adj)
+				break;
+
+			if (!memcmp(adj->sysid, sysid, ISIS_SYS_ID_LEN))
+				return true;
+			break;
+		default:
+			break;
+		}
+	}
+
+	return false;
+}
+
+DEFINE_HOOK(isis_adj_state_change_hook, (struct isis_adjacency *adj), (adj))
+
 void isis_delete_adj(void *arg)
 {
 	struct isis_adjacency *adj = arg;
@@ -129,18 +171,20 @@ void isis_delete_adj(void *arg)
 		return;
 
 	THREAD_TIMER_OFF(adj->t_expire);
+	if (adj->adj_state != ISIS_ADJ_DOWN) {
+		adj->adj_state = ISIS_ADJ_DOWN;
+		hook_call(isis_adj_state_change_hook, adj);
+	}
 
 	/* remove from SPF trees */
 	spftree_area_adj_del(adj->circuit->area, adj);
 
-	if (adj->area_addresses)
-		XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->area_addresses);
-	if (adj->ipv4_addresses)
-		XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->ipv4_addresses);
-	if (adj->ipv6_addresses)
-		XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->ipv6_addresses);
+	XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->area_addresses);
+	XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->ipv4_addresses);
+	XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->ipv6_addresses);
 
 	adj_mt_finish(adj);
+	list_delete(&adj->adj_sids);
 
 	XFREE(MTYPE_ISIS_ADJACENCY, adj);
 	return;
@@ -193,33 +237,41 @@ void isis_adj_process_threeway(struct isis_adjacency *adj,
 		}
 	}
 
+	if (next_tw_state != ISIS_THREEWAY_DOWN)
+		fabricd_initial_sync_hello(adj->circuit);
+
 	if (next_tw_state == ISIS_THREEWAY_DOWN) {
-		isis_adj_state_change(adj, ISIS_ADJ_DOWN, "Neighbor restarted");
+		isis_adj_state_change(&adj, ISIS_ADJ_DOWN,
+				      "Neighbor restarted");
 		return;
 	}
 
 	if (next_tw_state == ISIS_THREEWAY_UP) {
 		if (adj->adj_state != ISIS_ADJ_UP) {
-			isis_adj_state_change(adj, ISIS_ADJ_UP, NULL);
+			isis_adj_state_change(&adj, ISIS_ADJ_UP, NULL);
 			adj->adj_usage = adj_usage;
 		}
 	}
 
+	if (adj->threeway_state != next_tw_state) {
+		send_hello_sched(adj->circuit, 0, TRIGGERED_IIH_DELAY);
+	}
+
 	adj->threeway_state = next_tw_state;
 }
 
-void isis_adj_state_change(struct isis_adjacency *adj,
+void isis_adj_state_change(struct isis_adjacency **padj,
 			   enum isis_adj_state new_state, const char *reason)
 {
-	int old_state;
-	int level;
-	struct isis_circuit *circuit;
-	bool del;
+	struct isis_adjacency *adj = *padj;
+	enum isis_adj_state old_state = adj->adj_state;
+	struct isis_circuit *circuit = adj->circuit;
+	bool del = false;
 
-	old_state = adj->adj_state;
 	adj->adj_state = new_state;
-
-	circuit = adj->circuit;
+	if (new_state != old_state) {
+		send_hello_sched(circuit, adj->level, TRIGGERED_IIH_DELAY);
+	}
 
 	if (isis->debugs & DEBUG_ADJ_PACKETS) {
 		zlog_debug("ISIS-Adj (%s): Adjacency state change %d->%d: %s",
@@ -245,15 +297,19 @@ void isis_adj_state_change(struct isis_adjacency *adj,
 			reason ? reason : "unspecified");
 	}
 
+	circuit->adj_state_changes++;
+#ifndef FABRICD
+	/* send northbound notification */
+	isis_notif_adj_state_change(adj, new_state, reason);
+#endif /* ifndef FABRICD */
+
 	if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
-		del = false;
-		for (level = IS_LEVEL_1; level <= IS_LEVEL_2; level++) {
+		for (int level = IS_LEVEL_1; level <= IS_LEVEL_2; level++) {
 			if ((adj->level & level) == 0)
 				continue;
 			if (new_state == ISIS_ADJ_UP) {
 				circuit->upadjcount[level - 1]++;
-				isis_event_adjacency_state_change(adj,
-								  new_state);
+				hook_call(isis_adj_state_change_hook, adj);
 				/* update counter & timers for debugging
 				 * purposes */
 				adj->last_flap = time(NULL);
@@ -264,10 +320,9 @@ void isis_adj_state_change(struct isis_adjacency *adj,
 
 				circuit->upadjcount[level - 1]--;
 				if (circuit->upadjcount[level - 1] == 0)
-					isis_circuit_lsp_queue_clean(circuit);
+					isis_tx_queue_clean(circuit->tx_queue);
 
-				isis_event_adjacency_state_change(adj,
-								  new_state);
+				hook_call(isis_adj_state_change_hook, adj);
 				del = true;
 			}
 
@@ -285,52 +340,45 @@ void isis_adj_state_change(struct isis_adjacency *adj,
 				lsp_regenerate_schedule_pseudo(circuit, level);
 		}
 
-		if (del)
-			isis_delete_adj(adj);
-
-		adj = NULL;
 	} else if (circuit->circ_type == CIRCUIT_T_P2P) {
-		del = false;
-		for (level = IS_LEVEL_1; level <= IS_LEVEL_2; level++) {
+		for (int level = IS_LEVEL_1; level <= IS_LEVEL_2; level++) {
 			if ((adj->level & level) == 0)
 				continue;
 			if (new_state == ISIS_ADJ_UP) {
 				circuit->upadjcount[level - 1]++;
-				isis_event_adjacency_state_change(adj,
-								  new_state);
-
-				if (adj->sys_type == ISIS_SYSTYPE_UNKNOWN)
-					send_hello(circuit, level);
+				hook_call(isis_adj_state_change_hook, adj);
 
 				/* update counter & timers for debugging
 				 * purposes */
 				adj->last_flap = time(NULL);
 				adj->flaps++;
 
-				/* 7.3.17 - going up on P2P -> send CSNP */
-				/* FIXME: yup, I know its wrong... but i will do
-				 * it! (for now) */
-				send_csnp(circuit, level);
+				if (level == IS_LEVEL_1) {
+					thread_add_timer(master, send_l1_csnp,
+							 circuit, 0,
+							 &circuit->t_send_csnp[0]);
+				} else {
+					thread_add_timer(master, send_l2_csnp,
+							 circuit, 0,
+							 &circuit->t_send_csnp[1]);
+				}
 			} else if (new_state == ISIS_ADJ_DOWN) {
 				if (adj->circuit->u.p2p.neighbor == adj)
 					adj->circuit->u.p2p.neighbor = NULL;
 				circuit->upadjcount[level - 1]--;
 				if (circuit->upadjcount[level - 1] == 0)
-					isis_circuit_lsp_queue_clean(circuit);
+					isis_tx_queue_clean(circuit->tx_queue);
 
-				isis_event_adjacency_state_change(adj,
-								  new_state);
+				hook_call(isis_adj_state_change_hook, adj);
 				del = true;
 			}
 		}
-
-		if (del)
-			isis_delete_adj(adj);
-
-		adj = NULL;
 	}
 
-	return;
+	if (del) {
+		isis_delete_adj(adj);
+		*padj = NULL;
+	}
 }
 
 
@@ -344,7 +392,7 @@ void isis_adj_print(struct isis_adjacency *adj)
 	if (dyn)
 		zlog_debug("%s", dyn->hostname);
 
-	zlog_debug("SystemId %20s SNPA %s, level %d\nHolding Time %d",
+	zlog_debug("SystemId %20s SNPA %s, level %d; Holding Time %d",
 		   sysid_print(adj->sysid), snpa_print(adj->snpa), adj->level,
 		   adj->hold_time);
 	if (adj->ipv4_address_count) {
@@ -367,6 +415,20 @@ void isis_adj_print(struct isis_adjacency *adj)
 	return;
 }
 
+const char *isis_adj_yang_state(enum isis_adj_state state)
+{
+	switch (state) {
+	case ISIS_ADJ_DOWN:
+		return "down";
+	case ISIS_ADJ_UP:
+		return "up";
+	case ISIS_ADJ_INITIALIZING:
+		return "init";
+	default:
+		return "failed";
+	}
+}
+
 int isis_adj_expire(struct thread *thread)
 {
 	struct isis_adjacency *adj;
@@ -379,7 +441,7 @@ int isis_adj_expire(struct thread *thread)
 	adj->t_expire = NULL;
 
 	/* trigger the adj expire event */
-	isis_adj_state_change(adj, ISIS_ADJ_DOWN, "holding time expired");
+	isis_adj_state_change(&adj, ISIS_ADJ_DOWN, "holding time expired");
 
 	return 0;
 }
@@ -419,6 +481,9 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 	}
 
 	if (detail == ISIS_UI_LEVEL_DETAIL) {
+		struct sr_adjacency *sra;
+		struct listnode *anode;
+
 		level = adj->level;
 		vty_out(vty, "\n");
 		if (adj->circuit)
@@ -507,6 +572,31 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 				vty_out(vty, "      %s\n", buf);
 			}
 		}
+		for (ALL_LIST_ELEMENTS_RO(adj->adj_sids, anode, sra)) {
+			const char *adj_type;
+			const char *backup;
+			uint32_t sid;
+
+			switch (sra->adj->circuit->circ_type) {
+			case CIRCUIT_T_BROADCAST:
+				adj_type = "LAN Adjacency-SID";
+				sid = sra->u.ladj_sid->sid;
+				break;
+			case CIRCUIT_T_P2P:
+				adj_type = "Adjacency-SID";
+				sid = sra->u.adj_sid->sid;
+				break;
+			default:
+				continue;
+			}
+			backup = (sra->type == ISIS_SR_LAN_BACKUP) ? " (backup)"
+								   : "";
+
+			vty_out(vty, "    %s %s%s: %u\n",
+				(sra->nexthop.family == AF_INET) ? "IPv4"
+								 : "IPv6",
+				adj_type, backup, sid);
+		}
 		vty_out(vty, "\n");
 	}
 	return;
