@@ -30,65 +30,72 @@
 #include "vty.h"
 #include "ldp_vty.h"
 
+static int	 ldp_config_write(struct vty *);
 static void	 ldp_af_iface_config_write(struct vty *, int);
 static void	 ldp_af_config_write(struct vty *, int, struct ldpd_conf *,
 		    struct ldpd_af_conf *);
+static int	 ldp_l2vpn_config_write(struct vty *);
 static void	 ldp_l2vpn_pw_config_write(struct vty *, struct l2vpn_pw *);
 static int	 ldp_vty_get_af(struct vty *);
 static int	 ldp_iface_is_configured(struct ldpd_conf *, const char *);
 
-struct cmd_node ldp_node =
-{
-	LDP_NODE,
-	"%s(config-ldp)# ",
-	1,
+struct cmd_node ldp_node = {
+	.name = "ldp",
+	.node = LDP_NODE,
+	.parent_node = CONFIG_NODE,
+	.prompt = "%s(config-ldp)# ",
+	.config_write = ldp_config_write,
 };
 
-struct cmd_node ldp_ipv4_node =
-{
-	LDP_IPV4_NODE,
-	"%s(config-ldp-af)# ",
-	1,
+struct cmd_node ldp_ipv4_node = {
+	.name = "ldp ipv4",
+	.node = LDP_IPV4_NODE,
+	.parent_node = LDP_NODE,
+	.prompt = "%s(config-ldp-af)# ",
 };
 
-struct cmd_node ldp_ipv6_node =
-{
-	LDP_IPV6_NODE,
-	"%s(config-ldp-af)# ",
-	1,
+struct cmd_node ldp_ipv6_node = {
+	.name = "ldp ipv6",
+	.node = LDP_IPV6_NODE,
+	.parent_node = LDP_NODE,
+	.prompt = "%s(config-ldp-af)# ",
 };
 
-struct cmd_node ldp_ipv4_iface_node =
-{
-	LDP_IPV4_IFACE_NODE,
-	"%s(config-ldp-af-if)# ",
-	1,
+struct cmd_node ldp_ipv4_iface_node = {
+	.name = "ldp ipv4 interface",
+	.node = LDP_IPV4_IFACE_NODE,
+	.parent_node = LDP_IPV4_NODE,
+	.prompt = "%s(config-ldp-af-if)# ",
 };
 
-struct cmd_node ldp_ipv6_iface_node =
-{
-	LDP_IPV6_IFACE_NODE,
-	"%s(config-ldp-af-if)# ",
-	1,
+struct cmd_node ldp_ipv6_iface_node = {
+	.name = "ldp ipv6 interface",
+	.node = LDP_IPV6_IFACE_NODE,
+	.parent_node = LDP_IPV6_NODE,
+	.prompt = "%s(config-ldp-af-if)# ",
 };
 
-struct cmd_node ldp_l2vpn_node =
-{
-	LDP_L2VPN_NODE,
-	"%s(config-l2vpn)# ",
-	1,
+struct cmd_node ldp_l2vpn_node = {
+	.name = "ldp l2vpn",
+	.node = LDP_L2VPN_NODE,
+	.parent_node = CONFIG_NODE,
+	.prompt = "%s(config-l2vpn)# ",
+	.config_write = ldp_l2vpn_config_write,
 };
 
-struct cmd_node ldp_pseudowire_node =
-{
-	LDP_PSEUDOWIRE_NODE,
-	"%s(config-l2vpn-pw)# ",
-	1,
+struct cmd_node ldp_pseudowire_node = {
+	.name = "ldp",
+	.node = LDP_PSEUDOWIRE_NODE,
+	.parent_node = LDP_L2VPN_NODE,
+	.prompt = "%s(config-l2vpn-pw)# ",
 };
 
 int
 ldp_get_address(const char *str, int *af, union ldpd_addr *addr)
 {
+	if (!str || !af || !addr)
+		return (-1);
+
 	memset(addr, 0, sizeof(*addr));
 
 	if (inet_pton(AF_INET, str, &addr->v4) == 1) {
@@ -237,7 +244,7 @@ ldp_af_config_write(struct vty *vty, int af, struct ldpd_conf *conf,
 	vty_out(vty, " exit-address-family\n");
 }
 
-int
+static int
 ldp_config_write(struct vty *vty)
 {
 	struct nbr_params	*nbrp;
@@ -247,9 +254,8 @@ ldp_config_write(struct vty *vty)
 
 	vty_out (vty, "mpls ldp\n");
 
-	if (ldpd_conf->rtr_id.s_addr != 0)
-		vty_out (vty, " router-id %s\n",
-		    inet_ntoa(ldpd_conf->rtr_id));
+	if (ldpd_conf->rtr_id.s_addr != INADDR_ANY)
+		vty_out(vty, " router-id %s\n", inet_ntoa(ldpd_conf->rtr_id));
 
 	if (ldpd_conf->lhello_holdtime != LINK_DFLT_HOLDTIME &&
 	    ldpd_conf->lhello_holdtime != 0)
@@ -276,6 +282,9 @@ ldp_config_write(struct vty *vty)
 	if (ldpd_conf->flags & F_LDPD_DS_CISCO_INTEROP)
 		vty_out (vty, " dual-stack cisco-interop\n");
 
+	if (ldpd_conf->flags & F_LDPD_ORDERED_CONTROL)
+		vty_out (vty, " ordered-control\n");
+
 	RB_FOREACH(nbrp, nbrp_head, &ldpd_conf->nbrp_tree) {
 		if (nbrp->flags & F_NBRP_KEEPALIVE)
 			vty_out (vty, " neighbor %s session holdtime %u\n",
@@ -340,7 +349,7 @@ ldp_l2vpn_pw_config_write(struct vty *vty, struct l2vpn_pw *pw)
 		vty_out (vty,"  ! Incomplete config, specify a pw-id\n");
 }
 
-int
+static int
 ldp_l2vpn_config_write(struct vty *vty)
 {
 	struct l2vpn		*l2vpn;
@@ -428,6 +437,9 @@ ldp_vty_address_family(struct vty *vty, const char *negate, const char *af_str)
 	struct ldpd_af_conf	*af_conf;
 	int			 af;
 
+	if (af_str == NULL)
+		return (CMD_WARNING_CONFIG_FAILED);
+
 	if (strcmp(af_str, "ipv4") == 0) {
 		af = AF_INET;
 		af_conf = &vty_conf->ipv4;
@@ -709,6 +721,11 @@ ldp_vty_interface(struct vty *vty, const char *negate, const char *ifname)
 	struct iface		*iface;
 	struct iface_af		*ia;
 
+	if (ifname == NULL) {
+		vty_out (vty, "%% Missing IF name\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	af = ldp_vty_get_af(vty);
 	iface = if_lookup_name(vty_conf, ifname);
 
@@ -776,8 +793,9 @@ ldp_vty_trans_addr(struct vty *vty, const char *negate, const char *addr_str)
 	if (negate)
 		memset(&af_conf->trans_addr, 0, sizeof(af_conf->trans_addr));
 	else {
-		if (inet_pton(af, addr_str, &af_conf->trans_addr) != 1 ||
-		    bad_addr(af, &af_conf->trans_addr)) {
+		if (addr_str == NULL
+		    || inet_pton(af, addr_str, &af_conf->trans_addr) != 1
+		    || bad_addr(af, &af_conf->trans_addr)) {
 			vty_out (vty, "%% Malformed address\n");
 			return (CMD_SUCCESS);
 		}
@@ -797,7 +815,7 @@ ldp_vty_neighbor_targeted(struct vty *vty, const char *negate, const char *addr_
 
 	af = ldp_vty_get_af(vty);
 
-	if (inet_pton(af, addr_str, &addr) != 1 ||
+	if (addr_str == NULL || inet_pton(af, addr_str, &addr) != 1 ||
 	    bad_addr(af, &addr)) {
 		vty_out (vty, "%% Malformed address\n");
 		return (CMD_WARNING_CONFIG_FAILED);
@@ -985,6 +1003,19 @@ ldp_vty_router_id(struct vty *vty, const char *negate, struct in_addr address)
 	return (CMD_SUCCESS);
 }
 
+int
+ldp_vty_ordered_control(struct vty *vty, const char *negate)
+{
+	if (negate)
+		vty_conf->flags &= ~F_LDPD_ORDERED_CONTROL;
+	else
+		vty_conf->flags |= F_LDPD_ORDERED_CONTROL;
+
+	ldp_config_apply(vty, vty_conf);
+
+	return (CMD_SUCCESS);
+}
+
 int
 ldp_vty_ds_cisco_interop(struct vty *vty, const char * negate)
 {
@@ -1018,6 +1049,11 @@ ldp_vty_neighbor_password(struct vty *vty, const char *negate, struct in_addr ls
 	size_t			 password_len;
 	struct nbr_params	*nbrp;
 
+	if (password_str == NULL) {
+		vty_out (vty, "%% Missing password\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	if (bad_addr_v4(lsr_id)) {
 		vty_out (vty, "%% Malformed address\n");
 		return (CMD_WARNING_CONFIG_FAILED);
@@ -1045,7 +1081,7 @@ ldp_vty_neighbor_password(struct vty *vty, const char *negate, struct in_addr ls
 		if (password_len >= sizeof(nbrp->auth.md5key))
 			vty_out(vty, "%% password has been truncated to %zu "
 			    "characters.", sizeof(nbrp->auth.md5key) - 1);
-		nbrp->auth.md5key_len = password_len;
+		nbrp->auth.md5key_len = strlen(nbrp->auth.md5key);
 		nbrp->auth.method = AUTH_MD5SIG;
 	}
 
@@ -1113,6 +1149,11 @@ ldp_vty_l2vpn(struct vty *vty, const char *negate, const char *name_str)
 	struct l2vpn_if		*lif;
 	struct l2vpn_pw		*pw;
 
+	if (name_str == NULL) {
+		vty_out (vty, "%% Missing name\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	l2vpn = l2vpn_find(vty_conf, name_str);
 
 	if (negate) {
@@ -1158,8 +1199,13 @@ ldp_vty_l2vpn_bridge(struct vty *vty, const char *negate, const char *ifname)
 
 	if (negate)
 		memset(l2vpn->br_ifname, 0, sizeof(l2vpn->br_ifname));
-	else
+	else {
+		if (ifname == NULL) {
+			vty_out (vty, "%% Missing IF name\n");
+			return (CMD_WARNING_CONFIG_FAILED);
+		}
 		strlcpy(l2vpn->br_ifname, ifname, sizeof(l2vpn->br_ifname));
+	}
 
 	ldp_config_apply(vty, vty_conf);
 
@@ -1187,6 +1233,11 @@ ldp_vty_l2vpn_pwtype(struct vty *vty, const char *negate, const char *type_str)
 	VTY_DECLVAR_CONTEXT(l2vpn, l2vpn);
 	int			 pw_type;
 
+	if (type_str == NULL) {
+		vty_out (vty, "%% Missing type\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	if (strcmp(type_str, "ethernet") == 0)
 		pw_type = PW_TYPE_ETHERNET;
 	else
@@ -1208,6 +1259,11 @@ ldp_vty_l2vpn_interface(struct vty *vty, const char *negate, const char *ifname)
 	VTY_DECLVAR_CONTEXT(l2vpn, l2vpn);
 	struct l2vpn_if		*lif;
 
+	if (ifname == NULL) {
+		vty_out (vty, "%% Missing IF name\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	lif = l2vpn_if_find(l2vpn, ifname);
 
 	if (negate) {
@@ -1246,6 +1302,11 @@ ldp_vty_l2vpn_pseudowire(struct vty *vty, const char *negate, const char *ifname
 	VTY_DECLVAR_CONTEXT(l2vpn, l2vpn);
 	struct l2vpn_pw		*pw;
 
+	if (ifname == NULL) {
+		vty_out (vty, "%% Missing IF name\n");
+		return (CMD_WARNING_CONFIG_FAILED);
+	}
+
 	pw = l2vpn_pw_find(l2vpn, ifname);
 
 	if (negate) {
@@ -1294,6 +1355,10 @@ ldp_vty_l2vpn_pw_cword(struct vty *vty, const char *negate, const char *preferen
 	if (negate)
 		pw->flags |= F_PW_CWORD_CONF;
 	else {
+		if (!preference_str) {
+			vty_out (vty, "%% Missing preference\n");
+			return (CMD_WARNING_CONFIG_FAILED);
+		}
 		if (preference_str[0] == 'e')
 			pw->flags &= ~F_PW_CWORD_CONF;
 		else
