@@ -151,7 +151,9 @@ enum imsg_type {
 	IMSG_LOG,
 	IMSG_ACL_CHECK,
 	IMSG_INIT,
-	IMSG_PW_UPDATE
+	IMSG_PW_UPDATE,
+	IMSG_FILTER_UPDATE,
+	IMSG_NBR_SHUTDOWN
 };
 
 struct ldpd_init {
@@ -162,6 +164,11 @@ struct ldpd_init {
 	unsigned short instance;
 };
 
+struct ldp_access {
+	char			 name[ACL_NAMSIZ];
+	enum access_type	 type;
+};
+
 union ldpd_addr {
 	struct in_addr	v4;
 	struct in6_addr	v6;
@@ -306,7 +313,7 @@ struct iface_af {
 struct iface {
 	RB_ENTRY(iface)		 entry;
 	char			 name[IF_NAMESIZE];
-	unsigned int		 ifindex;
+	ifindex_t		 ifindex;
 	struct if_addr_head	 addr_list;
 	struct in6_addr		 linklocal;
 	enum iface_type		 type;
@@ -391,7 +398,7 @@ struct l2vpn_if {
 	RB_ENTRY(l2vpn_if)	 entry;
 	struct l2vpn		*l2vpn;
 	char			 ifname[IF_NAMESIZE];
-	unsigned int		 ifindex;
+	ifindex_t		 ifindex;
 	int			 operative;
 	uint8_t			 mac[ETH_ALEN];
 	QOBJ_FIELDS
@@ -408,7 +415,7 @@ struct l2vpn_pw {
 	union ldpd_addr		 addr;
 	uint32_t		 pwid;
 	char			 ifname[IF_NAMESIZE];
-	unsigned int		 ifindex;
+	ifindex_t		 ifindex;
 	bool			 enabled;
 	uint32_t		 remote_group;
 	uint16_t		 remote_mtu;
@@ -433,7 +440,7 @@ struct l2vpn {
 	int			 pw_type;
 	int			 mtu;
 	char			 br_ifname[IF_NAMESIZE];
-	unsigned int		 br_ifindex;
+	ifindex_t		 br_ifindex;
 	struct l2vpn_if_head	 if_tree;
 	struct l2vpn_pw_head	 pw_tree;
 	struct l2vpn_pw_head	 pw_inactive_tree;
@@ -446,7 +453,7 @@ DECLARE_QOBJ_TYPE(l2vpn)
 #define L2VPN_TYPE_VPLS		2
 
 /* ldp_conf */
-enum ldpd_process {
+extern enum ldpd_process {
 	PROC_MAIN,
 	PROC_LDP_ENGINE,
 	PROC_LDE_ENGINE
@@ -511,6 +518,8 @@ DECLARE_QOBJ_TYPE(ldpd_conf)
 #define	F_LDPD_NO_FIB_UPDATE	0x0001
 #define	F_LDPD_DS_CISCO_INTEROP	0x0002
 #define	F_LDPD_ENABLED		0x0004
+#define	F_LDPD_ORDERED_CONTROL  0x0008
+
 
 struct ldpd_af_global {
 	struct thread		*disc_ev;
@@ -542,14 +551,15 @@ struct kroute {
 	union ldpd_addr		 nexthop;
 	uint32_t		 local_label;
 	uint32_t		 remote_label;
-	unsigned short		 ifindex;
-	uint8_t			 priority;
+	ifindex_t		 ifindex;
+	uint8_t			 route_type;
+	uint8_t			 route_instance;
 	uint16_t		 flags;
 };
 
 struct kaddr {
 	char			 ifname[IF_NAMESIZE];
-	unsigned short		 ifindex;
+	ifindex_t		 ifindex;
 	int			 af;
 	union ldpd_addr		 addr;
 	uint8_t			 prefixlen;
@@ -558,7 +568,7 @@ struct kaddr {
 
 struct kif {
 	char			 ifname[IF_NAMESIZE];
-	unsigned short		 ifindex;
+	ifindex_t		 ifindex;
 	int			 flags;
 	int			 operative;
 	uint8_t			 mac[ETH_ALEN];
@@ -576,7 +586,7 @@ struct acl_check {
 struct ctl_iface {
 	int			 af;
 	char			 name[IF_NAMESIZE];
-	unsigned int		 ifindex;
+	ifindex_t		 ifindex;
 	int			 state;
 	enum iface_type		 type;
 	uint16_t		 hello_holdtime;
@@ -759,7 +769,7 @@ int		 sock_set_bindany(int, int);
 int		 sock_set_md5sig(int, int, union ldpd_addr *, const char *);
 int		 sock_set_ipv4_tos(int, int);
 int		 sock_set_ipv4_pktinfo(int, int);
-int		 sock_set_ipv4_recvdstaddr(int, int);
+int		 sock_set_ipv4_recvdstaddr(int fd, ifindex_t ifindex);
 int		 sock_set_ipv4_recvif(int, int);
 int		 sock_set_ipv4_minttl(int, int);
 int		 sock_set_ipv4_ucast_ttl(int fd, int);
@@ -782,7 +792,8 @@ struct fec;
 
 const char	*log_sockaddr(void *);
 const char	*log_in6addr(const struct in6_addr *);
-const char	*log_in6addr_scope(const struct in6_addr *, unsigned int);
+const char	*log_in6addr_scope(const struct in6_addr *addr,
+				   ifindex_t ifidx);
 const char	*log_addr(int, const union ldpd_addr *);
 char		*log_label(uint32_t);
 const char	*log_time(time_t);
