@@ -41,7 +41,8 @@ static int	 ldpe_dispatch_pfkey(struct thread *);
 #endif
 static void	 ldpe_setup_sockets(int, int, int, int);
 static void	 ldpe_close_sockets(int);
-static void	 ldpe_iface_af_ctl(struct ctl_conn *, int, unsigned int);
+static void	 ldpe_iface_af_ctl(struct ctl_conn *c, int af, ifindex_t ifidx);
+static void	 ldpe_check_filter_af(int, struct ldpd_af_conf *, const char *);
 
 struct ldpd_conf	*leconf;
 #ifdef __OpenBSD__
@@ -54,9 +55,6 @@ static struct imsgev	*iev_lde;
 static struct thread	*pfkey_ev;
 #endif
 
-/* Master of threads. */
-struct thread_master *master;
-
 /* ldpe privileges */
 static zebra_capabilities_t _caps_p [] =
 {
@@ -97,6 +95,8 @@ static struct quagga_signal_t ldpe_signals[] =
 	},
 };
 
+char *pkt_ptr; /* packet buffer */
+
 /* label distribution protocol engine */
 void
 ldpe(void)
@@ -109,7 +109,7 @@ ldpe(void)
 	ldpd_process = PROC_LDP_ENGINE;
 	log_procname = log_procnames[ldpd_process];
 
-  	master = thread_master_create();
+	master = thread_master_create(NULL);
 
 	/* setup signal handler */
 	signal_init(master, array_size(ldpe_signals), ldpe_signals);
@@ -119,8 +119,9 @@ ldpe(void)
 		fatal(NULL);
 	imsg_init(&iev_main->ibuf, LDPD_FD_ASYNC);
 	iev_main->handler_read = ldpe_dispatch_main;
-	iev_main->ev_read = thread_add_read(master, iev_main->handler_read,
-	    iev_main, iev_main->ibuf.fd);
+	iev_main->ev_read = NULL;
+	thread_add_read(master, iev_main->handler_read, iev_main, iev_main->ibuf.fd,
+		        &iev_main->ev_read);
 	iev_main->handler_write = ldp_write_handler;
 
 	if ((iev_main_sync = calloc(1, sizeof(struct imsgev))) == NULL)
@@ -138,9 +139,20 @@ ldpe(void)
 void
 ldpe_init(struct ldpd_init *init)
 {
+#ifdef __OpenBSD__
+	/* This socket must be open before dropping privileges. */
+	global.pfkeysock = pfkey_init();
+	if (sysdep.no_pfkey == 0) {
+		pfkey_ev = NULL;
+		thread_add_read(master, ldpe_dispatch_pfkey, NULL, global.pfkeysock,
+				&pfkey_ev);
+	}
+#endif
+
 	/* drop privileges */
 	ldpe_privs.user = init->user;
 	ldpe_privs.group = init->group;
+	zprivs_preinit(&ldpe_privs);
 	zprivs_init(&ldpe_privs);
 
 	/* listen on ldpd control socket */
@@ -151,18 +163,12 @@ ldpe_init(struct ldpd_init *init)
 	control_listen();
 
 	LIST_INIT(&global.addr_list);
-	RB_INIT(&global.adj_tree);
+	RB_INIT(global_adj_head, &global.adj_tree);
 	TAILQ_INIT(&global.pending_conns);
 	if (inet_pton(AF_INET, AllRouters_v4, &global.mcast_addr_v4) != 1)
 		fatal("inet_pton");
 	if (inet_pton(AF_INET6, AllRouters_v6, &global.mcast_addr_v6) != 1)
 		fatal("inet_pton");
-#ifdef __OpenBSD__
-	global.pfkeysock = pfkey_init();
-	if (sysdep.no_pfkey == 0)
-		pfkey_ev = thread_add_read(master, ldpe_dispatch_pfkey,
-		    NULL, global.pfkeysock);
-#endif
 
 	/* mark sockets as closed */
 	global.ipv4.ldp_disc_socket = -1;
@@ -186,15 +192,16 @@ ldpe_shutdown(void)
 
 	/* close pipes */
 	if (iev_lde) {
-		msgbuf_write(&iev_lde->ibuf.w);
 		msgbuf_clear(&iev_lde->ibuf.w);
 		close(iev_lde->ibuf.fd);
+		iev_lde->ibuf.fd = -1;
 	}
-	msgbuf_write(&iev_main->ibuf.w);
 	msgbuf_clear(&iev_main->ibuf.w);
 	close(iev_main->ibuf.fd);
+	iev_main->ibuf.fd = -1;
 	msgbuf_clear(&iev_main_sync->ibuf.w);
 	close(iev_main_sync->ibuf.fd);
+	iev_main_sync->ibuf.fd = -1;
 
 	control_cleanup(ctl_sock_path);
 	config_clear(leconf);
@@ -211,10 +218,14 @@ ldpe_shutdown(void)
 	/* remove addresses from global list */
 	while ((if_addr = LIST_FIRST(&global.addr_list)) != NULL) {
 		LIST_REMOVE(if_addr, entry);
+		assert(if_addr != LIST_FIRST(&global.addr_list));
 		free(if_addr);
 	}
-	while ((adj = RB_ROOT(&global.adj_tree)) != NULL)
+	while (!RB_EMPTY(global_adj_head, &global.adj_tree)) {
+		adj = RB_ROOT(global_adj_head, &global.adj_tree);
+
 		adj_del(adj, S_SHUTDOWN);
+	}
 
 	/* clean up */
 	if (iev_lde)
@@ -231,12 +242,16 @@ ldpe_shutdown(void)
 int
 ldpe_imsg_compose_parent(int type, pid_t pid, void *data, uint16_t datalen)
 {
+	if (iev_main->ibuf.fd == -1)
+		return (0);
 	return (imsg_compose_event(iev_main, type, 0, pid, -1, data, datalen));
 }
 
 void
 ldpe_imsg_compose_parent_sync(int type, pid_t pid, void *data, uint16_t datalen)
 {
+	if (iev_main_sync->ibuf.fd == -1)
+		return;
 	imsg_compose_event(iev_main_sync, type, 0, pid, -1, data, datalen);
 	imsg_flush(&iev_main_sync->ibuf);
 }
@@ -245,6 +260,8 @@ int
 ldpe_imsg_compose_lde(int type, uint32_t peerid, pid_t pid, void *data,
     uint16_t datalen)
 {
+	if (iev_lde->ibuf.fd == -1)
+		return (0);
 	return (imsg_compose_event(iev_lde, type, peerid, pid, -1,
 	    data, datalen));
 }
@@ -261,7 +278,7 @@ ldpe_dispatch_main(struct thread *thread)
 	struct l2vpn_if		*lif, *nlif;
 	struct l2vpn_pw		*pw, *npw;
 	struct imsg		 imsg;
-	int			 fd = THREAD_FD(thread);
+	int			 fd;
 	struct imsgev		*iev = THREAD_ARG(thread);
 	struct imsgbuf		*ibuf = &iev->ibuf;
 	struct iface		*iface = NULL;
@@ -276,7 +293,8 @@ ldpe_dispatch_main(struct thread *thread)
 	struct nbr_params	*nbrp;
 #endif
 	int			 n, shut = 0;
-
+	struct ldp_access       *laccess;
+	
 	iev->ev_read = NULL;
 
 	if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
@@ -348,8 +366,9 @@ ldpe_dispatch_main(struct thread *thread)
 				fatal(NULL);
 			imsg_init(&iev_lde->ibuf, fd);
 			iev_lde->handler_read = ldpe_dispatch_lde;
-			iev_lde->ev_read = thread_add_read(master,
-			    iev_lde->handler_read, iev_lde, iev_lde->ibuf.fd);
+			iev_lde->ev_read = NULL;
+			thread_add_read(master, iev_lde->handler_read, iev_lde, iev_lde->ibuf.fd,
+					&iev_lde->ev_read);
 			iev_lde->handler_write = ldp_write_handler;
 			iev_lde->ev_write = NULL;
 			break;
@@ -452,10 +471,10 @@ ldpe_dispatch_main(struct thread *thread)
 				fatal(NULL);
 			memcpy(nconf, imsg.data, sizeof(struct ldpd_conf));
 
-			RB_INIT(&nconf->iface_tree);
-			RB_INIT(&nconf->tnbr_tree);
-			RB_INIT(&nconf->nbrp_tree);
-			RB_INIT(&nconf->l2vpn_tree);
+			RB_INIT(iface_head, &nconf->iface_tree);
+			RB_INIT(tnbr_head, &nconf->tnbr_tree);
+			RB_INIT(nbrp_head, &nconf->nbrp_tree);
+			RB_INIT(l2vpn_head, &nconf->l2vpn_tree);
 			break;
 		case IMSG_RECONF_IFACE:
 			if ((niface = malloc(sizeof(struct iface))) == NULL)
@@ -483,9 +502,9 @@ ldpe_dispatch_main(struct thread *thread)
 				fatal(NULL);
 			memcpy(nl2vpn, imsg.data, sizeof(struct l2vpn));
 
-			RB_INIT(&nl2vpn->if_tree);
-			RB_INIT(&nl2vpn->pw_tree);
-			RB_INIT(&nl2vpn->pw_inactive_tree);
+			RB_INIT(l2vpn_if_head, &nl2vpn->if_tree);
+			RB_INIT(l2vpn_pw_head, &nl2vpn->pw_tree);
+			RB_INIT(l2vpn_pw_head, &nl2vpn->pw_inactive_tree);
 
 			RB_INSERT(l2vpn_head, &nconf->l2vpn_tree, nl2vpn);
 			break;
@@ -527,6 +546,18 @@ ldpe_dispatch_main(struct thread *thread)
 			}
 			memcpy(&ldp_debug, imsg.data, sizeof(ldp_debug));
 			break;
+		case IMSG_FILTER_UPDATE:
+			if (imsg.hdr.len != IMSG_HEADER_SIZE +
+			    sizeof(struct ldp_access)) {
+				log_warnx("%s: wrong imsg len", __func__);
+				break;
+			}
+			laccess = imsg.data;
+			ldpe_check_filter_af(AF_INET, &leconf->ipv4,
+				laccess->name);
+			ldpe_check_filter_af(AF_INET6, &leconf->ipv6,
+				laccess->name);
+			break;
 		default:
 			log_debug("ldpe_dispatch_main: error handling imsg %d",
 			    imsg.hdr.type);
@@ -663,6 +694,17 @@ ldpe_dispatch_lde(struct thread *thread)
 		case IMSG_CTL_SHOW_L2VPN_BINDING:
 			control_imsg_relay(&imsg);
 			break;
+		case IMSG_NBR_SHUTDOWN:
+			nbr = nbr_find_peerid(imsg.hdr.peerid);
+			if (nbr == NULL) {
+				log_debug("ldpe_dispatch_lde: cannot find "
+				    "neighbor");
+				break;
+			}
+			if (nbr->state != NBR_STA_OPER)
+				break;
+			session_shutdown(nbr,S_SHUTDOWN,0,0);
+			break;
 		default:
 			log_debug("ldpe_dispatch_lde: error handling imsg %d",
 			    imsg.hdr.type);
@@ -689,8 +731,9 @@ ldpe_dispatch_pfkey(struct thread *thread)
 {
 	int	 fd = THREAD_FD(thread);
 
-	pfkey_ev = thread_add_read(master, ldpe_dispatch_pfkey,
-	    NULL, global.pfkeysock);
+	pfkey_ev = NULL;
+	thread_add_read(master, ldpe_dispatch_pfkey, NULL, global.pfkeysock,
+			&pfkey_ev);
 
 	if (pfkey_read(fd, NULL) == -1)
 		fatal("pfkey_read failed, exiting...");
@@ -709,13 +752,15 @@ ldpe_setup_sockets(int af, int disc_socket, int edisc_socket,
 
 	/* discovery socket */
 	af_global->ldp_disc_socket = disc_socket;
-	af_global->disc_ev = thread_add_read(master, disc_recv_packet,
-	    &af_global->disc_ev, af_global->ldp_disc_socket);
+	af_global->disc_ev = NULL;
+	thread_add_read(master, disc_recv_packet, &af_global->disc_ev, af_global->ldp_disc_socket,
+			&af_global->disc_ev);
 
 	/* extended discovery socket */
 	af_global->ldp_edisc_socket = edisc_socket;
-	af_global->edisc_ev = thread_add_read(master, disc_recv_packet,
-	    &af_global->edisc_ev, af_global->ldp_edisc_socket);
+	af_global->edisc_ev = NULL;
+	thread_add_read(master, disc_recv_packet, &af_global->edisc_ev, af_global->ldp_edisc_socket,
+			&af_global->edisc_ev);
 
 	/* session socket */
 	af_global->ldp_session_socket = session_socket;
@@ -807,7 +852,7 @@ ldpe_stop_init_backoff(int af)
 }
 
 static void
-ldpe_iface_af_ctl(struct ctl_conn *c, int af, unsigned int idx)
+ldpe_iface_af_ctl(struct ctl_conn *c, int af, ifindex_t idx)
 {
 	struct iface		*iface;
 	struct iface_af		*ia;
@@ -827,7 +872,7 @@ ldpe_iface_af_ctl(struct ctl_conn *c, int af, unsigned int idx)
 }
 
 void
-ldpe_iface_ctl(struct ctl_conn *c, unsigned int idx)
+ldpe_iface_ctl(struct ctl_conn *c, ifindex_t idx)
 {
 	ldpe_iface_af_ctl(c, AF_INET, idx);
 	ldpe_iface_af_ctl(c, AF_INET6, idx);
@@ -869,8 +914,8 @@ ldpe_adj_detail_ctl(struct ctl_conn *c)
 			continue;
 
 		strlcpy(ictl.name, iface->name, sizeof(ictl.name));
-		if (RB_EMPTY(&iface->ipv4.adj_tree) &&
-		    RB_EMPTY(&iface->ipv6.adj_tree))
+		if (RB_EMPTY(ia_adj_head, &iface->ipv4.adj_tree) &&
+		    RB_EMPTY(ia_adj_head, &iface->ipv6.adj_tree))
 			ictl.no_adj = 1;
 		imsg_compose_event(&c->iev, IMSG_CTL_SHOW_DISC_IFACE, 0, 0,
 		    -1, &ictl, sizeof(ictl));
@@ -956,6 +1001,15 @@ mapping_list_clr(struct mapping_head *mh)
 
 	while ((me = TAILQ_FIRST(mh)) != NULL) {
 		TAILQ_REMOVE(mh, me, entry);
+		assert(me != TAILQ_FIRST(mh));
 		free(me);
 	}
 }
+
+void
+ldpe_check_filter_af(int af, struct ldpd_af_conf *af_conf,
+    const char *filter_name)
+{
+	if (strcmp(af_conf->acl_thello_accept_from, filter_name) == 0)
+		ldpe_remove_dynamic_tnbrs(af);
+}
