@@ -34,8 +34,8 @@
 
 DEFINE_MTYPE_STATIC(LIB, BFD_INFO, "BFD info")
 
-int bfd_debug = 0;
-struct bfd_gbl bfd_gbl;
+static int bfd_debug = 0;
+static struct bfd_gbl bfd_gbl;
 
 /*
  * bfd_gbl_init - Initialize the BFD global structure
@@ -74,10 +74,7 @@ struct bfd_info *bfd_info_create(void)
  */
 void bfd_info_free(struct bfd_info **bfd_info)
 {
-	if (*bfd_info) {
-		XFREE(MTYPE_BFD_INFO, *bfd_info);
-		*bfd_info = NULL;
-	}
+	XFREE(MTYPE_BFD_INFO, *bfd_info);
 }
 
 /*
@@ -127,19 +124,19 @@ void bfd_set_param(struct bfd_info **bfd_info, uint32_t min_rx, uint32_t min_tx,
  */
 void bfd_peer_sendmsg(struct zclient *zclient, struct bfd_info *bfd_info,
 		      int family, void *dst_ip, void *src_ip, char *if_name,
-		      int ttl, int multihop, int command, int set_flag,
-		      vrf_id_t vrf_id)
+		      int ttl, int multihop, int cbit, int command,
+		      int set_flag, vrf_id_t vrf_id)
 {
 	struct stream *s;
 	int ret;
 	int len;
 
-	/* Individual reg/dereg messages are supressed during shutdown. */
+	/* Individual reg/dereg messages are suppressed during shutdown. */
 	if (CHECK_FLAG(bfd_gbl.flags, BFD_GBL_FLAG_IN_SHUTDOWN)) {
 		if (bfd_debug)
 			zlog_debug(
 				"%s: Suppressing BFD peer reg/dereg messages",
-				__FUNCTION__);
+				__func__);
 		return;
 	}
 
@@ -149,7 +146,7 @@ void bfd_peer_sendmsg(struct zclient *zclient, struct bfd_info *bfd_info,
 			zlog_debug(
 				"%s: Can't send BFD peer register, Zebra client not "
 				"established",
-				__FUNCTION__);
+				__func__);
 		return;
 	}
 
@@ -208,6 +205,11 @@ void bfd_peer_sendmsg(struct zclient *zclient, struct bfd_info *bfd_info,
 			stream_putc(s, 0);
 		}
 	}
+	/* cbit */
+	if (cbit)
+		stream_putc(s, 1);
+	else
+		stream_putc(s, 0);
 
 	stream_putw_at(s, 0, stream_get_endp(s));
 
@@ -253,11 +255,13 @@ const char *bfd_get_command_dbg_str(int command)
  */
 struct interface *bfd_get_peer_info(struct stream *s, struct prefix *dp,
 				    struct prefix *sp, int *status,
+				    int *remote_cbit,
 				    vrf_id_t vrf_id)
 {
 	unsigned int ifindex;
 	struct interface *ifp = NULL;
 	int plen;
+	int local_remote_cbit;
 
 	/* Get interface index. */
 	ifindex = stream_getl(s);
@@ -292,6 +296,9 @@ struct interface *bfd_get_peer_info(struct stream *s, struct prefix *dp,
 		stream_get(&sp->u.prefix, s, plen);
 		sp->prefixlen = stream_getc(s);
 	}
+	local_remote_cbit = stream_getc(s);
+	if (remote_cbit)
+		*remote_cbit = local_remote_cbit;
 	return ifp;
 }
 
@@ -305,6 +312,8 @@ const char *bfd_get_status_str(int status)
 		return "Down";
 	case BFD_STATUS_UP:
 		return "Up";
+	case BFD_STATUS_ADMIN_DOWN:
+		return "Admin Down";
 	case BFD_STATUS_UNKNOWN:
 	default:
 		return "Unknown";
@@ -319,7 +328,7 @@ static void bfd_last_update(time_t last_update, char *buf, size_t len)
 {
 	time_t curr;
 	time_t diff;
-	struct tm *tm;
+	struct tm tm;
 	struct timeval tv;
 
 	/* If no BFD satatus update has ever been received, print `never'. */
@@ -332,17 +341,17 @@ static void bfd_last_update(time_t last_update, char *buf, size_t len)
 	monotime(&tv);
 	curr = tv.tv_sec;
 	diff = curr - last_update;
-	tm = gmtime(&diff);
+	gmtime_r(&diff, &tm);
 
-	snprintf(buf, len, "%d:%02d:%02d:%02d", tm->tm_yday, tm->tm_hour,
-		 tm->tm_min, tm->tm_sec);
+	snprintf(buf, len, "%d:%02d:%02d:%02d", tm.tm_yday, tm.tm_hour,
+		 tm.tm_min, tm.tm_sec);
 }
 
 /*
  * bfd_show_param - Show the BFD parameter information.
  */
 void bfd_show_param(struct vty *vty, struct bfd_info *bfd_info, int bfd_tag,
-		    int extra_space, uint8_t use_json, json_object *json_obj)
+		    int extra_space, bool use_json, json_object *json_obj)
 {
 	json_object *json_bfd = NULL;
 
@@ -378,7 +387,7 @@ void bfd_show_param(struct vty *vty, struct bfd_info *bfd_info, int bfd_tag,
  * bfd_show_status - Show the BFD status information.
  */
 static void bfd_show_status(struct vty *vty, struct bfd_info *bfd_info,
-			    int bfd_tag, int extra_space, uint8_t use_json,
+			    int bfd_tag, int extra_space, bool use_json,
 			    json_object *json_bfd)
 {
 	char time_buf[32];
@@ -402,7 +411,7 @@ static void bfd_show_status(struct vty *vty, struct bfd_info *bfd_info,
  * bfd_show_info - Show the BFD information.
  */
 void bfd_show_info(struct vty *vty, struct bfd_info *bfd_info, int multihop,
-		   int extra_space, uint8_t use_json, json_object *json_obj)
+		   int extra_space, bool use_json, json_object *json_obj)
 {
 	json_object *json_bfd = NULL;
 
@@ -433,7 +442,8 @@ void bfd_show_info(struct vty *vty, struct bfd_info *bfd_info, int multihop,
  * bfd_client_sendmsg - Format and send a client register
  *                    command to Zebra to be forwarded to BFD
  */
-void bfd_client_sendmsg(struct zclient *zclient, int command)
+void bfd_client_sendmsg(struct zclient *zclient, int command,
+			vrf_id_t vrf_id)
 {
 	struct stream *s;
 	int ret;
@@ -444,13 +454,13 @@ void bfd_client_sendmsg(struct zclient *zclient, int command)
 			zlog_debug(
 				"%s: Can't send BFD client register, Zebra client not "
 				"established",
-				__FUNCTION__);
+				__func__);
 		return;
 	}
 
 	s = zclient->obuf;
 	stream_reset(s);
-	zclient_create_header(s, command, VRF_DEFAULT);
+	zclient_create_header(s, command, vrf_id);
 
 	stream_putl(s, getpid());
 
