@@ -25,6 +25,8 @@
 #include "buffer.h"
 #include "log.h"
 #include "network.h"
+#include "lib_errors.h"
+
 #include <stddef.h>
 
 DEFINE_MTYPE_STATIC(LIB, BUFFER, "Buffer")
@@ -112,12 +114,6 @@ char *buffer_getstr(struct buffer *b)
 	return s;
 }
 
-/* Return 1 if buffer is empty. */
-int buffer_empty(struct buffer *b)
-{
-	return (b->head == NULL);
-}
-
 /* Clear and free all allocated data. */
 void buffer_reset(struct buffer *b)
 {
@@ -272,19 +268,12 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 	if (!b->head)
 		return BUFFER_EMPTY;
 
-	if (height < 1) {
-		zlog_warn(
-			"%s called with non-positive window height %d, forcing to 1",
-			__func__, height);
+	if (height < 1)
 		height = 1;
-	} else if (height >= 2)
+	else if (height >= 2)
 		height--;
-	if (width < 1) {
-		zlog_warn(
-			"%s called with non-positive window width %d, forcing to 1",
-			__func__, width);
+	if (width < 1)
 		width = 1;
-	}
 
 	/* For erase and more data add two to b's buffer_data count.*/
 	if (b->head->next == NULL) {
@@ -299,7 +288,7 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 	/* Previously print out is performed. */
 	if (erase_flag) {
 		iov[iov_index].iov_base = erase;
-		iov[iov_index].iov_len = sizeof erase;
+		iov[iov_index].iov_len = sizeof(erase);
 		iov_index++;
 	}
 
@@ -332,16 +321,12 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 		{
 			iov_alloc *= 2;
 			if (iov != small_iov) {
-				zlog_warn(
-					"%s: growing iov array to %d; "
-					"width %d, height %d, size %lu",
-					__func__, iov_alloc, width, height,
-					(unsigned long)b->size);
 				iov = XREALLOC(MTYPE_TMP, iov,
 					       iov_alloc * sizeof(*iov));
 			} else {
 				/* This should absolutely never occur. */
-				zlog_err(
+				flog_err_sys(
+					EC_LIB_SYSTEM_CALL,
 					"%s: corruption detected: iov_small overflowed; "
 					"head %p, tail %p, head->next %p",
 					__func__, (void *)b->head,
@@ -356,7 +341,7 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 	/* In case of `more' display need. */
 	if (b->tail && (b->tail->sp < b->tail->cp) && !no_more_flag) {
 		iov[iov_index].iov_base = more;
-		iov[iov_index].iov_len = sizeof more;
+		iov[iov_index].iov_len = sizeof(more);
 		iov_index++;
 	}
 
@@ -374,8 +359,9 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 			iov_size =
 				((iov_index > IOV_MAX) ? IOV_MAX : iov_index);
 			if ((nbytes = writev(fd, c_iov, iov_size)) < 0) {
-				zlog_warn("%s: writev to fd %d failed: %s",
-					  __func__, fd, safe_strerror(errno));
+				flog_err(EC_LIB_SOCKET,
+					 "%s: writev to fd %d failed: %s",
+					 __func__, fd, safe_strerror(errno));
 				break;
 			}
 
@@ -386,8 +372,8 @@ buffer_status_t buffer_flush_window(struct buffer *b, int fd, int width,
 	}
 #else  /* IOV_MAX */
 	if ((nbytes = writev(fd, iov, iov_index)) < 0)
-		zlog_warn("%s: writev to fd %d failed: %s", __func__, fd,
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET, "%s: writev to fd %d failed: %s",
+			 __func__, fd, safe_strerror(errno));
 #endif /* IOV_MAX */
 
 	/* Free printed buffer data. */
@@ -447,18 +433,17 @@ in one shot. */
 		if (ERRNO_IO_RETRY(errno))
 			/* Calling code should try again later. */
 			return BUFFER_PENDING;
-		zlog_warn("%s: write error on fd %d: %s", __func__, fd,
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET, "%s: write error on fd %d: %s",
+			 __func__, fd, safe_strerror(errno));
 		return BUFFER_ERROR;
 	}
 
 	/* Free printed buffer data. */
 	while (written > 0) {
-		struct buffer_data *d;
 		if (!(d = b->head)) {
-			zlog_err(
-				"%s: corruption detected: buffer queue empty, "
-				"but written is %lu",
+			flog_err(
+				EC_LIB_DEVELOPMENT,
+				"%s: corruption detected: buffer queue empty, but written is %lu",
 				__func__, (unsigned long)written);
 			break;
 		}
@@ -502,8 +487,8 @@ buffer_status_t buffer_write(struct buffer *b, int fd, const void *p,
 		if (ERRNO_IO_RETRY(errno))
 			nbytes = 0;
 		else {
-			zlog_warn("%s: write error on fd %d: %s", __func__, fd,
-				  safe_strerror(errno));
+			flog_err(EC_LIB_SOCKET, "%s: write error on fd %d: %s",
+				 __func__, fd, safe_strerror(errno));
 			return BUFFER_ERROR;
 		}
 	}
