@@ -218,7 +218,7 @@ static struct prefix_list *prefix_list_insert(afi_t afi, int orf,
 	/* If name is made by all digit character.  We treat it as
 	   number. */
 	for (number = 0, i = 0; i < strlen(name); i++) {
-		if (isdigit((int)name[i]))
+		if (isdigit((unsigned char)name[i]))
 			number = (number * 10) + (name[i] - '0');
 		else
 			break;
@@ -303,6 +303,8 @@ static void prefix_list_delete(struct prefix_list *plist)
 
 	/* If prefix-list contain prefix_list_entry free all of it. */
 	for (pentry = plist->head; pentry; pentry = next) {
+		route_map_notify_pentry_dependencies(plist->name, pentry,
+						     RMAP_EVENT_PLIST_DELETED);
 		next = pentry->next;
 		prefix_list_trie_del(plist, pentry);
 		prefix_list_entry_free(pentry);
@@ -326,8 +328,7 @@ static void prefix_list_delete(struct prefix_list *plist)
 	else
 		list->head = plist->next;
 
-	if (plist->desc)
-		XFREE(MTYPE_TMP, plist->desc);
+	XFREE(MTYPE_TMP, plist->desc);
 
 	/* Make sure master's recent changed prefix-list information is
 	   cleared. */
@@ -338,8 +339,7 @@ static void prefix_list_delete(struct prefix_list *plist)
 	if (master->delete_hook)
 		(*master->delete_hook)(plist);
 
-	if (plist->name)
-		XFREE(MTYPE_MPREFIX_LIST_STR, plist->name);
+	XFREE(MTYPE_MPREFIX_LIST_STR, plist->name);
 
 	XFREE(MTYPE_PREFIX_LIST_TRIE, plist->trie);
 
@@ -348,14 +348,14 @@ static void prefix_list_delete(struct prefix_list *plist)
 
 static struct prefix_list_entry *
 prefix_list_entry_make(struct prefix *prefix, enum prefix_list_type type,
-		       int64_t seq, int le, int ge, int any)
+		       int64_t seq, int le, int ge, bool any)
 {
 	struct prefix_list_entry *pentry;
 
 	pentry = prefix_list_entry_new();
 
 	if (any)
-		pentry->any = 1;
+		pentry->any = true;
 
 	prefix_copy(&pentry->prefix, prefix);
 	pentry->type = type;
@@ -387,7 +387,7 @@ static int64_t prefix_new_seq_get(struct prefix_list *plist)
 	int64_t newseq;
 	struct prefix_list_entry *pentry;
 
-	maxseq = newseq = 0;
+	maxseq = 0;
 
 	for (pentry = plist->head; pentry; pentry = pentry->next) {
 		if (maxseq < pentry->seq)
@@ -396,7 +396,7 @@ static int64_t prefix_new_seq_get(struct prefix_list *plist)
 
 	newseq = ((maxseq / 5) * 5) + 5;
 
-	return newseq;
+	return (newseq > UINT_MAX) ? UINT_MAX : newseq;
 }
 
 /* Return prefix list entry which has same seq number. */
@@ -477,7 +477,7 @@ static void prefix_list_trie_del(struct prefix_list *plist,
 				 struct prefix_list_entry *pentry)
 {
 	size_t depth, maxdepth = plist->master->trie_depth;
-	uint8_t *bytes = &pentry->prefix.u.prefix;
+	uint8_t *bytes = pentry->prefix.u.val;
 	size_t validbits = pentry->prefix.prefixlen;
 	struct pltrie_table *table, **tables[PLC_MAXLEVEL];
 
@@ -498,7 +498,6 @@ static void prefix_list_trie_del(struct prefix_list *plist,
 	for (; depth > 0; depth--)
 		if (trie_table_empty(*tables[depth])) {
 			XFREE(MTYPE_PREFIX_LIST_TRIE, *tables[depth]);
-			*tables[depth] = NULL;
 		}
 }
 
@@ -521,6 +520,8 @@ static void prefix_list_entry_delete(struct prefix_list *plist,
 	else
 		plist->tail = pentry->prev;
 
+	route_map_notify_pentry_dependencies(plist->name, pentry,
+					     RMAP_EVENT_PLIST_DELETED);
 	prefix_list_entry_free(pentry);
 
 	plist->count--;
@@ -565,7 +566,7 @@ static void prefix_list_trie_add(struct prefix_list *plist,
 				 struct prefix_list_entry *pentry)
 {
 	size_t depth = plist->master->trie_depth;
-	uint8_t *bytes = &pentry->prefix.u.prefix;
+	uint8_t *bytes = pentry->prefix.u.val;
 	size_t validbits = pentry->prefix.prefixlen;
 	struct pltrie_table *table;
 
@@ -634,6 +635,9 @@ static void prefix_list_entry_add(struct prefix_list *plist,
 	/* Increment count. */
 	plist->count++;
 
+	route_map_notify_pentry_dependencies(plist->name, pentry,
+					     RMAP_EVENT_PLIST_ADDED);
+
 	/* Run hook function. */
 	if (plist->master->add_hook)
 		(*plist->master->add_hook)(plist);
@@ -656,7 +660,7 @@ static const char *prefix_list_type_str(struct prefix_list_entry *pentry)
 }
 
 static int prefix_list_entry_match(struct prefix_list_entry *pentry,
-				   struct prefix *p)
+				   const struct prefix *p)
 {
 	int ret;
 
@@ -683,14 +687,15 @@ static int prefix_list_entry_match(struct prefix_list_entry *pentry,
 	return 1;
 }
 
-enum prefix_list_type prefix_list_apply_which_prefix(struct prefix_list *plist,
-						     struct prefix **which,
-						     void *object)
+enum prefix_list_type prefix_list_apply_which_prefix(
+	struct prefix_list *plist,
+	const struct prefix **which,
+	const void *object)
 {
 	struct prefix_list_entry *pentry, *pbest = NULL;
 
-	struct prefix *p = (struct prefix *)object;
-	uint8_t *byte = &p->u.prefix;
+	const struct prefix *p = (const struct prefix *)object;
+	const uint8_t *byte = p->u.val;
 	size_t depth;
 	size_t validbits = p->prefixlen;
 	struct pltrie_table *table;
@@ -751,6 +756,7 @@ enum prefix_list_type prefix_list_apply_which_prefix(struct prefix_list *plist,
 	if (pbest == NULL)
 		return PREFIX_DENY;
 
+	pbest->hitcnt++;
 	return pbest->type;
 }
 
@@ -772,9 +778,9 @@ static void __attribute__((unused)) prefix_list_print(struct prefix_list *plist)
 
 			p = &pentry->prefix;
 
-			printf("  seq %" PRId64 " %s %s/%d", pentry->seq,
+			printf("  seq %lld %s %s/%d", (long long)pentry->seq,
 			       prefix_list_type_str(pentry),
-			       inet_ntop(p->family, &p->u.prefix, buf, BUFSIZ),
+			       inet_ntop(p->family, p->u.val, buf, BUFSIZ),
 			       p->prefixlen);
 			if (pentry->ge)
 				printf(" ge %d", pentry->ge);
@@ -790,7 +796,7 @@ static struct prefix_list_entry *
 prefix_entry_dup_check(struct prefix_list *plist, struct prefix_list_entry *new)
 {
 	size_t depth, maxdepth = plist->master->trie_depth;
-	uint8_t byte, *bytes = &new->prefix.u.prefix;
+	uint8_t byte, *bytes = new->prefix.u.val;
 	size_t validbits = new->prefix.prefixlen;
 	struct pltrie_table *table;
 	struct prefix_list_entry *pentry;
@@ -845,11 +851,16 @@ static int vty_prefix_list_install(struct vty *vty, afi_t afi, const char *name,
 	struct prefix_list_entry *pentry;
 	struct prefix_list_entry *dup;
 	struct prefix p, p_tmp;
-	int any = 0;
+	bool any = false;
 	int64_t seqnum = -1;
 	int lenum = 0;
 	int genum = 0;
 
+	if (name == NULL || prefix == NULL || typestr == NULL) {
+		vty_out(vty, "%% Missing prefix or type\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	/* Sequential number. */
 	if (seq)
 		seqnum = (int64_t)atol(seq);
@@ -878,7 +889,7 @@ static int vty_prefix_list_install(struct vty *vty, afi_t afi, const char *name,
 					      (struct prefix_ipv4 *)&p);
 			genum = 0;
 			lenum = IPV4_MAX_BITLEN;
-			any = 1;
+			any = true;
 		} else
 			ret = str2prefix_ipv4(prefix, (struct prefix_ipv4 *)&p);
 
@@ -897,7 +908,7 @@ static int vty_prefix_list_install(struct vty *vty, afi_t afi, const char *name,
 			ret = str2prefix_ipv6("::/0", (struct prefix_ipv6 *)&p);
 			genum = 0;
 			lenum = IPV6_MAX_BITLEN;
-			any = 1;
+			any = true;
 		} else
 			ret = str2prefix_ipv6(prefix, (struct prefix_ipv6 *)&p);
 
@@ -915,7 +926,6 @@ static int vty_prefix_list_install(struct vty *vty, afi_t afi, const char *name,
 	default:
 		vty_out(vty, "%% Unrecognized AFI (%d)\n", afi);
 		return CMD_WARNING_CONFIG_FAILED;
-		break;
 	}
 
 	/* If prefix has bits not under the mask, adjust it to fit */
@@ -924,7 +934,10 @@ static int vty_prefix_list_install(struct vty *vty, afi_t afi, const char *name,
 		char buf_tmp[PREFIX2STR_BUFFER];
 		prefix2str(&p, buf, sizeof(buf));
 		prefix2str(&p_tmp, buf_tmp, sizeof(buf_tmp));
-		zlog_warn(
+		vty_out(vty,
+			"%% Prefix-list %s prefix changed from %s to %s to match length\n",
+			name, buf, buf_tmp);
+		zlog_info(
 			"Prefix-list %s prefix changed from %s to %s to match length",
 			name, buf, buf_tmp);
 		p = p_tmp;
@@ -991,22 +1004,36 @@ static int vty_prefix_list_uninstall(struct vty *vty, afi_t afi,
 		return CMD_SUCCESS;
 	}
 
-	/* We must have, at a minimum, both the type and prefix here */
-	if ((typestr == NULL) || (prefix == NULL)) {
-		vty_out(vty, "%% Both prefix and type required\n");
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
 	/* Check sequence number. */
 	if (seq)
 		seqnum = (int64_t)atol(seq);
 
+	/* Sequence number specified, but nothing else. */
+	if (seq && typestr == NULL && prefix == NULL && ge == NULL
+	    && le == NULL) {
+		pentry = prefix_seq_check(plist, seqnum);
+
+		if (pentry == NULL) {
+			vty_out(vty,
+				"%% Can't find prefix-list %s with sequence number %" PRIu64 "\n",
+				name, seqnum);
+			return CMD_WARNING_CONFIG_FAILED;
+		}
+
+		prefix_list_entry_delete(plist, pentry, 1);
+		return CMD_SUCCESS;
+	}
+
 	/* ge and le number */
 	if (ge)
 		genum = atoi(ge);
 	if (le)
 		lenum = atoi(le);
 
+	/* We must have, at a minimum, both the type and prefix here */
+	if ((typestr == NULL) || (prefix == NULL))
+		return CMD_WARNING_CONFIG_FAILED;
+
 	/* Check of filter type. */
 	if (strncmp("permit", typestr, 1) == 0)
 		type = PREFIX_PERMIT;
@@ -1071,10 +1098,7 @@ static int vty_prefix_list_desc_unset(struct vty *vty, afi_t afi,
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	if (plist->desc) {
-		XFREE(MTYPE_TMP, plist->desc);
-		plist->desc = NULL;
-	}
+	XFREE(MTYPE_TMP, plist->desc);
 
 	if (plist->head == NULL && plist->tail == NULL && plist->desc == NULL)
 		prefix_list_delete(plist);
@@ -1140,7 +1164,7 @@ static void vty_show_prefix_entry(struct vty *vty, afi_t afi,
 				char buf[BUFSIZ];
 
 				vty_out(vty, "%s/%d",
-					inet_ntop(p->family, &p->u.prefix, buf,
+					inet_ntop(p->family, p->u.val, buf,
 						  BUFSIZ),
 					p->prefixlen);
 
@@ -1243,13 +1267,13 @@ static int vty_show_prefix_list_prefix(struct vty *vty, afi_t afi,
 			if (pentry->any)
 				vty_out(vty, "any");
 			else {
-				struct prefix *p = &pentry->prefix;
+				struct prefix *pf = &pentry->prefix;
 				char buf[BUFSIZ];
 
 				vty_out(vty, "%s/%d",
-					inet_ntop(p->family, &p->u.prefix, buf,
+					inet_ntop(pf->family, pf->u.val, buf,
 						  BUFSIZ),
-					p->prefixlen);
+					pf->prefixlen);
 
 				if (pentry->ge)
 					vty_out(vty, " ge %d", pentry->ge);
@@ -1368,6 +1392,17 @@ DEFPY (no_ip_prefix_list,
 					 action, dest, ge_str, le_str);
 }
 
+DEFPY(no_ip_prefix_list_seq, no_ip_prefix_list_seq_cmd,
+      "no ip prefix-list WORD seq (1-4294967295)",
+      NO_STR IP_STR PREFIX_LIST_STR
+      "Name of a prefix list\n"
+      "sequence number of an entry\n"
+      "Sequence number\n")
+{
+	return vty_prefix_list_uninstall(vty, AFI_IP, prefix_list, seq_str,
+					 NULL, NULL, NULL, NULL);
+}
+
 DEFPY (no_ip_prefix_list_all,
        no_ip_prefix_list_all_cmd,
        "no ip prefix-list WORD",
@@ -1756,7 +1791,7 @@ static int config_write_prefix_afi(afi_t afi, struct vty *vty)
 				char buf[BUFSIZ];
 
 				vty_out(vty, "%s/%d",
-					inet_ntop(p->family, &p->u.prefix, buf,
+					inet_ntop(p->family, p->u.val, buf,
 						  BUFSIZ),
 					p->prefixlen);
 
@@ -1795,7 +1830,7 @@ static int config_write_prefix_afi(afi_t afi, struct vty *vty)
 				char buf[BUFSIZ];
 
 				vty_out(vty, " %s/%d",
-					inet_ntop(p->family, &p->u.prefix, buf,
+					inet_ntop(p->family, p->u.val, buf,
 						  BUFSIZ),
 					p->prefixlen);
 
@@ -1858,10 +1893,12 @@ int prefix_bgp_orf_set(char *name, afi_t afi, struct orf_prefix *orfp,
 	if (!plist)
 		return CMD_WARNING_CONFIG_FAILED;
 
+	apply_mask(&orfp->p);
+
 	if (set) {
 		pentry = prefix_list_entry_make(
 			&orfp->p, (permit ? PREFIX_PERMIT : PREFIX_DENY),
-			orfp->seq, orfp->le, orfp->ge, 0);
+			orfp->seq, orfp->le, orfp->ge, false);
 
 		if (prefix_entry_dup_check(plist, pentry)) {
 			prefix_list_entry_free(pentry);
@@ -1894,7 +1931,7 @@ void prefix_bgp_orf_remove_all(afi_t afi, char *name)
 
 /* return prefix count */
 int prefix_bgp_show_prefix_list(struct vty *vty, afi_t afi, char *name,
-				uint8_t use_json)
+				bool use_json)
 {
 	struct prefix_list *plist;
 	struct prefix_list_entry *pentry;
@@ -1924,10 +1961,9 @@ int prefix_bgp_show_prefix_list(struct vty *vty, afi_t afi, char *name,
 			char buf_a[BUFSIZ];
 			char buf_b[BUFSIZ];
 
-			sprintf(buf_a, "%s/%d",
-				inet_ntop(p->family, &p->u.prefix, buf_b,
-					  BUFSIZ),
-				p->prefixlen);
+			snprintf(buf_a, sizeof(buf_a), "%s/%d",
+				 inet_ntop(p->family, p->u.val, buf_b, BUFSIZ),
+				 p->prefixlen);
 
 			json_object_int_add(json_list, "seq", pentry->seq);
 			json_object_string_add(json_list, "seqPrefixListType",
@@ -1963,7 +1999,7 @@ int prefix_bgp_show_prefix_list(struct vty *vty, afi_t afi, char *name,
 			vty_out(vty, "   seq %" PRId64 " %s %s/%d",
 				pentry->seq,
 				prefix_list_type_str(pentry),
-				inet_ntop(p->family, &p->u.prefix, buf, BUFSIZ),
+				inet_ntop(p->family, p->u.val, buf, BUFSIZ),
 				p->prefixlen);
 
 			if (pentry->ge)
@@ -2002,15 +2038,19 @@ static void prefix_list_reset_afi(afi_t afi, int orf)
 	assert(master->str.head == NULL);
 	assert(master->str.tail == NULL);
 
-	master->seqnum = 1;
+	master->seqnum = true;
 	master->recent = NULL;
 }
 
 
+static int config_write_prefix_ipv4(struct vty *vty);
 /* Prefix-list node. */
-static struct cmd_node prefix_node = {PREFIX_NODE,
-				      "", /* Prefix list has no interface. */
-				      1};
+static struct cmd_node prefix_node = {
+	.name = "ipv4 prefix list",
+	.node = PREFIX_NODE,
+	.prompt = "",
+	.config_write = config_write_prefix_ipv4,
+};
 
 static int config_write_prefix_ipv4(struct vty *vty)
 {
@@ -2048,10 +2088,11 @@ static const struct cmd_variable_handler plist_var_handlers[] = {
 
 static void prefix_list_init_ipv4(void)
 {
-	install_node(&prefix_node, config_write_prefix_ipv4);
+	install_node(&prefix_node);
 
 	install_element(CONFIG_NODE, &ip_prefix_list_cmd);
 	install_element(CONFIG_NODE, &no_ip_prefix_list_cmd);
+	install_element(CONFIG_NODE, &no_ip_prefix_list_seq_cmd);
 	install_element(CONFIG_NODE, &no_ip_prefix_list_all_cmd);
 
 	install_element(CONFIG_NODE, &ip_prefix_list_description_cmd);
@@ -2069,10 +2110,14 @@ static void prefix_list_init_ipv4(void)
 	install_element(ENABLE_NODE, &clear_ip_prefix_list_cmd);
 }
 
+static int config_write_prefix_ipv6(struct vty *vty);
 /* Prefix-list node. */
 static struct cmd_node prefix_ipv6_node = {
-	PREFIX_IPV6_NODE, "", /* Prefix list has no interface. */
-	1};
+	.name = "ipv6 prefix list",
+	.node = PREFIX_IPV6_NODE,
+	.prompt = "",
+	.config_write = config_write_prefix_ipv6,
+};
 
 static int config_write_prefix_ipv6(struct vty *vty)
 {
@@ -2081,7 +2126,7 @@ static int config_write_prefix_ipv6(struct vty *vty)
 
 static void prefix_list_init_ipv6(void)
 {
-	install_node(&prefix_ipv6_node, config_write_prefix_ipv6);
+	install_node(&prefix_ipv6_node);
 
 	install_element(CONFIG_NODE, &ipv6_prefix_list_cmd);
 	install_element(CONFIG_NODE, &no_ipv6_prefix_list_cmd);
@@ -2102,7 +2147,7 @@ static void prefix_list_init_ipv6(void)
 	install_element(ENABLE_NODE, &clear_ipv6_prefix_list_cmd);
 }
 
-void prefix_list_init()
+void prefix_list_init(void)
 {
 	cmd_variable_handler_register(plist_var_handlers);
 
@@ -2110,7 +2155,7 @@ void prefix_list_init()
 	prefix_list_init_ipv6();
 }
 
-void prefix_list_reset()
+void prefix_list_reset(void)
 {
 	prefix_list_reset_afi(AFI_IP, 0);
 	prefix_list_reset_afi(AFI_IP6, 0);
