@@ -26,6 +26,7 @@
 #include "memory.h"
 #include "log.h"
 #include "jhash.h"
+#include "lib_errors.h"
 
 DEFINE_MTYPE_STATIC(LIB, SOCKUNION, "Socket union")
 
@@ -46,6 +47,9 @@ int str2sockunion(const char *str, union sockunion *su)
 {
 	int ret;
 
+	if (str == NULL)
+		return -1;
+
 	memset(su, 0, sizeof(union sockunion));
 
 	ret = inet_pton(AF_INET, str, &su->sin.sin_addr);
@@ -120,7 +124,6 @@ static const char *sockunion_log(const union sockunion *su, char *buf,
 
 	case AF_INET6:
 		return inet_ntop(AF_INET6, &(su->sin6.sin6_addr), buf, len);
-		break;
 
 	default:
 		snprintf(buf, len, "af_unknown %d ", su->sa.sa_family);
@@ -136,9 +139,9 @@ int sockunion_socket(const union sockunion *su)
 	sock = socket(su->sa.sa_family, SOCK_STREAM, 0);
 	if (sock < 0) {
 		char buf[SU_ADDRSTRLEN];
-		zlog_warn("Can't make socket for %s : %s",
-			  sockunion_log(su, buf, SU_ADDRSTRLEN),
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET, "Can't make socket for %s : %s",
+			 sockunion_log(su, buf, SU_ADDRSTRLEN),
+			 safe_strerror(errno));
 		return -1;
 	}
 
@@ -159,7 +162,7 @@ int sockunion_accept(int sock, union sockunion *su)
 }
 
 /* Return sizeof union sockunion.  */
-static int sockunion_sizeof(const union sockunion *su)
+int sockunion_sizeof(const union sockunion *su)
 {
 	int ret;
 
@@ -211,7 +214,7 @@ enum connect_result sockunion_connect(int fd, const union sockunion *peersu,
 		if (errno != EINPROGRESS) {
 			char str[SU_ADDRSTRLEN];
 			zlog_info("can't connect to %s fd %d : %s",
-				  sockunion_log(&su, str, sizeof str), fd,
+				  sockunion_log(&su, str, sizeof(str)), fd,
 				  safe_strerror(errno));
 			return connect_error;
 		}
@@ -231,7 +234,8 @@ int sockunion_stream_socket(union sockunion *su)
 	sock = socket(su->sa.sa_family, SOCK_STREAM, 0);
 
 	if (sock < 0)
-		zlog_warn("can't make socket sockunion_stream_socket");
+		flog_err(EC_LIB_SOCKET,
+			 "can't make socket sockunion_stream_socket");
 
 	return sock;
 }
@@ -269,9 +273,9 @@ int sockunion_bind(int sock, union sockunion *su, unsigned short port,
 	ret = bind(sock, (struct sockaddr *)su, size);
 	if (ret < 0) {
 		char buf[SU_ADDRSTRLEN];
-		zlog_warn("can't bind socket for %s : %s",
-			  sockunion_log(su, buf, SU_ADDRSTRLEN),
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET, "can't bind socket for %s : %s",
+			 sockunion_log(su, buf, SU_ADDRSTRLEN),
+			 safe_strerror(errno));
 	}
 
 	return ret;
@@ -285,7 +289,8 @@ int sockopt_reuseaddr(int sock)
 	ret = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&on,
 			 sizeof(on));
 	if (ret < 0) {
-		zlog_warn("can't set sockopt SO_REUSEADDR to socket %d", sock);
+		flog_err(EC_LIB_SOCKET,
+			 "can't set sockopt SO_REUSEADDR to socket %d", sock);
 		return -1;
 	}
 	return 0;
@@ -300,7 +305,8 @@ int sockopt_reuseport(int sock)
 	ret = setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, (void *)&on,
 			 sizeof(on));
 	if (ret < 0) {
-		zlog_warn("can't set sockopt SO_REUSEPORT to socket %d", sock);
+		flog_err(EC_LIB_SOCKET,
+			 "can't set sockopt SO_REUSEPORT to socket %d", sock);
 		return -1;
 	}
 	return 0;
@@ -321,8 +327,9 @@ int sockopt_ttl(int family, int sock, int ttl)
 		ret = setsockopt(sock, IPPROTO_IP, IP_TTL, (void *)&ttl,
 				 sizeof(int));
 		if (ret < 0) {
-			zlog_warn("can't set sockopt IP_TTL %d to socket %d",
-				  ttl, sock);
+			flog_err(EC_LIB_SOCKET,
+				 "can't set sockopt IP_TTL %d to socket %d",
+				 ttl, sock);
 			return -1;
 		}
 		return 0;
@@ -332,7 +339,8 @@ int sockopt_ttl(int family, int sock, int ttl)
 		ret = setsockopt(sock, IPPROTO_IPV6, IPV6_UNICAST_HOPS,
 				 (void *)&ttl, sizeof(int));
 		if (ret < 0) {
-			zlog_warn(
+			flog_err(
+				EC_LIB_SOCKET,
 				"can't set sockopt IPV6_UNICAST_HOPS %d to socket %d",
 				ttl, sock);
 			return -1;
@@ -347,7 +355,7 @@ int sockopt_ttl(int family, int sock, int ttl)
  * Which on linux is a no-op since it is enabled by
  * default and on BSD it uses TCP_NOPUSH to do
  * the same thing( which it was not configured to
- * use).  This cleanup of the api occured on 8/1/17
+ * use).  This cleanup of the api occurred on 8/1/17
  * I imagine if after more than 1 year of no-one
  * complaining, and a major upgrade release we
  * can deprecate and remove this function call
@@ -357,25 +365,6 @@ int sockopt_cork(int sock, int onoff)
 	return 0;
 }
 
-int sockopt_mark_default(int sock, int mark, struct zebra_privs_t *cap)
-{
-#ifdef SO_MARK
-	int ret;
-
-	if (cap->change(ZPRIVS_RAISE))
-		zlog_err("routing_socket: Can't raise privileges");
-
-	ret = setsockopt(sock, SOL_SOCKET, SO_MARK, &mark, sizeof(mark));
-
-	if (cap->change(ZPRIVS_LOWER))
-		zlog_err("routing_socket: Can't lower privileges");
-
-	return ret;
-#else
-	return 0;
-#endif
-}
-
 int sockopt_minttl(int family, int sock, int minttl)
 {
 #ifdef IP_MINTTL
@@ -383,7 +372,8 @@ int sockopt_minttl(int family, int sock, int minttl)
 		int ret = setsockopt(sock, IPPROTO_IP, IP_MINTTL, &minttl,
 				     sizeof(minttl));
 		if (ret < 0)
-			zlog_warn(
+			flog_err(
+				EC_LIB_SOCKET,
 				"can't set sockopt IP_MINTTL to %d on socket %d: %s",
 				minttl, sock, safe_strerror(errno));
 		return ret;
@@ -394,7 +384,8 @@ int sockopt_minttl(int family, int sock, int minttl)
 		int ret = setsockopt(sock, IPPROTO_IPV6, IPV6_MINHOPCOUNT,
 				     &minttl, sizeof(minttl));
 		if (ret < 0)
-			zlog_warn(
+			flog_err(
+				EC_LIB_SOCKET,
 				"can't set sockopt IPV6_MINHOPCOUNT to %d on socket %d: %s",
 				minttl, sock, safe_strerror(errno));
 		return ret;
@@ -414,10 +405,10 @@ int sockopt_v6only(int family, int sock)
 		ret = setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (void *)&on,
 				 sizeof(int));
 		if (ret < 0) {
-			zlog_warn(
-				"can't set sockopt IPV6_V6ONLY "
-				"to socket %d",
-				sock);
+			flog_err(EC_LIB_SOCKET,
+				 "can't set sockopt IPV6_V6ONLY "
+				 "to socket %d",
+				 sock);
 			return -1;
 		}
 		return 0;
@@ -465,7 +456,7 @@ unsigned int sockunion_hash(const union sockunion *su)
 		return jhash_1word(su->sin.sin_addr.s_addr, 0);
 	case AF_INET6:
 		return jhash2(su->sin6.sin6_addr.s6_addr32,
-			      ZEBRA_NUM_OF(su->sin6.sin6_addr.s6_addr32), 0);
+			      array_size(su->sin6.sin6_addr.s6_addr32), 0);
 	}
 	return 0;
 }
@@ -527,13 +518,14 @@ union sockunion *sockunion_getsockname(int fd)
 	} name;
 	union sockunion *su;
 
-	memset(&name, 0, sizeof name);
-	len = sizeof name;
+	memset(&name, 0, sizeof(name));
+	len = sizeof(name);
 
 	ret = getsockname(fd, (struct sockaddr *)&name, &len);
 	if (ret < 0) {
-		zlog_warn("Can't get local address and port by getsockname: %s",
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET,
+			 "Can't get local address and port by getsockname: %s",
+			 safe_strerror(errno));
 		return NULL;
 	}
 
@@ -564,12 +556,12 @@ union sockunion *sockunion_getpeername(int fd)
 	} name;
 	union sockunion *su;
 
-	memset(&name, 0, sizeof name);
-	len = sizeof name;
+	memset(&name, 0, sizeof(name));
+	len = sizeof(name);
 	ret = getpeername(fd, (struct sockaddr *)&name, &len);
 	if (ret < 0) {
-		zlog_warn("Can't get remote address and port: %s",
-			  safe_strerror(errno));
+		flog_err(EC_LIB_SOCKET, "Can't get remote address and port: %s",
+			 safe_strerror(errno));
 		return NULL;
 	}
 
