@@ -12,61 +12,65 @@
 #include "zclient.h"
 #include "stream.h"
 #include "filter.h"
+#include "json.h"
 
 #include "nhrpd.h"
 #include "netlink.h"
 
+static int nhrp_config_write(struct vty *vty);
 static struct cmd_node zebra_node = {
-	.node   = ZEBRA_NODE,
+	.name = "zebra",
+	.node = ZEBRA_NODE,
+	.parent_node = CONFIG_NODE,
 	.prompt = "%s(config-router)# ",
-	.vtysh  = 1,
+	.config_write = nhrp_config_write,
 };
 
+static int interface_config_write(struct vty *vty);
 static struct cmd_node nhrp_interface_node = {
-	.node   = INTERFACE_NODE,
+	.name = "interface",
+	.node = INTERFACE_NODE,
+	.parent_node = CONFIG_NODE,
 	.prompt = "%s(config-if)# ",
-	.vtysh  = 1,
+	.config_write = interface_config_write,
 };
 
 #define NHRP_DEBUG_FLAGS_CMD "<all|common|event|interface|kernel|route|vici>"
 
-#define NHRP_DEBUG_FLAGS_STR		\
-	"All messages\n"		\
-	"Common messages (default)\n"	\
-	"Event manager messages\n"	\
-	"Interface messages\n"		\
-	"Kernel messages\n"		\
-	"Route messages\n"		\
+#define NHRP_DEBUG_FLAGS_STR                                                   \
+	"All messages\n"                                                       \
+	"Common messages (default)\n"                                          \
+	"Event manager messages\n"                                             \
+	"Interface messages\n"                                                 \
+	"Kernel messages\n"                                                    \
+	"Route messages\n"                                                     \
 	"VICI messages\n"
 
 static const struct message debug_flags_desc[] = {
-	{ NHRP_DEBUG_ALL, "all" },
-	{ NHRP_DEBUG_COMMON, "common" },
-	{ NHRP_DEBUG_IF, "interface" },
-	{ NHRP_DEBUG_KERNEL, "kernel" },
-	{ NHRP_DEBUG_ROUTE, "route" },
-	{ NHRP_DEBUG_VICI, "vici" },
-	{ NHRP_DEBUG_EVENT, "event" },
-	{ 0 }
-};
+	{NHRP_DEBUG_ALL, "all"},      {NHRP_DEBUG_COMMON, "common"},
+	{NHRP_DEBUG_IF, "interface"}, {NHRP_DEBUG_KERNEL, "kernel"},
+	{NHRP_DEBUG_ROUTE, "route"},  {NHRP_DEBUG_VICI, "vici"},
+	{NHRP_DEBUG_EVENT, "event"},  {0}};
 
 static const struct message interface_flags_desc[] = {
-	{ NHRP_IFF_SHORTCUT, "shortcut" },
-	{ NHRP_IFF_REDIRECT, "redirect" },
-	{ NHRP_IFF_REG_NO_UNIQUE, "registration no-unique" },
-	{ 0 }
-};
+	{NHRP_IFF_SHORTCUT, "shortcut"},
+	{NHRP_IFF_REDIRECT, "redirect"},
+	{NHRP_IFF_REG_NO_UNIQUE, "registration no-unique"},
+	{0}};
 
 static int nhrp_vty_return(struct vty *vty, int ret)
 {
-	static const char * const errmsgs[] = {
-		[NHRP_ERR_FAIL]				= "Command failed",
-		[NHRP_ERR_NO_MEMORY]			= "Out of memory",
-		[NHRP_ERR_UNSUPPORTED_INTERFACE]	= "NHRP not supported on this interface",
-		[NHRP_ERR_NHRP_NOT_ENABLED]		= "NHRP not enabled (set 'nhrp network-id' first)",
-		[NHRP_ERR_ENTRY_EXISTS]			= "Entry exists already",
-		[NHRP_ERR_ENTRY_NOT_FOUND]		= "Entry not found",
-		[NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH]	= "Protocol address family does not match command (ip/ipv6 mismatch)",
+	static const char *const errmsgs[] = {
+			[NHRP_ERR_FAIL] = "Command failed",
+			[NHRP_ERR_NO_MEMORY] = "Out of memory",
+			[NHRP_ERR_UNSUPPORTED_INTERFACE] =
+				"NHRP not supported on this interface",
+			[NHRP_ERR_NHRP_NOT_ENABLED] =
+				"NHRP not enabled (set 'nhrp network-id' first)",
+			[NHRP_ERR_ENTRY_EXISTS] = "Entry exists already",
+			[NHRP_ERR_ENTRY_NOT_FOUND] = "Entry not found",
+			[NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH] =
+				"Protocol address family does not match command (ip/ipv6 mismatch)",
 	};
 	const char *str = NULL;
 	char buf[256];
@@ -74,7 +78,7 @@ static int nhrp_vty_return(struct vty *vty, int ret)
 	if (ret == NHRP_OK)
 		return CMD_SUCCESS;
 
-	if (ret > 0 && ret <= (int)ZEBRA_NUM_OF(errmsgs))
+	if (ret > 0 && ret <= NHRP_ERR_MAX)
 		if (errmsgs[ret])
 			str = errmsgs[ret];
 
@@ -83,14 +87,14 @@ static int nhrp_vty_return(struct vty *vty, int ret)
 		snprintf(buf, sizeof(buf), "Unknown error %d", ret);
 	}
 
-	vty_out (vty, "%% %s%s", str, VTY_NEWLINE);
+	vty_out(vty, "%% %s\n", str);
 
-	return CMD_WARNING;
+	return CMD_WARNING_CONFIG_FAILED;
+	;
 }
 
-static int toggle_flag(
-	struct vty *vty, const struct message *flag_desc,
-	const char *name, int on_off, unsigned *flags)
+static int toggle_flag(struct vty *vty, const struct message *flag_desc,
+		       const char *name, int on_off, unsigned *flags)
 {
 	int i;
 
@@ -104,21 +108,22 @@ static int toggle_flag(
 		return CMD_SUCCESS;
 	}
 
-	vty_out(vty, "%% Invalid value %s%s", name, VTY_NEWLINE);
-	return CMD_WARNING;
+	vty_out(vty, "%% Invalid value %s\n", name);
+	return CMD_WARNING_CONFIG_FAILED;
+	;
 }
 
 #ifndef NO_DEBUG
 
-DEFUN(show_debugging_nhrp, show_debugging_nhrp_cmd,
-	"show debugging nhrp",
-	SHOW_STR
-	"Debugging information\n"
-	"NHRP configuration\n")
+DEFUN_NOSH(show_debugging_nhrp, show_debugging_nhrp_cmd,
+	   "show debugging [nhrp]",
+	   SHOW_STR
+	   "Debugging information\n"
+	   "NHRP configuration\n")
 {
 	int i;
 
-	vty_out(vty, "NHRP debugging status:%s", VTY_NEWLINE);
+	vty_out(vty, "NHRP debugging status:\n");
 
 	for (i = 0; debug_flags_desc[i].str != NULL; i++) {
 		if (debug_flags_desc[i].key == NHRP_DEBUG_ALL)
@@ -126,8 +131,8 @@ DEFUN(show_debugging_nhrp, show_debugging_nhrp_cmd,
 		if (!(debug_flags_desc[i].key & debug_flags))
 			continue;
 
-		vty_out(vty, "  NHRP %s debugging is on%s",
-			debug_flags_desc[i].str, VTY_NEWLINE);
+		vty_out(vty, "  NHRP %s debugging is on\n",
+			debug_flags_desc[i].str);
 	}
 
 	return CMD_SUCCESS;
@@ -139,7 +144,8 @@ DEFUN(debug_nhrp, debug_nhrp_cmd,
 	"NHRP information\n"
 	NHRP_DEBUG_FLAGS_STR)
 {
-	return toggle_flag(vty, debug_flags_desc, argv[2]->text, 1, &debug_flags);
+	return toggle_flag(vty, debug_flags_desc, argv[2]->text, 1,
+			   &debug_flags);
 }
 
 DEFUN(no_debug_nhrp, no_debug_nhrp_cmd,
@@ -149,7 +155,8 @@ DEFUN(no_debug_nhrp, no_debug_nhrp_cmd,
 	"NHRP information\n"
 	NHRP_DEBUG_FLAGS_STR)
 {
-	return toggle_flag(vty, debug_flags_desc, argv[3]->text, 0, &debug_flags);
+	return toggle_flag(vty, debug_flags_desc, argv[3]->text, 0,
+			   &debug_flags);
 }
 
 #endif /* NO_DEBUG */
@@ -158,7 +165,7 @@ static int nhrp_config_write(struct vty *vty)
 {
 #ifndef NO_DEBUG
 	if (debug_flags == NHRP_DEBUG_ALL) {
-		vty_out(vty, "debug nhrp all%s", VTY_NEWLINE);
+		vty_out(vty, "debug nhrp all\n");
 	} else {
 		int i;
 
@@ -167,19 +174,18 @@ static int nhrp_config_write(struct vty *vty)
 				continue;
 			if (!(debug_flags & debug_flags_desc[i].key))
 				continue;
-			vty_out(vty, "debug nhrp %s%s", debug_flags_desc[i].str, VTY_NEWLINE);
+			vty_out(vty, "debug nhrp %s\n",
+				debug_flags_desc[i].str);
 		}
 	}
-	vty_out(vty, "!%s", VTY_NEWLINE);
+	vty_out(vty, "!\n");
 #endif /* NO_DEBUG */
 
 	if (nhrp_event_socket_path) {
-		vty_out(vty, "nhrp event socket %s%s",
-			nhrp_event_socket_path, VTY_NEWLINE);
+		vty_out(vty, "nhrp event socket %s\n", nhrp_event_socket_path);
 	}
 	if (netlink_nflog_group) {
-		vty_out(vty, "nhrp nflog-group %d%s",
-			netlink_nflog_group, VTY_NEWLINE);
+		vty_out(vty, "nhrp nflog-group %d\n", netlink_nflog_group);
 	}
 
 	return 0;
@@ -198,7 +204,8 @@ static afi_t cmd_to_afi(const struct cmd_token *tok)
 
 static const char *afi_to_cmd(afi_t afi)
 {
-	if (afi == AFI_IP6) return "ipv6";
+	if (afi == AFI_IP6)
+		return "ipv6";
 	return "ip";
 }
 
@@ -207,7 +214,7 @@ DEFUN(nhrp_event_socket, nhrp_event_socket_cmd,
 	NHRP_STR
 	"Event Manager commands\n"
 	"Event Manager unix socket path\n"
-	"Unix path for the socket")
+	"Unix path for the socket\n")
 {
 	evmgr_set_socket(argv[3]->arg);
 	return CMD_SUCCESS;
@@ -219,7 +226,7 @@ DEFUN(no_nhrp_event_socket, no_nhrp_event_socket_cmd,
 	NHRP_STR
 	"Event Manager commands\n"
 	"Event Manager unix socket path\n"
-	"Unix path for the socket")
+	"Unix path for the socket\n")
 {
 	evmgr_set_socket(NULL);
 	return CMD_SUCCESS;
@@ -233,7 +240,7 @@ DEFUN(nhrp_nflog_group, nhrp_nflog_group_cmd,
 {
 	uint32_t nfgroup;
 
-	VTY_GET_INTEGER_RANGE("nflog-group", nfgroup, argv[2]->arg, 1, 65535);
+	nfgroup = strtoul(argv[2]->arg, NULL, 10);
 	netlink_set_nflog_group(nfgroup);
 
 	return CMD_SUCCESS;
@@ -263,7 +270,7 @@ DEFUN(tunnel_protection, tunnel_protection_cmd,
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 
 	nhrp_interface_set_protection(ifp, argv[4]->arg,
-			argc > 6 ? argv[6]->arg : NULL);
+				      argc > 6 ? argv[6]->arg : NULL);
 	return CMD_SUCCESS;
 }
 
@@ -312,7 +319,7 @@ DEFUN(if_nhrp_network_id, if_nhrp_network_id_cmd,
 	struct nhrp_interface *nifp = ifp->info;
 	afi_t afi = cmd_to_afi(argv[0]);
 
-	VTY_GET_INTEGER_RANGE("network-id", nifp->afi[afi].network_id, argv[3]->arg, 1, 4294967295);
+	nifp->afi[afi].network_id = strtoul(argv[3]->arg, NULL, 10);
 	nhrp_interface_update(ifp);
 
 	return CMD_SUCCESS;
@@ -347,7 +354,8 @@ DEFUN(if_nhrp_flags, if_nhrp_flags_cmd,
 	struct nhrp_interface *nifp = ifp->info;
 	afi_t afi = cmd_to_afi(argv[0]);
 
-	return toggle_flag(vty, interface_flags_desc, argv[2]->text, 1, &nifp->afi[afi].flags);
+	return toggle_flag(vty, interface_flags_desc, argv[2]->text, 1,
+			   &nifp->afi[afi].flags);
 }
 
 DEFUN(if_no_nhrp_flags, if_no_nhrp_flags_cmd,
@@ -362,7 +370,8 @@ DEFUN(if_no_nhrp_flags, if_no_nhrp_flags_cmd,
 	struct nhrp_interface *nifp = ifp->info;
 	afi_t afi = cmd_to_afi(argv[1]);
 
-	return toggle_flag(vty, interface_flags_desc, argv[3]->text, 0, &nifp->afi[afi].flags);
+	return toggle_flag(vty, interface_flags_desc, argv[3]->text, 0,
+			   &nifp->afi[afi].flags);
 }
 
 DEFUN(if_nhrp_reg_flags, if_nhrp_reg_flags_cmd,
@@ -377,7 +386,8 @@ DEFUN(if_nhrp_reg_flags, if_nhrp_reg_flags_cmd,
 	afi_t afi = cmd_to_afi(argv[0]);
 	char name[256];
 	snprintf(name, sizeof(name), "registration %s", argv[3]->text);
-	return toggle_flag(vty, interface_flags_desc, name, 1, &nifp->afi[afi].flags);
+	return toggle_flag(vty, interface_flags_desc, name, 1,
+			   &nifp->afi[afi].flags);
 }
 
 DEFUN(if_no_nhrp_reg_flags, if_no_nhrp_reg_flags_cmd,
@@ -393,7 +403,8 @@ DEFUN(if_no_nhrp_reg_flags, if_no_nhrp_reg_flags_cmd,
 	afi_t afi = cmd_to_afi(argv[1]);
 	char name[256];
 	snprintf(name, sizeof(name), "registration %s", argv[4]->text);
-	return toggle_flag(vty, interface_flags_desc, name, 0, &nifp->afi[afi].flags);
+	return toggle_flag(vty, interface_flags_desc, name, 0,
+			   &nifp->afi[afi].flags);
 }
 
 DEFUN(if_nhrp_holdtime, if_nhrp_holdtime_cmd,
@@ -407,7 +418,7 @@ DEFUN(if_nhrp_holdtime, if_nhrp_holdtime_cmd,
 	struct nhrp_interface *nifp = ifp->info;
 	afi_t afi = cmd_to_afi(argv[0]);
 
-	VTY_GET_INTEGER_RANGE("holdtime", nifp->afi[afi].holdtime, argv[3]->arg, 1, 65000);
+	nifp->afi[afi].holdtime = strtoul(argv[3]->arg, NULL, 10);
 	nhrp_interface_update(ifp);
 
 	return CMD_SUCCESS;
@@ -437,7 +448,7 @@ DEFUN(if_nhrp_mtu, if_nhrp_mtu_cmd,
 	NHRP_STR
 	"Configure NHRP advertised MTU\n"
 	"MTU value\n"
-	"Advertise bound interface MTU similar to OpenNHRP")
+	"Advertise bound interface MTU similar to OpenNHRP\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct nhrp_interface *nifp = ifp->info;
@@ -445,7 +456,8 @@ DEFUN(if_nhrp_mtu, if_nhrp_mtu_cmd,
 	if (argv[3]->arg[0] == 'o') {
 		nifp->afi[AFI_IP].configured_mtu = -1;
 	} else {
-		VTY_GET_INTEGER_RANGE("mtu", nifp->afi[AFI_IP].configured_mtu, argv[3]->arg, 576, 1500);
+		nifp->afi[AFI_IP].configured_mtu =
+			strtoul(argv[3]->arg, NULL, 10);
 	}
 	nhrp_interface_update_mtu(ifp, AFI_IP);
 
@@ -459,7 +471,7 @@ DEFUN(if_no_nhrp_mtu, if_no_nhrp_mtu_cmd,
 	NHRP_STR
 	"Configure NHRP advertised MTU\n"
 	"MTU value\n"
-	"Advertise bound interface MTU similar to OpenNHRP")
+	"Advertise bound interface MTU similar to OpenNHRP\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct nhrp_interface *nifp = ifp->info;
@@ -484,8 +496,8 @@ DEFUN(if_nhrp_map, if_nhrp_map_cmd,
 	union sockunion proto_addr, nbma_addr;
 	struct nhrp_cache *c;
 
-	if (str2sockunion(argv[3]->arg, &proto_addr) < 0 ||
-	    afi2family(afi) != sockunion_family(&proto_addr))
+	if (str2sockunion(argv[3]->arg, &proto_addr) < 0
+	    || afi2family(afi) != sockunion_family(&proto_addr))
 		return nhrp_vty_return(vty, NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH);
 
 	c = nhrp_cache_get(ifp, &proto_addr, 1);
@@ -493,41 +505,46 @@ DEFUN(if_nhrp_map, if_nhrp_map_cmd,
 		return nhrp_vty_return(vty, NHRP_ERR_FAIL);
 
 	c->map = 1;
-	if (strcmp(argv[4]->text, "local") == 0) {
-		nhrp_cache_update_binding(c, NHRP_CACHE_LOCAL, 0, NULL, 0, NULL);
-	} else{
+	if (strmatch(argv[4]->text, "local")) {
+		nhrp_cache_update_binding(c, NHRP_CACHE_LOCAL, 0, NULL, 0,
+					  NULL);
+	} else {
 		if (str2sockunion(argv[4]->arg, &nbma_addr) < 0)
 			return nhrp_vty_return(vty, NHRP_ERR_FAIL);
 		nhrp_cache_update_binding(c, NHRP_CACHE_STATIC, 0,
-			nhrp_peer_get(ifp, &nbma_addr), 0, NULL);
+					  nhrp_peer_get(ifp, &nbma_addr), 0,
+					  NULL);
 	}
 
 	return CMD_SUCCESS;
 }
 
 DEFUN(if_no_nhrp_map, if_no_nhrp_map_cmd,
-	"no " AFI_CMD " nhrp map <A.B.C.D|X:X::X:X>",
+	"no " AFI_CMD " nhrp map <A.B.C.D|X:X::X:X> [<A.B.C.D|local>]",
 	NO_STR
 	AFI_STR
 	NHRP_STR
 	"Nexthop Server configuration\n"
 	"IPv4 protocol address\n"
-	"IPv6 protocol address\n")
+	"IPv6 protocol address\n"
+	"IPv4 NBMA address\n"
+	"Handle protocol address locally\n")
 {
-	VTY_DECLVAR_CONTEXT(interface,ifp);
+	VTY_DECLVAR_CONTEXT(interface, ifp);
 	afi_t afi = cmd_to_afi(argv[1]);
-	union sockunion proto_addr;
+	union sockunion proto_addr, nbma_addr;
 	struct nhrp_cache *c;
 
-	if (str2sockunion(argv[4]->arg, &proto_addr) < 0 ||
-	    afi2family(afi) != sockunion_family(&proto_addr))
+	if (str2sockunion(argv[4]->arg, &proto_addr) < 0
+	    || afi2family(afi) != sockunion_family(&proto_addr))
 		return nhrp_vty_return(vty, NHRP_ERR_PROTOCOL_ADDRESS_MISMATCH);
 
 	c = nhrp_cache_get(ifp, &proto_addr, 0);
 	if (!c || !c->map)
 		return nhrp_vty_return(vty, NHRP_ERR_ENTRY_NOT_FOUND);
 
-	nhrp_cache_update_binding(c, c->cur.type, -1, NULL, 0, NULL);
+	nhrp_cache_update_binding(c, c->cur.type, -1,
+				  nhrp_peer_get(ifp, &nbma_addr), 0, NULL);
 	return CMD_SUCCESS;
 }
 
@@ -584,6 +601,7 @@ struct info_ctx {
 	struct vty *vty;
 	afi_t afi;
 	int count;
+	struct json_object *json;
 };
 
 static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
@@ -591,56 +609,100 @@ static void show_ip_nhrp_cache(struct nhrp_cache *c, void *pctx)
 	struct info_ctx *ctx = pctx;
 	struct vty *vty = ctx->vty;
 	char buf[2][SU_ADDRSTRLEN];
+	struct json_object *json = NULL;
 
 	if (ctx->afi != family2afi(sockunion_family(&c->remote_addr)))
 		return;
 
-	if (!ctx->count) {
-		vty_out(vty, "%-8s %-8s %-24s %-24s %-6s %s%s",
-			"Iface",
-			"Type",
-			"Protocol",
-			"NBMA",
-			"Flags",
-			"Identity",
-			VTY_NEWLINE);
+
+	if (!ctx->count && !ctx->json) {
+		vty_out(vty, "%-8s %-8s %-24s %-24s %-6s %s\n", "Iface", "Type",
+			"Protocol", "NBMA", "Flags", "Identity");
 	}
 	ctx->count++;
 
-	vty_out(ctx->vty, "%-8s %-8s %-24s %-24s %c%c%c    %s%s",
-		c->ifp->name,
+	sockunion2str(&c->remote_addr, buf[0], sizeof(buf[0]));
+	if (c->cur.peer)
+		sockunion2str(&c->cur.peer->vc->remote.nbma,
+			      buf[1], sizeof(buf[1]));
+	else
+		snprintf(buf[1], sizeof(buf[1]), "-");
+
+	if (ctx->json) {
+		json = json_object_new_object();
+		json_object_string_add(json, "interface", c->ifp->name);
+		json_object_string_add(json, "type",
+				       nhrp_cache_type_str[c->cur.type]);
+		json_object_string_add(json, "protocol", buf[0]);
+		json_object_string_add(json, "nbma", buf[1]);
+
+		if (c->used)
+			json_object_boolean_true_add(json, "used");
+		else
+			json_object_boolean_false_add(json, "used");
+
+		if (c->t_timeout)
+			json_object_boolean_true_add(json, "timeout");
+		else
+			json_object_boolean_false_add(json, "timeout");
+
+		if (c->t_auth)
+			json_object_boolean_true_add(json, "auth");
+		else
+			json_object_boolean_false_add(json, "auth");
+
+		if (c->cur.peer)
+			json_object_string_add(json, "identity",
+					       c->cur.peer->vc->remote.id);
+		else
+			json_object_string_add(json, "identity", "-");
+
+		json_object_array_add(ctx->json, json);
+		return;
+	}
+	vty_out(ctx->vty, "%-8s %-8s %-24s %-24s %c%c%c    %s\n", c->ifp->name,
 		nhrp_cache_type_str[c->cur.type],
-		sockunion2str(&c->remote_addr, buf[0], sizeof buf[0]),
-		c->cur.peer ? sockunion2str(&c->cur.peer->vc->remote.nbma, buf[1], sizeof buf[1]) : "-",
-		c->used ? 'U' : ' ',
-		c->t_timeout ? 'T' : ' ',
+		buf[0], buf[1],
+		c->used ? 'U' : ' ', c->t_timeout ? 'T' : ' ',
 		c->t_auth ? 'A' : ' ',
-		c->cur.peer ? c->cur.peer->vc->remote.id : "-",
-		VTY_NEWLINE);
+		c->cur.peer ? c->cur.peer->vc->remote.id : "-");
 }
 
-static void show_ip_nhrp_nhs(struct nhrp_nhs *n, struct nhrp_registration *reg, void *pctx)
+static void show_ip_nhrp_nhs(struct nhrp_nhs *n, struct nhrp_registration *reg,
+			     void *pctx)
 {
 	struct info_ctx *ctx = pctx;
 	struct vty *vty = ctx->vty;
 	char buf[2][SU_ADDRSTRLEN];
+	struct json_object *json = NULL;
 
-	if (!ctx->count) {
-		vty_out(vty, "%-8s %-24s %-16s %-16s%s",
-			"Iface",
-			"FQDN",
-			"NBMA",
-			"Protocol",
-			VTY_NEWLINE);
+	if (!ctx->count && !ctx->json) {
+		vty_out(vty, "%-8s %-24s %-16s %-16s\n", "Iface", "FQDN",
+			"NBMA", "Protocol");
 	}
 	ctx->count++;
 
-	vty_out(vty, "%-8s %-24s %-16s %-16s%s",
-		n->ifp->name,
-		n->nbma_fqdn,
-		(reg && reg->peer) ? sockunion2str(&reg->peer->vc->remote.nbma, buf[0], sizeof buf[0]) : "-",
-		sockunion2str(reg ? &reg->proto_addr : &n->proto_addr, buf[1], sizeof buf[1]),
-		VTY_NEWLINE);
+	if (reg && reg->peer)
+		sockunion2str(&reg->peer->vc->remote.nbma,
+			      buf[0], sizeof(buf[0]));
+	else
+		snprintf(buf[0], sizeof(buf[0]), "-");
+	sockunion2str(reg ? &reg->proto_addr : &n->proto_addr, buf[1],
+		      sizeof(buf[1]));
+
+	if (ctx->json) {
+		json = json_object_new_object();
+		json_object_string_add(json, "interface", n->ifp->name);
+		json_object_string_add(json, "fqdn", n->nbma_fqdn);
+		json_object_string_add(json, "nbma", buf[0]);
+		json_object_string_add(json, "protocol", buf[1]);
+
+		json_object_array_add(ctx->json, json);
+		return;
+	}
+
+	vty_out(vty, "%-8s %-24s %-16s %-16s\n", n->ifp->name, n->nbma_fqdn,
+		buf[0], buf[1]);
 }
 
 static void show_ip_nhrp_shortcut(struct nhrp_shortcut *s, void *pctx)
@@ -649,141 +711,244 @@ static void show_ip_nhrp_shortcut(struct nhrp_shortcut *s, void *pctx)
 	struct nhrp_cache *c;
 	struct vty *vty = ctx->vty;
 	char buf1[PREFIX_STRLEN], buf2[SU_ADDRSTRLEN];
+	struct json_object *json = NULL;
 
 	if (!ctx->count) {
-		vty_out(vty, "%-8s %-24s %-24s %s%s",
-			"Type",
-			"Prefix",
-			"Via",
-			"Identity",
-			VTY_NEWLINE);
+		vty_out(vty, "%-8s %-24s %-24s %s\n", "Type", "Prefix", "Via",
+			"Identity");
 	}
 	ctx->count++;
 
 	c = s->cache;
-	vty_out(ctx->vty, "%-8s %-24s %-24s %s%s",
+	if (c)
+		sockunion2str(&c->remote_addr, buf2, sizeof(buf2));
+	prefix2str(s->p, buf1, sizeof(buf1));
+
+	if (ctx->json) {
+		json = json_object_new_object();
+		json_object_string_add(json, "type",
+				       nhrp_cache_type_str[s->type]);
+		json_object_string_add(json, "prefix", buf1);
+
+		if (c)
+			json_object_string_add(json, "via", buf2);
+
+		if (c && c->cur.peer)
+			json_object_string_add(json, "identity",
+					       c->cur.peer->vc->remote.id);
+		else
+			json_object_string_add(json, "identity", "");
+
+		json_object_array_add(ctx->json, json);
+		return;
+	}
+
+	vty_out(ctx->vty, "%-8s %-24s %-24s %s\n",
 		nhrp_cache_type_str[s->type],
-		prefix2str(s->p, buf1, sizeof buf1),
-		c ? sockunion2str(&c->remote_addr, buf2, sizeof buf2) : "",
-		(c && c->cur.peer) ? c->cur.peer->vc->remote.id : "",
-		VTY_NEWLINE);
+		buf1, buf2,
+		(c && c->cur.peer) ? c->cur.peer->vc->remote.id : "");
 }
 
 static void show_ip_opennhrp_cache(struct nhrp_cache *c, void *pctx)
 {
 	struct info_ctx *ctx = pctx;
-	struct vty *vty = ctx->vty;
-	char buf[SU_ADDRSTRLEN];
+	char buf[3][SU_ADDRSTRLEN];
+	struct json_object *json = NULL;
+
 
 	if (ctx->afi != family2afi(sockunion_family(&c->remote_addr)))
 		return;
 
+	sockunion2str(&c->remote_addr, buf[0], sizeof(buf[0]));
+	if (c->cur.peer)
+		sockunion2str(&c->cur.peer->vc->remote.nbma, buf[1],
+			      sizeof(buf[1]));
+	if (sockunion_family(&c->cur.remote_nbma_natoa) != AF_UNSPEC)
+		sockunion2str(&c->cur.remote_nbma_natoa, buf[2],
+			      sizeof(buf[2]));
+	if (ctx->json) {
+		json = json_object_new_object();
+		json_object_string_add(json, "type",
+				       nhrp_cache_type_str[c->cur.type]);
+
+		if (c->cur.peer && c->cur.peer->online)
+			json_object_boolean_true_add(json, "up");
+		else
+			json_object_boolean_false_add(json, "up");
+
+		if (c->used)
+			json_object_boolean_true_add(json, "used");
+		else
+			json_object_boolean_false_add(json, "used");
+
+		json_object_string_add(json, "protocolAddress", buf[0]);
+		json_object_int_add(json, "protocolAddressSize",
+				    8 * family2addrsize(sockunion_family
+							(&c->remote_addr)));
+
+		if (c->cur.peer)
+			json_object_string_add(json, "nbmaAddress", buf[1]);
+
+		if (sockunion_family(&c->cur.remote_nbma_natoa) != AF_UNSPEC)
+			json_object_string_add(json, "nbmaNatOaAddress",
+					       buf[2]);
+
+		json_object_array_add(ctx->json, json);
+		return;
+	}
 	vty_out(ctx->vty,
-		"Type: %s%s"
-		"Flags:%s%s%s"
-		"Protocol-Address: %s/%zu%s",
+		"Type: %s\n"
+		"Flags:%s%s\n"
+		"Protocol-Address: %s/%zu\n",
 		nhrp_cache_type_str[c->cur.type],
-		VTY_NEWLINE,
-		(c->cur.peer && c->cur.peer->online) ? " up": "",
-		c->used ? " used": "",
-		VTY_NEWLINE,
-		sockunion2str(&c->remote_addr, buf, sizeof buf),
-		8 * family2addrsize(sockunion_family(&c->remote_addr)),
-		VTY_NEWLINE);
+		(c->cur.peer && c->cur.peer->online) ? " up" : "",
+		c->used ? " used" : "",
+		buf[0],
+		8 * family2addrsize(sockunion_family(&c->remote_addr)));
 
-	if (c->cur.peer) {
-		vty_out(ctx->vty,
-			"NBMA-Address: %s%s",
-			sockunion2str(&c->cur.peer->vc->remote.nbma, buf, sizeof buf),
-			VTY_NEWLINE);
-	}
+	if (c->cur.peer)
+		vty_out(ctx->vty, "NBMA-Address: %s\n", buf[1]);
 
-	if (sockunion_family(&c->cur.remote_nbma_natoa) != AF_UNSPEC) {
-		vty_out(ctx->vty,
-			"NBMA-NAT-OA-Address: %s%s",
-			sockunion2str(&c->cur.remote_nbma_natoa, buf, sizeof buf),
-			VTY_NEWLINE);
-	}
+	if (sockunion_family(&c->cur.remote_nbma_natoa) != AF_UNSPEC)
+		vty_out(ctx->vty, "NBMA-NAT-OA-Address: %s\n", buf[2]);
 
-	vty_out(ctx->vty, "%s", VTY_NEWLINE);
+	vty_out(ctx->vty, "\n\n");
 }
 
 DEFUN(show_ip_nhrp, show_ip_nhrp_cmd,
-	"show " AFI_CMD " nhrp [cache|nhs|shortcut|opennhrp]",
+	"show " AFI_CMD " nhrp [cache|nhs|shortcut|opennhrp] [json]",
 	SHOW_STR
 	AFI_STR
 	"NHRP information\n"
 	"Forwarding cache information\n"
 	"Next hop server information\n"
 	"Shortcut information\n"
-	"opennhrpctl style cache dump\n")
+	"opennhrpctl style cache dump\n"
+	JSON_STR)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 	struct info_ctx ctx = {
-		.vty = vty,
-		.afi = cmd_to_afi(argv[1]),
+		.vty = vty, .afi = cmd_to_afi(argv[1]), .json = NULL
 	};
+	bool uj = use_json(argc, argv);
+	struct json_object *json_path = NULL;
+	struct json_object *json_vrf = NULL, *json_vrf_path = NULL;
+	int ret = CMD_SUCCESS;
 
+	if (uj) {
+		json_vrf = json_object_new_object();
+		json_vrf_path = json_object_new_object();
+		json_path = json_object_new_array();
+		ctx.json = json_path;
+	}
 	if (argc <= 3 || argv[3]->text[0] == 'c') {
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			nhrp_cache_foreach(ifp, show_ip_nhrp_cache, &ctx);
 	} else if (argv[3]->text[0] == 'n') {
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			nhrp_nhs_foreach(ifp, ctx.afi, show_ip_nhrp_nhs, &ctx);
 	} else if (argv[3]->text[0] == 's') {
 		nhrp_shortcut_foreach(ctx.afi, show_ip_nhrp_shortcut, &ctx);
 	} else {
-		vty_out(vty, "Status: ok%s%s", VTY_NEWLINE, VTY_NEWLINE);
+		if (!ctx.json)
+			vty_out(vty, "Status: ok\n\n");
+		else
+			json_object_string_add(json_vrf, "status", "ok");
+
 		ctx.count++;
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			nhrp_cache_foreach(ifp, show_ip_opennhrp_cache, &ctx);
 	}
 
+	if (uj)
+		json_object_int_add(json_vrf, "entriesCount", ctx.count);
 	if (!ctx.count) {
-		vty_out(vty, "%% No entries%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		if (!ctx.json)
+			vty_out(vty, "%% No entries\n");
+		ret = CMD_WARNING;
 	}
-
-	return CMD_SUCCESS;
+	if (uj) {
+		json_object_object_add(json_vrf_path, "attr", json_vrf);
+		json_object_object_add(json_vrf_path, "table", ctx.json);
+		vty_out(vty, "%s",
+			json_object_to_json_string_ext(
+			       json_vrf_path, JSON_C_TO_STRING_PRETTY));
+		json_object_free(json_vrf_path);
+	}
+	return ret;
 }
 
+struct dmvpn_cfg {
+	struct vty *vty;
+	struct json_object *json;
+};
+
 static void show_dmvpn_entry(struct nhrp_vc *vc, void *ctx)
 {
-	struct vty *vty = ctx;
+	struct dmvpn_cfg *ctxt = ctx;
+	struct vty *vty;
 	char buf[2][SU_ADDRSTRLEN];
+	struct json_object *json = NULL;
 
-	vty_out(vty, "%-24s %-24s %c      %-4d %-24s%s",
-		sockunion2str(&vc->local.nbma, buf[0], sizeof buf[0]),
-		sockunion2str(&vc->remote.nbma, buf[1], sizeof buf[1]),
-		notifier_active(&vc->notifier_list) ? 'n' : ' ',
-		vc->ipsec,
-		vc->remote.id,
-		VTY_NEWLINE);
+	if (!ctxt || !ctxt->vty)
+		return;
+	vty = ctxt->vty;
+	sockunion2str(&vc->local.nbma, buf[0], sizeof(buf[0]));
+	sockunion2str(&vc->remote.nbma, buf[1], sizeof(buf[1]));
+	if (ctxt->json) {
+		json = json_object_new_object();
+		json_object_string_add(json, "src", buf[0]);
+		json_object_string_add(json, "dst", buf[1]);
+
+		if (notifier_active(&vc->notifier_list))
+			json_object_boolean_true_add(json, "notifierActive");
+		else
+			json_object_boolean_false_add(json, "notifierActive");
+
+		json_object_int_add(json, "sas", vc->ipsec);
+		json_object_string_add(json, "identity", vc->remote.id);
+		json_object_array_add(ctxt->json, json);
+	} else {
+		vty_out(vty, "%-24s %-24s %c      %-4d %-24s\n",
+			buf[0], buf[1], notifier_active(&vc->notifier_list) ?
+			'n' : ' ', vc->ipsec, vc->remote.id);
+	}
 }
 
 DEFUN(show_dmvpn, show_dmvpn_cmd,
-	"show dmvpn",
+	"show dmvpn [json]",
 	SHOW_STR
-	"DMVPN information\n")
+	"DMVPN information\n"
+	JSON_STR)
 {
-	vty_out(vty, "%-24s %-24s %-6s %-4s %-24s%s",
-		"Src",
-		"Dst",
-		"Flags",
-		"SAs",
-		"Identity",
-		VTY_NEWLINE);
-
-	nhrp_vc_foreach(show_dmvpn_entry, vty);
+	bool uj = use_json(argc, argv);
+	struct dmvpn_cfg ctxt;
+	struct json_object *json_path = NULL;
 
+	ctxt.vty = vty;
+	if (!uj) {
+		ctxt.json = NULL;
+		vty_out(vty, "%-24s %-24s %-6s %-4s %-24s\n",
+			"Src", "Dst", "Flags", "SAs", "Identity");
+	} else {
+		json_path = json_object_new_array();
+		ctxt.json = json_path;
+	}
+	nhrp_vc_foreach(show_dmvpn_entry, &ctxt);
+	if (uj) {
+		vty_out(vty, "%s",
+			json_object_to_json_string_ext(
+			       json_path, JSON_C_TO_STRING_PRETTY));
+		json_object_free(json_path);
+	}
 	return CMD_SUCCESS;
 }
 
 static void clear_nhrp_cache(struct nhrp_cache *c, void *data)
 {
 	struct info_ctx *ctx = data;
-	if (c->cur.type <= NHRP_CACHE_CACHED) {
+	if (c->cur.type <= NHRP_CACHE_DYNAMIC) {
 		nhrp_cache_update_binding(c, c->cur.type, -1, NULL, 0, NULL);
 		ctx->count++;
 	}
@@ -804,27 +969,25 @@ DEFUN(clear_nhrp, clear_nhrp_cmd,
 	"Dynamic cache entries\n"
 	"Shortcut entries\n")
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 	struct info_ctx ctx = {
-		.vty = vty,
-		.afi = cmd_to_afi(argv[1]),
-		.count = 0,
+		.vty = vty, .afi = cmd_to_afi(argv[1]), .count = 0,
 	};
 
 	if (argc <= 3 || argv[3]->text[0] == 'c') {
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			nhrp_cache_foreach(ifp, clear_nhrp_cache, &ctx);
 	} else {
 		nhrp_shortcut_foreach(ctx.afi, clear_nhrp_shortcut, &ctx);
 	}
 
 	if (!ctx.count) {
-		vty_out(vty, "%% No entries%s", VTY_NEWLINE);
+		vty_out(vty, "%% No entries\n");
 		return CMD_WARNING;
 	}
 
-	vty_out(vty, "%% %d entries cleared%s", ctx.count, VTY_NEWLINE);
+	vty_out(vty, "%% %d entries cleared\n", ctx.count);
 	return CMD_SUCCESS;
 }
 
@@ -840,21 +1003,23 @@ static void interface_config_write_nhrp_map(struct nhrp_cache *c, void *data)
 	struct vty *vty = ctx->vty;
 	char buf[2][SU_ADDRSTRLEN];
 
-	if (!c->map) return;
-	if (sockunion_family(&c->remote_addr) != ctx->family) return;
+	if (!c->map)
+		return;
+	if (sockunion_family(&c->remote_addr) != ctx->family)
+		return;
 
-	vty_out(vty, " %s nhrp map %s %s%s",
-		ctx->aficmd,
-		sockunion2str(&c->remote_addr, buf[0], sizeof buf[0]),
-		c->cur.type == NHRP_CACHE_LOCAL ? "local" :
-		sockunion2str(&c->cur.peer->vc->remote.nbma, buf[1], sizeof buf[1]),
-		VTY_NEWLINE);
+	vty_out(vty, " %s nhrp map %s %s\n", ctx->aficmd,
+		sockunion2str(&c->remote_addr, buf[0], sizeof(buf[0])),
+		c->cur.type == NHRP_CACHE_LOCAL
+			? "local"
+			: sockunion2str(&c->cur.peer->vc->remote.nbma, buf[1],
+					sizeof(buf[1])));
 }
 
 static int interface_config_write(struct vty *vty)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct write_map_ctx mapctx;
-	struct listnode *node;
 	struct interface *ifp;
 	struct nhrp_interface *nifp;
 	struct nhrp_nhs *nhs;
@@ -863,10 +1028,10 @@ static int interface_config_write(struct vty *vty)
 	char buf[SU_ADDRSTRLEN];
 	int i;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
-		vty_out(vty, "interface %s%s", ifp->name, VTY_NEWLINE);
+	FOR_ALL_INTERFACES (vrf, ifp) {
+		vty_frame(vty, "interface %s\n", ifp->name);
 		if (ifp->desc)
-			vty_out(vty, " description %s%s", ifp->desc, VTY_NEWLINE);
+			vty_out(vty, " description %s\n", ifp->desc);
 
 		nifp = ifp->info;
 		if (nifp->ipsec_profile) {
@@ -875,11 +1040,10 @@ static int interface_config_write(struct vty *vty)
 			if (nifp->ipsec_fallback_profile)
 				vty_out(vty, " fallback-profile %s",
 					nifp->ipsec_fallback_profile);
-			vty_out(vty, "%s", VTY_NEWLINE);
+			vty_out(vty, "\n");
 		}
 		if (nifp->source)
-			vty_out(vty, " tunnel source %s%s",
-				nifp->source, VTY_NEWLINE);
+			vty_out(vty, " tunnel source %s\n", nifp->source);
 
 		for (afi = 0; afi < AFI_MAX; afi++) {
 			struct nhrp_afi_data *ad = &nifp->afi[afi];
@@ -887,47 +1051,50 @@ static int interface_config_write(struct vty *vty)
 			aficmd = afi_to_cmd(afi);
 
 			if (ad->network_id)
-				vty_out(vty, " %s nhrp network-id %u%s",
-					aficmd, ad->network_id,
-					VTY_NEWLINE);
+				vty_out(vty, " %s nhrp network-id %u\n", aficmd,
+					ad->network_id);
 
 			if (ad->holdtime != NHRPD_DEFAULT_HOLDTIME)
-				vty_out(vty, " %s nhrp holdtime %u%s",
-					aficmd, ad->holdtime,
-					VTY_NEWLINE);
+				vty_out(vty, " %s nhrp holdtime %u\n", aficmd,
+					ad->holdtime);
 
 			if (ad->configured_mtu < 0)
-				vty_out(vty, " %s nhrp mtu opennhrp%s",
-					aficmd, VTY_NEWLINE);
+				vty_out(vty, " %s nhrp mtu opennhrp\n", aficmd);
 			else if (ad->configured_mtu)
-				vty_out(vty, " %s nhrp mtu %u%s",
-					aficmd, ad->configured_mtu,
-					VTY_NEWLINE);
+				vty_out(vty, " %s nhrp mtu %u\n", aficmd,
+					ad->configured_mtu);
 
 			for (i = 0; interface_flags_desc[i].str != NULL; i++) {
 				if (!(ad->flags & interface_flags_desc[i].key))
 					continue;
-				vty_out(vty, " %s nhrp %s%s",
-					aficmd, interface_flags_desc[i].str, VTY_NEWLINE);
+				vty_out(vty, " %s nhrp %s\n", aficmd,
+					interface_flags_desc[i].str);
 			}
 
-			mapctx = (struct write_map_ctx) {
+			mapctx = (struct write_map_ctx){
 				.vty = vty,
 				.family = afi2family(afi),
 				.aficmd = aficmd,
 			};
-			nhrp_cache_foreach(ifp, interface_config_write_nhrp_map, &mapctx);
+			nhrp_cache_foreach(ifp, interface_config_write_nhrp_map,
+					   &mapctx);
 
-			list_for_each_entry(nhs, &ad->nhslist_head, nhslist_entry) {
-				vty_out(vty, " %s nhrp nhs %s nbma %s%s",
+			list_for_each_entry(nhs, &ad->nhslist_head,
+					    nhslist_entry)
+			{
+				vty_out(vty, " %s nhrp nhs %s nbma %s\n",
 					aficmd,
-					sockunion_family(&nhs->proto_addr) == AF_UNSPEC ? "dynamic" : sockunion2str(&nhs->proto_addr, buf, sizeof buf),
-					nhs->nbma_fqdn,
-					VTY_NEWLINE);
+					sockunion_family(&nhs->proto_addr)
+							== AF_UNSPEC
+						? "dynamic"
+						: sockunion2str(
+							  &nhs->proto_addr, buf,
+							  sizeof(buf)),
+					nhs->nbma_fqdn);
 			}
 		}
 
-		vty_out (vty, "!%s", VTY_NEWLINE);
+		vty_endframe(vty, "!\n");
 	}
 
 	return 0;
@@ -935,11 +1102,11 @@ static int interface_config_write(struct vty *vty)
 
 void nhrp_config_init(void)
 {
-	install_node(&zebra_node, nhrp_config_write);
+	install_node(&zebra_node);
 	install_default(ZEBRA_NODE);
 
 	/* access-list commands */
-	access_list_init ();
+	access_list_init();
 
 	/* global commands */
 	install_element(VIEW_NODE, &show_debugging_nhrp_cmd);
@@ -959,7 +1126,7 @@ void nhrp_config_init(void)
 	install_element(CONFIG_NODE, &no_nhrp_nflog_group_cmd);
 
 	/* interface specific commands */
-	install_node(&nhrp_interface_node, interface_config_write);
+	install_node(&nhrp_interface_node);
 
 	if_cmd_init();
 	install_element(INTERFACE_NODE, &tunnel_protection_cmd);
