@@ -166,7 +166,7 @@ int ospf6_orig_as_external_lsa(struct thread *thread)
 		if (IS_OSPF6_DEBUG_ASBR)
 			zlog_debug(
 				"%s: Send update of AS-External LSA %s seq 0x%x",
-				__PRETTY_FUNCTION__, lsa->name,
+				__func__, lsa->name,
 				ntohl(lsa->header->seqnum));
 
 		ospf6_flood_interface(NULL, lsa, oi);
@@ -245,7 +245,7 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 					   sizeof(buf));
 				zlog_debug(
 					"%s: route %s cost old %u new %u is not same, replace route",
-					__PRETTY_FUNCTION__, buf, o_path->cost,
+					__func__, buf, o_path->cost,
 					route->path.cost);
 			}
 
@@ -260,12 +260,12 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 					listnode_delete(old_route->nh_list,
 							rnh);
 					ospf6_nexthop_delete(rnh);
-					route_updated = true;
 				}
 			}
 
 			listnode_delete(old_route->paths, o_path);
 			ospf6_path_free(o_path);
+			route_updated = true;
 
 			/* Current route's path (adv_router info) is similar
 			 * to route being added.
@@ -273,6 +273,19 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 			 * Update FIB with effective NHs.
 			 */
 			if (listcount(old_route->paths)) {
+				for (ALL_LIST_ELEMENTS(old_route->paths,
+						anode, anext, o_path)) {
+					ospf6_merge_nexthops(
+						old_route->nh_list,
+						o_path->nh_list);
+				}
+				/* Update RIB/FIB with effective
+				 * nh_list
+				 */
+				if (ospf6->route_table->hook_add)
+					(*ospf6->route_table->hook_add)
+						(old_route);
+
 				if (old_route->path.origin.id
 					    == route->path.origin.id
 				    && old_route->path.origin.adv_router
@@ -290,36 +303,18 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 					old_route->path.origin.adv_router =
 						h_path->origin.adv_router;
 				}
-
-				if (route_updated) {
-					for (ALL_LIST_ELEMENTS(old_route->paths,
-							       anode, anext,
-							       o_path)) {
-						ospf6_merge_nexthops(
-							old_route->nh_list,
-							o_path->nh_list);
-					}
-					/* Update RIB/FIB with effective
-					 * nh_list
-					 */
-					if (ospf6->route_table->hook_add)
-						(*ospf6->route_table->hook_add)(
-							old_route);
-					break;
-				}
 			} else {
 				if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
 					prefix2str(&old_route->prefix, buf,
 						   sizeof(buf));
 					zlog_debug(
 						"%s: route %s old cost %u new cost %u, delete old entry.",
-						__PRETTY_FUNCTION__, buf,
+						__func__, buf,
 						old_route->path.cost,
 						route->path.cost);
 				}
 				ospf6_route_remove(old_route,
 						   ospf6->route_table);
-				break;
 			}
 		}
 		if (route_updated)
@@ -345,8 +340,7 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 					   sizeof(buf));
 				zlog_debug(
 					"%s: old route %s path  cost %u e2 %u",
-					__PRETTY_FUNCTION__, buf,
-					old_route->path.cost,
+					__func__, buf, old_route->path.cost,
 					old_route->path.u.cost_e2);
 			}
 			route_found = true;
@@ -374,13 +368,6 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 				/* Add a nh_list to new ecmp path */
 				ospf6_copy_nexthops(ecmp_path->nh_list,
 						    route->nh_list);
-				/* Merge nexthop to existing route's nh_list */
-				ospf6_route_merge_nexthops(old_route, route);
-
-				/* Update RIB/FIB */
-				if (ospf6->route_table->hook_add)
-					(*ospf6->route_table->hook_add)(
-						old_route);
 
 				/* Add the new path to route's path list */
 				listnode_add_sort(old_route->paths, ecmp_path);
@@ -390,56 +377,63 @@ void ospf6_asbr_update_route_ecmp_path(struct ospf6_route *old,
 						   sizeof(buf));
 					zlog_debug(
 						"%s: route %s another path added with nh %u, effective paths %u nh %u",
-						__PRETTY_FUNCTION__, buf,
+						__func__, buf,
 						listcount(ecmp_path->nh_list),
-						old_route->paths
-							? listcount(
-								  old_route
-									  ->paths)
-							: 0,
+						old_route->paths ? listcount(
+							old_route->paths)
+								 : 0,
 						listcount(old_route->nh_list));
 				}
 			} else {
-				for (ALL_LIST_ELEMENTS_RO(o_path->nh_list,
-							  nnode, nh)) {
-					for (ALL_LIST_ELEMENTS(
-						     old_route->nh_list, rnode,
-						     rnext, rnh)) {
-						if (!ospf6_nexthop_is_same(rnh,
-									   nh))
-							continue;
-
-						listnode_delete(
-							old_route->nh_list,
-							rnh);
-						ospf6_nexthop_delete(rnh);
-					}
-				}
 				list_delete_all_node(o_path->nh_list);
 				ospf6_copy_nexthops(o_path->nh_list,
 						    route->nh_list);
-
-				/* Merge nexthop to existing route's nh_list */
-				ospf6_route_merge_nexthops(old_route, route);
-
-				if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
-					prefix2str(&route->prefix, buf,
-						   sizeof(buf));
-					zlog_debug(
-						"%s: existing route %s with effective nh count %u",
-						__PRETTY_FUNCTION__, buf,
-						old_route->nh_list
-							? listcount(
-								  old_route
-									  ->nh_list)
-							: 0);
-				}
-
-				/* Update RIB/FIB */
-				if (ospf6->route_table->hook_add)
-					(*ospf6->route_table->hook_add)(
-						old_route);
 			}
+
+			/* Reset nexthop lists, rebuild from brouter table
+			 * for each adv. router.
+			 */
+			list_delete_all_node(old_route->nh_list);
+
+			for (ALL_LIST_ELEMENTS_RO(old_route->paths, anode,
+						  o_path)) {
+				struct ospf6_route *asbr_entry;
+
+				asbr_entry = ospf6_route_lookup(
+							&o_path->ls_prefix,
+							ospf6->brouter_table);
+				if (asbr_entry == NULL) {
+					if (IS_OSPF6_DEBUG_EXAMIN(
+							AS_EXTERNAL)) {
+						prefix2str(&old_route->prefix,
+							   buf, sizeof(buf));
+						zlog_debug(
+							"%s: ls_prfix %s asbr_entry not found.",
+							__func__, buf);
+					}
+					continue;
+				}
+				ospf6_route_merge_nexthops(old_route,
+							   asbr_entry);
+			}
+
+			if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
+				prefix2str(&route->prefix, buf, sizeof(buf));
+				zlog_debug(
+					"%s: route %s with effective paths %u nh %u",
+					__func__, buf,
+					old_route->paths
+						? listcount(old_route->paths)
+						: 0,
+					old_route->nh_list
+						? listcount(old_route->nh_list)
+						: 0);
+			}
+
+			/* Update RIB/FIB */
+			if (ospf6->route_table->hook_add)
+				(*ospf6->route_table->hook_add)(old_route);
+
 			/* Delete the new route its info added to existing
 			 * route.
 			 */
@@ -503,14 +497,16 @@ void ospf6_asbr_lsa_add(struct ospf6_lsa *lsa)
 	route->type = OSPF6_DEST_TYPE_NETWORK;
 	route->prefix.family = AF_INET6;
 	route->prefix.prefixlen = external->prefix.prefix_length;
-	ospf6_prefix_in6_addr(&route->prefix.u.prefix6, &external->prefix);
+	ospf6_prefix_in6_addr(&route->prefix.u.prefix6, external,
+			      &external->prefix);
 
 	route->path.area_id = asbr_entry->path.area_id;
 	route->path.origin.type = lsa->header->type;
 	route->path.origin.id = lsa->header->id;
 	route->path.origin.adv_router = lsa->header->adv_router;
-
 	route->path.prefix_options = external->prefix.prefix_options;
+	memcpy(&route->path.ls_prefix, &asbr_id, sizeof(struct prefix));
+
 	if (CHECK_FLAG(external->bits_metric, OSPF6_ASBR_BIT_E)) {
 		route->path.type = OSPF6_PATH_TYPE_EXTERNAL2;
 		route->path.metric_type = 2;
@@ -536,7 +532,7 @@ void ospf6_asbr_lsa_add(struct ospf6_lsa *lsa)
 	if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
 		prefix2str(&route->prefix, buf, sizeof(buf));
 		zlog_debug("%s: AS-External %u route add %s cost %u(%u) nh %u",
-			   __PRETTY_FUNCTION__,
+			   __func__,
 			   (route->path.type == OSPF6_PATH_TYPE_EXTERNAL1) ? 1
 									   : 2,
 			   buf, route->path.cost, route->path.u.cost_e2,
@@ -581,7 +577,7 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 	route_to_del->type = OSPF6_DEST_TYPE_NETWORK;
 	route_to_del->prefix.family = AF_INET6;
 	route_to_del->prefix.prefixlen = external->prefix.prefix_length;
-	ospf6_prefix_in6_addr(&route_to_del->prefix.u.prefix6,
+	ospf6_prefix_in6_addr(&route_to_del->prefix.u.prefix6, external,
 			      &external->prefix);
 
 	route_to_del->path.origin.type = lsa->header->type;
@@ -608,7 +604,7 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 	memset(&prefix, 0, sizeof(struct prefix));
 	prefix.family = AF_INET6;
 	prefix.prefixlen = external->prefix.prefix_length;
-	ospf6_prefix_in6_addr(&prefix.u.prefix6, &external->prefix);
+	ospf6_prefix_in6_addr(&prefix.u.prefix6, external, &external->prefix);
 
 	route = ospf6_route_lookup(&prefix, ospf6->route_table);
 	if (route == NULL) {
@@ -625,9 +621,8 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 		prefix2str(&prefix, buf, sizeof(buf));
 		zlog_debug(
 			"%s: Current route %s cost %u e2 %u, route to del cost %u e2 %u",
-			__PRETTY_FUNCTION__, buf, route->path.cost,
-			route->path.u.cost_e2, route_to_del->path.cost,
-			route_to_del->path.u.cost_e2);
+			__func__, buf, route->path.cost, route->path.u.cost_e2,
+			route_to_del->path.cost, route_to_del->path.u.cost_e2);
 	}
 
 	for (ospf6_route_lock(route);
@@ -674,8 +669,8 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 							   sizeof(buf));
 						zlog_debug(
 							"%s: route %s to delete is not same, cost %u del cost %u. skip",
-							__PRETTY_FUNCTION__,
-							buf, route->path.cost,
+							__func__, buf,
+							route->path.cost,
 							route_to_del->path
 								.cost);
 					}
@@ -686,8 +681,7 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 					prefix2str(&prefix, buf, sizeof(buf));
 					zlog_debug(
 						"%s: route %s path found with cost %u nh %u to remove.",
-						__PRETTY_FUNCTION__, buf,
-						route->path.cost,
+						__func__, buf, route->path.cost,
 						listcount(o_path->nh_list));
 				}
 
@@ -730,13 +724,15 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 						   sizeof(buf));
 					zlog_debug(
 						"%s: AS-External %u route %s update paths %u nh %u",
-						__PRETTY_FUNCTION__,
+						__func__,
 						(route->path.type
 						 == OSPF6_PATH_TYPE_EXTERNAL1)
 							? 1
 							: 2,
 						buf, listcount(route->paths),
-						listcount(route->nh_list));
+						route->nh_list ? listcount(
+							route->nh_list)
+							       : 0);
 				}
 
 				if (listcount(route->paths)) {
@@ -788,8 +784,7 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 					prefix2str(&prefix, buf, sizeof(buf));
 					zlog_debug(
 						"%s: route %s to delete is not same, cost %u del cost %u. skip",
-						__PRETTY_FUNCTION__, buf,
-						route->path.cost,
+						__func__, buf, route->path.cost,
 						route_to_del->path.cost);
 				}
 				continue;
@@ -805,7 +800,7 @@ void ospf6_asbr_lsa_remove(struct ospf6_lsa *lsa,
 			prefix2str(&route->prefix, buf, sizeof(buf));
 			zlog_debug(
 				"%s: AS-External %u route remove %s cost %u(%u) nh %u",
-				__PRETTY_FUNCTION__,
+				__func__,
 				route->path.type == OSPF6_PATH_TYPE_EXTERNAL1
 					? 1
 					: 2,
@@ -859,16 +854,22 @@ void ospf6_asbr_lsentry_remove(struct ospf6_route *asbr_entry)
 
 static void ospf6_asbr_routemap_set(int type, const char *mapname)
 {
-	if (ospf6->rmap[type].name)
+	if (ospf6->rmap[type].name) {
+		route_map_counter_decrement(ospf6->rmap[type].map);
 		free(ospf6->rmap[type].name);
+	}
 	ospf6->rmap[type].name = strdup(mapname);
 	ospf6->rmap[type].map = route_map_lookup_by_name(mapname);
+	route_map_counter_increment(ospf6->rmap[type].map);
 }
 
 static void ospf6_asbr_routemap_unset(int type)
 {
 	if (ospf6->rmap[type].name)
 		free(ospf6->rmap[type].name);
+
+	route_map_counter_decrement(ospf6->rmap[type].map);
+
 	ospf6->rmap[type].name = NULL;
 	ospf6->rmap[type].map = NULL;
 }
@@ -889,8 +890,7 @@ static int ospf6_asbr_routemap_update_timer(struct thread *thread)
 	if (ospf6->rmap[arg_type].map) {
 		if (IS_OSPF6_DEBUG_ASBR)
 			zlog_debug("%s: route-map %s update, reset redist %s",
-				   __PRETTY_FUNCTION__,
-				   ospf6->rmap[arg_type].name,
+				   __func__, ospf6->rmap[arg_type].name,
 				   ZROUTE_NAME(arg_type));
 
 		ospf6_zebra_no_redistribute(arg_type);
@@ -914,12 +914,12 @@ void ospf6_asbr_distribute_list_update(int type)
 	args[1] = (void *)((ptrdiff_t)type);
 
 	if (IS_OSPF6_DEBUG_ASBR)
-		zlog_debug("%s: trigger redistribute %s reset thread",
-			   __PRETTY_FUNCTION__, ZROUTE_NAME(type));
+		zlog_debug("%s: trigger redistribute %s reset thread", __func__,
+			   ZROUTE_NAME(type));
 
 	ospf6->t_distribute_update = NULL;
-	thread_add_timer_msec(master, ospf6_asbr_routemap_update_timer,
-			      (void **)args, OSPF_MIN_LS_INTERVAL,
+	thread_add_timer_msec(master, ospf6_asbr_routemap_update_timer, args,
+			      OSPF_MIN_LS_INTERVAL,
 			      &ospf6->t_distribute_update);
 }
 
@@ -940,8 +940,12 @@ static void ospf6_asbr_routemap_update(const char *mapname)
 				if (IS_OSPF6_DEBUG_ASBR)
 					zlog_debug(
 						"%s: route-map %s update, reset redist %s",
-						__PRETTY_FUNCTION__, mapname,
+						__func__, mapname,
 						ZROUTE_NAME(type));
+
+				route_map_counter_increment(
+					ospf6->rmap[type].map);
+
 				ospf6_asbr_distribute_list_update(type);
 			}
 		} else
@@ -949,7 +953,7 @@ static void ospf6_asbr_routemap_update(const char *mapname)
 	}
 }
 
-static void ospf6_asbr_routemap_event(route_map_event_t event, const char *name)
+static void ospf6_asbr_routemap_event(const char *name)
 {
 	int type;
 
@@ -999,7 +1003,7 @@ void ospf6_asbr_send_externals_to_area(struct ospf6_area *oa)
 
 	for (ALL_LSDB(oa->ospf6->lsdb, lsa)) {
 		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_AS_EXTERNAL) {
-			zlog_debug("%s: Flooding AS-External LSA %s\n",
+			zlog_debug("%s: Flooding AS-External LSA %s",
 				   __func__, lsa->name);
 			ospf6_flood_area(NULL, lsa, oa);
 		}
@@ -1011,7 +1015,7 @@ void ospf6_asbr_redistribute_add(int type, ifindex_t ifindex,
 				 unsigned int nexthop_num,
 				 struct in6_addr *nexthop, route_tag_t tag)
 {
-	int ret;
+	route_map_result_t ret;
 	struct ospf6_route troute;
 	struct ospf6_external_info tinfo;
 	struct ospf6_route *route, *match;
@@ -1348,8 +1352,9 @@ static void ospf6_redistribute_show_config(struct vty *vty)
 
 
 /* Routemap Functions */
-static route_map_result_t
-ospf6_routemap_rule_match_address_prefixlist(void *rule, struct prefix *prefix,
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_match_address_prefixlist(void *rule,
+					     const struct prefix *prefix,
 					     route_map_object_t type,
 					     void *object)
 {
@@ -1377,7 +1382,8 @@ static void ospf6_routemap_rule_match_address_prefixlist_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd ospf6_routemap_rule_match_address_prefixlist_cmd = {
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_match_address_prefixlist_cmd = {
 	"ipv6 address prefix-list",
 	ospf6_routemap_rule_match_address_prefixlist,
 	ospf6_routemap_rule_match_address_prefixlist_compile,
@@ -1387,8 +1393,8 @@ struct route_map_rule_cmd ospf6_routemap_rule_match_address_prefixlist_cmd = {
 /* `match interface IFNAME' */
 /* Match function should return 1 if match is success else return
    zero. */
-static route_map_result_t
-ospf6_routemap_rule_match_interface(void *rule, struct prefix *prefix,
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_match_interface(void *rule, const struct prefix *prefix,
 				    route_map_object_t type, void *object)
 {
 	struct interface *ifp;
@@ -1419,16 +1425,18 @@ static void ospf6_routemap_rule_match_interface_free(void *rule)
 }
 
 /* Route map commands for interface matching. */
-struct route_map_rule_cmd ospf6_routemap_rule_match_interface_cmd = {
-	"interface", ospf6_routemap_rule_match_interface,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_match_interface_cmd = {
+	"interface",
+	ospf6_routemap_rule_match_interface,
 	ospf6_routemap_rule_match_interface_compile,
-	ospf6_routemap_rule_match_interface_free};
+	ospf6_routemap_rule_match_interface_free
+};
 
 /* Match function for matching route tags */
-static route_map_result_t ospf6_routemap_rule_match_tag(void *rule,
-							struct prefix *prefix,
-							route_map_object_t type,
-							void *object)
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_match_tag(void *rule, const struct prefix *p,
+			      route_map_object_t type, void *object)
 {
 	route_tag_t *tag = rule;
 	struct ospf6_route *route = object;
@@ -1440,13 +1448,16 @@ static route_map_result_t ospf6_routemap_rule_match_tag(void *rule,
 	return RMAP_NOMATCH;
 }
 
-static struct route_map_rule_cmd ospf6_routemap_rule_match_tag_cmd = {
-	"tag", ospf6_routemap_rule_match_tag, route_map_rule_tag_compile,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_match_tag_cmd = {
+	"tag",
+	ospf6_routemap_rule_match_tag,
+	route_map_rule_tag_compile,
 	route_map_rule_tag_free,
 };
 
-static route_map_result_t
-ospf6_routemap_rule_set_metric_type(void *rule, struct prefix *prefix,
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_set_metric_type(void *rule, const struct prefix *prefix,
 				    route_map_object_t type, void *object)
 {
 	char *metric_type = rule;
@@ -1475,14 +1486,16 @@ static void ospf6_routemap_rule_set_metric_type_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd ospf6_routemap_rule_set_metric_type_cmd = {
-	"metric-type", ospf6_routemap_rule_set_metric_type,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_set_metric_type_cmd = {
+	"metric-type",
+	ospf6_routemap_rule_set_metric_type,
 	ospf6_routemap_rule_set_metric_type_compile,
 	ospf6_routemap_rule_set_metric_type_free,
 };
 
-static route_map_result_t
-ospf6_routemap_rule_set_metric(void *rule, struct prefix *prefix,
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_set_metric(void *rule, const struct prefix *prefix,
 			       route_map_object_t type, void *object)
 {
 	char *metric = rule;
@@ -1510,14 +1523,16 @@ static void ospf6_routemap_rule_set_metric_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd ospf6_routemap_rule_set_metric_cmd = {
-	"metric", ospf6_routemap_rule_set_metric,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_set_metric_cmd = {
+	"metric",
+	ospf6_routemap_rule_set_metric,
 	ospf6_routemap_rule_set_metric_compile,
 	ospf6_routemap_rule_set_metric_free,
 };
 
-static route_map_result_t
-ospf6_routemap_rule_set_forwarding(void *rule, struct prefix *prefix,
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_set_forwarding(void *rule, const struct prefix *prefix,
 				   route_map_object_t type, void *object)
 {
 	char *forwarding = rule;
@@ -1548,16 +1563,17 @@ static void ospf6_routemap_rule_set_forwarding_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd ospf6_routemap_rule_set_forwarding_cmd = {
-	"forwarding-address", ospf6_routemap_rule_set_forwarding,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_set_forwarding_cmd = {
+	"forwarding-address",
+	ospf6_routemap_rule_set_forwarding,
 	ospf6_routemap_rule_set_forwarding_compile,
 	ospf6_routemap_rule_set_forwarding_free,
 };
 
-static route_map_result_t ospf6_routemap_rule_set_tag(void *rule,
-						      struct prefix *prefix,
-						      route_map_object_t type,
-						      void *object)
+static enum route_map_cmd_result_t
+ospf6_routemap_rule_set_tag(void *rule, const struct prefix *p,
+			    route_map_object_t type, void *object)
 {
 	route_tag_t *tag = rule;
 	struct ospf6_route *route = object;
@@ -1570,22 +1586,23 @@ static route_map_result_t ospf6_routemap_rule_set_tag(void *rule,
 	return RMAP_OKAY;
 }
 
-static struct route_map_rule_cmd ospf6_routemap_rule_set_tag_cmd = {
-	"tag", ospf6_routemap_rule_set_tag, route_map_rule_tag_compile,
+static const struct route_map_rule_cmd
+		ospf6_routemap_rule_set_tag_cmd = {
+	"tag",
+	ospf6_routemap_rule_set_tag,
+	route_map_rule_tag_compile,
 	route_map_rule_tag_free,
 };
 
-static int route_map_command_status(struct vty *vty, int ret)
+static int route_map_command_status(struct vty *vty, enum rmap_compile_rets ret)
 {
 	switch (ret) {
 	case RMAP_RULE_MISSING:
 		vty_out(vty, "OSPF6 Can't find rule.\n");
 		return CMD_WARNING_CONFIG_FAILED;
-		break;
 	case RMAP_COMPILE_ERROR:
 		vty_out(vty, "OSPF6 Argument is malformed.\n");
 		return CMD_WARNING_CONFIG_FAILED;
-		break;
 	case RMAP_COMPILE_SUCCESS:
 		break;
 	}
@@ -1604,8 +1621,10 @@ DEFUN (ospf6_routemap_set_metric_type,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, route_map_index);
 	int idx_external = 2;
-	int ret = route_map_add_set(route_map_index, "metric-type",
-				    argv[idx_external]->arg);
+	enum rmap_compile_rets ret = route_map_add_set(route_map_index,
+						       "metric-type",
+						       argv[idx_external]->arg);
+
 	return route_map_command_status(vty, ret);
 }
 
@@ -1621,7 +1640,9 @@ DEFUN (ospf6_routemap_no_set_metric_type,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, route_map_index);
 	char *ext = (argc == 4) ? argv[3]->text : NULL;
-	int ret = route_map_delete_set(route_map_index, "metric-type", ext);
+	enum rmap_compile_rets ret = route_map_delete_set(route_map_index,
+							  "metric-type", ext);
+
 	return route_map_command_status(vty, ret);
 }
 
@@ -1635,8 +1656,10 @@ DEFUN (ospf6_routemap_set_forwarding,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, route_map_index);
 	int idx_ipv6 = 2;
-	int ret = route_map_add_set(route_map_index, "forwarding-address",
-				    argv[idx_ipv6]->arg);
+	enum rmap_compile_rets ret = route_map_add_set(route_map_index,
+						       "forwarding-address",
+						       argv[idx_ipv6]->arg);
+
 	return route_map_command_status(vty, ret);
 }
 
@@ -1651,8 +1674,10 @@ DEFUN (ospf6_routemap_no_set_forwarding,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, route_map_index);
 	int idx_ipv6 = 3;
-	int ret = route_map_delete_set(route_map_index, "forwarding-address",
-				       argv[idx_ipv6]->arg);
+	enum rmap_compile_rets ret = route_map_delete_set(route_map_index,
+							  "forwarding-address",
+							  argv[idx_ipv6]->arg);
+
 	return route_map_command_status(vty, ret);
 }
 
@@ -1710,7 +1735,8 @@ static char *ospf6_as_external_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 			lsa->header);
 
 		if (pos == 0) {
-			ospf6_prefix_in6_addr(&in6, &external->prefix);
+			ospf6_prefix_in6_addr(&in6, external,
+					      &external->prefix);
 			prefix_length = external->prefix.prefix_length;
 		} else {
 			in6 = *((struct in6_addr
@@ -1827,7 +1853,7 @@ DEFUN (show_ipv6_ospf6_redistribute,
 	return CMD_SUCCESS;
 }
 
-struct ospf6_lsa_handler as_external_handler = {
+static struct ospf6_lsa_handler as_external_handler = {
 	.lh_type = OSPF6_LSTYPE_AS_EXTERNAL,
 	.lh_name = "AS-External",
 	.lh_short_name = "ASE",
@@ -1863,9 +1889,6 @@ void ospf6_asbr_redistribute_reset(void)
 void ospf6_asbr_terminate(void)
 {
 	/* Cleanup route maps */
-	route_map_add_hook(NULL);
-	route_map_delete_hook(NULL);
-	route_map_event_hook(NULL);
 	route_map_finish();
 }
 
@@ -1901,7 +1924,7 @@ int config_write_ospf6_debug_asbr(struct vty *vty)
 	return 0;
 }
 
-void install_element_ospf6_debug_asbr()
+void install_element_ospf6_debug_asbr(void)
 {
 	install_element(ENABLE_NODE, &debug_ospf6_asbr_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf6_asbr_cmd);
