@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -210,16 +209,16 @@ static oid ospfv3_oid[] = {OSPFv3MIB};
 static oid ospfv3_trap_oid[] = {OSPFv3MIB, 0};
 
 /* Hook functions. */
-static u_char *ospfv3GeneralGroup(struct variable *, oid *, size_t *, int,
+static uint8_t *ospfv3GeneralGroup(struct variable *, oid *, size_t *, int,
+				   size_t *, WriteMethod **);
+static uint8_t *ospfv3AreaEntry(struct variable *, oid *, size_t *, int,
+				size_t *, WriteMethod **);
+static uint8_t *ospfv3WwLsdbEntry(struct variable *, oid *, size_t *, int,
 				  size_t *, WriteMethod **);
-static u_char *ospfv3AreaEntry(struct variable *, oid *, size_t *, int,
+static uint8_t *ospfv3NbrEntry(struct variable *, oid *, size_t *, int,
 			       size_t *, WriteMethod **);
-static u_char *ospfv3WwLsdbEntry(struct variable *, oid *, size_t *, int,
-				 size_t *, WriteMethod **);
-static u_char *ospfv3NbrEntry(struct variable *, oid *, size_t *, int, size_t *,
+static uint8_t *ospfv3IfEntry(struct variable *, oid *, size_t *, int, size_t *,
 			      WriteMethod **);
-static u_char *ospfv3IfEntry(struct variable *, oid *, size_t *, int, size_t *,
-			     WriteMethod **);
 
 static struct variable ospfv3_variables[] = {
 	/* OSPF general variables */
@@ -632,12 +631,12 @@ static struct variable ospfv3_variables[] = {
 	 {1, 9, 1, 15}},
 };
 
-static u_char *ospfv3GeneralGroup(struct variable *v, oid *name, size_t *length,
-				  int exact, size_t *var_len,
-				  WriteMethod **write_method)
+static uint8_t *ospfv3GeneralGroup(struct variable *v, oid *name,
+				   size_t *length, int exact, size_t *var_len,
+				   WriteMethod **write_method)
 {
-	u_int16_t sum;
-	u_int32_t count;
+	uint16_t sum;
+	uint32_t count;
 	struct ospf6_lsa *lsa = NULL;
 
 	/* Check whether the instance identifier is valid */
@@ -679,8 +678,8 @@ static u_char *ospfv3GeneralGroup(struct variable *v, oid *name, size_t *length,
 		return SNMP_INTEGER(0);
 	case OSPFv3ASSCOPELSACHECKSUMSUM:
 		if (ospf6) {
-			for (sum = 0, lsa = ospf6_lsdb_head(ospf6->lsdb); lsa;
-			     lsa = ospf6_lsdb_next(lsa))
+			sum = 0;
+			for (ALL_LSDB(ospf6->lsdb, lsa))
 				sum += ntohs(lsa->header->checksum);
 			return SNMP_INTEGER(sum);
 		}
@@ -693,12 +692,10 @@ static u_char *ospfv3GeneralGroup(struct variable *v, oid *name, size_t *length,
 			0); /* Don't know where to get this value... */
 	case OSPFv3EXTLSACOUNT:
 		if (ospf6) {
-			for (count = 0,
-			    lsa = ospf6_lsdb_type_head(
-				    htons(OSPF6_LSTYPE_AS_EXTERNAL),
-				    ospf6->lsdb);
-			     lsa; lsa = ospf6_lsdb_type_next(
-					  htons(OSPF6_LSTYPE_AS_EXTERNAL), lsa))
+			count = 0;
+			for (ALL_LSDB_TYPED(ospf6->lsdb,
+					    htons(OSPF6_LSTYPE_AS_EXTERNAL),
+					    lsa))
 				count += 1;
 			return SNMP_INTEGER(count);
 		}
@@ -712,8 +709,8 @@ static u_char *ospfv3GeneralGroup(struct variable *v, oid *name, size_t *length,
 	case OSPFv3REFERENCEBANDWIDTH:
 		if (ospf6)
 			return SNMP_INTEGER(ospf6->ref_bandwidth);
-		/* Otherwise, like for "not implemented". */
-		/* fallthru */
+	/* Otherwise, like for "not implemented". */
+	/* fallthru */
 	case OSPFv3RESTARTSUPPORT:
 	case OSPFv3RESTARTINTERVAL:
 	case OSPFv3RESTARTSTRICTLSACHECKING:
@@ -731,15 +728,15 @@ static u_char *ospfv3GeneralGroup(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
-			       int exact, size_t *var_len,
-			       WriteMethod **write_method)
+static uint8_t *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
+				int exact, size_t *var_len,
+				WriteMethod **write_method)
 {
 	struct ospf6_area *oa, *area = NULL;
 	struct ospf6_lsa *lsa = NULL;
-	u_int32_t area_id = 0;
-	u_int32_t count;
-	u_int16_t sum;
+	uint32_t area_id = 0;
+	uint32_t count;
+	uint16_t sum;
 	struct listnode *node;
 	unsigned int len;
 	char a[16];
@@ -761,7 +758,7 @@ static u_char *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
 
 	inet_ntop(AF_INET, &area_id, a, sizeof(a));
 	zlog_debug("SNMP access by area: %s, exact=%d len=%d length=%lu", a,
-		   exact, len, (u_long)*length);
+		   exact, len, (unsigned long)*length);
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, node, oa)) {
 		if (area == NULL) {
@@ -782,7 +779,7 @@ static u_char *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
 
 	inet_ntop(AF_INET, &area->area_id, a, sizeof(a));
 	zlog_debug("SNMP found area: %s, exact=%d len=%d length=%lu", a, exact,
-		   len, (u_long)*length);
+		   len, (unsigned long)*length);
 
 	switch (v->magic) {
 	case OSPFv3IMPORTASEXTERN:
@@ -810,8 +807,8 @@ static u_char *ospfv3AreaEntry(struct variable *v, oid *name, size_t *length,
 	case OSPFv3AREASCOPELSACOUNT:
 		return SNMP_INTEGER(area->lsdb->count);
 	case OSPFv3AREASCOPELSACKSUMSUM:
-		for (sum = 0, lsa = ospf6_lsdb_head(area->lsdb); lsa;
-		     lsa = ospf6_lsdb_next(lsa))
+		sum = 0;
+		for (ALL_LSDB(area->lsdb, lsa))
 			sum += ntohs(lsa->header->checksum);
 		return SNMP_INTEGER(sum);
 	case OSPFv3AREASUMMARY:
@@ -836,14 +833,15 @@ static int if_icmp_func(struct interface *ifp1, struct interface *ifp2)
 	return (ifp1->ifindex - ifp2->ifindex);
 }
 
-static u_char *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
-				 int exact, size_t *var_len,
-				 WriteMethod **write_method)
+static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
+				  int exact, size_t *var_len,
+				  WriteMethod **write_method)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct ospf6_lsa *lsa = NULL;
 	ifindex_t ifindex;
 	uint32_t area_id, id, instid, adv_router;
-	u_int16_t type;
+	uint16_t type;
 	int len;
 	oid *offset;
 	int offsetlen;
@@ -916,7 +914,7 @@ static u_char *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	if (len)
 		id = htonl(*offset);
 	offset += len;
-	offsetlen -= len;
+	// offsetlen -= len;  // Add back in if we need it again
 
 	if (exact) {
 		if (v->magic & OSPFv3WWASTABLE) {
@@ -958,8 +956,7 @@ static u_char *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 			if (!ifslist)
 				return NULL;
 			ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
-			for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node,
-						  iif))
+			FOR_ALL_INTERFACES (vrf, iif)
 				listnode_add_sort(ifslist, iif);
 
 			for (ALL_LIST_ELEMENTS_RO(ifslist, node, iif)) {
@@ -1020,18 +1017,14 @@ static u_char *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic & OSPFv3WWCOLUMN) {
 	case OSPFv3WWLSDBSEQUENCE:
 		return SNMP_INTEGER(ntohl(lsa->header->seqnum));
-		break;
 	case OSPFv3WWLSDBAGE:
 		ospf6_lsa_age_current(lsa);
 		return SNMP_INTEGER(ntohs(lsa->header->age));
-		break;
 	case OSPFv3WWLSDBCHECKSUM:
 		return SNMP_INTEGER(ntohs(lsa->header->checksum));
-		break;
 	case OSPFv3WWLSDBADVERTISEMENT:
 		*var_len = ntohs(lsa->header->length);
-		return (u_char *)lsa->header;
-		break;
+		return (uint8_t *)lsa->header;
 	case OSPFv3WWLSDBTYPEKNOWN:
 		return SNMP_INTEGER(OSPF6_LSA_IS_KNOWN(lsa->header->type)
 					    ? SNMP_TRUE
@@ -1041,10 +1034,11 @@ static u_char *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
-			     int exact, size_t *var_len,
-			     WriteMethod **write_method)
+static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
+			      int exact, size_t *var_len,
+			      WriteMethod **write_method)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	ifindex_t ifindex = 0;
 	unsigned int instid = 0;
 	struct ospf6_interface *oi = NULL;
@@ -1054,7 +1048,7 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	struct list *ifslist;
 	oid *offset;
 	int offsetlen, len;
-	u_int32_t sum;
+	uint32_t sum;
 
 	if (smux_header_table(v, name, length, exact, var_len, write_method)
 	    == MATCH_FAILED)
@@ -1082,8 +1076,8 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	len = (offsetlen < 1 ? 0 : 1);
 	if (len)
 		instid = *offset;
-	offset += len;
-	offsetlen -= len;
+	// offset += len; // Add back in if we ever start using again
+	// offsetlen -= len;
 
 	if (exact) {
 		oi = ospf6_interface_lookup_by_ifindex(ifindex);
@@ -1095,7 +1089,7 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 		if (!ifslist)
 			return NULL;
 		ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), i, iif))
+		FOR_ALL_INTERFACES (vrf, iif)
 			listnode_add_sort(ifslist, iif);
 
 		for (ALL_LIST_ELEMENTS_RO(ifslist, i, iif)) {
@@ -1132,9 +1126,9 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(ntohl(oi->area->area_id));
 		break;
 	case OSPFv3IFTYPE:
-		if (if_is_broadcast(oi->interface))
+		if (oi->type == OSPF_IFTYPE_BROADCAST)
 			return SNMP_INTEGER(1);
-		else if (if_is_pointopoint(oi->interface))
+		else if (oi->type == OSPF_IFTYPE_POINTOPOINT)
 			return SNMP_INTEGER(3);
 		else
 			break; /* Unknown, don't put anything */
@@ -1172,8 +1166,8 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	case OSPFv3IFLINKSCOPELSACOUNT:
 		return SNMP_INTEGER(oi->lsdb->count);
 	case OSPFv3IFLINKLSACKSUMSUM:
-		for (sum = 0, lsa = ospf6_lsdb_head(oi->lsdb); lsa;
-		     lsa = ospf6_lsdb_next(lsa))
+		sum = 0;
+		for (ALL_LSDB(oi->lsdb, lsa))
 			sum += ntohs(lsa->header->checksum);
 		return SNMP_INTEGER(sum);
 	case OSPFv3IFDEMANDNBRPROBE:
@@ -1193,10 +1187,11 @@ static u_char *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
-			      int exact, size_t *var_len,
-			      WriteMethod **write_method)
+static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
+			       int exact, size_t *var_len,
+			       WriteMethod **write_method)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	ifindex_t ifindex = 0;
 	unsigned int instid, rtrid;
 	struct ospf6_interface *oi = NULL;
@@ -1242,8 +1237,8 @@ static u_char *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 	len = (offsetlen < 1 ? 0 : 1);
 	if (len)
 		rtrid = htonl(*offset);
-	offset += len;
-	offsetlen -= len;
+	// offset += len; // Add back in if we ever start looking at data
+	// offsetlen -= len;
 
 	if (exact) {
 		oi = ospf6_interface_lookup_by_ifindex(ifindex);
@@ -1256,7 +1251,7 @@ static u_char *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 		if (!ifslist)
 			return NULL;
 		ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), i, iif))
+		FOR_ALL_INTERFACES (vrf, iif)
 			listnode_add_sort(ifslist, iif);
 
 		for (ALL_LIST_ELEMENTS_RO(ifslist, i, iif)) {
@@ -1302,7 +1297,7 @@ static u_char *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 		return SNMP_INTEGER(2); /* IPv6 only */
 	case OSPFv3NBRADDRESS:
 		*var_len = sizeof(struct in6_addr);
-		return (u_char *)&on->linklocal_addr;
+		return (uint8_t *)&on->linklocal_addr;
 	case OSPFv3NBROPTIONS:
 		return SNMP_INTEGER(on->options[2]);
 	case OSPFv3NBRPRIORITY:
@@ -1359,13 +1354,10 @@ static int ospf6TrapNbrStateChange(struct ospf6_neighbor *on, int next_state,
 	index[1] = on->ospf6_if->instance_id;
 	index[2] = ntohl(on->router_id);
 
-	smux_trap(ospfv3_variables,
-		  sizeof ospfv3_variables / sizeof(struct variable),
-		  ospfv3_trap_oid, sizeof ospfv3_trap_oid / sizeof(oid),
-		  ospfv3_oid, sizeof ospfv3_oid / sizeof(oid), index, 3,
-		  ospf6NbrTrapList,
-		  sizeof ospf6NbrTrapList / sizeof(struct trap_object),
-		  NBRSTATECHANGE);
+	smux_trap(ospfv3_variables, array_size(ospfv3_variables),
+		  ospfv3_trap_oid, array_size(ospfv3_trap_oid), ospfv3_oid,
+		  sizeof(ospfv3_oid) / sizeof(oid), index, 3, ospf6NbrTrapList,
+		  array_size(ospf6NbrTrapList), NBRSTATECHANGE);
 	return 0;
 }
 
@@ -1384,13 +1376,10 @@ static int ospf6TrapIfStateChange(struct ospf6_interface *oi, int next_state,
 	index[0] = oi->interface->ifindex;
 	index[1] = oi->instance_id;
 
-	smux_trap(ospfv3_variables,
-		  sizeof ospfv3_variables / sizeof(struct variable),
-		  ospfv3_trap_oid, sizeof ospfv3_trap_oid / sizeof(oid),
-		  ospfv3_oid, sizeof ospfv3_oid / sizeof(oid), index, 2,
-		  ospf6IfTrapList,
-		  sizeof ospf6IfTrapList / sizeof(struct trap_object),
-		  IFSTATECHANGE);
+	smux_trap(ospfv3_variables, array_size(ospfv3_variables),
+		  ospfv3_trap_oid, array_size(ospfv3_trap_oid), ospfv3_oid,
+		  sizeof(ospfv3_oid) / sizeof(oid), index, 2, ospf6IfTrapList,
+		  array_size(ospf6IfTrapList), IFSTATECHANGE);
 	return 0;
 }
 
