@@ -27,9 +27,9 @@
 #include "command.h"
 #include "vty.h"
 #include "prefix.h"
-#include "pqueue.h"
 #include "linklist.h"
 #include "thread.h"
+#include "lib_errors.h"
 
 #include "ospf6_lsa.h"
 #include "ospf6_lsdb.h"
@@ -75,16 +75,18 @@ static unsigned int ospf6_spf_get_ifindex_from_nh(struct ospf6_vertex *v)
 	return 0;
 }
 
-static int ospf6_vertex_cmp(void *a, void *b)
+static int ospf6_vertex_cmp(const struct ospf6_vertex *va,
+		const struct ospf6_vertex *vb)
 {
-	struct ospf6_vertex *va = (struct ospf6_vertex *)a;
-	struct ospf6_vertex *vb = (struct ospf6_vertex *)b;
-
 	/* ascending order */
 	if (va->cost != vb->cost)
 		return (va->cost - vb->cost);
-	return (va->hops - vb->hops);
+	if (va->hops != vb->hops)
+		return (va->hops - vb->hops);
+	return 0;
 }
+DECLARE_SKIPLIST_NONUNIQ(vertex_pqueue, struct ospf6_vertex, pqi,
+		ospf6_vertex_cmp)
 
 static int ospf6_vertex_id_cmp(void *a, void *b)
 {
@@ -106,8 +108,7 @@ static struct ospf6_vertex *ospf6_vertex_create(struct ospf6_lsa *lsa)
 {
 	struct ospf6_vertex *v;
 
-	v = (struct ospf6_vertex *)XMALLOC(MTYPE_OSPF6_VERTEX,
-					   sizeof(struct ospf6_vertex));
+	v = XMALLOC(MTYPE_OSPF6_VERTEX, sizeof(struct ospf6_vertex));
 
 	/* type */
 	if (ntohs(lsa->header->type) == OSPF6_LSTYPE_ROUTER) {
@@ -157,8 +158,8 @@ static struct ospf6_vertex *ospf6_vertex_create(struct ospf6_lsa *lsa)
 
 static void ospf6_vertex_delete(struct ospf6_vertex *v)
 {
-	list_delete_and_null(&v->nh_list);
-	list_delete_and_null(&v->child_list);
+	list_delete(&v->nh_list);
+	list_delete(&v->child_list);
 	XFREE(MTYPE_OSPF6_VERTEX, v);
 }
 
@@ -272,7 +273,8 @@ static void ospf6_nexthop_calc(struct ospf6_vertex *w, struct ospf6_vertex *v,
 	ifindex = (VERTEX_IS_TYPE(NETWORK, v) ? ospf6_spf_get_ifindex_from_nh(v)
 					      : ROUTER_LSDESC_GET_IFID(lsdesc));
 	if (ifindex == 0) {
-		zlog_err("No nexthop ifindex at vertex %s", v->name);
+		flog_err(EC_LIB_DEVELOPMENT, "No nexthop ifindex at vertex %s",
+			 v->name);
 		return;
 	}
 
@@ -349,7 +351,7 @@ static int ospf6_spf_install(struct ospf6_vertex *v,
 			if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
 				zlog_debug(
 					"%s: V lsa %s id %u, route id %u are different",
-					__PRETTY_FUNCTION__, v->lsa->name,
+					__func__, v->lsa->name,
 					ntohl(v->lsa->header->id),
 					ntohl(route->path.origin.id));
 			}
@@ -433,7 +435,7 @@ void ospf6_spf_table_finish(struct ospf6_route_table *result_table)
 	}
 }
 
-static const char *ospf6_spf_reason_str[] = {
+static const char *const ospf6_spf_reason_str[] = {
 	"R+", "R-", "N+", "N-", "L+", "L-", "R*", "N*",
 };
 
@@ -460,7 +462,7 @@ void ospf6_spf_calculation(uint32_t router_id,
 			   struct ospf6_route_table *result_table,
 			   struct ospf6_area *oa)
 {
-	struct pqueue *candidate_list;
+	struct vertex_pqueue_head candidate_list;
 	struct ospf6_vertex *root, *v, *w;
 	int size;
 	caddr_t lsdesc;
@@ -474,14 +476,13 @@ void ospf6_spf_calculation(uint32_t router_id,
 	lsa = ospf6_create_single_router_lsa(oa, oa->lsdb_self, router_id);
 	if (lsa == NULL) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
-			zlog_debug("%s: No router LSA for area %s\n", __func__,
+			zlog_debug("%s: No router LSA for area %s", __func__,
 				   oa->name);
 		return;
 	}
 
 	/* initialize */
-	candidate_list = pqueue_create();
-	candidate_list->cmp = ospf6_vertex_cmp;
+	vertex_pqueue_init(&candidate_list);
 
 	root = ospf6_vertex_create(lsa);
 	root->area = oa;
@@ -491,13 +492,10 @@ void ospf6_spf_calculation(uint32_t router_id,
 	inet_pton(AF_INET6, "::1", &address);
 
 	/* Actually insert root to the candidate-list as the only candidate */
-	pqueue_enqueue(root, candidate_list);
+	vertex_pqueue_add(&candidate_list, root);
 
 	/* Iterate until candidate-list becomes empty */
-	while (candidate_list->size) {
-		/* get closest candidate from priority queue */
-		v = pqueue_dequeue(candidate_list);
-
+	while ((v = vertex_pqueue_pop(&candidate_list))) {
 		/* installing may result in merging or rejecting of the vertex
 		 */
 		if (ospf6_spf_install(v, result_table) < 0)
@@ -556,12 +554,11 @@ void ospf6_spf_calculation(uint32_t router_id,
 				zlog_debug(
 					"  New candidate: %s hops %d cost %d",
 					w->name, w->hops, w->cost);
-			pqueue_enqueue(w, candidate_list);
+			vertex_pqueue_add(&candidate_list, w);
 		}
 	}
 
-
-	pqueue_delete(candidate_list);
+	//vertex_pqueue_fini(&candidate_list);
 
 	ospf6_remove_temp_router_lsa(oa);
 
@@ -677,6 +674,10 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 {
 	unsigned long delay, elapsed, ht;
 
+	/* OSPF instance does not exist. */
+	if (ospf6 == NULL)
+		return;
+
 	ospf6_set_spf_reason(ospf6, reason);
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME)) {
@@ -686,10 +687,6 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 			   rbuf);
 	}
 
-	/* OSPF instance does not exist. */
-	if (ospf6 == NULL)
-		return;
-
 	/* SPF calculation timer is already scheduled. */
 	if (ospf6->t_spf_calc) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME))
@@ -992,36 +989,29 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 			rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
 			continue;
 		}
-		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		lsa_header = rtr_lsa->header;
 		total_lsa_length += (ntohs(lsa_header->length) - lsa_length);
 		num_lsa++;
 		rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
 	}
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
-		zlog_debug("%s: adv_router %s num_lsa %u to convert.",
-			   __PRETTY_FUNCTION__, ifbuf, num_lsa);
+		zlog_debug("%s: adv_router %s num_lsa %u to convert.", __func__,
+			   ifbuf, num_lsa);
 	if (num_lsa == 1)
 		return lsa;
 
 	if (num_lsa == 0) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
 			zlog_debug("%s: adv_router %s not found in LSDB.",
-				   __PRETTY_FUNCTION__, ifbuf);
+				   __func__, ifbuf);
 		return NULL;
 	}
 
 	/* Allocate memory for this LSA */
 	new_header = XMALLOC(MTYPE_OSPF6_LSA_HEADER, total_lsa_length);
-	if (!new_header)
-		return NULL;
 
 	/* LSA information structure */
-	lsa = (struct ospf6_lsa *)XCALLOC(MTYPE_OSPF6_LSA,
-					  sizeof(struct ospf6_lsa));
-	if (!lsa) {
-		free(new_header);
-		return NULL;
-	}
+	lsa = XCALLOC(MTYPE_OSPF6_LSA, sizeof(struct ospf6_lsa));
 
 	lsa->header = (struct ospf6_lsa_header *)new_header;
 
@@ -1037,7 +1027,7 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 	assert(rtr_lsa);
 	if (!OSPF6_LSA_IS_MAXAGE(rtr_lsa)) {
 		/* Append first Link State ID LSA */
-		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		lsa_header = rtr_lsa->header;
 		memcpy(new_header, lsa_header, ntohs(lsa_header->length));
 		/* Assign new lsa length as aggregated length. */
 		((struct ospf6_lsa_header *)new_header)->length =
@@ -1062,12 +1052,12 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 			inet_ntop(AF_INET, &interface_id, ifbuf, sizeof(ifbuf));
 			zlog_debug(
 				"%s: Next Router LSA %s to aggreat with len %u interface_id %s",
-				__PRETTY_FUNCTION__, rtr_lsa->name,
+				__func__, rtr_lsa->name,
 				ntohs(lsa_header->length), ifbuf);
 		}
 
 		/* Append Next Link State ID LSA */
-		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		lsa_header = rtr_lsa->header;
 		memcpy(new_header, (OSPF6_LSA_HEADER_END(rtr_lsa->header) + 4),
 		       (ntohs(lsa_header->length) - lsa_length));
 		new_header += (ntohs(lsa_header->length) - lsa_length);
@@ -1084,9 +1074,9 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
 		zlog_debug("%s: LSA %s id %u type 0%x len %u num_lsa %u",
-			   __PRETTY_FUNCTION__, lsa->name,
-			   ntohl(lsa->header->id), ntohs(lsa->header->type),
-			   ntohs(lsa->header->length), num_lsa);
+			   __func__, lsa->name, ntohl(lsa->header->id),
+			   ntohs(lsa->header->type), ntohs(lsa->header->length),
+			   num_lsa);
 
 	return lsa;
 }
@@ -1099,7 +1089,7 @@ void ospf6_remove_temp_router_lsa(struct ospf6_area *area)
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
 			zlog_debug(
 				"%s Remove LSA %s lsa->lock %u lsdb count %u",
-				__PRETTY_FUNCTION__, lsa->name, lsa->lock,
+				__func__, lsa->name, lsa->lock,
 				area->temp_router_lsa_lsdb->count);
 		ospf6_lsdb_remove(lsa, area->temp_router_lsa_lsdb);
 	}
