@@ -69,8 +69,12 @@ struct route_node *route_prev(struct route_node *node)
 	return prev;
 }
 
+static int config_write_ospf6_debug(struct vty *vty);
 static struct cmd_node debug_node = {
-	DEBUG_NODE, "", 1 /* VTYSH */
+	.name = "debug",
+	.node = DEBUG_NODE,
+	.prompt = "",
+	.config_write = config_write_ospf6_debug,
 };
 
 static int config_write_ospf6_debug(struct vty *vty)
@@ -97,7 +101,7 @@ DEFUN_NOSH (show_debugging_ospf6,
 	    DEBUG_STR
 	    OSPF6_STR)
 {
-	vty_out(vty, "OSPF6 debugging status:");
+	vty_out(vty, "OSPF6 debugging status:\n");
 
 	config_write_ospf6_debug(vty);
 
@@ -1216,7 +1220,7 @@ void ospf6_init(void)
 	prefix_list_delete_hook(ospf6_plist_del);
 
 	ospf6_bfd_init();
-	install_node(&debug_node, config_write_ospf6_debug);
+	install_node(&debug_node);
 
 	install_element_ospf6_debug_message();
 	install_element_ospf6_debug_lsa();
