@@ -49,6 +49,7 @@
 #include "ospfd/ospf_ase.h"
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_dump.h"
+#include "ospfd/ospf_errors.h"
 
 static struct ospf_area_range *ospf_area_range_new(struct prefix_ipv4 *p)
 {
@@ -195,6 +196,8 @@ int ospf_area_range_set(struct ospf *ospf, struct in_addr area_id,
 
 	range = ospf_area_range_lookup(area, p);
 	if (range != NULL) {
+		if (!CHECK_FLAG(advertise, OSPF_AREA_RANGE_ADVERTISE))
+			range->cost_config = OSPF_AREA_RANGE_COST_UNSPEC;
 		if ((CHECK_FLAG(range->flags, OSPF_AREA_RANGE_ADVERTISE)
 		     && !CHECK_FLAG(advertise, OSPF_AREA_RANGE_ADVERTISE))
 		    || (!CHECK_FLAG(range->flags, OSPF_AREA_RANGE_ADVERTISE)
@@ -208,8 +211,10 @@ int ospf_area_range_set(struct ospf *ospf, struct in_addr area_id,
 
 	if (CHECK_FLAG(advertise, OSPF_AREA_RANGE_ADVERTISE))
 		SET_FLAG(range->flags, OSPF_AREA_RANGE_ADVERTISE);
-	else
+	else {
 		UNSET_FLAG(range->flags, OSPF_AREA_RANGE_ADVERTISE);
+		range->cost_config = OSPF_AREA_RANGE_COST_UNSPEC;
+	}
 
 	return 1;
 }
@@ -305,7 +310,7 @@ int ospf_area_range_substitute_unset(struct ospf *ospf, struct in_addr area_id,
 			ospf_schedule_abr_task(ospf);
 
 	UNSET_FLAG(range->flags, OSPF_AREA_RANGE_SUBSTITUTE);
-	range->subst_addr.s_addr = 0;
+	range->subst_addr.s_addr = INADDR_ANY;
 	range->subst_masklen = 0;
 
 	return 1;
@@ -665,8 +670,7 @@ static int ospf_abr_translate_nssa(struct ospf_area *area, struct ospf_lsa *lsa)
 		 * originate translated LSA
 		 */
 
-		if ((new = ospf_translated_nssa_originate(area->ospf, lsa))
-		    == NULL) {
+		if (ospf_translated_nssa_originate(area->ospf, lsa) == NULL) {
 			if (IS_DEBUG_OSPF_NSSA)
 				zlog_debug(
 					"ospf_abr_translate_nssa(): Could not translate "
@@ -704,8 +708,7 @@ void ospf_abr_announce_network_to_area(struct prefix_ipv4 *p, uint32_t cost,
 	else
 		full_cost = cost;
 
-	old = ospf_lsa_lookup_by_prefix(area->lsdb, OSPF_SUMMARY_LSA,
-					(struct prefix_ipv4 *)p,
+	old = ospf_lsa_lookup_by_prefix(area->lsdb, OSPF_SUMMARY_LSA, p,
 					area->ospf->router_id);
 	if (old) {
 		if (IS_DEBUG_OSPF_EVENT)
@@ -742,7 +745,8 @@ void ospf_abr_announce_network_to_area(struct prefix_ipv4 *p, uint32_t cost,
 
 				prefix2str((struct prefix *)p, buf,
 					   sizeof(buf));
-				zlog_warn("%s: Could not refresh %s to %s",
+				flog_warn(EC_OSPF_LSA_MISSING,
+					  "%s: Could not refresh %s to %s",
 					  __func__, buf,
 					  inet_ntoa(area->area_id));
 				return;
@@ -756,15 +760,15 @@ void ospf_abr_announce_network_to_area(struct prefix_ipv4 *p, uint32_t cost,
 			zlog_debug(
 				"ospf_abr_announce_network_to_area(): "
 				"creating new summary");
-		lsa = ospf_summary_lsa_originate((struct prefix_ipv4 *)p,
-						 full_cost, area);
+		lsa = ospf_summary_lsa_originate(p, full_cost, area);
 		/* This will flood through area. */
 
 		if (!lsa) {
 			char buf[PREFIX2STR_BUFFER];
 
 			prefix2str((struct prefix *)p, buf, sizeof(buf));
-			zlog_warn("%s: Could not originate %s to %s", __func__,
+			flog_warn(EC_OSPF_LSA_MISSING,
+				  "%s: Could not originate %s to %s", __func__,
 				  buf, inet_ntoa(area->area_id));
 			return;
 		}
@@ -1132,7 +1136,8 @@ static void ospf_abr_announce_rtr_to_area(struct prefix_ipv4 *p, uint32_t cost,
 			char buf[PREFIX2STR_BUFFER];
 
 			prefix2str((struct prefix *)p, buf, sizeof(buf));
-			zlog_warn("%s: Could not refresh/originate %s to %s",
+			flog_warn(EC_OSPF_LSA_MISSING,
+				  "%s: Could not refresh/originate %s to %s",
 				  __func__, buf, inet_ntoa(area->area_id));
 			return;
 		}
