@@ -73,34 +73,13 @@ void ospf_external_route_remove(struct ospf *ospf, struct prefix_ipv4 *p)
 		  p->prefixlen);
 }
 
-/* Lookup external route. */
-struct ospf_route *ospf_external_route_lookup(struct ospf *ospf,
-					      struct prefix_ipv4 *p)
-{
-	struct route_node *rn;
-
-	rn = route_node_lookup(ospf->old_external_route, (struct prefix *)p);
-	if (rn) {
-		route_unlock_node(rn);
-		if (rn->info)
-			return rn->info;
-	}
-
-	zlog_warn("Route[%s/%d]: lookup, no such prefix", inet_ntoa(p->prefix),
-		  p->prefixlen);
-
-	return NULL;
-}
-
-
 /* Add an External info for AS-external-LSA. */
 struct external_info *ospf_external_info_new(uint8_t type,
 					     unsigned short instance)
 {
 	struct external_info *new;
 
-	new = (struct external_info *)XCALLOC(MTYPE_OSPF_EXTERNAL_INFO,
-					      sizeof(struct external_info));
+	new = XCALLOC(MTYPE_OSPF_EXTERNAL_INFO, sizeof(struct external_info));
 	new->type = type;
 	new->instance = instance;
 
@@ -155,13 +134,13 @@ ospf_external_info_add(struct ospf *ospf, uint8_t type, unsigned short instance,
 
 			inet_ntop(AF_INET, (void *)&nexthop.s_addr, inetbuf,
 				  INET6_BUFSIZ);
-			zlog_warn(
-				"Redistribute[%s][%d][%u]: %s/%d discarding old info with NH %s.",
-				ospf_redist_string(type), instance,
-				ospf->vrf_id, inet_ntoa(p.prefix), p.prefixlen,
-				inetbuf);
+			if (IS_DEBUG_OSPF(lsa, LSA_GENERATE))
+				zlog_debug(
+					"Redistribute[%s][%d][%u]: %s/%d discarding old info with NH %s.",
+					ospf_redist_string(type), instance,
+					ospf->vrf_id, inet_ntoa(p.prefix),
+					p.prefixlen, inetbuf);
 			XFREE(MTYPE_OSPF_EXTERNAL_INFO, rn->info);
-			rn->info = NULL;
 		}
 
 	/* Create new External info instance. */
@@ -258,20 +237,23 @@ struct ospf_lsa *ospf_external_info_find_lsa(struct ospf *ospf,
 /* Update ASBR status. */
 void ospf_asbr_status_update(struct ospf *ospf, uint8_t status)
 {
-	zlog_info("ASBR[Status:%d]: Update", status);
+	zlog_info("ASBR[%s:Status:%d]: Update",
+		  ospf_get_name(ospf), status);
 
 	/* ASBR on. */
 	if (status) {
 		/* Already ASBR. */
 		if (IS_OSPF_ASBR(ospf)) {
-			zlog_info("ASBR[Status:%d]: Already ASBR", status);
+			zlog_info("ASBR[%s:Status:%d]: Already ASBR",
+				  ospf_get_name(ospf), status);
 			return;
 		}
 		SET_FLAG(ospf->flags, OSPF_FLAG_ASBR);
 	} else {
 		/* Already non ASBR. */
 		if (!IS_OSPF_ASBR(ospf)) {
-			zlog_info("ASBR[Status:%d]: Already non ASBR", status);
+			zlog_info("ASBR[%s:Status:%d]: Already non ASBR",
+				  ospf_get_name(ospf), status);
 			return;
 		}
 		UNSET_FLAG(ospf->flags, OSPF_FLAG_ASBR);
