@@ -77,7 +77,7 @@ struct ospf_route *ospf_find_asbr_route(struct ospf *ospf,
 
 	/* If none is found -- look through all. */
 	if (listcount(chosen) == 0) {
-		list_delete_and_null(&chosen);
+		list_delete(&chosen);
 		chosen = rn->info;
 	}
 
@@ -98,7 +98,7 @@ struct ospf_route *ospf_find_asbr_route(struct ospf *ospf,
 		}
 
 	if (chosen != rn->info)
-		list_delete_and_null(&chosen);
+		list_delete(&chosen);
 
 	return best;
 }
@@ -190,7 +190,7 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
 	zlog_debug ("ospf_ase_calculate(): Originating router is not an ASBR");
       return NULL;
     }
-   
+
   if (al->e[0].fwd_addr.s_addr != 0)
     {
       if (IS_DEBUG_OSPF (lsa, LSA))
@@ -215,7 +215,7 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
       asbr.prefixlen = IPV4_MAX_BITLEN;
 
       rn = route_node_match (rt_network, (struct prefix *) &asbr);
-   
+
       if (rn == NULL)
 	{
 	  if (IS_DEBUG_OSPF (lsa, LSA))
@@ -288,6 +288,7 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 	struct prefix_ipv4 asbr, p;
 	struct route_node *rn;
 	struct ospf_route *new, * or ;
+	char buf1[INET_ADDRSTRLEN];
 	int ret;
 
 	assert(lsa);
@@ -304,10 +305,14 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 		return 0;
 	}
 
-	if (IS_DEBUG_OSPF(lsa, LSA))
+	if (IS_DEBUG_OSPF(lsa, LSA)) {
+		snprintf(buf1, sizeof(buf1), "%s",
+			 inet_ntoa(al->header.adv_router));
 		zlog_debug(
-			"Route[External]: Calculate AS-external-LSA to %s/%d",
-			inet_ntoa(al->header.id), ip_masklen(al->mask));
+			"Route[External]: Calculate AS-external-LSA to %s/%d adv_router %s",
+			inet_ntoa(al->header.id), ip_masklen(al->mask), buf1);
+	}
+
 	/* (1) If the cost specified by the LSA is LSInfinity, or if the
 	       LSA's LS age is equal to MaxAge, then examine the next LSA. */
 	if ((metric = GET_METRIC(al->e[0].metric)) >= OSPF_LS_INFINITY) {
@@ -365,7 +370,7 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 	       external-LSA.  This indicates the IP address to which
 	       packets for the destination should be forwarded. */
 
-	if (al->e[0].fwd_addr.s_addr == 0) {
+	if (al->e[0].fwd_addr.s_addr == INADDR_ANY) {
 		/* If the forwarding address is set to 0.0.0.0, packets should
 		   be sent to the ASBR itself. Among the multiple routing table
 		   entries for the ASBR, select the preferred entry as follows.
@@ -459,12 +464,13 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 
 	if (!rn || (or = rn->info) == NULL) {
 		if (IS_DEBUG_OSPF(lsa, LSA))
-			zlog_debug("Route[External]: Adding a new route %s/%d",
-				   inet_ntoa(p.prefix), p.prefixlen);
+			zlog_debug("Route[External]: Adding a new route %s/%d with paths %u",
+				   inet_ntoa(p.prefix), p.prefixlen,
+				   listcount(asbr_route->paths));
 
 		ospf_route_add(ospf->new_external_route, &p, new, asbr_route);
 
-		if (al->e[0].fwd_addr.s_addr)
+		if (al->e[0].fwd_addr.s_addr != INADDR_ANY)
 			ospf_ase_complete_direct_routes(new, al->e[0].fwd_addr);
 		return 0;
 	} else {
@@ -506,7 +512,7 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 				zlog_debug(
 					"Route[External]: New route is better");
 			ospf_route_subst(rn, new, asbr_route);
-			if (al->e[0].fwd_addr.s_addr)
+			if (al->e[0].fwd_addr.s_addr != INADDR_ANY)
 				ospf_ase_complete_direct_routes(
 					new, al->e[0].fwd_addr);
 			or = new;
@@ -524,7 +530,7 @@ int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 			if (IS_DEBUG_OSPF(lsa, LSA))
 				zlog_debug("Route[External]: Routes are equal");
 			ospf_route_copy_nexthops(or, asbr_route->paths);
-			if (al->e[0].fwd_addr.s_addr)
+			if (al->e[0].fwd_addr.s_addr != INADDR_ANY)
 				ospf_ase_complete_direct_routes(
 					or, al->e[0].fwd_addr);
 		}
@@ -543,7 +549,7 @@ static int ospf_ase_route_match_same(struct route_table *rt,
 				     struct ospf_route *newor)
 {
 	struct route_node *rn;
-	struct ospf_route * or ;
+	struct ospf_route *or;
 	struct ospf_path *op;
 	struct ospf_path *newop;
 	struct listnode *n1;
@@ -559,6 +565,9 @@ static int ospf_ase_route_match_same(struct route_table *rt,
 	route_unlock_node(rn);
 
 	or = rn->info;
+
+	assert(or);
+
 	if (or->path_type != newor->path_type)
 		return 0;
 
@@ -577,12 +586,14 @@ static int ospf_ase_route_match_same(struct route_table *rt,
 		return 0;
 	}
 
+	assert(or->paths);
+
 	if (or->paths->count != newor->paths->count)
 		return 0;
 
 	/* Check each path. */
 	for (n1 = listhead(or->paths), n2 = listhead(newor->paths); n1 && n2;
-	     n1 = listnextnode(n1), n2 = listnextnode(n2)) {
+	     n1 = listnextnode_unchecked(n1), n2 = listnextnode_unchecked(n2)) {
 		op = listgetdata(n1);
 		newop = listgetdata(n2);
 
@@ -680,7 +691,7 @@ static int ospf_ase_calculate_timer(struct thread *t)
 
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_info(
-				"SPF Processing Time(usecs): External Routes: %lld\n",
+				"SPF Processing Time(usecs): External Routes: %lld",
 				(stop_time.tv_sec - start_time.tv_sec)
 						* 1000000LL
 					+ (stop_time.tv_usec
@@ -764,7 +775,7 @@ void ospf_ase_external_lsas_finish(struct route_table *rt)
 		if ((lst = rn->info) != NULL) {
 			for (ALL_LIST_ELEMENTS(lst, node, nnode, lsa))
 				ospf_lsa_unlock(&lsa); /* external_lsas lst */
-			list_delete_and_null(&lst);
+			list_delete(&lst);
 		}
 
 	route_table_finish(rt);
