@@ -82,9 +82,8 @@ const char *ospf_area_name_string(struct ospf_area *area)
 		return "-";
 
 	area_id = ntohl(area->area_id.s_addr);
-	snprintf(buf, OSPF_AREA_STRING_MAXLEN, "%d.%d.%d.%d",
-		 (area_id >> 24) & 0xff, (area_id >> 16) & 0xff,
-		 (area_id >> 8) & 0xff, area_id & 0xff);
+	snprintf(buf, sizeof(buf), "%d.%d.%d.%d", (area_id >> 24) & 0xff,
+		 (area_id >> 16) & 0xff, (area_id >> 8) & 0xff, area_id & 0xff);
 	return buf;
 }
 
@@ -100,11 +99,11 @@ const char *ospf_area_desc_string(struct ospf_area *area)
 	type = area->external_routing;
 	switch (type) {
 	case OSPF_AREA_NSSA:
-		snprintf(buf, OSPF_AREA_DESC_STRING_MAXLEN, "%s [NSSA]",
+		snprintf(buf, sizeof(buf), "%s [NSSA]",
 			 ospf_area_name_string(area));
 		break;
 	case OSPF_AREA_STUB:
-		snprintf(buf, OSPF_AREA_DESC_STRING_MAXLEN, "%s [Stub]",
+		snprintf(buf, sizeof(buf), "%s [Stub]",
 			 ospf_area_name_string(area));
 		break;
 	default:
@@ -127,7 +126,7 @@ const char *ospf_if_name_string(struct ospf_interface *oi)
 		return oi->ifp->name;
 
 	ifaddr = ntohl(oi->address->u.prefix4.s_addr);
-	snprintf(buf, OSPF_IF_STRING_MAXLEN, "%s:%d.%d.%d.%d", oi->ifp->name,
+	snprintf(buf, sizeof(buf), "%s:%d.%d.%d.%d", oi->ifp->name,
 		 (ifaddr >> 24) & 0xff, (ifaddr >> 16) & 0xff,
 		 (ifaddr >> 8) & 0xff, ifaddr & 0xff);
 	return buf;
@@ -146,8 +145,6 @@ void ospf_nbr_state_message(struct ospf_neighbor *nbr, char *buf, size_t size)
 	else
 		state = ISM_DROther;
 
-	memset(buf, 0, size);
-
 	snprintf(buf, size, "%s/%s",
 		 lookup_msg(ospf_nsm_state_msg, nbr->state, NULL),
 		 lookup_msg(ospf_ism_state_msg, state, NULL));
@@ -160,12 +157,12 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 #define HOUR_IN_SECONDS		(60*MINUTE_IN_SECONDS)
 #define DAY_IN_SECONDS		(24*HOUR_IN_SECONDS)
 #define WEEK_IN_SECONDS		(7*DAY_IN_SECONDS)
-	unsigned long w, d, h, m, s, ms, us;
+	unsigned long w, d, h, m, ms, us;
 
 	if (!t)
 		return "inactive";
 
-	w = d = h = m = s = ms = us = 0;
+	w = d = h = m = ms = 0;
 	memset(buf, 0, size);
 
 	us = t->tv_usec;
@@ -201,17 +198,17 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 	}
 
 	if (w > 99)
-		snprintf(buf, size, "%ldw%1ldd", w, d);
+		snprintf(buf, size, "%luw%1lud", w, d);
 	else if (w)
-		snprintf(buf, size, "%ldw%1ldd%02ldh", w, d, h);
+		snprintf(buf, size, "%luw%1lud%02luh", w, d, h);
 	else if (d)
-		snprintf(buf, size, "%1ldd%02ldh%02ldm", d, h, m);
+		snprintf(buf, size, "%1lud%02luh%02lum", d, h, m);
 	else if (h)
-		snprintf(buf, size, "%ldh%02ldm%02lds", h, m, (long)t->tv_sec);
+		snprintf(buf, size, "%luh%02lum%02lds", h, m, (long)t->tv_sec);
 	else if (m)
-		snprintf(buf, size, "%ldm%02lds", m, (long)t->tv_sec);
+		snprintf(buf, size, "%lum%02lds", m, (long)t->tv_sec);
 	else if (ms)
-		snprintf(buf, size, "%ld.%03lds", (long)t->tv_sec, ms);
+		snprintf(buf, size, "%ld.%03lus", (long)t->tv_sec, ms);
 	else
 		snprintf(buf, size, "%ld usecs", (long)t->tv_usec);
 
@@ -254,8 +251,6 @@ static void ospf_packet_hello_dump(struct stream *s, uint16_t length)
 
 static char *ospf_dd_flags_dump(uint8_t flags, char *buf, size_t size)
 {
-	memset(buf, 0, size);
-
 	snprintf(buf, size, "%s|%s|%s", (flags & OSPF_DD_FLAG_I) ? "I" : "-",
 		 (flags & OSPF_DD_FLAG_M) ? "M" : "-",
 		 (flags & OSPF_DD_FLAG_MS) ? "MS" : "-");
@@ -265,8 +260,6 @@ static char *ospf_dd_flags_dump(uint8_t flags, char *buf, size_t size)
 
 static char *ospf_router_lsa_flags_dump(uint8_t flags, char *buf, size_t size)
 {
-	memset(buf, 0, size);
-
 	snprintf(buf, size, "%s|%s|%s",
 		 (flags & ROUTER_LSA_VIRTUAL) ? "V" : "-",
 		 (flags & ROUTER_LSA_EXTERNAL) ? "E" : "-",
@@ -392,7 +385,7 @@ static void ospf_packet_db_desc_dump(struct stream *s, uint16_t length)
 	zlog_debug("  Options %d (%s)", dd->options,
 		   ospf_options_dump(dd->options));
 	zlog_debug("  Flags %d (%s)", dd->flags,
-		   ospf_dd_flags_dump(dd->flags, dd_flags, sizeof dd_flags));
+		   ospf_dd_flags_dump(dd->flags, dd_flags, sizeof(dd_flags)));
 	zlog_debug("  Sequence Number 0x%08lx",
 		   (unsigned long)ntohl(dd->dd_seqnum));
 
@@ -507,23 +500,6 @@ static void ospf_packet_ls_ack_dump(struct stream *s, uint16_t length)
 	stream_set_getp(s, sp);
 }
 
-/* Expects header to be in host order */
-void ospf_ip_header_dump(struct ip *iph)
-{
-	/* IP Header dump. */
-	zlog_debug("ip_v %d", iph->ip_v);
-	zlog_debug("ip_hl %d", iph->ip_hl);
-	zlog_debug("ip_tos %d", iph->ip_tos);
-	zlog_debug("ip_len %d", iph->ip_len);
-	zlog_debug("ip_id %u", (uint32_t)iph->ip_id);
-	zlog_debug("ip_off %u", (uint32_t)iph->ip_off);
-	zlog_debug("ip_ttl %d", iph->ip_ttl);
-	zlog_debug("ip_p %d", iph->ip_p);
-	zlog_debug("ip_sum 0x%x", (uint32_t)iph->ip_sum);
-	zlog_debug("ip_src %s", inet_ntoa(iph->ip_src));
-	zlog_debug("ip_dst %s", inet_ntoa(iph->ip_dst));
-}
-
 static void ospf_header_dump(struct ospf_header *ospfh)
 {
 	char buf[9];
@@ -544,8 +520,7 @@ static void ospf_header_dump(struct ospf_header *ospfh)
 	case OSPF_AUTH_NULL:
 		break;
 	case OSPF_AUTH_SIMPLE:
-		memset(buf, 0, 9);
-		strncpy(buf, (char *)ospfh->u.auth_data, 8);
+		strlcpy(buf, (char *)ospfh->u.auth_data, sizeof(buf));
 		zlog_debug("  Simple Password %s", buf);
 		break;
 	case OSPF_AUTH_CRYPTOGRAPHIC:
@@ -1664,9 +1639,13 @@ DEFUN_NOSH (show_debugging_ospf_instance,
 	return show_debugging_ospf_common(vty, ospf);
 }
 
+static int config_write_debug(struct vty *vty);
 /* Debug node. */
 static struct cmd_node debug_node = {
-	DEBUG_NODE, "", 1 /* VTYSH */
+	.name = "debug",
+	.node = DEBUG_NODE,
+	.prompt = "",
+	.config_write = config_write_debug,
 };
 
 static int config_write_debug(struct vty *vty)
@@ -1689,7 +1668,7 @@ static int config_write_debug(struct vty *vty)
 		return CMD_SUCCESS;
 
 	if (ospf->instance)
-		sprintf(str, " %d", ospf->instance);
+		snprintf(str, sizeof(str), " %u", ospf->instance);
 
 	/* debug ospf ism (status|events|timers). */
 	if (IS_CONF_DEBUG_OSPF(ism, ISM) == OSPF_DEBUG_ISM)
@@ -1805,9 +1784,9 @@ static int config_write_debug(struct vty *vty)
 }
 
 /* Initialize debug commands. */
-void debug_init()
+void ospf_debug_init(void)
 {
-	install_node(&debug_node, config_write_debug);
+	install_node(&debug_node);
 
 	install_element(ENABLE_NODE, &show_debugging_ospf_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_ism_cmd);
