@@ -38,6 +38,7 @@
 #include "bfd.h"
 #include "libfrr.h"
 #include "defaults.h"
+#include "lib_errors.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_network.h"
@@ -51,10 +52,10 @@
 #include "ospfd/ospf_spf.h"
 #include "ospfd/ospf_packet.h"
 #include "ospfd/ospf_dump.h"
+#include "ospfd/ospf_route.h"
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_abr.h"
 #include "ospfd/ospf_flood.h"
-#include "ospfd/ospf_route.h"
 #include "ospfd/ospf_ase.h"
 
 
@@ -90,7 +91,6 @@ void ospf_router_id_update(struct ospf *ospf)
 	struct ospf_interface *oi;
 	struct interface *ifp;
 	struct listnode *node;
-	int type;
 
 	if (!ospf->oi_running) {
 		if (IS_DEBUG_OSPF_EVENT)
@@ -134,24 +134,6 @@ void ospf_router_id_update(struct ospf *ospf)
 			ospf_nbr_self_reset(oi, router_id);
 		}
 
-		/* If AS-external-LSA is queued, then flush those LSAs. */
-		if (router_id_old.s_addr == 0 && ospf->external_origin) {
-			/* Originate each redistributed external route. */
-			for (type = 0; type < ZEBRA_ROUTE_MAX; type++)
-				if (ospf->external_origin & (1 << type))
-					thread_add_event(
-						master,
-						ospf_external_lsa_originate_timer,
-						ospf, type, NULL);
-			/* Originate Deafult. */
-			if (ospf->external_origin & (1 << ZEBRA_ROUTE_MAX))
-				thread_add_event(master,
-						 ospf_default_originate_timer,
-						 ospf, 0, NULL);
-
-			ospf->external_origin = 0;
-		}
-
 		/* Flush (inline) all external LSAs based on the OSPF_LSA_SELF
 		 * flag */
 		if (ospf->lsdb) {
@@ -170,7 +152,7 @@ void ospf_router_id_update(struct ospf *ospf)
 
 		/* Flush (inline) all external LSAs which now match the new
 		   router-id,
-		   need to adjust the OSPF_LSA_SELF flag, so the flush doesnt
+		   need to adjust the OSPF_LSA_SELF flag, so the flush doesn't
 		   hit
 		   asserts in ospf_refresher_unregister_lsa(). This step is
 		   needed
@@ -195,20 +177,14 @@ void ospf_router_id_update(struct ospf *ospf)
 			}
 		}
 
-		/* Originate each redistributed external route. */
-		for (type = 0; type < ZEBRA_ROUTE_MAX; type++)
-			thread_add_event(master,
-					 ospf_external_lsa_originate_timer,
-					 ospf, type, NULL);
-		thread_add_event(master, ospf_default_originate_timer, ospf, 0,
-				 NULL);
-
 		/* update router-lsa's for each area */
 		ospf_router_lsa_update(ospf);
 
 		/* update ospf_interface's */
 		FOR_ALL_INTERFACES (vrf, ifp)
 			ospf_if_update(ospf, ifp);
+
+		ospf_external_lsa_rid_change(ospf);
 	}
 }
 
@@ -233,23 +209,26 @@ static struct ospf *ospf_new(unsigned short instance, const char *name)
 	new->instance = instance;
 	new->router_id.s_addr = htonl(0);
 	new->router_id_static.s_addr = htonl(0);
-
 	if (name) {
-		new->vrf_id = VRF_UNKNOWN;
+		vrf = vrf_lookup_by_name(name);
+		if (vrf)
+			new->vrf_id = vrf->vrf_id;
+		else
+			new->vrf_id = VRF_UNKNOWN;
 		/* Freed in ospf_finish_final */
 		new->name = XSTRDUP(MTYPE_OSPF_TOP, name);
-		vrf = vrf_lookup_by_name(new->name);
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
 				"%s: Create new ospf instance with vrf_name %s vrf_id %u",
-				__PRETTY_FUNCTION__, name, new->vrf_id);
-		if (vrf)
-			ospf_vrf_link(new, vrf);
+				__func__, name, new->vrf_id);
 	} else {
 		new->vrf_id = VRF_DEFAULT;
 		vrf = vrf_lookup_by_id(VRF_DEFAULT);
-		ospf_vrf_link(new, vrf);
 	}
+
+	if (vrf)
+		ospf_vrf_link(new, vrf);
+
 	ospf_zebra_vrf_register(new);
 
 	new->abr_type = OSPF_ABR_DEFAULT;
@@ -308,27 +287,19 @@ static struct ospf *ospf_new(unsigned short instance, const char *name)
 			 new->lsa_refresh_interval, &new->t_lsa_refresher);
 	new->lsa_refresher_started = monotime(NULL);
 
-	if ((new->ibuf = stream_new(OSPF_MAX_PACKET_SIZE + 1)) == NULL) {
-		zlog_err(
-			"ospf_new: fatal error: stream_new(%u) failed allocating ibuf",
-			OSPF_MAX_PACKET_SIZE + 1);
-		exit(1);
-	}
+	new->ibuf = stream_new(OSPF_MAX_PACKET_SIZE + 1);
+
 	new->t_read = NULL;
 	new->oi_write_q = list_new();
 	new->write_oi_count = OSPF_WRITE_INTERFACE_COUNT_DEFAULT;
 
-/* Enable "log-adjacency-changes" */
-#if DFLT_OSPF_LOG_ADJACENCY_CHANGES
-	SET_FLAG(new->config, OSPF_LOG_ADJACENCY_CHANGES);
-#endif
-
 	QOBJ_REG(new, ospf);
 
 	new->fd = -1;
 	if ((ospf_sock_init(new)) < 0) {
 		if (new->vrf_id != VRF_UNKNOWN)
-			zlog_warn(
+			flog_err(
+				EC_LIB_SOCKET,
 				"%s: ospf_sock_init is unable to open a socket",
 				__func__);
 		return new;
@@ -358,7 +329,7 @@ struct ospf *ospf_lookup_instance(unsigned short instance)
 static int ospf_is_ready(struct ospf *ospf)
 {
 	/* OSPF must be on and Router-ID must be configured. */
-	if (!ospf || ospf->router_id.s_addr == 0)
+	if (!ospf || ospf->router_id.s_addr == INADDR_ANY)
 		return 0;
 
 	return 1;
@@ -379,6 +350,9 @@ struct ospf *ospf_lookup_by_inst_name(unsigned short instance, const char *name)
 	struct ospf *ospf = NULL;
 	struct listnode *node, *nnode;
 
+	if (name == NULL || strmatch(name, VRF_DEFAULT_NAME))
+		return ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
 	for (ALL_LIST_ELEMENTS(om->ospf, node, nnode, ospf)) {
 		if ((ospf->instance == instance)
 		    && ((ospf->name == NULL && name == NULL)
@@ -389,7 +363,7 @@ struct ospf *ospf_lookup_by_inst_name(unsigned short instance, const char *name)
 	return NULL;
 }
 
-struct ospf *ospf_get(unsigned short instance, const char *name)
+struct ospf *ospf_get(unsigned short instance, const char *name, bool *created)
 {
 	struct ospf *ospf;
 
@@ -400,11 +374,12 @@ struct ospf *ospf_get(unsigned short instance, const char *name)
 	else
 		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
+	*created = (ospf == NULL);
 	if (ospf == NULL) {
 		ospf = ospf_new(instance, name);
 		ospf_add(ospf);
 
-		if (ospf->router_id_static.s_addr == 0)
+		if (ospf->router_id_static.s_addr == INADDR_ANY)
 			ospf_router_id_update(ospf);
 
 		ospf_opaque_type11_lsa_init(ospf);
@@ -413,27 +388,18 @@ struct ospf *ospf_get(unsigned short instance, const char *name)
 	return ospf;
 }
 
-struct ospf *ospf_get_instance(unsigned short instance)
+struct ospf *ospf_get_instance(unsigned short instance, bool *created)
 {
 	struct ospf *ospf;
 
 	ospf = ospf_lookup_instance(instance);
+	*created = (ospf == NULL);
 	if (ospf == NULL) {
 		ospf = ospf_new(instance, NULL /* VRF_DEFAULT*/);
 		ospf_add(ospf);
 
-		if (ospf->router_id_static.s_addr == 0) {
-			if (vrf_lookup_by_id(ospf->vrf_id))
-				ospf_router_id_update(ospf);
-			else {
-				if (IS_DEBUG_OSPF_EVENT)
-					zlog_debug(
-						"%s: ospf VRF (id %d) is not active yet, skip router id update",
-						__PRETTY_FUNCTION__,
-						ospf->vrf_id);
-			}
+		if (ospf->router_id_static.s_addr == INADDR_ANY)
 			ospf_router_id_update(ospf);
-		}
 
 		ospf_opaque_type11_lsa_init(ospf);
 	}
@@ -551,18 +517,15 @@ void ospf_terminate(void)
 
 	SET_FLAG(om->options, OSPF_MASTER_SHUTDOWN);
 
-	/* exit immediately if OSPF not actually running */
+	/* Skip some steps if OSPF not actually running */
 	if (listcount(om->ospf) == 0)
-		exit(0);
+		goto done;
 
 	bfd_gbl_exit();
 	for (ALL_LIST_ELEMENTS(om->ospf, node, nnode, ospf))
 		ospf_finish(ospf);
 
 	/* Cleanup route maps */
-	route_map_add_hook(NULL);
-	route_map_delete_hook(NULL);
-	route_map_event_hook(NULL);
 	route_map_finish();
 
 	/* reverse prefix_list_init */
@@ -580,6 +543,7 @@ void ospf_terminate(void)
 	zclient_stop(zclient);
 	zclient_free(zclient);
 
+done:
 	frr_fini();
 }
 
@@ -608,7 +572,6 @@ static void ospf_finish_final(struct ospf *ospf)
 	struct ospf_vl_data *vl_data;
 	struct listnode *node, *nnode;
 	int i;
-	unsigned short instance = 0;
 
 	QOBJ_UNREG(ospf);
 
@@ -627,10 +590,12 @@ static void ospf_finish_final(struct ospf *ospf)
 		if (!red_list)
 			continue;
 
-		for (ALL_LIST_ELEMENTS(red_list, node, nnode, red))
+		for (ALL_LIST_ELEMENTS(red_list, node, nnode, red)) {
 			ospf_redistribute_unset(ospf, i, red->instance);
+			ospf_redist_del(ospf, i, red->instance);
+		}
 	}
-	ospf_redistribute_default_unset(ospf);
+	ospf_redistribute_default_set(ospf, DEFAULT_ORIGINATE_NONE, 0, 0);
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area))
 		ospf_remove_vls_through_area(ospf, area);
@@ -638,7 +603,7 @@ static void ospf_finish_final(struct ospf *ospf)
 	for (ALL_LIST_ELEMENTS(ospf->vlinks, node, nnode, vl_data))
 		ospf_vl_delete(ospf, vl_data);
 
-	list_delete_and_null(&ospf->vlinks);
+	list_delete(&ospf->vlinks);
 
 	/* Remove any ospf interface config params */
 	FOR_ALL_INTERFACES (vrf, ifp) {
@@ -652,7 +617,8 @@ static void ospf_finish_final(struct ospf *ospf)
 	/* Reset interface. */
 	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi))
 		ospf_if_free(oi);
-	list_delete_and_null(&ospf->oiflist);
+	list_delete(&ospf->oiflist);
+	ospf->oi_running = 0;
 
 	/* De-Register VRF */
 	ospf_zebra_vrf_deregister(ospf);
@@ -696,7 +662,8 @@ static void ospf_finish_final(struct ospf *ospf)
 	}
 
 	/* Cancel all timers. */
-	OSPF_TIMER_OFF(ospf->t_external_lsa);
+	OSPF_TIMER_OFF(ospf->t_read);
+	OSPF_TIMER_OFF(ospf->t_write);
 	OSPF_TIMER_OFF(ospf->t_spf_calc);
 	OSPF_TIMER_OFF(ospf->t_ase_calc);
 	OSPF_TIMER_OFF(ospf->t_maxage);
@@ -705,14 +672,9 @@ static void ospf_finish_final(struct ospf *ospf)
 	OSPF_TIMER_OFF(ospf->t_asbr_check);
 	OSPF_TIMER_OFF(ospf->t_distribute_update);
 	OSPF_TIMER_OFF(ospf->t_lsa_refresher);
-	OSPF_TIMER_OFF(ospf->t_read);
-	OSPF_TIMER_OFF(ospf->t_write);
 	OSPF_TIMER_OFF(ospf->t_opaque_lsa_self);
 	OSPF_TIMER_OFF(ospf->t_sr_update);
 
-	close(ospf->fd);
-	stream_free(ospf->ibuf);
-
 	LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
 		ospf_discard_from_db(ospf, ospf->lsdb, lsa);
 	LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
@@ -722,8 +684,6 @@ static void ospf_finish_final(struct ospf *ospf)
 	ospf_lsdb_free(ospf->lsdb);
 
 	for (rn = route_top(ospf->maxage_lsa); rn; rn = route_next(rn)) {
-		struct ospf_lsa *lsa;
-
 		if ((lsa = rn->info) != NULL) {
 			ospf_lsa_unlock(&lsa);
 			rn->info = NULL;
@@ -754,19 +714,15 @@ static void ospf_finish_final(struct ospf *ospf)
 		ospf_ase_external_lsas_finish(ospf->external_lsas);
 	}
 
-	list_delete_and_null(&ospf->areas);
-	list_delete_and_null(&ospf->oi_write_q);
-
 	for (i = ZEBRA_ROUTE_SYSTEM; i <= ZEBRA_ROUTE_MAX; i++) {
 		struct list *ext_list;
-		struct listnode *node;
 		struct ospf_external *ext;
 
 		ext_list = ospf->external[i];
 		if (!ext_list)
 			continue;
 
-		for (ALL_LIST_ELEMENTS_RO(ext_list, node, ext)) {
+		for (ALL_LIST_ELEMENTS(ext_list, node, nnode, ext)) {
 			if (ext->external_info)
 				for (rn = route_top(ext->external_info); rn;
 				     rn = route_next(rn)) {
@@ -778,15 +734,20 @@ static void ospf_finish_final(struct ospf *ospf)
 					rn->info = NULL;
 					route_unlock_node(rn);
 				}
+
+			ospf_external_del(ospf, i, ext->instance);
 		}
 	}
 
 	ospf_distance_reset(ospf);
 	route_table_finish(ospf->distance_table);
 
-	if (!CHECK_FLAG(om->options, OSPF_MASTER_SHUTDOWN))
-		instance = ospf->instance;
+	list_delete(&ospf->areas);
+	list_delete(&ospf->oi_write_q);
 
+	close(ospf->fd);
+	stream_free(ospf->ibuf);
+	ospf->fd = -1;
 	ospf_delete(ospf);
 
 	if (ospf->name) {
@@ -801,9 +762,6 @@ static void ospf_finish_final(struct ospf *ospf)
 	}
 
 	XFREE(MTYPE_OSPF_TOP, ospf);
-
-	if (!CHECK_FLAG(om->options, OSPF_MASTER_SHUTDOWN))
-		ospf_get_instance(instance);
 }
 
 
@@ -872,7 +830,7 @@ static void ospf_area_free(struct ospf_area *area)
 	ospf_lsa_unlock(&area->router_lsa_self);
 
 	route_table_finish(area->ranges);
-	list_delete_and_null(&area->oiflist);
+	list_delete(&area->oiflist);
 
 	if (EXPORT_NAME(area))
 		free(EXPORT_NAME(area));
@@ -896,6 +854,7 @@ void ospf_area_check_free(struct ospf *ospf, struct in_addr area_id)
 
 	area = ospf_area_lookup_by_area_id(ospf, area_id);
 	if (area && listcount(area->oiflist) == 0 && area->ranges->top == NULL
+	    && !ospf_vl_count(ospf, area)
 	    && area->shortcut_configured == OSPF_SHORTCUT_DEFAULT
 	    && area->external_routing == OSPF_AREA_DEFAULT
 	    && area->no_summary == 0 && area->default_cost == 1
@@ -980,7 +939,8 @@ static void add_ospf_interface(struct connected *co, struct ospf_area *area)
 	 * ospf_router_id_update() will call ospf_if_update
 	 * whenever r-id is configured instead.
 	 */
-	if ((area->ospf->router_id.s_addr != 0) && if_is_operative(co->ifp))
+	if ((area->ospf->router_id.s_addr != INADDR_ANY)
+	    && if_is_operative(co->ifp))
 		ospf_if_up(oi);
 }
 
@@ -1154,6 +1114,32 @@ void ospf_interface_area_unset(struct ospf *ospf, struct interface *ifp)
 	update_redistributed(ospf, 0); /* interfaces possibly removed */
 }
 
+bool ospf_interface_area_is_already_set(struct ospf *ospf,
+					struct interface *ifp)
+{
+	struct route_node *rn_oi;
+
+	if (!ospf)
+		return false; /* Ospf not ready yet */
+
+	/* Find interfaces that may need to be removed. */
+	for (rn_oi = route_top(IF_OIFS(ifp)); rn_oi;
+	     rn_oi = route_next(rn_oi)) {
+		struct ospf_interface *oi = rn_oi->info;
+
+		if (oi == NULL)
+			continue;
+
+		if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
+			continue;
+		/* at least one route covered by interface
+		 * that implies already done
+		 */
+		return true;
+	}
+	return false;
+}
+
 /* Check whether interface matches given network
  * returns: 1, true. 0, false
  */
@@ -1283,7 +1269,7 @@ static void ospf_network_run(struct prefix *p, struct ospf_area *area)
 	struct interface *ifp;
 
 	/* Schedule Router ID Update. */
-	if (area->ospf->router_id.s_addr == 0)
+	if (area->ospf->router_id.s_addr == INADDR_ANY)
 		ospf_router_id_update(area->ospf);
 
 	/* Get target interface. */
@@ -1303,7 +1289,7 @@ void ospf_ls_upd_queue_empty(struct ospf_interface *oi)
 		if ((lst = (struct list *)rn->info)) {
 			for (ALL_LIST_ELEMENTS(lst, node, nnode, lsa))
 				ospf_lsa_unlock(&lsa); /* oi->ls_upd_queue */
-			list_delete_and_null(&lst);
+			list_delete(&lst);
 			rn->info = NULL;
 		}
 
@@ -1323,7 +1309,7 @@ void ospf_if_update(struct ospf *ospf, struct interface *ifp)
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
 			"%s: interface %s ifp->vrf_id %u ospf vrf %s vrf_id %u router_id %s",
-			__PRETTY_FUNCTION__, ifp->name, ifp->vrf_id,
+			__func__, ifp->name, ifp->vrf_id,
 			ospf_vrf_id_to_name(ospf->vrf_id), ospf->vrf_id,
 			inet_ntoa(ospf->router_id));
 
@@ -1335,6 +1321,7 @@ void ospf_if_update(struct ospf *ospf, struct interface *ifp)
 
 	/* Update connected redistribute. */
 	update_redistributed(ospf, 1);
+
 }
 
 void ospf_remove_vls_through_area(struct ospf *ospf, struct ospf_area *area)
@@ -1801,7 +1788,7 @@ static void ospf_nbr_nbma_add(struct ospf_nbr_nbma *nbr_nbma,
 	p.prefixlen = IPV4_MAX_BITLEN;
 	p.u.prefix4 = nbr_nbma->addr;
 
-	rn = route_node_get(oi->nbrs, (struct prefix *)&p);
+	rn = route_node_get(oi->nbrs, &p);
 	if (rn->info) {
 		nbr = rn->info;
 		nbr->nbr_nbma = nbr_nbma;
@@ -2033,8 +2020,8 @@ void ospf_vrf_unlink(struct ospf *ospf, struct vrf *vrf)
 static int ospf_vrf_new(struct vrf *vrf)
 {
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("%s: VRF Created: %s(%u)", __PRETTY_FUNCTION__,
-			   vrf->name, vrf->vrf_id);
+		zlog_debug("%s: VRF Created: %s(%u)", __func__, vrf->name,
+			   vrf->vrf_id);
 
 	return 0;
 }
@@ -2043,8 +2030,8 @@ static int ospf_vrf_new(struct vrf *vrf)
 static int ospf_vrf_delete(struct vrf *vrf)
 {
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("%s: VRF Deletion: %s(%u)", __PRETTY_FUNCTION__,
-			   vrf->name, vrf->vrf_id);
+		zlog_debug("%s: VRF Deletion: %s(%u)", __func__, vrf->name,
+			   vrf->vrf_id);
 
 	return 0;
 }
@@ -2074,40 +2061,35 @@ static int ospf_vrf_enable(struct vrf *vrf)
 	int ret = 0;
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("%s: VRF %s id %u enabled", __PRETTY_FUNCTION__,
-			   vrf->name, vrf->vrf_id);
+		zlog_debug("%s: VRF %s id %u enabled", __func__, vrf->name,
+			   vrf->vrf_id);
 
 	ospf = ospf_lookup_by_name(vrf->name);
 	if (ospf) {
+		if (ospf->name && strmatch(vrf->name, VRF_DEFAULT_NAME)) {
+			XFREE(MTYPE_OSPF_TOP, ospf->name);
+			ospf->name = NULL;
+		}
 		old_vrf_id = ospf->vrf_id;
 		/* We have instance configured, link to VRF and make it "up". */
 		ospf_vrf_link(ospf, vrf);
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
 				"%s: ospf linked to vrf %s vrf_id %u (old id %u)",
-				__PRETTY_FUNCTION__, vrf->name, ospf->vrf_id,
-				old_vrf_id);
+				__func__, vrf->name, ospf->vrf_id, old_vrf_id);
 
 		if (old_vrf_id != ospf->vrf_id) {
-			if (ospfd_privs.change(ZPRIVS_RAISE)) {
-				zlog_err(
-					"ospf_sock_init: could not raise privs, %s",
-					safe_strerror(errno));
-			}
+			frr_with_privs(&ospfd_privs) {
+				/* stop zebra redist to us for old vrf */
+				zclient_send_dereg_requests(zclient,
+							    old_vrf_id);
 
-			/* stop zebra redist to us for old vrf */
-			zclient_send_dereg_requests(zclient, old_vrf_id);
+				ospf_set_redist_vrf_bitmaps(ospf);
 
-			ospf_set_redist_vrf_bitmaps(ospf);
+				/* start zebra redist to us for new vrf */
+				ospf_zebra_vrf_register(ospf);
 
-			/* start zebra redist to us for new vrf */
-			ospf_zebra_vrf_register(ospf);
-
-			ret = ospf_sock_init(ospf);
-			if (ospfd_privs.change(ZPRIVS_LOWER)) {
-				zlog_err(
-					"ospf_sock_init: could not lower privs, %s",
-					safe_strerror(errno));
+				ret = ospf_sock_init(ospf);
 			}
 			if (ret < 0 || ospf->fd <= 0)
 				return 0;
@@ -2131,8 +2113,8 @@ static int ospf_vrf_disable(struct vrf *vrf)
 		return 0;
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("%s: VRF %s id %d disabled.", __PRETTY_FUNCTION__,
-			   vrf->name, vrf->vrf_id);
+		zlog_debug("%s: VRF %s id %d disabled.", __func__, vrf->name,
+			   vrf->vrf_id);
 
 	ospf = ospf_lookup_by_name(vrf->name);
 	if (ospf) {
@@ -2144,8 +2126,8 @@ static int ospf_vrf_disable(struct vrf *vrf)
 		ospf_vrf_unlink(ospf, vrf);
 		ospf->oi_running = 0;
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug("%s: ospf old_vrf_id %d unlinked",
-				   __PRETTY_FUNCTION__, old_vrf_id);
+			zlog_debug("%s: ospf old_vrf_id %d unlinked", __func__,
+				   old_vrf_id);
 		thread_cancel(ospf->t_read);
 		close(ospf->fd);
 		ospf->fd = -1;
@@ -2158,7 +2140,7 @@ static int ospf_vrf_disable(struct vrf *vrf)
 void ospf_vrf_init(void)
 {
 	vrf_init(ospf_vrf_new, ospf_vrf_enable, ospf_vrf_disable,
-		 ospf_vrf_delete);
+		 ospf_vrf_delete, ospf_vrf_enable);
 }
 
 void ospf_vrf_terminate(void)
@@ -2172,3 +2154,11 @@ const char *ospf_vrf_id_to_name(vrf_id_t vrf_id)
 
 	return vrf ? vrf->name : "NIL";
 }
+
+const char *ospf_get_name(const struct ospf *ospf)
+{
+	if (ospf->name)
+		return ospf->name;
+	else
+		return VRF_DEFAULT_NAME;
+}
