@@ -21,7 +21,8 @@
 
 #include <log.h>
 #include <nexthop.h>
-#include <nexthop_group.h>
+#include "nexthop_group.h"
+#include "nexthop_group_private.h"
 #include <hash.h>
 #include <jhash.h>
 #include <vty.h>
@@ -50,7 +51,7 @@ static void pbr_nht_install_nexthop_group(struct pbr_nexthop_group_cache *pnhgc,
 static void
 pbr_nht_uninstall_nexthop_group(struct pbr_nexthop_group_cache *pnhgc,
 				struct nexthop_group nhg,
-				enum nexthop_types_t nh_afi);
+				enum nexthop_types_t nh_type);
 
 /*
  * Nexthop refcount.
@@ -66,10 +67,12 @@ static void *pbr_nhrc_hash_alloc(void *p)
 {
 	struct nhrc *nhrc = XCALLOC(MTYPE_PBR_NHG, sizeof(struct nhrc));
 	nhrc->nexthop = *(struct nexthop *)p;
+	nhrc->nexthop.next = NULL;
+	nhrc->nexthop.prev = NULL;
 	return nhrc;
 }
 
-static int pbr_nhrc_hash_equal(const void *arg1, const void *arg2)
+static bool pbr_nhrc_hash_equal(const void *arg1, const void *arg2)
 {
 	const struct nexthop *nh1, *nh2;
 
@@ -94,8 +97,7 @@ static void *pbr_nh_alloc(void *p)
 	/* Decremented again in pbr_nh_delete */
 	++nhrc->refcount;
 
-	DEBUGD(&pbr_dbg_nht, "%s: Sending nexthop to Zebra",
-	       __PRETTY_FUNCTION__);
+	DEBUGD(&pbr_dbg_nht, "%s: Sending nexthop to Zebra", __func__);
 
 	pbr_send_rnh(new->nexthop, true);
 
@@ -113,7 +115,7 @@ static void pbr_nh_delete(struct pbr_nexthop_cache **pnhc)
 		--nhrc->refcount;
 	if (!nhrc || nhrc->refcount == 0) {
 		DEBUGD(&pbr_dbg_nht, "%s: Removing nexthop from Zebra",
-		       __PRETTY_FUNCTION__);
+		       __func__);
 		pbr_send_rnh((*pnhc)->nexthop, false);
 	}
 	if (nhrc && nhrc->refcount == 0) {
@@ -124,22 +126,22 @@ static void pbr_nh_delete(struct pbr_nexthop_cache **pnhc)
 	XFREE(MTYPE_PBR_NHG, *pnhc);
 }
 
-static void pbr_nh_delete_iterate(struct hash_backet *b, void *p)
+static void pbr_nh_delete_iterate(struct hash_bucket *b, void *p)
 {
 	pbr_nh_delete((struct pbr_nexthop_cache **)&b->data);
 }
 
-static uint32_t pbr_nh_hash_key(void *arg)
+static uint32_t pbr_nh_hash_key(const void *arg)
 {
 	uint32_t key;
-	struct pbr_nexthop_cache *pbrnc = (struct pbr_nexthop_cache *)arg;
+	const struct pbr_nexthop_cache *pbrnc = arg;
 
 	key = nexthop_hash(pbrnc->nexthop);
 
 	return key;
 }
 
-static int pbr_nh_hash_equal(const void *arg1, const void *arg2)
+static bool pbr_nh_hash_equal(const void *arg1, const void *arg2)
 {
 	const struct pbr_nexthop_cache *pbrnc1 =
 		(const struct pbr_nexthop_cache *)arg1;
@@ -147,17 +149,17 @@ static int pbr_nh_hash_equal(const void *arg1, const void *arg2)
 		(const struct pbr_nexthop_cache *)arg2;
 
 	if (pbrnc1->nexthop->vrf_id != pbrnc2->nexthop->vrf_id)
-		return 0;
+		return false;
 
 	if (pbrnc1->nexthop->ifindex != pbrnc2->nexthop->ifindex)
-		return 0;
+		return false;
 
 	if (pbrnc1->nexthop->type != pbrnc2->nexthop->type)
-		return 0;
+		return false;
 
 	switch (pbrnc1->nexthop->type) {
 	case NEXTHOP_TYPE_IFINDEX:
-		return 1;
+		return pbrnc1->nexthop->ifindex == pbrnc2->nexthop->ifindex;
 	case NEXTHOP_TYPE_IPV4_IFINDEX:
 	case NEXTHOP_TYPE_IPV4:
 		return pbrnc1->nexthop->gate.ipv4.s_addr
@@ -173,7 +175,7 @@ static int pbr_nh_hash_equal(const void *arg1, const void *arg2)
 	/*
 	 * We should not get here
 	 */
-	return 0;
+	return false;
 }
 
 static void pbr_nhgc_delete(struct pbr_nexthop_group_cache *p)
@@ -191,11 +193,11 @@ static void *pbr_nhgc_alloc(void *p)
 
 	new = XCALLOC(MTYPE_PBR_NHG, sizeof(*new));
 
-	strcpy(new->name, pnhgc->name);
-	new->table_id = pbr_nht_get_next_tableid();
+	strlcpy(new->name, pnhgc->name, sizeof(pnhgc->name));
+	new->table_id = pbr_nht_get_next_tableid(false);
 
-	DEBUGD(&pbr_dbg_nht, "%s: NHT: %s assigned Table ID: %u",
-	       __PRETTY_FUNCTION__, new->name, new->table_id);
+	DEBUGD(&pbr_dbg_nht, "%s: NHT: %s assigned Table ID: %u", __func__,
+	       new->name, new->table_id);
 
 	new->nhh = hash_create_size(8, pbr_nh_hash_key, pbr_nh_hash_equal,
 				    "PBR NH Cache Hash");
@@ -212,16 +214,17 @@ void pbr_nhgroup_add_cb(const char *name)
 
 	if (!nhgc) {
 		DEBUGD(&pbr_dbg_nht, "%s: Could not find nhgc with name: %s\n",
-		       __PRETTY_FUNCTION__, name);
+		       __func__, name);
 		return;
 	}
 
 	pnhgc = pbr_nht_add_group(name);
 
-	DEBUGD(&pbr_dbg_nht, "%s: Added nexthop-group %s", __PRETTY_FUNCTION__,
-	       name);
+	if (!pnhgc)
+		return;
+
+	DEBUGD(&pbr_dbg_nht, "%s: Added nexthop-group %s", __func__, name);
 
-	pbr_nht_install_nexthop_group(pnhgc, nhgc->nhg);
 	pbr_map_check_nh_group_change(name);
 }
 
@@ -234,6 +237,13 @@ void pbr_nhgroup_add_nexthop_cb(const struct nexthop_group_cmd *nhgc,
 	struct pbr_nexthop_cache pnhc_find = {};
 	struct pbr_nexthop_cache *pnhc;
 
+	if (!pbr_nht_get_next_tableid(true)) {
+		zlog_warn(
+			"%s: Exhausted all table identifiers; cannot create nexthop-group cache for nexthop-group '%s'",
+			__func__, nhgc->name);
+		return;
+	}
+
 	/* find pnhgc by name */
 	strlcpy(pnhgc_find.name, nhgc->name, sizeof(pnhgc_find.name));
 	pnhgc = hash_get(pbr_nhg_hash, &pnhgc_find, pbr_nhgc_alloc);
@@ -249,11 +259,21 @@ void pbr_nhgroup_add_nexthop_cb(const struct nexthop_group_cmd *nhgc,
 	if (DEBUG_MODE_CHECK(&pbr_dbg_nht, DEBUG_MODE_ALL)) {
 		nexthop2str(nhop, debugstr, sizeof(debugstr));
 		DEBUGD(&pbr_dbg_nht, "%s: Added %s to nexthop-group %s",
-		       __PRETTY_FUNCTION__, debugstr, nhgc->name);
+		       __func__, debugstr, nhgc->name);
 	}
 
 	pbr_nht_install_nexthop_group(pnhgc, nhgc->nhg);
 	pbr_map_check_nh_group_change(nhgc->name);
+
+	if (nhop->type == NEXTHOP_TYPE_IFINDEX
+	    || (nhop->type == NEXTHOP_TYPE_IPV6_IFINDEX
+		&& IN6_IS_ADDR_LINKLOCAL(&nhop->gate.ipv6))) {
+		struct interface *ifp;
+
+		ifp = if_lookup_by_index(nhop->ifindex, nhop->vrf_id);
+		if (ifp)
+			pbr_nht_nexthop_interface_update(ifp);
+	}
 }
 
 void pbr_nhgroup_del_nexthop_cb(const struct nexthop_group_cmd *nhgc,
@@ -264,11 +284,11 @@ void pbr_nhgroup_del_nexthop_cb(const struct nexthop_group_cmd *nhgc,
 	struct pbr_nexthop_group_cache *pnhgc;
 	struct pbr_nexthop_cache pnhc_find = {};
 	struct pbr_nexthop_cache *pnhc;
-	enum nexthop_types_t nh_afi = nhop->type;
+	enum nexthop_types_t nh_type = nhop->type;
 
 	/* find pnhgc by name */
 	strlcpy(pnhgc_find.name, nhgc->name, sizeof(pnhgc_find.name));
-	pnhgc = hash_get(pbr_nhg_hash, &pnhgc_find, pbr_nhgc_alloc);
+	pnhgc = hash_lookup(pbr_nhg_hash, &pnhgc_find);
 
 	/* delete pnhc from pnhgc->nhh */
 	pnhc_find.nexthop = (struct nexthop *)nhop;
@@ -280,21 +300,20 @@ void pbr_nhgroup_del_nexthop_cb(const struct nexthop_group_cmd *nhgc,
 	if (DEBUG_MODE_CHECK(&pbr_dbg_nht, DEBUG_MODE_ALL)) {
 		nexthop2str(nhop, debugstr, sizeof(debugstr));
 		DEBUGD(&pbr_dbg_nht, "%s: Removed %s from nexthop-group %s",
-		       __PRETTY_FUNCTION__, debugstr, nhgc->name);
+		       __func__, debugstr, nhgc->name);
 	}
 
 	if (pnhgc->nhh->count)
 		pbr_nht_install_nexthop_group(pnhgc, nhgc->nhg);
 	else
-		pbr_nht_uninstall_nexthop_group(pnhgc, nhgc->nhg, nh_afi);
+		pbr_nht_uninstall_nexthop_group(pnhgc, nhgc->nhg, nh_type);
 
 	pbr_map_check_nh_group_change(nhgc->name);
 }
 
 void pbr_nhgroup_delete_cb(const char *name)
 {
-	DEBUGD(&pbr_dbg_nht, "%s: Removed nexthop-group %s",
-	       __PRETTY_FUNCTION__, name);
+	DEBUGD(&pbr_dbg_nht, "%s: Removed nexthop-group %s", __func__, name);
 
 	/* delete group from all pbrms's */
 	pbr_nht_delete_group(name);
@@ -309,7 +328,7 @@ static struct pbr_nexthop_cache *pbr_nht_lookup_nexthop(struct nexthop *nexthop)
 }
 #endif
 
-static void pbr_nht_find_nhg_from_table_install(struct hash_backet *b,
+static void pbr_nht_find_nhg_from_table_install(struct hash_bucket *b,
 						void *data)
 {
 	struct pbr_nexthop_group_cache *pnhgc =
@@ -317,8 +336,8 @@ static void pbr_nht_find_nhg_from_table_install(struct hash_backet *b,
 	uint32_t *table_id = (uint32_t *)data;
 
 	if (pnhgc->table_id == *table_id) {
-		DEBUGD(&pbr_dbg_nht, "%s: Table ID (%u) matches %s",
-		       __PRETTY_FUNCTION__, *table_id, pnhgc->name);
+		DEBUGD(&pbr_dbg_nht, "%s: Table ID (%u) matches %s", __func__,
+		       *table_id, pnhgc->name);
 
 		/*
 		 * If the table has been re-handled by zebra
@@ -338,7 +357,7 @@ void pbr_nht_route_installed_for_table(uint32_t table_id)
 		     &table_id);
 }
 
-static void pbr_nht_find_nhg_from_table_remove(struct hash_backet *b,
+static void pbr_nht_find_nhg_from_table_remove(struct hash_bucket *b,
 					       void *data)
 {
 	;
@@ -362,49 +381,63 @@ void pbr_nht_route_removed_for_table(uint32_t table_id)
  *    - AFI_MAX on error
  */
 static afi_t pbr_nht_which_afi(struct nexthop_group nhg,
-			       enum nexthop_types_t nh_afi)
+			       enum nexthop_types_t nh_type)
 {
 	struct nexthop *nexthop;
 	afi_t install_afi = AFI_MAX;
 	bool v6, v4, bh;
 
+	if (nh_type) {
+		switch (nh_type) {
+		case NEXTHOP_TYPE_IPV4:
+		case NEXTHOP_TYPE_IPV4_IFINDEX:
+			return AFI_IP;
+		case NEXTHOP_TYPE_IPV6:
+		case NEXTHOP_TYPE_IPV6_IFINDEX:
+			return AFI_IP6;
+		case NEXTHOP_TYPE_IFINDEX:
+		case NEXTHOP_TYPE_BLACKHOLE:
+			return AFI_MAX;
+		}
+	}
+
 	v6 = v4 = bh = false;
 
-	if (!nh_afi) {
-		for (ALL_NEXTHOPS(nhg, nexthop)) {
-			nh_afi = nexthop->type;
+	for (ALL_NEXTHOPS(nhg, nexthop)) {
+		nh_type = nexthop->type;
+
+		switch (nh_type) {
+		case NEXTHOP_TYPE_IFINDEX:
+			break;
+		case NEXTHOP_TYPE_IPV4:
+		case NEXTHOP_TYPE_IPV4_IFINDEX:
+			v6 = true;
+			install_afi = AFI_IP;
+			break;
+		case NEXTHOP_TYPE_IPV6:
+		case NEXTHOP_TYPE_IPV6_IFINDEX:
+			v4 = true;
+			install_afi = AFI_IP6;
+			break;
+		case NEXTHOP_TYPE_BLACKHOLE:
+			bh = true;
 			break;
 		}
 	}
 
-	switch (nh_afi) {
-	case NEXTHOP_TYPE_IFINDEX:
-		break;
-	case NEXTHOP_TYPE_IPV4:
-	case NEXTHOP_TYPE_IPV4_IFINDEX:
-		v6 = true;
-		install_afi = AFI_IP;
-		break;
-	case NEXTHOP_TYPE_IPV6:
-	case NEXTHOP_TYPE_IPV6_IFINDEX:
-		v4 = true;
-		install_afi = AFI_IP6;
-		break;
-	case NEXTHOP_TYPE_BLACKHOLE:
-		bh = true;
+	/* Interface and/or blackhole nexthops only. */
+	if (!v4 && !v6)
 		install_afi = AFI_MAX;
-		break;
-	}
 
 	if (!bh && v6 && v4)
 		DEBUGD(&pbr_dbg_nht,
-		       "%s: Saw both V6 and V4 nexthops...using %s",
-		       __PRETTY_FUNCTION__, afi2str(install_afi));
+		       "%s: Saw both V6 and V4 nexthops...using %s", __func__,
+		       afi2str(install_afi));
 	if (bh && (v6 || v4))
 		DEBUGD(&pbr_dbg_nht,
 		       "%s: Saw blackhole nexthop(s) with %s%s%s nexthop(s), using AFI_MAX.",
-		       __PRETTY_FUNCTION__, v4 ? "v4" : "",
-		       (v4 && v6) ? " and " : "", v6 ? "v6" : "");
+		       __func__, v4 ? "v4" : "", (v4 && v6) ? " and " : "",
+		       v6 ? "v6" : "");
 
 	return install_afi;
 }
@@ -413,9 +446,9 @@ static void pbr_nht_install_nexthop_group(struct pbr_nexthop_group_cache *pnhgc,
 					  struct nexthop_group nhg)
 {
 	afi_t install_afi;
-	enum nexthop_types_t nh_afi = 0;
+	enum nexthop_types_t nh_type = 0;
 
-	install_afi = pbr_nht_which_afi(nhg, nh_afi);
+	install_afi = pbr_nht_which_afi(nhg, nh_type);
 
 	route_add(pnhgc, nhg, install_afi);
 }
@@ -423,11 +456,11 @@ static void pbr_nht_install_nexthop_group(struct pbr_nexthop_group_cache *pnhgc,
 static void
 pbr_nht_uninstall_nexthop_group(struct pbr_nexthop_group_cache *pnhgc,
 				struct nexthop_group nhg,
-				enum nexthop_types_t nh_afi)
+				enum nexthop_types_t nh_type)
 {
 	afi_t install_afi;
 
-	install_afi = pbr_nht_which_afi(nhg, nh_afi);
+	install_afi = pbr_nht_which_afi(nhg, nh_type);
 
 	pnhgc->installed = false;
 	pnhgc->valid = false;
@@ -452,7 +485,7 @@ void pbr_nht_change_group(const char *name)
 	if (!pnhgc) {
 		DEBUGD(&pbr_dbg_nht,
 		       "%s: Could not find nexthop-group cache w/ name '%s'",
-		       __PRETTY_FUNCTION__, name);
+		       __func__, name);
 		return;
 	}
 
@@ -477,16 +510,38 @@ char *pbr_nht_nexthop_make_name(char *name, size_t l,
 	return buffer;
 }
 
-void pbr_nht_add_individual_nexthop(struct pbr_map_sequence *pbrms)
+void pbr_nht_add_individual_nexthop(struct pbr_map_sequence *pbrms,
+				    const struct nexthop *nhop)
 {
 	struct pbr_nexthop_group_cache *pnhgc;
 	struct pbr_nexthop_group_cache find;
 	struct pbr_nexthop_cache *pnhc;
 	struct pbr_nexthop_cache lookup;
+	struct nexthop *nh;
+	char buf[PBR_NHC_NAMELEN];
+
+	pbrms->nhg = nexthop_group_new();
+	pbrms->internal_nhg_name = XSTRDUP(
+		MTYPE_TMP,
+		pbr_nht_nexthop_make_name(pbrms->parent->name, PBR_NHC_NAMELEN,
+					  pbrms->seqno, buf));
+
+	nh = nexthop_new();
+	memcpy(nh, nhop, sizeof(*nh));
+
+	nexthop_group_add_sorted(pbrms->nhg, nh);
 
 	memset(&find, 0, sizeof(find));
 	pbr_nht_nexthop_make_name(pbrms->parent->name, PBR_NHC_NAMELEN,
 				  pbrms->seqno, find.name);
+
+	if (!pbr_nht_get_next_tableid(true)) {
+		zlog_warn(
+			"%s: Exhausted all table identifiers; cannot create nexthop-group cache for nexthop-group '%s'",
+			__func__, find.name);
+		return;
+	}
+
 	if (!pbrms->internal_nhg_name)
 		pbrms->internal_nhg_name = XSTRDUP(MTYPE_TMP, find.name);
 
@@ -498,48 +553,42 @@ void pbr_nht_add_individual_nexthop(struct pbr_map_sequence *pbrms)
 	pbr_nht_install_nexthop_group(pnhgc, *pbrms->nhg);
 }
 
-void pbr_nht_delete_individual_nexthop(struct pbr_map_sequence *pbrms)
+static void pbr_nht_release_individual_nexthop(struct pbr_map_sequence *pbrms)
 {
 	struct pbr_nexthop_group_cache *pnhgc;
 	struct pbr_nexthop_group_cache find;
 	struct pbr_nexthop_cache *pnhc;
 	struct pbr_nexthop_cache lup;
-	struct pbr_map *pbrm = pbrms->parent;
-	struct listnode *node;
-	struct pbr_map_interface *pmi;
 	struct nexthop *nh;
-	enum nexthop_types_t nh_afi = 0;
-
-	if (pbrm->valid && pbrms->nhs_installed && pbrm->incoming->count) {
-		for (ALL_LIST_ELEMENTS_RO(pbrm->incoming, node, pmi))
-			pbr_send_pbr_map(pbrms, pmi, false);
-	}
-
-	pbrm->valid = false;
-	pbrms->nhs_installed = false;
-	pbrms->reason |= PBR_MAP_INVALID_NO_NEXTHOPS;
+	enum nexthop_types_t nh_type = 0;
 
 	memset(&find, 0, sizeof(find));
 	snprintf(find.name, sizeof(find.name), "%s", pbrms->internal_nhg_name);
 	pnhgc = hash_lookup(pbr_nhg_hash, &find);
 
 	nh = pbrms->nhg->nexthop;
-	nh_afi = nh->type;
+	nh_type = nh->type;
 	lup.nexthop = nh;
 	pnhc = hash_lookup(pnhgc->nhh, &lup);
 	pnhc->parent = NULL;
 	hash_release(pnhgc->nhh, pnhc);
 	pbr_nh_delete(&pnhc);
-	pbr_nht_uninstall_nexthop_group(pnhgc, *pbrms->nhg, nh_afi);
+	pbr_nht_uninstall_nexthop_group(pnhgc, *pbrms->nhg, nh_type);
 
 	hash_release(pbr_nhg_hash, pnhgc);
+	pbr_nhgc_delete(pnhgc);
 
-	nexthop_del(pbrms->nhg, nh);
-	nexthop_free(nh);
 	nexthop_group_delete(&pbrms->nhg);
 	XFREE(MTYPE_TMP, pbrms->internal_nhg_name);
 }
 
+void pbr_nht_delete_individual_nexthop(struct pbr_map_sequence *pbrms)
+{
+	pbr_map_delete_nexthops(pbrms);
+
+	pbr_nht_release_individual_nexthop(pbrms);
+}
+
 struct pbr_nexthop_group_cache *pbr_nht_add_group(const char *name)
 {
 	struct nexthop *nhop;
@@ -547,27 +596,33 @@ struct pbr_nexthop_group_cache *pbr_nht_add_group(const char *name)
 	struct pbr_nexthop_group_cache *pnhgc;
 	struct pbr_nexthop_group_cache lookup;
 
+	if (!pbr_nht_get_next_tableid(true)) {
+		zlog_warn(
+			"%s: Exhausted all table identifiers; cannot create nexthop-group cache for nexthop-group '%s'",
+			__func__, name);
+		return NULL;
+	}
+
 	nhgc = nhgc_find(name);
 
 	if (!nhgc) {
-		zlog_warn("%s: Could not find group %s to add",
-			  __PRETTY_FUNCTION__, name);
+		DEBUGD(&pbr_dbg_nht, "%s: Could not find nhgc with name: %s\n",
+		       __func__, name);
 		return NULL;
 	}
 
 	snprintf(lookup.name, sizeof(lookup.name), "%s", name);
 	pnhgc = hash_get(pbr_nhg_hash, &lookup, pbr_nhgc_alloc);
-	DEBUGD(&pbr_dbg_nht, "%s: Retrieved NHGC @ %p", __PRETTY_FUNCTION__,
-	       pnhgc);
+	DEBUGD(&pbr_dbg_nht, "%s: Retrieved NHGC @ %p", __func__, pnhgc);
 
 	for (ALL_NEXTHOPS(nhgc->nhg, nhop)) {
-		struct pbr_nexthop_cache lookup;
+		struct pbr_nexthop_cache lookupc;
 		struct pbr_nexthop_cache *pnhc;
 
-		lookup.nexthop = nhop;
-		pnhc = hash_lookup(pnhgc->nhh, &lookup);
+		lookupc.nexthop = nhop;
+		pnhc = hash_lookup(pnhgc->nhh, &lookupc);
 		if (!pnhc) {
-			pnhc = hash_get(pnhgc->nhh, &lookup, pbr_nh_alloc);
+			pnhc = hash_get(pnhgc->nhh, &lookupc, pbr_nh_alloc);
 			pnhc->parent = pnhgc;
 		}
 	}
@@ -588,7 +643,6 @@ void pbr_nht_delete_group(const char *name)
 			if (pbrms->nhgrp_name
 			    && strmatch(pbrms->nhgrp_name, name)) {
 				pbrms->reason |= PBR_MAP_INVALID_NO_NEXTHOPS;
-				nexthop_group_delete(&pbrms->nhg);
 				pbrms->nhg = NULL;
 				pbrms->internal_nhg_name = NULL;
 				pbrm->valid = false;
@@ -603,7 +657,7 @@ void pbr_nht_delete_group(const char *name)
 
 bool pbr_nht_nexthop_valid(struct nexthop_group *nhg)
 {
-	DEBUGD(&pbr_dbg_nht, "%s: %p", __PRETTY_FUNCTION__, nhg);
+	DEBUGD(&pbr_dbg_nht, "%s: %p", __func__, nhg);
 	return true;
 }
 
@@ -612,13 +666,13 @@ bool pbr_nht_nexthop_group_valid(const char *name)
 	struct pbr_nexthop_group_cache *pnhgc;
 	struct pbr_nexthop_group_cache lookup;
 
-	DEBUGD(&pbr_dbg_nht, "%s: %s", __PRETTY_FUNCTION__, name);
+	DEBUGD(&pbr_dbg_nht, "%s: %s", __func__, name);
 
 	snprintf(lookup.name, sizeof(lookup.name), "%s", name);
 	pnhgc = hash_get(pbr_nhg_hash, &lookup, NULL);
 	if (!pnhgc)
 		return false;
-	DEBUGD(&pbr_dbg_nht, "%s: \t%d %d", __PRETTY_FUNCTION__, pnhgc->valid,
+	DEBUGD(&pbr_dbg_nht, "%s: \t%d %d", __func__, pnhgc->valid,
 	       pnhgc->installed);
 	if (pnhgc->valid && pnhgc->installed)
 		return true;
@@ -628,11 +682,119 @@ bool pbr_nht_nexthop_group_valid(const char *name)
 
 struct pbr_nht_individual {
 	struct zapi_route *nhr;
+	struct interface *ifp;
 
 	uint32_t valid;
 };
 
-static void pbr_nht_individual_nexthop_update_lookup(struct hash_backet *b,
+static bool
+pbr_nht_individual_nexthop_gw_update(struct pbr_nexthop_cache *pnhc,
+				     const struct pbr_nht_individual *pnhi)
+{
+	bool is_valid = pnhc->valid;
+
+	if (!pnhi->nhr) /* It doesn't care about non-nexthop updates */
+		goto done;
+
+	switch (pnhi->nhr->prefix.family) {
+	case AF_INET:
+		if (pnhc->nexthop->gate.ipv4.s_addr
+		    != pnhi->nhr->prefix.u.prefix4.s_addr)
+			goto done; /* Unrelated change */
+		break;
+	case AF_INET6:
+		if (memcmp(&pnhc->nexthop->gate.ipv6,
+			   &pnhi->nhr->prefix.u.prefix6, 16)
+		    != 0)
+			goto done; /* Unrelated change */
+		break;
+	}
+
+	if (!pnhi->nhr->nexthop_num) {
+		is_valid = false;
+		goto done;
+	}
+
+	if (pnhc->nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX
+	    || pnhc->nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX) {
+
+		/* GATEWAY_IFINDEX type shouldn't resolve to group */
+		if (pnhi->nhr->nexthop_num > 1) {
+			is_valid = false;
+			goto done;
+		}
+
+		/* If whatever we resolved to wasn't on the interface we
+		 * specified. (i.e. not a connected route), its invalid.
+		 */
+		if (pnhi->nhr->nexthops[0].ifindex != pnhc->nexthop->ifindex) {
+			is_valid = false;
+			goto done;
+		}
+	}
+
+	is_valid = true;
+
+done:
+	pnhc->valid = is_valid;
+
+	return pnhc->valid;
+}
+
+static bool pbr_nht_individual_nexthop_interface_update(
+	struct pbr_nexthop_cache *pnhc, const struct pbr_nht_individual *pnhi)
+{
+	bool is_valid = pnhc->valid;
+
+	if (!pnhi->ifp) /* It doesn't care about non-interface updates */
+		goto done;
+
+	if (pnhc->nexthop->ifindex
+	    != pnhi->ifp->ifindex) /* Un-related interface */
+		goto done;
+
+	is_valid = !!if_is_up(pnhi->ifp);
+
+done:
+	pnhc->valid = is_valid;
+
+	return pnhc->valid;
+}
+
+/* Given this update either from interface or nexthop tracking, re-validate this
+ * nexthop.
+ *
+ * If the update is un-related, the subroutines shoud just return their cached
+ * valid state.
+ */
+static void
+pbr_nht_individual_nexthop_update(struct pbr_nexthop_cache *pnhc,
+				  const struct pbr_nht_individual *pnhi)
+{
+	assert(pnhi->nhr || pnhi->ifp); /* Either nexthop or interface update */
+
+	switch (pnhc->nexthop->type) {
+	case NEXTHOP_TYPE_IFINDEX:
+		pbr_nht_individual_nexthop_interface_update(pnhc, pnhi);
+		break;
+	case NEXTHOP_TYPE_IPV6_IFINDEX:
+		if (IN6_IS_ADDR_LINKLOCAL(&pnhc->nexthop->gate.ipv6)) {
+			pbr_nht_individual_nexthop_interface_update(pnhc, pnhi);
+			break;
+		}
+		/* Intentional fall thru */
+	case NEXTHOP_TYPE_IPV4_IFINDEX:
+	case NEXTHOP_TYPE_IPV4:
+	case NEXTHOP_TYPE_IPV6:
+		pbr_nht_individual_nexthop_gw_update(pnhc, pnhi);
+		break;
+	case NEXTHOP_TYPE_BLACKHOLE:
+		pnhc->valid = true;
+		break;
+	}
+}
+
+static void pbr_nht_individual_nexthop_update_lookup(struct hash_bucket *b,
 						     void *data)
 {
 	struct pbr_nexthop_cache *pnhc = b->data;
@@ -642,19 +804,7 @@ static void pbr_nht_individual_nexthop_update_lookup(struct hash_backet *b,
 
 	old_valid = pnhc->valid;
 
-	switch (pnhi->nhr->prefix.family) {
-	case AF_INET:
-		if (pnhc->nexthop->gate.ipv4.s_addr
-		    == pnhi->nhr->prefix.u.prefix4.s_addr)
-			pnhc->valid = !!pnhi->nhr->nexthop_num;
-		break;
-	case AF_INET6:
-		if (memcmp(&pnhc->nexthop->gate.ipv6,
-			   &pnhi->nhr->prefix.u.prefix6, 16)
-		    == 0)
-			pnhc->valid = !!pnhi->nhr->nexthop_num;
-		break;
-	}
+	pbr_nht_individual_nexthop_update(pnhc, pnhi);
 
 	DEBUGD(&pbr_dbg_nht, "\tFound %s: old: %d new: %d",
 	       prefix2str(&pnhi->nhr->prefix, buf, sizeof(buf)), old_valid,
@@ -664,10 +814,30 @@ static void pbr_nht_individual_nexthop_update_lookup(struct hash_backet *b,
 		pnhi->valid += 1;
 }
 
-static void pbr_nht_nexthop_update_lookup(struct hash_backet *b, void *data)
+static void pbr_nexthop_group_cache_iterate_to_group(struct hash_bucket *b,
+						     void *data)
+{
+	struct pbr_nexthop_cache *pnhc = b->data;
+	struct nexthop_group *nhg = data;
+	struct nexthop *nh = NULL;
+
+	copy_nexthops(&nh, pnhc->nexthop, NULL);
+
+	_nexthop_add(&nhg->nexthop, nh);
+}
+
+static void
+pbr_nexthop_group_cache_to_nexthop_group(struct nexthop_group *nhg,
+					 struct pbr_nexthop_group_cache *pnhgc)
+{
+	hash_iterate(pnhgc->nhh, pbr_nexthop_group_cache_iterate_to_group, nhg);
+}
+
+static void pbr_nht_nexthop_update_lookup(struct hash_bucket *b, void *data)
 {
 	struct pbr_nexthop_group_cache *pnhgc = b->data;
-	struct pbr_nht_individual pnhi;
+	struct pbr_nht_individual pnhi = {};
+	struct nexthop_group nhg = {};
 	bool old_valid;
 
 	old_valid = pnhgc->valid;
@@ -682,6 +852,13 @@ static void pbr_nht_nexthop_update_lookup(struct hash_backet *b, void *data)
 	 */
 	pnhgc->valid = !!pnhi.valid;
 
+	if (pnhgc->valid) {
+		pbr_nexthop_group_cache_to_nexthop_group(&nhg, pnhgc);
+		pbr_nht_install_nexthop_group(pnhgc, nhg);
+		/* Don't need copied nexthops anymore */
+		nexthops_free(nhg.nexthop);
+	}
+
 	if (old_valid != pnhgc->valid)
 		pbr_map_check_nh_group_change(pnhgc->name);
 }
@@ -691,15 +868,62 @@ void pbr_nht_nexthop_update(struct zapi_route *nhr)
 	hash_iterate(pbr_nhg_hash, pbr_nht_nexthop_update_lookup, nhr);
 }
 
-static uint32_t pbr_nhg_hash_key(void *arg)
+static void
+pbr_nht_individual_nexthop_interface_update_lookup(struct hash_bucket *b,
+						   void *data)
 {
-	struct pbr_nexthop_group_cache *nhgc =
-		(struct pbr_nexthop_group_cache *)arg;
+	struct pbr_nexthop_cache *pnhc = b->data;
+	struct pbr_nht_individual *pnhi = data;
+	bool old_valid;
+
+	old_valid = pnhc->valid;
+
+	pbr_nht_individual_nexthop_update(pnhc, pnhi);
+
+	DEBUGD(&pbr_dbg_nht, "\tFound %s: old: %d new: %d", pnhi->ifp->name,
+	       old_valid, pnhc->valid);
+
+	if (pnhc->valid)
+		pnhi->valid += 1;
+}
+
+static void pbr_nht_nexthop_interface_update_lookup(struct hash_bucket *b,
+						    void *data)
+{
+	struct pbr_nexthop_group_cache *pnhgc = b->data;
+	struct pbr_nht_individual pnhi = {};
+	bool old_valid;
+
+	old_valid = pnhgc->valid;
+
+	pnhi.ifp = data;
+	pnhi.valid = 0;
+	hash_iterate(pnhgc->nhh,
+		     pbr_nht_individual_nexthop_interface_update_lookup, &pnhi);
+
+	/*
+	 * If any of the specified nexthops are valid we are valid
+	 */
+	pnhgc->valid = !!pnhi.valid;
+
+	if (old_valid != pnhgc->valid)
+		pbr_map_check_nh_group_change(pnhgc->name);
+}
+
+void pbr_nht_nexthop_interface_update(struct interface *ifp)
+{
+	hash_iterate(pbr_nhg_hash, pbr_nht_nexthop_interface_update_lookup,
+		     ifp);
+}
+
+static uint32_t pbr_nhg_hash_key(const void *arg)
+{
+	const struct pbr_nexthop_group_cache *nhgc = arg;
 
 	return jhash(&nhgc->name, strlen(nhgc->name), 0x52c34a96);
 }
 
-static int pbr_nhg_hash_equal(const void *arg1, const void *arg2)
+static bool pbr_nhg_hash_equal(const void *arg1, const void *arg2)
 {
 	const struct pbr_nexthop_group_cache *nhgc1 =
 		(const struct pbr_nexthop_group_cache *)arg1;
@@ -709,21 +933,20 @@ static int pbr_nhg_hash_equal(const void *arg1, const void *arg2)
 	return !strcmp(nhgc1->name, nhgc2->name);
 }
 
-
-uint32_t pbr_nht_get_next_tableid(void)
+uint32_t pbr_nht_get_next_tableid(bool peek)
 {
 	uint32_t i;
 	bool found = false;
 
 	for (i = pbr_nhg_low_table; i <= pbr_nhg_high_table; i++) {
-		if (nhg_tableid[i] == false) {
+		if (!nhg_tableid[i]) {
 			found = true;
 			break;
 		}
 	}
 
 	if (found) {
-		nhg_tableid[i] = true;
+		nhg_tableid[i] = !peek;
 		return i;
 	} else
 		return 0;
@@ -775,7 +998,7 @@ uint32_t pbr_nht_get_table(const char *name)
 	if (!pnhgc) {
 		DEBUGD(&pbr_dbg_nht,
 		       "%s: Could not find nexthop-group cache w/ name '%s'",
-		       __PRETTY_FUNCTION__, name);
+		       __func__, name);
 		return 5000;
 	}
 
@@ -798,7 +1021,7 @@ bool pbr_nht_get_installed(const char *name)
 	return pnhgc->installed;
 }
 
-static void pbr_nht_show_nhg_nexthops(struct hash_backet *b, void *data)
+static void pbr_nht_show_nhg_nexthops(struct hash_bucket *b, void *data)
 {
 	struct pbr_nexthop_cache *pnhc = b->data;
 	struct vty *vty = data;
@@ -812,7 +1035,7 @@ struct pbr_nht_show {
 	const char *name;
 };
 
-static void pbr_nht_show_nhg(struct hash_backet *b, void *data)
+static void pbr_nht_show_nhg(struct hash_bucket *b, void *data)
 {
 	struct pbr_nexthop_group_cache *pnhgc = b->data;
 	struct pbr_nht_show *pns = data;
@@ -843,7 +1066,7 @@ void pbr_nht_init(void)
 	pbr_nhg_hash = hash_create_size(
 		16, pbr_nhg_hash_key, pbr_nhg_hash_equal, "PBR NHG Cache Hash");
 	pbr_nhrc_hash =
-		hash_create_size(16, (unsigned int (*)(void *))nexthop_hash,
+		hash_create_size(16, (unsigned int (*)(const void *))nexthop_hash,
 				 pbr_nhrc_hash_equal, "PBR NH Hash");
 
 	pbr_nhg_low_table = PBR_NHT_DEFAULT_LOW_TABLEID;
