@@ -25,9 +25,10 @@
 #include "vrf.h"
 #include "nexthop.h"
 #include "nexthop_group.h"
+#include "nexthop_group_private.h"
 #include "log.h"
-#include "json.h"
 #include "debug.h"
+#include "pbr.h"
 
 #include "pbrd/pbr_nht.h"
 #include "pbrd/pbr_map.h"
@@ -38,7 +39,7 @@
 #include "pbrd/pbr_vty_clippy.c"
 #endif
 
-DEFUN_NOSH(pbr_map, pbr_map_cmd, "pbr-map WORD seq (1-1000)",
+DEFUN_NOSH(pbr_map, pbr_map_cmd, "pbr-map PBRMAP seq (1-700)",
 	   "Create pbr-map or enter pbr-map command mode\n"
 	   "The name of the PBR MAP\n"
 	   "Sequence to insert in existing pbr-map entry\n"
@@ -54,7 +55,7 @@ DEFUN_NOSH(pbr_map, pbr_map_cmd, "pbr-map WORD seq (1-1000)",
 	return CMD_SUCCESS;
 }
 
-DEFUN_NOSH(no_pbr_map, no_pbr_map_cmd, "no pbr-map WORD [seq (1-65535)]",
+DEFUN_NOSH(no_pbr_map, no_pbr_map_cmd, "no pbr-map PBRMAP [seq (1-700)]",
 	   NO_STR
 	   "Delete pbr-map\n"
 	   "The name of the PBR MAP\n"
@@ -85,6 +86,46 @@ DEFUN_NOSH(no_pbr_map, no_pbr_map_cmd, "no pbr-map WORD [seq (1-65535)]",
 	return CMD_SUCCESS;
 }
 
+DEFPY(pbr_set_table_range,
+      pbr_set_table_range_cmd,
+      "pbr table range (10000-4294966272)$lb (10000-4294966272)$ub",
+      PBR_STR
+      "Set table ID range\n"
+      "Set table ID range\n"
+      "Lower bound for table ID range\n"
+      "Upper bound for table ID range\n")
+{
+	/* upper bound is 2^32 - 2^10 */
+	int ret = CMD_WARNING;
+	const int minrange = 1000;
+
+	/* validate given bounds */
+	if (lb > ub)
+		vty_out(vty, "%% Lower bound must be less than upper bound\n");
+	else if (ub - lb < minrange)
+		vty_out(vty, "%% Range breadth must be at least %d\n", minrange);
+	else {
+		ret = CMD_SUCCESS;
+		pbr_nht_set_tableid_range((uint32_t) lb, (uint32_t) ub);
+	}
+
+	return ret;
+}
+
+DEFPY(no_pbr_set_table_range, no_pbr_set_table_range_cmd,
+      "no pbr table range [(10000-4294966272)$lb (10000-4294966272)$ub]",
+      NO_STR
+      PBR_STR
+      "Set table ID range\n"
+      "Set table ID range\n"
+      "Lower bound for table ID range\n"
+      "Upper bound for table ID range\n")
+{
+	pbr_nht_set_tableid_range(PBR_NHT_DEFAULT_LOW_TABLEID,
+				  PBR_NHT_DEFAULT_HIGH_TABLEID);
+	return CMD_SUCCESS;
+}
+
 DEFPY(pbr_map_match_src, pbr_map_match_src_cmd,
 	"[no] match src-ip <A.B.C.D/M|X:X::X:X/M>$prefix",
 	NO_STR
@@ -98,18 +139,17 @@ DEFPY(pbr_map_match_src, pbr_map_match_src_cmd,
 	pbrms->family = prefix->family;
 
 	if (!no) {
-		if (prefix_same(pbrms->src, prefix))
-			return CMD_SUCCESS;
-
-		if (!pbrms->src)
+		if (pbrms->src) {
+			if (prefix_same(pbrms->src, prefix))
+				return CMD_SUCCESS;
+		} else
 			pbrms->src = prefix_new();
-		prefix_copy(pbrms->src, prefix);
-	} else {
-		prefix_free(pbrms->src);
-		pbrms->src = 0;
-	}
 
-	pbr_map_check(pbrms);
+		prefix_copy(pbrms->src, prefix);
+	} else
+		prefix_free(&pbrms->src);
+
+	pbr_map_check(pbrms, true);
 
 	return CMD_SUCCESS;
 }
@@ -118,7 +158,7 @@ DEFPY(pbr_map_match_dst, pbr_map_match_dst_cmd,
 	"[no] match dst-ip <A.B.C.D/M|X:X::X:X/M>$prefix",
 	NO_STR
 	"Match the rest of the command\n"
-	"Choose the src ip or ipv6 prefix to use\n"
+	"Choose the dst ip or ipv6 prefix to use\n"
 	"v4 Prefix\n"
 	"v6 Prefix\n")
 {
@@ -127,192 +167,296 @@ DEFPY(pbr_map_match_dst, pbr_map_match_dst_cmd,
 	pbrms->family = prefix->family;
 
 	if (!no) {
-		if (prefix_same(pbrms->dst, prefix))
-			return CMD_SUCCESS;
-
-		if (!pbrms->dst)
+		if (pbrms->dst) {
+			if (prefix_same(pbrms->dst, prefix))
+				return CMD_SUCCESS;
+		} else
 			pbrms->dst = prefix_new();
-		prefix_copy(pbrms->dst, prefix);
-	} else {
-		prefix_free(pbrms->dst);
-		pbrms->dst = NULL;
-	}
 
-	pbr_map_check(pbrms);
+		prefix_copy(pbrms->dst, prefix);
+	} else
+		prefix_free(&pbrms->dst);
+
+	pbr_map_check(pbrms, true);
 
 	return CMD_SUCCESS;
 }
 
-DEFPY(pbr_map_nexthop_group, pbr_map_nexthop_group_cmd,
-	"[no] set nexthop-group NAME$name",
+DEFPY(pbr_map_match_mark, pbr_map_match_mark_cmd,
+	"[no] match mark (1-4294967295)$mark",
 	NO_STR
-	"Set for the PBR-MAP\n"
-	"nexthop-group to use\n"
-	"The name of the nexthop-group\n")
+	"Match the rest of the command\n"
+	"Choose the mark value to use\n"
+	"mark\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+#ifndef GNU_LINUX
+	vty_out(vty, "pbr marks are not supported on this platform");
+	return CMD_WARNING_CONFIG_FAILED;
+#endif
+
+	if (!no) {
+		if (pbrms->mark)
+			if (pbrms->mark == (uint32_t)mark)
+				return CMD_SUCCESS;
+
+		pbrms->mark = (uint32_t)mark;
+	} else
+		pbrms->mark = 0;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+static void pbrms_clear_set_vrf_config(struct pbr_map_sequence *pbrms)
+{
+	if (pbrms->vrf_lookup || pbrms->vrf_unchanged) {
+		pbr_map_delete_vrf(pbrms);
+		pbrms->vrf_name[0] = '\0';
+		pbrms->vrf_lookup = false;
+		pbrms->vrf_unchanged = false;
+	}
+}
+
+static void pbrms_clear_set_nhg_config(struct pbr_map_sequence *pbrms)
+{
+	if (pbrms->nhgrp_name)
+		pbr_map_delete_nexthops(pbrms);
+}
+
+static void pbrms_clear_set_nexthop_config(struct pbr_map_sequence *pbrms)
+{
+	if (pbrms->nhg)
+		pbr_nht_delete_individual_nexthop(pbrms);
+}
+
+static void pbrms_clear_set_config(struct pbr_map_sequence *pbrms)
+{
+	pbrms_clear_set_vrf_config(pbrms);
+	pbrms_clear_set_nhg_config(pbrms);
+	pbrms_clear_set_nexthop_config(pbrms);
+
+	pbrms->nhs_installed = false;
+}
+
+DEFPY(pbr_map_nexthop_group, pbr_map_nexthop_group_cmd,
+      "set nexthop-group NHGNAME$name",
+      "Set for the PBR-MAP\n"
+      "nexthop-group to use\n"
+      "The name of the nexthop-group\n")
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 	struct nexthop_group_cmd *nhgc;
 
-	if (pbrms->nhg) {
-		vty_out(vty,
-			"A `set nexthop XX` command already exists, please remove that first\n");
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
 	nhgc = nhgc_find(name);
 	if (!nhgc) {
 		vty_out(vty, "Specified nexthop-group %s does not exist\n",
 			name);
-		vty_out(vty, "PBR-MAP will not be applied until it is created\n");
+		vty_out(vty,
+			"PBR-MAP will not be applied until it is created\n");
 	}
 
-	if (no) {
-		if (pbrms->nhgrp_name && strcmp(name, pbrms->nhgrp_name) == 0)
-			pbr_map_delete_nexthop_group(pbrms);
-		else {
-			vty_out(vty,
-				"Nexthop Group specified: %s does not exist to remove",
-				name);
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-	} else {
-		if (pbrms->nhgrp_name) {
-			if (strcmp(name, pbrms->nhgrp_name) != 0) {
-				vty_out(vty,
-					"Please delete current nexthop group before modifying current one");
-				return CMD_WARNING_CONFIG_FAILED;
-			}
+	if (pbrms->nhgrp_name && strcmp(name, pbrms->nhgrp_name) == 0)
+		return CMD_SUCCESS;
 
-			return CMD_SUCCESS;
-		}
-		pbrms->nhgrp_name = XSTRDUP(MTYPE_TMP, name);
-		pbr_map_check(pbrms);
-	}
+	/* This is new/replacement config */
+	pbrms_clear_set_config(pbrms);
+
+	pbrms->nhgrp_name = XSTRDUP(MTYPE_TMP, name);
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(no_pbr_map_nexthop_group, no_pbr_map_nexthop_group_cmd,
+      "no set nexthop-group [NHGNAME$name]",
+      NO_STR
+      "Set for the PBR-MAP\n"
+      "nexthop-group to use\n"
+      "The name of the nexthop-group\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	pbrms_clear_set_config(pbrms);
 
 	return CMD_SUCCESS;
 }
 
 DEFPY(pbr_map_nexthop, pbr_map_nexthop_cmd,
-      "[no] set nexthop <A.B.C.D|X:X::X:X>$addr [INTERFACE]$intf [nexthop-vrf NAME$name]",
-      NO_STR
+      "set nexthop\
+        <\
+	  <A.B.C.D|X:X::X:X>$addr [INTERFACE$intf]\
+	  |INTERFACE$intf\
+	>\
+        [nexthop-vrf NAME$vrf_name]",
       "Set for the PBR-MAP\n"
       "Specify one of the nexthops in this map\n"
       "v4 Address\n"
       "v6 Address\n"
       "Interface to use\n"
+      "Interface to use\n"
       "If the nexthop is in a different vrf tell us\n"
       "The nexthop-vrf Name\n")
 {
 	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 	struct vrf *vrf;
 	struct nexthop nhop;
-	struct nexthop *nh;
+	struct nexthop *nh = NULL;
 
-	if (pbrms->nhgrp_name) {
-		vty_out(vty,
-			"Please unconfigure the nexthop group before adding an individual nexthop");
-		return CMD_WARNING_CONFIG_FAILED;
-	}
-
-	if (name)
-		vrf = vrf_lookup_by_name(name);
+	if (vrf_name)
+		vrf = vrf_lookup_by_name(vrf_name);
 	else
 		vrf = vrf_lookup_by_id(VRF_DEFAULT);
 
 	if (!vrf) {
-		vty_out(vty, "Specified: %s is non-existent\n", name);
+		vty_out(vty, "Specified: %s is non-existent\n", vrf_name);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	memset(&nhop, 0, sizeof(nhop));
 	nhop.vrf_id = vrf->vrf_id;
 
-	/*
-	 * Make SA happy.  CLIPPY is not going to give us a NULL
-	 * addr.
-	 */
-	assert(addr);
-	if (addr->sa.sa_family == AF_INET) {
-		nhop.gate.ipv4.s_addr = addr->sin.sin_addr.s_addr;
-		if (intf) {
-			nhop.type = NEXTHOP_TYPE_IPV4_IFINDEX;
-			nhop.ifindex = ifname2ifindex(intf, vrf->vrf_id);
-			if (nhop.ifindex == IFINDEX_INTERNAL) {
-				vty_out(vty,
-					"Specified Intf %s does not exist in vrf: %s\n",
-					intf, vrf->name);
-				return CMD_WARNING_CONFIG_FAILED;
-			}
-		} else
-			nhop.type = NEXTHOP_TYPE_IPV4;
-	} else {
-		memcpy(&nhop.gate.ipv6, &addr->sin6.sin6_addr, 16);
-		if (intf) {
-			nhop.type = NEXTHOP_TYPE_IPV6_IFINDEX;
-			nhop.ifindex = ifname2ifindex(intf, vrf->vrf_id);
-			if (nhop.ifindex == IFINDEX_INTERNAL) {
-				vty_out(vty,
-					"Specified Intf %s does not exist in vrf: %s\n",
-					intf, vrf->name);
-				return CMD_WARNING_CONFIG_FAILED;
-			}
-		} else {
-			if (IN6_IS_ADDR_LINKLOCAL(&nhop.gate.ipv6)) {
-				vty_out(vty,
-					"Specified a v6 LL with no interface, rejecting\n");
-				return CMD_WARNING_CONFIG_FAILED;
-			}
-			nhop.type = NEXTHOP_TYPE_IPV6;
+	if (intf) {
+		nhop.ifindex = ifname2ifindex(intf, vrf->vrf_id);
+		if (nhop.ifindex == IFINDEX_INTERNAL) {
+			vty_out(vty,
+				"Specified Intf %s does not exist in vrf: %s\n",
+				intf, vrf->name);
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 	}
 
+	if (addr) {
+		if (addr->sa.sa_family == AF_INET) {
+			nhop.gate.ipv4.s_addr = addr->sin.sin_addr.s_addr;
+			if (intf)
+				nhop.type = NEXTHOP_TYPE_IPV4_IFINDEX;
+			else
+				nhop.type = NEXTHOP_TYPE_IPV4;
+		} else {
+			nhop.gate.ipv6 = addr->sin6.sin6_addr;
+			if (intf)
+				nhop.type = NEXTHOP_TYPE_IPV6_IFINDEX;
+			else {
+				if (IN6_IS_ADDR_LINKLOCAL(&nhop.gate.ipv6)) {
+					vty_out(vty,
+						"Specified a v6 LL with no interface, rejecting\n");
+					return CMD_WARNING_CONFIG_FAILED;
+				}
+				nhop.type = NEXTHOP_TYPE_IPV6;
+			}
+		}
+	} else
+		nhop.type = NEXTHOP_TYPE_IFINDEX;
+
 	if (pbrms->nhg)
 		nh = nexthop_exists(pbrms->nhg, &nhop);
-	else {
-		char buf[PBR_NHC_NAMELEN];
 
-		if (no) {
-			vty_out(vty, "No nexthops to delete");
-			return CMD_WARNING_CONFIG_FAILED;
-		}
+	if (nh) /* Same config re-entered */
+		goto done;
 
-		pbrms->nhg = nexthop_group_new();
-		pbrms->internal_nhg_name =
-			XSTRDUP(MTYPE_TMP,
-				pbr_nht_nexthop_make_name(pbrms->parent->name,
-							  PBR_NHC_NAMELEN,
-							  pbrms->seqno,
-							  buf));
-		nh = NULL;
+	/* This is new/replacement config */
+	pbrms_clear_set_config(pbrms);
+
+	pbr_nht_add_individual_nexthop(pbrms, &nhop);
+
+	pbr_map_check(pbrms, true);
+
+done:
+	if (nhop.type == NEXTHOP_TYPE_IFINDEX
+	    || (nhop.type == NEXTHOP_TYPE_IPV6_IFINDEX
+		&& IN6_IS_ADDR_LINKLOCAL(&nhop.gate.ipv6))) {
+		struct interface *ifp;
+
+		ifp = if_lookup_by_index(nhop.ifindex, nhop.vrf_id);
+		if (ifp)
+			pbr_nht_nexthop_interface_update(ifp);
 	}
 
-	if (no) {
-		if (nh)
-			pbr_nht_delete_individual_nexthop(pbrms);
-	} else if (!nh) {
+	return CMD_SUCCESS;
+}
 
-		if (pbrms->nhg->nexthop) {
-			vty_out(vty,
-				"If you would like more than one nexthop please use nexthop-groups");
-			return CMD_WARNING_CONFIG_FAILED;
-		}
+DEFPY(no_pbr_map_nexthop, no_pbr_map_nexthop_cmd,
+      "no set nexthop\
+        [<\
+	  <A.B.C.D|X:X::X:X>$addr [INTERFACE$intf]\
+	  |INTERFACE$intf\
+	>\
+        [nexthop-vrf NAME$vrf_name]]",
+      NO_STR
+      "Set for the PBR-MAP\n"
+      "Specify one of the nexthops in this map\n"
+      "v4 Address\n"
+      "v6 Address\n"
+      "Interface to use\n"
+      "Interface to use\n"
+      "If the nexthop is in a different vrf tell us\n"
+      "The nexthop-vrf Name\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
 
-		/* must be adding new nexthop since !no and !nexthop_exists */
-		nh = nexthop_new();
+	pbrms_clear_set_config(pbrms);
 
-		memcpy(nh, &nhop, sizeof(nhop));
-		nexthop_add(&pbrms->nhg->nexthop, nh);
+	return CMD_SUCCESS;
+}
 
-		pbr_nht_add_individual_nexthop(pbrms);
-		pbr_map_check(pbrms);
+DEFPY(pbr_map_vrf, pbr_map_vrf_cmd,
+      "set vrf <NAME$vrf_name|unchanged>",
+      "Set for the PBR-MAP\n"
+      "Specify the VRF for this map\n"
+      "The VRF Name\n"
+      "Use the interface's VRF for lookup\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	/*
+	 * If an equivalent set vrf * exists, just return success.
+	 */
+	if (vrf_name && pbrms->vrf_lookup
+	    && strncmp(pbrms->vrf_name, vrf_name, sizeof(pbrms->vrf_name)) == 0)
+		return CMD_SUCCESS;
+	else if (!vrf_name && pbrms->vrf_unchanged) /* Unchanged already set */
+		return CMD_SUCCESS;
+
+	if (vrf_name && !pbr_vrf_lookup_by_name(vrf_name)) {
+		vty_out(vty, "Specified: %s is non-existent\n", vrf_name);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
+	/* This is new/replacement config */
+	pbrms_clear_set_config(pbrms);
+
+	if (vrf_name) {
+		pbrms->vrf_lookup = true;
+		strlcpy(pbrms->vrf_name, vrf_name, sizeof(pbrms->vrf_name));
+	} else
+		pbrms->vrf_unchanged = true;
+
+	pbr_map_check(pbrms, true);
+
+	return CMD_SUCCESS;
+}
+
+DEFPY(no_pbr_map_vrf, no_pbr_map_vrf_cmd,
+      "no set vrf [<NAME$vrf_name|unchanged>]",
+      NO_STR
+      "Set for the PBR-MAP\n"
+      "Specify the VRF for this map\n"
+      "The VRF Name\n"
+      "Use the interface's VRF for lookup\n")
+{
+	struct pbr_map_sequence *pbrms = VTY_GET_CONTEXT(pbr_map_sequence);
+
+	pbrms_clear_set_config(pbrms);
+
 	return CMD_SUCCESS;
 }
 
 DEFPY (pbr_policy,
 	pbr_policy_cmd,
-	"[no] pbr-policy NAME$mapname",
+	"[no] pbr-policy PBRMAP$mapname",
 	NO_STR
 	"Policy to use\n"
 	"Name of the pbr-map to apply\n")
@@ -321,6 +465,7 @@ DEFPY (pbr_policy,
 	struct pbr_map *pbrm, *old_pbrm;
 	struct pbr_interface *pbr_ifp = ifp->info;
 
+	old_pbrm = NULL;
 	pbrm = pbrm_find(mapname);
 
 	if (!pbr_ifp) {
@@ -341,12 +486,23 @@ DEFPY (pbr_policy,
 	} else {
 		if (strcmp(pbr_ifp->mapname, "") != 0) {
 			old_pbrm = pbrm_find(pbr_ifp->mapname);
-			if (old_pbrm)
+
+			/*
+			 * So if we have an old pbrm we should only
+			 * delete it if we are actually deleting and
+			 * moving to a new pbrm
+			 */
+			if (old_pbrm && old_pbrm != pbrm)
 				pbr_map_interface_delete(old_pbrm, ifp);
 		}
 		snprintf(pbr_ifp->mapname, sizeof(pbr_ifp->mapname),
 			 "%s", mapname);
-		if (pbrm)
+
+		/*
+		 * So only reinstall if the old_pbrm and this pbrm are
+		 * different.
+		 */
+		if (pbrm && pbrm != old_pbrm)
 			pbr_map_add_interface(pbrm, ifp);
 	}
 
@@ -355,10 +511,9 @@ DEFPY (pbr_policy,
 
 DEFPY (show_pbr,
 	show_pbr_cmd,
-	"show pbr [json$json]",
+	"show pbr",
 	SHOW_STR
-	"Policy Based Routing\n"
-	JSON_STR)
+	PBR_STR)
 {
 	pbr_nht_write_table_range(vty);
 	pbr_nht_write_rule_range(vty);
@@ -366,71 +521,104 @@ DEFPY (show_pbr,
 	return CMD_SUCCESS;
 }
 
-DEFPY (show_pbr_map,
-	show_pbr_map_cmd,
-	"show pbr map [NAME$name] [detail$detail] [json$json]",
-	SHOW_STR
-	"Policy Based Routing\n"
-	"PBR Map\n"
-	"PBR Map Name\n"
-	"Detailed information\n"
-	JSON_STR)
+static void vty_show_pbrms(struct vty *vty,
+			   const struct pbr_map_sequence *pbrms, bool detail)
 {
-	struct pbr_map_sequence *pbrms;
-	struct pbr_map *pbrm;
-	struct listnode *node;
 	char buf[PREFIX_STRLEN];
 	char rbuf[64];
 
+	if (pbrms->reason)
+		pbr_map_reason_string(pbrms->reason, rbuf, sizeof(rbuf));
+
+	vty_out(vty, "    Seq: %u rule: %u\n", pbrms->seqno, pbrms->ruleno);
+
+	if (detail)
+		vty_out(vty, "        Installed: %" PRIu64 "(%u) Reason: %s\n",
+			pbrms->installed, pbrms->unique,
+			pbrms->reason ? rbuf : "Valid");
+	else
+		vty_out(vty, "        Installed: %s Reason: %s\n",
+			pbrms->installed ? "yes" : "no",
+			pbrms->reason ? rbuf : "Valid");
+
+	if (pbrms->src)
+		vty_out(vty, "        SRC Match: %s\n",
+			prefix2str(pbrms->src, buf, sizeof(buf)));
+	if (pbrms->dst)
+		vty_out(vty, "        DST Match: %s\n",
+			prefix2str(pbrms->dst, buf, sizeof(buf)));
+	if (pbrms->mark)
+		vty_out(vty, "        MARK Match: %u\n", pbrms->mark);
+
+	if (pbrms->nhgrp_name) {
+		vty_out(vty, "        Nexthop-Group: %s\n", pbrms->nhgrp_name);
+
+		if (detail)
+			vty_out(vty,
+				"          Installed: %u(%d) Tableid: %d\n",
+				pbrms->nhs_installed,
+				pbr_nht_get_installed(pbrms->nhgrp_name),
+				pbr_nht_get_table(pbrms->nhgrp_name));
+		else
+			vty_out(vty, "          Installed: %s Tableid: %d\n",
+				pbr_nht_get_installed(pbrms->nhgrp_name) ? "yes"
+									 : "no",
+				pbr_nht_get_table(pbrms->nhgrp_name));
+
+	} else if (pbrms->nhg) {
+		vty_out(vty, "        ");
+		nexthop_group_write_nexthop(vty, pbrms->nhg->nexthop);
+		if (detail)
+			vty_out(vty,
+				"          Installed: %u(%d) Tableid: %d\n",
+				pbrms->nhs_installed,
+				pbr_nht_get_installed(pbrms->internal_nhg_name),
+				pbr_nht_get_table(pbrms->internal_nhg_name));
+		else
+			vty_out(vty, "          Installed: %s Tableid: %d\n",
+				pbr_nht_get_installed(pbrms->internal_nhg_name)
+					? "yes"
+					: "no",
+				pbr_nht_get_table(pbrms->internal_nhg_name));
+
+	} else if (pbrms->vrf_unchanged) {
+		vty_out(vty, "        VRF Unchanged (use interface vrf)\n");
+	} else if (pbrms->vrf_lookup) {
+		vty_out(vty, "        VRF Lookup: %s\n", pbrms->vrf_name);
+	} else {
+		vty_out(vty, "        Nexthop-Group: Unknown Installed: no\n");
+	}
+}
+
+static void vty_show_pbr_map(struct vty *vty, const struct pbr_map *pbrm,
+			     bool detail)
+{
+	struct pbr_map_sequence *pbrms;
+	struct listnode *node;
+
+	vty_out(vty, "  pbr-map %s valid: %s\n", pbrm->name,
+		pbrm->valid ? "yes" : "no");
+
+	for (ALL_LIST_ELEMENTS_RO(pbrm->seqnumbers, node, pbrms))
+		vty_show_pbrms(vty, pbrms, detail);
+}
+
+DEFPY (show_pbr_map,
+	show_pbr_map_cmd,
+	"show pbr map [NAME$name] [detail$detail]",
+	SHOW_STR
+	PBR_STR
+	"PBR Map\n"
+	"PBR Map Name\n"
+	"Detailed information\n")
+{
+	struct pbr_map *pbrm;
+
 	RB_FOREACH (pbrm, pbr_map_entry_head, &pbr_maps) {
 		if (name && strcmp(name, pbrm->name) != 0)
 			continue;
 
-		vty_out(vty, "  pbr-map %s valid: %d\n", pbrm->name,
-			pbrm->valid);
-
-		for (ALL_LIST_ELEMENTS_RO(pbrm->seqnumbers, node, pbrms)) {
-			if (pbrms->reason)
-				pbr_map_reason_string(pbrms->reason, rbuf,
-						      sizeof(rbuf));
-			vty_out(vty,
-				"    Seq: %u rule: %u Installed: %" PRIu64 "(%u) Reason: %s\n",
-				pbrms->seqno, pbrms->ruleno, pbrms->installed,
-				pbrms->unique, pbrms->reason ? rbuf : "Valid");
-
-			if (pbrms->src)
-				vty_out(vty, "\tSRC Match: %s\n",
-					prefix2str(pbrms->src, buf,
-						   sizeof(buf)));
-			if (pbrms->dst)
-				vty_out(vty, "\tDST Match: %s\n",
-					prefix2str(pbrms->dst, buf,
-						   sizeof(buf)));
-
-			if (pbrms->nhgrp_name) {
-				vty_out(vty,
-					"\tNexthop-Group: %s(%u) Installed: %u(%d)\n",
-					pbrms->nhgrp_name,
-					pbr_nht_get_table(pbrms->nhgrp_name),
-					pbrms->nhs_installed,
-					pbr_nht_get_installed(
-						pbrms->nhgrp_name));
-			} else if (pbrms->nhg) {
-				vty_out(vty, "     ");
-				nexthop_group_write_nexthop(
-					vty, pbrms->nhg->nexthop);
-				vty_out(vty,
-					"\tInstalled: %u(%d) Tableid: %d\n",
-					pbrms->nhs_installed,
-					pbr_nht_get_installed(
-						pbrms->internal_nhg_name),
-					pbr_nht_get_table(
-						pbrms->internal_nhg_name));
-			} else {
-				vty_out(vty,
-					"\tNexthop-Group: Unknown Installed: 0(0)\n");
-			}
-		}
+		vty_show_pbr_map(vty, pbrm, detail);
 	}
 	return CMD_SUCCESS;
 }
@@ -439,7 +627,7 @@ DEFPY(show_pbr_nexthop_group,
       show_pbr_nexthop_group_cmd,
       "show pbr nexthop-groups [WORD$word]",
       SHOW_STR
-      "Policy Based Routing\n"
+      PBR_STR
       "Nexthop Groups\n"
       "Optional Name of the nexthop group\n")
 {
@@ -450,12 +638,11 @@ DEFPY(show_pbr_nexthop_group,
 
 DEFPY (show_pbr_interface,
 	show_pbr_interface_cmd,
-	"show pbr interface [NAME$name] [json$json]",
+	"show pbr interface [NAME$name]",
 	SHOW_STR
-	"Policy Based Routing\n"
+	PBR_STR
 	"PBR Interface\n"
-	"PBR Interface Name\n"
-	JSON_STR)
+	"PBR Interface Name\n")
 {
 	struct interface *ifp;
 	struct vrf *vrf;
@@ -489,16 +676,20 @@ DEFPY (show_pbr_interface,
 }
 
 /* PBR debugging CLI ------------------------------------------------------- */
-/* clang-format off */
 
-static struct cmd_node debug_node = {DEBUG_NODE, "", 1};
+static struct cmd_node debug_node = {
+	.name = "debug",
+	.node = DEBUG_NODE,
+	.prompt = "",
+	.config_write = pbr_debug_config_write,
+};
 
 DEFPY(debug_pbr,
       debug_pbr_cmd,
       "[no] debug pbr [{map$map|zebra$zebra|nht$nht|events$events}]",
       NO_STR
       DEBUG_STR
-      "Policy Based Routing\n"
+      PBR_STR
       "Policy maps\n"
       "PBRD <-> Zebra communications\n"
       "Nexthop tracking\n"
@@ -527,7 +718,7 @@ DEFUN_NOSH(show_debugging_pbr,
 	   "show debugging [pbr]",
 	   SHOW_STR
 	   DEBUG_STR
-	   "Policy Based Routing\n")
+	   PBR_STR)
 {
 	vty_out(vty, "PBR debugging status:\n");
 
@@ -536,12 +727,16 @@ DEFUN_NOSH(show_debugging_pbr,
 	return CMD_SUCCESS;
 }
 
-/* clang-format on */
 /* ------------------------------------------------------------------------- */
 
 
+static int pbr_interface_config_write(struct vty *vty);
 static struct cmd_node interface_node = {
-	INTERFACE_NODE, "%s(config-if)# ", 1 /* vtysh ? yes */
+	.name = "interface",
+	.node = INTERFACE_NODE,
+	.parent_node = CONFIG_NODE,
+	.prompt = "%s(config-if)# ",
+	.config_write = pbr_interface_config_write,
 };
 
 static int pbr_interface_config_write(struct vty *vty)
@@ -557,6 +752,9 @@ static int pbr_interface_config_write(struct vty *vty)
 				vty_frame(vty, "interface %s vrf %s\n",
 					  ifp->name, vrf->name);
 
+			if (ifp->desc)
+				vty_out(vty, " description %s\n", ifp->desc);
+
 			pbr_map_write_interfaces(vty, ifp);
 
 			vty_endframe(vty, "!\n");
@@ -566,8 +764,15 @@ static int pbr_interface_config_write(struct vty *vty)
 	return 1;
 }
 
+static int pbr_vty_map_config_write(struct vty *vty);
 /* PBR map node structure. */
-static struct cmd_node pbr_map_node = {PBRMAP_NODE, "%s(config-pbr-map)# ", 1};
+static struct cmd_node pbr_map_node = {
+	.name = "pbr-map",
+	.node = PBRMAP_NODE,
+	.parent_node = CONFIG_NODE,
+	.prompt = "%s(config-pbr-map)# ",
+	.config_write = pbr_vty_map_config_write,
+};
 
 static int pbr_vty_map_config_write_sequence(struct vty *vty,
 					     struct pbr_map *pbrm,
@@ -578,18 +783,27 @@ static int pbr_vty_map_config_write_sequence(struct vty *vty,
 	vty_out(vty, "pbr-map %s seq %u\n", pbrm->name, pbrms->seqno);
 
 	if (pbrms->src)
-		vty_out(vty, "  match src-ip %s\n",
+		vty_out(vty, " match src-ip %s\n",
 			prefix2str(pbrms->src, buff, sizeof(buff)));
 
 	if (pbrms->dst)
-		vty_out(vty, "  match dst-ip %s\n",
+		vty_out(vty, " match dst-ip %s\n",
 			prefix2str(pbrms->dst, buff, sizeof(buff)));
 
+	if (pbrms->mark)
+		vty_out(vty, " match mark %u\n", pbrms->mark);
+
+	if (pbrms->vrf_unchanged)
+		vty_out(vty, " set vrf unchanged\n");
+
+	if (pbrms->vrf_lookup)
+		vty_out(vty, " set vrf %s\n", pbrms->vrf_name);
+
 	if (pbrms->nhgrp_name)
-		vty_out(vty, "  set nexthop-group %s\n", pbrms->nhgrp_name);
+		vty_out(vty, " set nexthop-group %s\n", pbrms->nhgrp_name);
 
 	if (pbrms->nhg) {
-		vty_out(vty, "  set ");
+		vty_out(vty, " set ");
 		nexthop_group_write_nexthop(vty, pbrms->nhg->nexthop);
 	}
 
@@ -615,17 +829,34 @@ static int pbr_vty_map_config_write(struct vty *vty)
 	return 1;
 }
 
+static void pbr_map_completer(vector comps, struct cmd_token *token)
+{
+	struct pbr_map *pbrm;
+
+	RB_FOREACH (pbrm, pbr_map_entry_head, &pbr_maps)
+		vector_set(comps, XSTRDUP(MTYPE_COMPLETION, pbrm->name));
+}
+
+static const struct cmd_variable_handler pbr_map_name[] = {
+	{
+		.tokenname = "PBRMAP", .completions = pbr_map_completer,
+	},
+	{
+		.completions = NULL
+	}
+};
+
 void pbr_vty_init(void)
 {
-	install_node(&interface_node,
-		     pbr_interface_config_write);
+	cmd_variable_handler_register(pbr_map_name);
+
+	install_node(&interface_node);
 	if_cmd_init();
 
-	install_node(&pbr_map_node,
-		     pbr_vty_map_config_write);
+	install_node(&pbr_map_node);
 
 	/* debug */
-	install_node(&debug_node, pbr_debug_config_write);
+	install_node(&debug_node);
 	install_element(VIEW_NODE, &debug_pbr_cmd);
 	install_element(CONFIG_NODE, &debug_pbr_cmd);
 	install_element(VIEW_NODE, &show_debugging_pbr_cmd);
@@ -634,11 +865,18 @@ void pbr_vty_init(void)
 
 	install_element(CONFIG_NODE, &pbr_map_cmd);
 	install_element(CONFIG_NODE, &no_pbr_map_cmd);
+	install_element(CONFIG_NODE, &pbr_set_table_range_cmd);
+	install_element(CONFIG_NODE, &no_pbr_set_table_range_cmd);
 	install_element(INTERFACE_NODE, &pbr_policy_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_match_src_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_match_dst_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_match_mark_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_nexthop_group_cmd);
+	install_element(PBRMAP_NODE, &no_pbr_map_nexthop_group_cmd);
 	install_element(PBRMAP_NODE, &pbr_map_nexthop_cmd);
+	install_element(PBRMAP_NODE, &no_pbr_map_nexthop_cmd);
+	install_element(PBRMAP_NODE, &pbr_map_vrf_cmd);
+	install_element(PBRMAP_NODE, &no_pbr_map_vrf_cmd);
 	install_element(VIEW_NODE, &show_pbr_cmd);
 	install_element(VIEW_NODE, &show_pbr_map_cmd);
 	install_element(VIEW_NODE, &show_pbr_interface_cmd);
