@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -48,6 +47,7 @@ void pim_ifassert_winner_set(struct pim_ifchannel *ch,
 			     struct in_addr winner,
 			     struct pim_assert_metric winner_metric)
 {
+	struct pim_interface *pim_ifp = ch->interface->info;
 	int winner_changed = (ch->ifassert_winner.s_addr != winner.s_addr);
 	int metric_changed = !pim_assert_metric_match(
 		&ch->ifassert_winner_metric, &winner_metric);
@@ -56,7 +56,7 @@ void pim_ifassert_winner_set(struct pim_ifchannel *ch,
 		if (ch->ifassert_state != new_state) {
 			zlog_debug(
 				"%s: (S,G)=%s assert state changed from %s to %s on interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str,
+				__func__, ch->sg_str,
 				pim_ifchannel_ifassert_name(ch->ifassert_state),
 				pim_ifchannel_ifassert_name(new_state),
 				ch->interface->name);
@@ -71,8 +71,8 @@ void pim_ifassert_winner_set(struct pim_ifchannel *ch,
 				       sizeof(winner_str));
 			zlog_debug(
 				"%s: (S,G)=%s assert winner changed from %s to %s on interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str, was_str,
-				winner_str, ch->interface->name);
+				__func__, ch->sg_str, was_str, winner_str,
+				ch->interface->name);
 		}
 	} /* PIM_DEBUG_PIM_EVENTS */
 
@@ -82,7 +82,7 @@ void pim_ifassert_winner_set(struct pim_ifchannel *ch,
 	ch->ifassert_creation = pim_time_monotonic_sec();
 
 	if (winner_changed || metric_changed) {
-		pim_upstream_update_join_desired(ch->upstream);
+		pim_upstream_update_join_desired(pim_ifp->pim, ch->upstream);
 		pim_ifchannel_update_could_assert(ch);
 		pim_ifchannel_update_assert_tracking_desired(ch);
 	}
@@ -130,7 +130,7 @@ static void if_could_assert_do_a1(const char *caller, struct pim_ifchannel *ch)
 		if (assert_action_a1(ch)) {
 			zlog_warn(
 				"%s: %s: (S,G)=%s assert_action_a1 failure on interface %s",
-				__PRETTY_FUNCTION__, caller, ch->sg_str,
+				__func__, caller, ch->sg_str,
 				ch->interface->name);
 			/* log warning only */
 		}
@@ -147,24 +147,18 @@ static int dispatch_assert(struct interface *ifp, struct in_addr source_addr,
 	memset(&sg, 0, sizeof(struct prefix_sg));
 	sg.src = source_addr;
 	sg.grp = group_addr;
-	ch = pim_ifchannel_add(ifp, &sg, 0);
-	if (!ch) {
-		zlog_warn(
-			"%s: (S,G)=%s failure creating channel on interface %s",
-			__PRETTY_FUNCTION__, pim_str_sg_dump(&sg), ifp->name);
-		return -1;
-	}
+	ch = pim_ifchannel_add(ifp, &sg, 0, 0);
 
 	switch (ch->ifassert_state) {
 	case PIM_IFASSERT_NOINFO:
 		if (recv_metric.rpt_bit_flag) {
 			/* RPT bit set */
-			if_could_assert_do_a1(__PRETTY_FUNCTION__, ch);
+			if_could_assert_do_a1(__func__, ch);
 		} else {
 			/* RPT bit clear */
 			if (inferior_assert(&ch->ifassert_my_metric,
 					    &recv_metric)) {
-				if_could_assert_do_a1(__PRETTY_FUNCTION__, ch);
+				if_could_assert_do_a1(__func__, ch);
 			} else if (acceptable_assert(&ch->ifassert_my_metric,
 						     &recv_metric)) {
 				if (PIM_IF_FLAG_TEST_ASSERT_TRACKING_DESIRED(
@@ -211,8 +205,7 @@ static int dispatch_assert(struct interface *ifp, struct in_addr source_addr,
 	default: {
 		zlog_warn(
 			"%s: (S,G)=%s invalid assert state %d on interface %s",
-			__PRETTY_FUNCTION__, ch->sg_str, ch->ifassert_state,
-			ifp->name);
+			__func__, ch->sg_str, ch->ifassert_state, ifp->name);
 	}
 		return -2;
 	}
@@ -229,8 +222,9 @@ int pim_assert_recv(struct interface *ifp, struct pim_neighbor *neigh,
 	int offset;
 	uint8_t *curr;
 	int curr_size;
+	struct pim_interface *pim_ifp = NULL;
 
-	on_trace(__PRETTY_FUNCTION__, ifp, src_addr);
+	on_trace(__func__, ifp, src_addr);
 
 	curr = buf;
 	curr_size = buf_size;
@@ -244,7 +238,7 @@ int pim_assert_recv(struct interface *ifp, struct pim_neighbor *neigh,
 		char src_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<src?>", src_addr, src_str, sizeof(src_str));
 		zlog_warn("%s: pim_parse_addr_group() failure: from %s on %s",
-			  __PRETTY_FUNCTION__, src_str, ifp->name);
+			  __func__, src_str, ifp->name);
 		return -1;
 	}
 	curr += offset;
@@ -258,18 +252,18 @@ int pim_assert_recv(struct interface *ifp, struct pim_neighbor *neigh,
 		char src_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<src?>", src_addr, src_str, sizeof(src_str));
 		zlog_warn("%s: pim_parse_addr_ucast() failure: from %s on %s",
-			  __PRETTY_FUNCTION__, src_str, ifp->name);
+			  __func__, src_str, ifp->name);
 		return -2;
 	}
 	curr += offset;
 	curr_size -= offset;
 
-	if (curr_size != 8) {
+	if (curr_size < 8) {
 		char src_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<src?>", src_addr, src_str, sizeof(src_str));
 		zlog_warn(
-			"%s: preference/metric size is not 8: size=%d from %s on interface %s",
-			__PRETTY_FUNCTION__, curr_size, src_str, ifp->name);
+			"%s: preference/metric size is less than 8 bytes: size=%d from %s on interface %s",
+			__func__, curr_size, src_str, ifp->name);
 		return -3;
 	}
 
@@ -302,14 +296,17 @@ int pim_assert_recv(struct interface *ifp, struct pim_neighbor *neigh,
 		pim_inet4_dump("<grp?>", sg.grp, group_str, sizeof(group_str));
 		zlog_debug(
 			"%s: from %s on %s: (S,G)=(%s,%s) pref=%u metric=%u rpt_bit=%u",
-			__PRETTY_FUNCTION__, neigh_str, ifp->name, source_str,
-			group_str, msg_metric.metric_preference,
-			msg_metric.route_metric,
+			__func__, neigh_str, ifp->name, source_str, group_str,
+			msg_metric.metric_preference, msg_metric.route_metric,
 			PIM_FORCE_BOOLEAN(msg_metric.rpt_bit_flag));
 	}
 
 	msg_metric.ip_address = src_addr;
 
+	pim_ifp = ifp->info;
+	zassert(pim_ifp);
+	++pim_ifp->pim_ifstat_assert_recv;
+
 	return dispatch_assert(ifp, msg_source_addr.u.prefix4, sg.grp,
 			       msg_metric);
 }
@@ -381,7 +378,7 @@ int pim_assert_build_msg(uint8_t *pim_msg, int buf_size, struct interface *ifp,
 			       sizeof(group_str));
 		zlog_warn(
 			"%s: failure encoding group address %s: space left=%d",
-			__PRETTY_FUNCTION__, group_str, remain);
+			__func__, group_str, remain);
 		return -1;
 	}
 
@@ -395,7 +392,7 @@ int pim_assert_build_msg(uint8_t *pim_msg, int buf_size, struct interface *ifp,
 			       sizeof(source_str));
 		zlog_warn(
 			"%s: failure encoding source address %s: space left=%d",
-			__PRETTY_FUNCTION__, source_str, remain);
+			__func__, source_str, remain);
 		return -2;
 	}
 
@@ -413,7 +410,7 @@ int pim_assert_build_msg(uint8_t *pim_msg, int buf_size, struct interface *ifp,
 	  Add PIM header
 	*/
 	pim_msg_size = pim_msg_curr - pim_msg;
-	pim_msg_build_header(pim_msg, pim_msg_size, PIM_MSG_TYPE_ASSERT);
+	pim_msg_build_header(pim_msg, pim_msg_size, PIM_MSG_TYPE_ASSERT, false);
 
 	return pim_msg_size;
 }
@@ -430,7 +427,7 @@ static int pim_assert_do(struct pim_ifchannel *ch,
 	if (!ifp) {
 		if (PIM_DEBUG_PIM_TRACE)
 			zlog_debug("%s: channel%s has no associated interface!",
-				   __PRETTY_FUNCTION__, ch->sg_str);
+				   __func__, ch->sg_str);
 		return -1;
 	}
 	pim_ifp = ifp->info;
@@ -438,7 +435,7 @@ static int pim_assert_do(struct pim_ifchannel *ch,
 		if (PIM_DEBUG_PIM_TRACE)
 			zlog_debug(
 				"%s: channel %s pim not enabled on interface: %s",
-				__PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+				__func__, ch->sg_str, ifp->name);
 		return -1;
 	}
 
@@ -449,7 +446,7 @@ static int pim_assert_do(struct pim_ifchannel *ch,
 	if (pim_msg_size < 1) {
 		zlog_warn(
 			"%s: failure building PIM assert message: msg_size=%d",
-			__PRETTY_FUNCTION__, pim_msg_size);
+			__func__, pim_msg_size);
 		return -2;
 	}
 
@@ -466,16 +463,17 @@ static int pim_assert_do(struct pim_ifchannel *ch,
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_debug("%s: to %s: (S,G)=%s pref=%u metric=%u rpt_bit=%u",
-			   __PRETTY_FUNCTION__, ifp->name, ch->sg_str,
+			   __func__, ifp->name, ch->sg_str,
 			   metric.metric_preference, metric.route_metric,
 			   PIM_FORCE_BOOLEAN(metric.rpt_bit_flag));
 	}
+	++pim_ifp->pim_ifstat_assert_send;
 
 	if (pim_msg_send(pim_ifp->pim_sock_fd, pim_ifp->primary_address,
 			 qpim_all_pim_routers_addr, pim_msg, pim_msg_size,
 			 ifp->name)) {
 		zlog_warn("%s: could not send PIM message on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
+			  __func__, ifp->name);
 		return -3;
 	}
 
@@ -516,7 +514,7 @@ static int on_assert_timer(struct thread *t)
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_debug("%s: (S,G)=%s timer expired on interface %s",
-			   __PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+			   __func__, ch->sg_str, ifp->name);
 	}
 
 	ch->t_ifassert_timer = NULL;
@@ -532,8 +530,8 @@ static int on_assert_timer(struct thread *t)
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: (S,G)=%s invalid assert state %d on interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str,
-				ch->ifassert_state, ifp->name);
+				__func__, ch->sg_str, ch->ifassert_state,
+				ifp->name);
 	}
 	}
 
@@ -546,8 +544,7 @@ static void assert_timer_off(struct pim_ifchannel *ch)
 		if (ch->t_ifassert_timer) {
 			zlog_debug(
 				"%s: (S,G)=%s cancelling timer on interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str,
-				ch->interface->name);
+				__func__, ch->sg_str, ch->interface->name);
 		}
 	}
 	THREAD_OFF(ch->t_ifassert_timer);
@@ -559,12 +556,11 @@ static void pim_assert_timer_set(struct pim_ifchannel *ch, int interval)
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_debug("%s: (S,G)=%s starting %u sec timer on interface %s",
-			   __PRETTY_FUNCTION__, ch->sg_str, interval,
-			   ch->interface->name);
+			   __func__, ch->sg_str, interval, ch->interface->name);
 	}
 
-	THREAD_TIMER_ON(master, ch->t_ifassert_timer, on_assert_timer, ch,
-			interval);
+	thread_add_timer(router->master, on_assert_timer, ch, interval,
+			 &ch->t_ifassert_timer);
 }
 
 static void pim_assert_timer_reset(struct pim_ifchannel *ch)
@@ -591,7 +587,7 @@ int assert_action_a1(struct pim_ifchannel *ch)
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
 		zlog_warn("%s: (S,G)=%s multicast not enabled on interface %s",
-			  __PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+			  __func__, ch->sg_str, ifp->name);
 		return -1; /* must return since pim_ifp is used below */
 	}
 
@@ -604,7 +600,7 @@ int assert_action_a1(struct pim_ifchannel *ch)
 	if (assert_action_a3(ch)) {
 		zlog_warn(
 			"%s: (S,G)=%s assert_action_a3 failure on interface %s",
-			__PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+			__func__, ch->sg_str, ifp->name);
 		/* warning only */
 	}
 
@@ -612,7 +608,7 @@ int assert_action_a1(struct pim_ifchannel *ch)
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s not in expected PIM_IFASSERT_I_AM_WINNER state",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 	}
 
 	return 0;
@@ -639,7 +635,7 @@ static void assert_action_a2(struct pim_ifchannel *ch,
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s not in expected PIM_IFASSERT_I_AM_LOSER state",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 	}
 }
 
@@ -657,7 +653,7 @@ static int assert_action_a3(struct pim_ifchannel *ch)
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s expected to be in PIM_IFASSERT_I_AM_WINNER state",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 		return -1;
 	}
 
@@ -665,7 +661,7 @@ static int assert_action_a3(struct pim_ifchannel *ch)
 
 	if (pim_assert_send(ch)) {
 		zlog_warn("%s: (S,G)=%s failure sending assert on interface %s",
-			  __PRETTY_FUNCTION__, ch->sg_str, ch->interface->name);
+			  __func__, ch->sg_str, ch->interface->name);
 		return -1;
 	}
 
@@ -686,7 +682,7 @@ void assert_action_a4(struct pim_ifchannel *ch)
 {
 	if (pim_assert_cancel(ch)) {
 		zlog_warn("%s: failure sending AssertCancel%s on interface %s",
-			  __PRETTY_FUNCTION__, ch->sg_str, ch->interface->name);
+			  __func__, ch->sg_str, ch->interface->name);
 		/* log warning only */
 	}
 
@@ -696,7 +692,7 @@ void assert_action_a4(struct pim_ifchannel *ch)
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s not in PIM_IFASSERT_NOINFO state as expected",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 	}
 }
 
@@ -715,7 +711,7 @@ void assert_action_a5(struct pim_ifchannel *ch)
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s not in PIM_IFSSERT_NOINFO state as expected",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 	}
 }
 
@@ -728,7 +724,7 @@ void assert_action_a5(struct pim_ifchannel *ch)
 	  winner metric as AssertWinnerMetric(S,G,I).
 	  Set Assert Timer to Assert_Time.
 	  If (I is RPF_interface(S)) AND (UpstreamJPState(S,G) == true)
-	  set SPTbit(S,G) to TRUE.
+	  set SPTbit(S,G) to true.
 */
 static void assert_action_a6(struct pim_ifchannel *ch,
 			     struct pim_assert_metric winner_metric)
@@ -737,7 +733,7 @@ static void assert_action_a6(struct pim_ifchannel *ch,
 
 	/*
 	  If (I is RPF_interface(S)) AND (UpstreamJPState(S,G) == true) set
-	  SPTbit(S,G) to TRUE.
+	  SPTbit(S,G) to true.
 	*/
 	if (ch->upstream->rpf.source_nexthop.interface == ch->interface)
 		if (ch->upstream->join_state == PIM_UPSTREAM_JOINED)
@@ -747,6 +743,6 @@ static void assert_action_a6(struct pim_ifchannel *ch,
 		if (PIM_DEBUG_PIM_EVENTS)
 			zlog_warn(
 				"%s: channel%s not in PIM_IFASSERT_I_AM_LOSER state as expected",
-				__PRETTY_FUNCTION__, ch->sg_str);
+				__func__, ch->sg_str);
 	}
 }
