@@ -28,6 +28,7 @@
 #include "plist.h"
 #include "hash.h"
 #include "ferr.h"
+#include "network.h"
 
 #include "pimd.h"
 #include "pim_instance.h"
@@ -46,6 +47,8 @@
 #include "pim_rp.h"
 #include "pim_nht.h"
 #include "pim_jp_agg.h"
+#include "pim_igmp_join.h"
+#include "pim_vxlan.h"
 
 static void pim_if_igmp_join_del_all(struct interface *ifp);
 static int igmp_join_sock(const char *ifname, ifindex_t ifindex,
@@ -62,38 +65,17 @@ void pim_if_init(struct pim_instance *pim)
 
 void pim_if_terminate(struct pim_instance *pim)
 {
-	// Nothing to do at this moment
-	return;
-}
+	struct interface *ifp;
 
-static void *if_list_clean(struct pim_interface *pim_ifp)
-{
-	struct pim_ifchannel *ch;
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
+		struct pim_interface *pim_ifp = ifp->info;
 
-	if (pim_ifp->igmp_join_list)
-		list_delete_and_null(&pim_ifp->igmp_join_list);
+		if (!pim_ifp)
+			continue;
 
-	if (pim_ifp->igmp_socket_list)
-		list_delete_and_null(&pim_ifp->igmp_socket_list);
-
-	if (pim_ifp->pim_neighbor_list)
-		list_delete_and_null(&pim_ifp->pim_neighbor_list);
-
-	if (pim_ifp->upstream_switch_list)
-		list_delete_and_null(&pim_ifp->upstream_switch_list);
-
-	if (pim_ifp->sec_addr_list)
-		list_delete_and_null(&pim_ifp->sec_addr_list);
-
-	while (!RB_EMPTY(pim_ifchannel_rb, &pim_ifp->ifchannel_rb)) {
-		ch = RB_ROOT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
-
-		pim_ifchannel_delete(ch);
+		pim_if_delete(ifp);
 	}
-
-	XFREE(MTYPE_PIM_INTERFACE, pim_ifp);
-
-	return 0;
+	return;
 }
 
 static void pim_sec_addr_free(struct pim_secondary_addr *sec_addr)
@@ -128,8 +110,8 @@ static int pim_sec_addr_comp(const void *p1, const void *p2)
 	return 0;
 }
 
-struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim,
-				 bool ispimreg)
+struct pim_interface *pim_if_new(struct interface *ifp, bool igmp, bool pim,
+				 bool ispimreg, bool is_vxlan_term)
 {
 	struct pim_interface *pim_ifp;
 
@@ -137,10 +119,6 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim,
 	zassert(!ifp->info);
 
 	pim_ifp = XCALLOC(MTYPE_PIM_INTERFACE, sizeof(*pim_ifp));
-	if (!pim_ifp) {
-		zlog_err("PIM XCALLOC(%zu) failure", sizeof(*pim_ifp));
-		return 0;
-	}
 
 	pim_ifp->options = 0;
 	pim_ifp->pim = pim_get_pim_instance(ifp->vrf_id);
@@ -154,6 +132,12 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim,
 		IGMP_QUERY_MAX_RESPONSE_TIME_DSEC;
 	pim_ifp->igmp_specific_query_max_response_time_dsec =
 		IGMP_SPECIFIC_QUERY_MAX_RESPONSE_TIME_DSEC;
+	pim_ifp->igmp_last_member_query_count =
+		IGMP_DEFAULT_ROBUSTNESS_VARIABLE;
+
+	/* BSM config on interface: true by default */
+	pim_ifp->bsm_enable = true;
+	pim_ifp->ucast_bsm_accept = true;
 
 	/*
 	  RFC 3376: 8.3. Query Response Interval
@@ -178,49 +162,32 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim,
 
 	/* list of struct igmp_sock */
 	pim_ifp->igmp_socket_list = list_new();
-	if (!pim_ifp->igmp_socket_list) {
-		zlog_err("%s: failure: igmp_socket_list=list_new()",
-			 __PRETTY_FUNCTION__);
-		return if_list_clean(pim_ifp);
-	}
 	pim_ifp->igmp_socket_list->del = (void (*)(void *))igmp_sock_free;
 
 	/* list of struct pim_neighbor */
 	pim_ifp->pim_neighbor_list = list_new();
-	if (!pim_ifp->pim_neighbor_list) {
-		zlog_err("%s: failure: pim_neighbor_list=list_new()",
-			 __PRETTY_FUNCTION__);
-		return if_list_clean(pim_ifp);
-	}
 	pim_ifp->pim_neighbor_list->del = (void (*)(void *))pim_neighbor_free;
 
 	pim_ifp->upstream_switch_list = list_new();
-	if (!pim_ifp->upstream_switch_list) {
-		zlog_err("%s: failure: upstream_switch_list=list_new()",
-			 __PRETTY_FUNCTION__);
-		return if_list_clean(pim_ifp);
-	}
 	pim_ifp->upstream_switch_list->del =
 		(void (*)(void *))pim_jp_agg_group_list_free;
 	pim_ifp->upstream_switch_list->cmp = pim_jp_agg_group_list_cmp;
 
 	pim_ifp->sec_addr_list = list_new();
-	if (!pim_ifp->sec_addr_list) {
-		zlog_err("%s: failure: secondary addresslist",
-			 __PRETTY_FUNCTION__);
-		return if_list_clean(pim_ifp);
-	}
 	pim_ifp->sec_addr_list->del = (void (*)(void *))pim_sec_addr_free;
 	pim_ifp->sec_addr_list->cmp =
 		(int (*)(void *, void *))pim_sec_addr_comp;
 
+	pim_ifp->activeactive = false;
+
 	RB_INIT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
 
 	ifp->info = pim_ifp;
 
 	pim_sock_reset(ifp);
 
-	pim_if_add_vif(ifp, ispimreg);
+	pim_if_add_vif(ifp, ispimreg, is_vxlan_term);
+	pim_ifp->pim->mcast_if_count++;
 
 	return pim_ifp;
 }
@@ -244,14 +211,14 @@ void pim_if_delete(struct interface *ifp)
 	pim_neighbor_delete_all(ifp, "Interface removed from configuration");
 
 	pim_if_del_vif(ifp);
+	pim_ifp->pim->mcast_if_count--;
 
-	list_delete_and_null(&pim_ifp->igmp_socket_list);
-	list_delete_and_null(&pim_ifp->pim_neighbor_list);
-	list_delete_and_null(&pim_ifp->upstream_switch_list);
-	list_delete_and_null(&pim_ifp->sec_addr_list);
+	list_delete(&pim_ifp->igmp_socket_list);
+	list_delete(&pim_ifp->pim_neighbor_list);
+	list_delete(&pim_ifp->upstream_switch_list);
+	list_delete(&pim_ifp->sec_addr_list);
 
-	if (pim_ifp->boundary_oil_plist)
-		XFREE(MTYPE_PIM_INTERFACE, pim_ifp->boundary_oil_plist);
+	XFREE(MTYPE_PIM_INTERFACE, pim_ifp->boundary_oil_plist);
 
 	while (!RB_EMPTY(pim_ifchannel_rb, &pim_ifp->ifchannel_rb)) {
 		ch = RB_ROOT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
@@ -311,7 +278,7 @@ static void pim_addr_change(struct interface *ifp)
 	  1) Before an interface goes down or changes primary IP address, a
 	  Hello message with a zero HoldTime should be sent immediately
 	  (with the old IP address if the IP address changed).
-	  -- FIXME See CAVEAT C13
+	  -- Done at the caller of the function as new ip already updated here
 
 	  2) After an interface has changed its IP address, it MUST send a
 	  Hello message with its new IP address.
@@ -350,12 +317,16 @@ static int detect_primary_address_change(struct interface *ifp,
 			       sizeof(new_prim_str));
 		pim_inet4_dump("<old?>", pim_ifp->primary_address, old_prim_str,
 			       sizeof(old_prim_str));
-		zlog_debug("%s: old=%s new=%s on interface %s: %s",
-			   __PRETTY_FUNCTION__, old_prim_str, new_prim_str,
-			   ifp->name, changed ? "changed" : "unchanged");
+		zlog_debug("%s: old=%s new=%s on interface %s: %s", __func__,
+			   old_prim_str, new_prim_str, ifp->name,
+			   changed ? "changed" : "unchanged");
 	}
 
 	if (changed) {
+		/* Before updating pim_ifp send Hello time with 0 hold time */
+		if (PIM_IF_TEST_PIM(pim_ifp->options)) {
+			pim_hello_send(ifp, 0 /* zero-sec holdtime */);
+		}
 		pim_ifp->primary_address = new_prim_addr;
 	}
 
@@ -396,8 +367,6 @@ static int pim_sec_addr_add(struct pim_interface *pim_ifp, struct prefix *addr)
 	}
 
 	sec_addr = XCALLOC(MTYPE_PIM_SEC_ADDR, sizeof(*sec_addr));
-	if (!sec_addr)
-		return changed;
 
 	changed = 1;
 	sec_addr->addr = *addr;
@@ -525,8 +494,7 @@ int pim_update_source_set(struct interface *ifp, struct in_addr source)
 	}
 
 	pim_ifp->update_source = source;
-	detect_address_change(ifp, 0 /* force_prim_as_any */,
-			      __PRETTY_FUNCTION__);
+	detect_address_change(ifp, 0 /* force_prim_as_any */, __func__);
 
 	return PIM_SUCCESS;
 }
@@ -536,6 +504,7 @@ void pim_if_addr_add(struct connected *ifc)
 	struct pim_interface *pim_ifp;
 	struct interface *ifp;
 	struct in_addr ifaddr;
+	bool vxlan_term;
 
 	zassert(ifc);
 
@@ -552,7 +521,7 @@ void pim_if_addr_add(struct connected *ifc)
 		char buf[BUFSIZ];
 		prefix2str(ifc->address, buf, BUFSIZ);
 		zlog_debug("%s: %s ifindex=%d connected IP address %s %s",
-			   __PRETTY_FUNCTION__, ifp->name, ifp->ifindex, buf,
+			   __func__, ifp->name, ifp->ifindex, buf,
 			   CHECK_FLAG(ifc->flags, ZEBRA_IFA_SECONDARY)
 				   ? "secondary"
 				   : "primary");
@@ -560,7 +529,7 @@ void pim_if_addr_add(struct connected *ifc)
 
 	ifaddr = ifc->address->u.prefix4;
 
-	detect_address_change(ifp, 0, __PRETTY_FUNCTION__);
+	detect_address_change(ifp, 0, __func__);
 
 	// if (ifc->address->family != AF_INET)
 	//  return;
@@ -608,8 +577,8 @@ void pim_if_addr_add(struct connected *ifc)
 						source_str, sizeof(source_str));
 					zlog_warn(
 						"%s: igmp_join_sock() failure for IGMP group %s source %s on interface %s",
-						__PRETTY_FUNCTION__, group_str,
-						source_str, ifp->name);
+						__func__, group_str, source_str,
+						ifp->name);
 					/* warning only */
 				} else
 					ij->sock_fd = join_fd;
@@ -673,7 +642,8 @@ void pim_if_addr_add(struct connected *ifc)
 	  address assigned, then try to create a vif_index.
 	*/
 	if (pim_ifp->mroute_vif_index < 0) {
-		pim_if_add_vif(ifp, false);
+		vxlan_term = pim_vxlan_is_term_dev_cfg(pim_ifp->pim, ifp);
+		pim_if_add_vif(ifp, false, vxlan_term);
 	}
 	pim_ifchannel_scan_forward_start(ifp);
 }
@@ -748,13 +718,13 @@ void pim_if_addr_del(struct connected *ifc, int force_prim_as_any)
 		char buf[BUFSIZ];
 		prefix2str(ifc->address, buf, BUFSIZ);
 		zlog_debug("%s: %s ifindex=%d disconnected IP address %s %s",
-			   __PRETTY_FUNCTION__, ifp->name, ifp->ifindex, buf,
+			   __func__, ifp->name, ifp->ifindex, buf,
 			   CHECK_FLAG(ifc->flags, ZEBRA_IFA_SECONDARY)
 				   ? "secondary"
 				   : "primary");
 	}
 
-	detect_address_change(ifp, force_prim_as_any, __PRETTY_FUNCTION__);
+	detect_address_change(ifp, force_prim_as_any, __func__);
 
 	pim_if_addr_del_igmp(ifc);
 	pim_if_addr_del_pim(ifc);
@@ -768,6 +738,7 @@ void pim_if_addr_add_all(struct interface *ifp)
 	int v4_addrs = 0;
 	int v6_addrs = 0;
 	struct pim_interface *pim_ifp = ifp->info;
+	bool vxlan_term;
 
 
 	/* PIM/IGMP enabled ? */
@@ -806,7 +777,8 @@ void pim_if_addr_add_all(struct interface *ifp)
 	 * address assigned, then try to create a vif_index.
 	 */
 	if (pim_ifp->mroute_vif_index < 0) {
-		pim_if_add_vif(ifp, false);
+		vxlan_term = pim_vxlan_is_term_dev_cfg(pim_ifp->pim, ifp);
+		pim_if_add_vif(ifp, false, vxlan_term);
 	}
 	pim_ifchannel_scan_forward_start(ifp);
 
@@ -911,7 +883,7 @@ struct in_addr pim_find_primary_addr(struct interface *ifp)
 		if (PIM_INADDR_IS_ANY(p->u.prefix4)) {
 			zlog_warn(
 				"%s: null IPv4 address connected to interface %s",
-				__PRETTY_FUNCTION__, ifp->name);
+				__func__, ifp->name);
 			continue;
 		}
 
@@ -929,15 +901,16 @@ struct in_addr pim_find_primary_addr(struct interface *ifp)
 	 * So let's grab the loopbacks v4 address
 	 * and use that as the primary address
 	 */
-	if (!v4_addrs && v6_addrs && !if_is_loopback(ifp)) {
+	if (!v4_addrs && v6_addrs) {
 		struct interface *lo_ifp;
+
 		// DBS - Come back and check here
 		if (ifp->vrf_id == VRF_DEFAULT)
 			lo_ifp = if_lookup_by_name("lo", vrf->vrf_id);
 		else
 			lo_ifp = if_lookup_by_name(vrf->name, vrf->vrf_id);
 
-		if (lo_ifp)
+		if (lo_ifp && (lo_ifp != ifp))
 			return pim_find_primary_addr(lo_ifp);
 	}
 
@@ -971,7 +944,7 @@ static int pim_iface_next_vif_index(struct interface *ifp)
 
   see also pim_if_find_vifindex_by_ifindex()
  */
-int pim_if_add_vif(struct interface *ifp, bool ispimreg)
+int pim_if_add_vif(struct interface *ifp, bool ispimreg, bool is_vxlan_term)
 {
 	struct pim_interface *pim_ifp = ifp->info;
 	struct in_addr ifaddr;
@@ -981,22 +954,22 @@ int pim_if_add_vif(struct interface *ifp, bool ispimreg)
 
 	if (pim_ifp->mroute_vif_index > 0) {
 		zlog_warn("%s: vif_index=%d > 0 on interface %s ifindex=%d",
-			  __PRETTY_FUNCTION__, pim_ifp->mroute_vif_index,
-			  ifp->name, ifp->ifindex);
+			  __func__, pim_ifp->mroute_vif_index, ifp->name,
+			  ifp->ifindex);
 		return -1;
 	}
 
 	if (ifp->ifindex < 0) {
-		zlog_warn("%s: ifindex=%d < 1 on interface %s",
-			  __PRETTY_FUNCTION__, ifp->ifindex, ifp->name);
+		zlog_warn("%s: ifindex=%d < 1 on interface %s", __func__,
+			  ifp->ifindex, ifp->name);
 		return -2;
 	}
 
 	ifaddr = pim_ifp->primary_address;
-	if (!ispimreg && PIM_INADDR_IS_ANY(ifaddr)) {
+	if (!ispimreg && !is_vxlan_term && PIM_INADDR_IS_ANY(ifaddr)) {
 		zlog_warn(
 			"%s: could not get address for interface %s ifindex=%d",
-			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex);
+			__func__, ifp->name, ifp->ifindex);
 		return -4;
 	}
 
@@ -1005,7 +978,7 @@ int pim_if_add_vif(struct interface *ifp, bool ispimreg)
 	if (pim_ifp->mroute_vif_index >= MAXVIFS) {
 		zlog_warn(
 			"%s: Attempting to configure more than MAXVIFS=%d on pim enabled interface %s",
-			__PRETTY_FUNCTION__, MAXVIFS, ifp->name);
+			__func__, MAXVIFS, ifp->name);
 		return -3;
 	}
 
@@ -1022,6 +995,10 @@ int pim_if_add_vif(struct interface *ifp, bool ispimreg)
 	}
 
 	pim_ifp->pim->iface_vif_index[pim_ifp->mroute_vif_index] = 1;
+
+	/* if the device qualifies as pim_vxlan iif/oif update vxlan entries */
+	pim_vxlan_add_vif(ifp);
+
 	return 0;
 }
 
@@ -1031,11 +1008,14 @@ int pim_if_del_vif(struct interface *ifp)
 
 	if (pim_ifp->mroute_vif_index < 1) {
 		zlog_warn("%s: vif_index=%d < 1 on interface %s ifindex=%d",
-			  __PRETTY_FUNCTION__, pim_ifp->mroute_vif_index,
-			  ifp->name, ifp->ifindex);
+			  __func__, pim_ifp->mroute_vif_index, ifp->name,
+			  ifp->ifindex);
 		return -1;
 	}
 
+	/* if the device was a pim_vxlan iif/oif update vxlan mroute entries */
+	pim_vxlan_del_vif(ifp);
+
 	pim_mroute_del_vif(ifp);
 
 	/*
@@ -1124,7 +1104,8 @@ int pim_if_t_override_msec(struct interface *ifp)
 	effective_override_interval_msec =
 		pim_if_effective_override_interval_msec(ifp);
 
-	t_override_msec = random() % (effective_override_interval_msec + 1);
+	t_override_msec =
+		frr_weak_random() % (effective_override_interval_msec + 1);
 
 	return t_override_msec;
 }
@@ -1156,8 +1137,8 @@ struct pim_neighbor *pim_if_find_neighbor(struct interface *ifp,
 
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
-		zlog_warn("%s: multicast not enabled on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
+		zlog_warn("%s: multicast not enabled on interface %s", __func__,
+			  ifp->name);
 		return 0;
 	}
 
@@ -1182,7 +1163,7 @@ struct pim_neighbor *pim_if_find_neighbor(struct interface *ifp,
 		pim_inet4_dump("<addr?>", addr, addr_str, sizeof(addr_str));
 		zlog_debug(
 			"%s: neighbor not found for address %s on interface %s",
-			__PRETTY_FUNCTION__, addr_str, ifp->name);
+			__func__, addr_str, ifp->name);
 	}
 
 	return NULL;
@@ -1202,8 +1183,8 @@ long pim_if_t_suppressed_msec(struct interface *ifp)
 		return 0;
 
 	/* t_suppressed = t_periodic * rand(1.1, 1.4) */
-	ramount = 1100 + (random() % (1400 - 1100 + 1));
-	t_suppressed_msec = qpim_t_periodic * ramount;
+	ramount = 1100 + (frr_weak_random() % (1400 - 1100 + 1));
+	t_suppressed_msec = router->t_periodic * ramount;
 
 	return t_suppressed_msec;
 }
@@ -1241,8 +1222,18 @@ static int igmp_join_sock(const char *ifname, ifindex_t ifindex,
 		return -1;
 	}
 
-	if (pim_socket_join_source(join_fd, ifindex, group_addr, source_addr,
-				   ifname)) {
+	if (pim_igmp_join_source(join_fd, ifindex, group_addr, source_addr)) {
+		char group_str[INET_ADDRSTRLEN];
+		char source_str[INET_ADDRSTRLEN];
+		pim_inet4_dump("<grp?>", group_addr, group_str,
+			       sizeof(group_str));
+		pim_inet4_dump("<src?>", source_addr, source_str,
+			       sizeof(source_str));
+		zlog_warn(
+			"%s: setsockopt(fd=%d) failure for IGMP group %s source %s ifindex %d on interface %s: errno=%d: %s",
+			__func__, join_fd, group_str, source_str, ifindex,
+			ifname, errno, safe_strerror(errno));
+
 		close(join_fd);
 		return -2;
 	}
@@ -1266,31 +1257,18 @@ static struct igmp_join *igmp_join_new(struct interface *ifp,
 	if (join_fd < 0) {
 		char group_str[INET_ADDRSTRLEN];
 		char source_str[INET_ADDRSTRLEN];
+
 		pim_inet4_dump("<grp?>", group_addr, group_str,
 			       sizeof(group_str));
 		pim_inet4_dump("<src?>", source_addr, source_str,
 			       sizeof(source_str));
 		zlog_warn(
 			"%s: igmp_join_sock() failure for IGMP group %s source %s on interface %s",
-			__PRETTY_FUNCTION__, group_str, source_str, ifp->name);
+			__func__, group_str, source_str, ifp->name);
 		return 0;
 	}
 
 	ij = XCALLOC(MTYPE_PIM_IGMP_JOIN, sizeof(*ij));
-	if (!ij) {
-		char group_str[INET_ADDRSTRLEN];
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<grp?>", group_addr, group_str,
-			       sizeof(group_str));
-		pim_inet4_dump("<src?>", source_addr, source_str,
-			       sizeof(source_str));
-		zlog_err(
-			"%s: XCALLOC(%zu) failure for IGMP group %s source %s on interface %s",
-			__PRETTY_FUNCTION__, sizeof(*ij), group_str, source_str,
-			ifp->name);
-		close(join_fd);
-		return 0;
-	}
 
 	ij->sock_fd = join_fd;
 	ij->group_addr = group_addr;
@@ -1316,9 +1294,6 @@ ferr_r pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
 
 	if (!pim_ifp->igmp_join_list) {
 		pim_ifp->igmp_join_list = list_new();
-		if (!pim_ifp->igmp_join_list) {
-			return ferr_cfg_invalid("Insufficient memory");
-		}
 		pim_ifp->igmp_join_list->del = (void (*)(void *))igmp_join_free;
 	}
 
@@ -1330,11 +1305,7 @@ ferr_r pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
 		return ferr_ok();
 	}
 
-	ij = igmp_join_new(ifp, group_addr, source_addr);
-	if (!ij) {
-		return ferr_cfg_invalid(
-			"Failure to create new join data structure, see log file for more information");
-	}
+	(void)igmp_join_new(ifp, group_addr, source_addr);
 
 	if (PIM_DEBUG_IGMP_EVENTS) {
 		char group_str[INET_ADDRSTRLEN];
@@ -1345,7 +1316,7 @@ ferr_r pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
 			       sizeof(source_str));
 		zlog_debug(
 			"%s: issued static igmp join for channel (S,G)=(%s,%s) on interface %s",
-			__PRETTY_FUNCTION__, source_str, group_str, ifp->name);
+			__func__, source_str, group_str, ifp->name);
 	}
 
 	return ferr_ok();
@@ -1360,14 +1331,14 @@ int pim_if_igmp_join_del(struct interface *ifp, struct in_addr group_addr,
 
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
-		zlog_warn("%s: multicast not enabled on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
+		zlog_warn("%s: multicast not enabled on interface %s", __func__,
+			  ifp->name);
 		return -1;
 	}
 
 	if (!pim_ifp->igmp_join_list) {
-		zlog_warn("%s: no IGMP join on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
+		zlog_warn("%s: no IGMP join on interface %s", __func__,
+			  ifp->name);
 		return -2;
 	}
 
@@ -1381,7 +1352,7 @@ int pim_if_igmp_join_del(struct interface *ifp, struct in_addr group_addr,
 			       sizeof(source_str));
 		zlog_warn(
 			"%s: could not find IGMP group %s source %s on interface %s",
-			__PRETTY_FUNCTION__, group_str, source_str, ifp->name);
+			__func__, group_str, source_str, ifp->name);
 		return -3;
 	}
 
@@ -1394,14 +1365,14 @@ int pim_if_igmp_join_del(struct interface *ifp, struct in_addr group_addr,
 			       sizeof(source_str));
 		zlog_warn(
 			"%s: failure closing sock_fd=%d for IGMP group %s source %s on interface %s: errno=%d: %s",
-			__PRETTY_FUNCTION__, ij->sock_fd, group_str, source_str,
-			ifp->name, errno, safe_strerror(errno));
+			__func__, ij->sock_fd, group_str, source_str, ifp->name,
+			errno, safe_strerror(errno));
 		/* warning only */
 	}
 	listnode_delete(pim_ifp->igmp_join_list, ij);
 	igmp_join_free(ij);
 	if (listcount(pim_ifp->igmp_join_list) < 1) {
-		list_delete_and_null(&pim_ifp->igmp_join_list);
+		list_delete(&pim_ifp->igmp_join_list);
 		pim_ifp->igmp_join_list = 0;
 	}
 
@@ -1417,8 +1388,8 @@ static void pim_if_igmp_join_del_all(struct interface *ifp)
 
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
-		zlog_warn("%s: multicast not enabled on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
+		zlog_warn("%s: multicast not enabled on interface %s", __func__,
+			  ifp->name);
 		return;
 	}
 
@@ -1517,10 +1488,18 @@ void pim_if_create_pimreg(struct pim_instance *pim)
 			snprintf(pimreg_name, sizeof(pimreg_name), "pimreg%u",
 				 pim->vrf->data.l.table_id);
 
-		pim->regiface = if_create(pimreg_name, pim->vrf_id);
+		pim->regiface = if_create_name(pimreg_name, pim->vrf_id);
 		pim->regiface->ifindex = PIM_OIF_PIM_REGISTER_VIF;
 
-		pim_if_new(pim->regiface, 0, 0, true);
+		pim_if_new(pim->regiface, false, false, true,
+			false /*vxlan_term*/);
+		/*
+		 * On vrf moves we delete the interface if there
+		 * is nothing going on with it.  We cannot have
+		 * the pimregiface deleted.
+		 */
+		pim->regiface->configured = true;
+
 	}
 }
 
@@ -1547,27 +1526,12 @@ int pim_if_connected_to_source(struct interface *ifp, struct in_addr src)
 	return 0;
 }
 
-int pim_if_is_loopback(struct pim_instance *pim, struct interface *ifp)
+bool pim_if_is_vrf_device(struct interface *ifp)
 {
-	if (if_is_loopback(ifp))
-		return 1;
+	if (if_is_vrf(ifp))
+		return true;
 
-	if (strcmp(ifp->name, pim->vrf->name) == 0)
-		return 1;
-
-	return 0;
-}
-
-int pim_if_is_vrf_device(struct interface *ifp)
-{
-	struct vrf *vrf;
-
-	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
-		if (strncmp(ifp->name, vrf->name, strlen(ifp->name)) == 0)
-			return 1;
-	}
-
-	return 0;
+	return false;
 }
 
 int pim_if_ifchannel_count(struct pim_interface *pim_ifp)
@@ -1581,3 +1545,173 @@ int pim_if_ifchannel_count(struct pim_interface *pim_ifp)
 
 	return count;
 }
+
+int pim_ifp_create(struct interface *ifp)
+{
+	struct pim_instance *pim;
+
+	pim = pim_get_pim_instance(ifp->vrf_id);
+	if (PIM_DEBUG_ZEBRA) {
+		zlog_debug(
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__func__, ifp->name, ifp->ifindex, ifp->vrf_id,
+			(long)ifp->flags, ifp->metric, ifp->mtu,
+			if_is_operative(ifp));
+	}
+
+	if (if_is_operative(ifp)) {
+		struct pim_interface *pim_ifp;
+
+		pim_ifp = ifp->info;
+		/*
+		 * If we have a pim_ifp already and this is an if_add
+		 * that means that we probably have a vrf move event
+		 * If that is the case, set the proper vrfness.
+		 */
+		if (pim_ifp)
+			pim_ifp->pim = pim;
+		pim_if_addr_add_all(ifp);
+	}
+
+	/*
+	 * If we are a vrf device that is up, open up the pim_socket for
+	 * listening
+	 * to incoming pim messages irrelevant if the user has configured us
+	 * for pim or not.
+	 */
+	if (pim_if_is_vrf_device(ifp)) {
+		struct pim_interface *pim_ifp;
+
+		if (!ifp->info) {
+			pim_ifp = pim_if_new(ifp, false, false, false,
+					     false /*vxlan_term*/);
+			ifp->info = pim_ifp;
+		}
+
+		pim_sock_add(ifp);
+	}
+
+	if (!strncmp(ifp->name, PIM_VXLAN_TERM_DEV_NAME,
+		     sizeof(PIM_VXLAN_TERM_DEV_NAME))) {
+		if (pim->mcast_if_count < MAXVIFS)
+			pim_vxlan_add_term_dev(pim, ifp);
+		else
+			zlog_warn(
+				"%s: Cannot enable pim on %s. MAXVIFS(%d) reached. Deleting and readding the vxlan termimation device after unconfiguring pim from other interfaces may succeed.",
+				__func__, ifp->name, MAXVIFS);
+	}
+
+	return 0;
+}
+
+int pim_ifp_up(struct interface *ifp)
+{
+	struct pim_interface *pim_ifp;
+	struct pim_instance *pim;
+	uint32_t table_id;
+
+	if (PIM_DEBUG_ZEBRA) {
+		zlog_debug(
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__func__, ifp->name, ifp->ifindex, ifp->vrf_id,
+			(long)ifp->flags, ifp->metric, ifp->mtu,
+			if_is_operative(ifp));
+	}
+
+	pim = pim_get_pim_instance(ifp->vrf_id);
+
+	pim_ifp = ifp->info;
+	/*
+	 * If we have a pim_ifp already and this is an if_add
+	 * that means that we probably have a vrf move event
+	 * If that is the case, set the proper vrfness.
+	 */
+	if (pim_ifp)
+		pim_ifp->pim = pim;
+
+	/*
+	  pim_if_addr_add_all() suffices for bringing up both IGMP and
+	  PIM
+	*/
+	pim_if_addr_add_all(ifp);
+
+	/*
+	 * If we have a pimreg device callback and it's for a specific
+	 * table set the master appropriately
+	 */
+	if (sscanf(ifp->name, "pimreg%" SCNu32, &table_id) == 1) {
+		struct vrf *vrf;
+		RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+			if ((table_id == vrf->data.l.table_id)
+			    && (ifp->vrf_id != vrf->vrf_id)) {
+				struct interface *master = if_lookup_by_name(
+					vrf->name, vrf->vrf_id);
+
+				if (!master) {
+					zlog_debug(
+						"%s: Unable to find Master interface for %s",
+						__func__, vrf->name);
+					return 0;
+				}
+				pim_zebra_interface_set_master(master, ifp);
+			}
+		}
+	}
+	return 0;
+}
+
+int pim_ifp_down(struct interface *ifp)
+{
+	if (PIM_DEBUG_ZEBRA) {
+		zlog_debug(
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__func__, ifp->name, ifp->ifindex, ifp->vrf_id,
+			(long)ifp->flags, ifp->metric, ifp->mtu,
+			if_is_operative(ifp));
+	}
+
+	if (!if_is_operative(ifp)) {
+		pim_ifchannel_delete_all(ifp);
+		/*
+		  pim_if_addr_del_all() suffices for shutting down IGMP,
+		  but not for shutting down PIM
+		*/
+		pim_if_addr_del_all(ifp);
+
+		/*
+		  pim_sock_delete() closes the socket, stops read and timer
+		  threads,
+		  and kills all neighbors.
+		*/
+		if (ifp->info) {
+			pim_sock_delete(ifp, "link down");
+		}
+	}
+
+	if (ifp->info)
+		pim_if_del_vif(ifp);
+
+	return 0;
+}
+
+int pim_ifp_destroy(struct interface *ifp)
+{
+	struct pim_instance *pim;
+
+	if (PIM_DEBUG_ZEBRA) {
+		zlog_debug(
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__func__, ifp->name, ifp->ifindex, ifp->vrf_id,
+			(long)ifp->flags, ifp->metric, ifp->mtu,
+			if_is_operative(ifp));
+	}
+
+	if (!if_is_operative(ifp))
+		pim_if_addr_del_all(ifp);
+
+	pim = pim_get_pim_instance(ifp->vrf_id);
+	if (pim && pim->vxlan.term_if == ifp)
+		pim_vxlan_del_term_dev(pim);
+
+	return 0;
+}
