@@ -24,6 +24,7 @@
 #include "if.h"
 #include "hash.h"
 #include "jhash.h"
+#include "lib_errors.h"
 
 #include "pimd.h"
 #include "pim_igmp.h"
@@ -63,9 +64,8 @@ static int igmp_sock_open(struct in_addr ifaddr, struct interface *ifp,
 		} else {
 			zlog_warn(
 				"%s %s: IGMP socket fd=%d interface %s: could not solve %s to group address: errno=%d: %s",
-				__FILE__, __PRETTY_FUNCTION__, fd,
-				inet_ntoa(ifaddr), PIM_ALL_ROUTERS, errno,
-				safe_strerror(errno));
+				__FILE__, __func__, fd, inet_ntoa(ifaddr),
+				PIM_ALL_ROUTERS, errno, safe_strerror(errno));
 		}
 	}
 
@@ -80,7 +80,7 @@ static int igmp_sock_open(struct in_addr ifaddr, struct interface *ifp,
 	} else {
 		zlog_warn(
 			"%s %s: IGMP socket fd=%d interface %s: could not solve %s to group address: errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__, fd, inet_ntoa(ifaddr),
+			__FILE__, __func__, fd, inet_ntoa(ifaddr),
 			PIM_ALL_SYSTEMS, errno, safe_strerror(errno));
 	}
 
@@ -91,12 +91,13 @@ static int igmp_sock_open(struct in_addr ifaddr, struct interface *ifp,
 	} else {
 		zlog_warn(
 			"%s %s: IGMP socket fd=%d interface %s: could not solve %s to group address: errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__, fd, inet_ntoa(ifaddr),
+			__FILE__, __func__, fd, inet_ntoa(ifaddr),
 			PIM_ALL_IGMP_ROUTERS, errno, safe_strerror(errno));
 	}
 
 	if (!join) {
-		zlog_err(
+		flog_err_sys(
+			EC_LIB_SOCKET,
 			"IGMP socket fd=%d could not join any group on interface address %s",
 			fd, inet_ntoa(ifaddr));
 		close(fd);
@@ -117,8 +118,8 @@ static void igmp_sock_dump(array_t *igmp_sock_array)
 		struct igmp_sock *igmp = array_get(igmp_sock_array, i);
 
 		zlog_debug("%s %s: [%d/%d] igmp_addr=%s fd=%d", __FILE__,
-			   __PRETTY_FUNCTION__, i, size,
-			   inet_ntoa(igmp->ifaddr), igmp->fd);
+			   __func__, i, size, inet_ntoa(igmp->ifaddr),
+			   igmp->fd);
 	}
 }
 #endif
@@ -164,8 +165,7 @@ static int pim_igmp_other_querier_expire(struct thread *t)
 		char ifaddr_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<ifaddr?>", igmp->ifaddr, ifaddr_str,
 			       sizeof(ifaddr_str));
-		zlog_debug("%s: Querier %s resuming", __PRETTY_FUNCTION__,
-			   ifaddr_str);
+		zlog_debug("%s: Querier %s resuming", __func__, ifaddr_str);
 	}
 
 	/*
@@ -248,8 +248,8 @@ void pim_igmp_other_querier_timer_on(struct igmp_sock *igmp)
 			other_querier_present_interval_msec % 1000);
 	}
 
-	thread_add_timer_msec(master, pim_igmp_other_querier_expire, igmp,
-			      other_querier_present_interval_msec,
+	thread_add_timer_msec(router->master, pim_igmp_other_querier_expire,
+			      igmp, other_querier_present_interval_msec,
 			      &igmp->t_other_querier_timer);
 }
 
@@ -303,6 +303,35 @@ static int igmp_recv_query(struct igmp_sock *igmp, int query_version,
 		return -1;
 	}
 
+	if (!pim_if_connected_to_source(ifp, from)) {
+		if (PIM_DEBUG_IGMP_PACKETS)
+			zlog_debug("Recv IGMP query on interface: %s from a non-connected source: %s",
+				   ifp->name, from_str);
+		return 0;
+	}
+
+	if (if_lookup_address(&from, AF_INET, ifp->vrf_id)) {
+		if (PIM_DEBUG_IGMP_PACKETS)
+			zlog_debug("Recv IGMP query on interface: %s from ourself %s",
+				   ifp->name, from_str);
+		return 0;
+	}
+
+	/* Collecting IGMP Rx stats */
+	switch (query_version) {
+	case 1:
+		igmp->rx_stats.query_v1++;
+		break;
+	case 2:
+		igmp->rx_stats.query_v2++;
+		break;
+	case 3:
+		igmp->rx_stats.query_v3++;
+		break;
+	default:
+		igmp->rx_stats.unsupported++;
+	}
+
 	/*
 	 * RFC 3376 defines some guidelines on operating in backwards
 	 * compatibility with older versions of IGMP but there are some gaps in
@@ -388,7 +417,7 @@ static int igmp_v1_recv_report(struct igmp_sock *igmp, struct in_addr from,
 	struct igmp_group *group;
 	struct in_addr group_addr;
 
-	on_trace(__PRETTY_FUNCTION__, igmp->interface, from);
+	on_trace(__func__, igmp->interface, from);
 
 	if (igmp->mtrace_only)
 		return 0;
@@ -400,9 +429,11 @@ static int igmp_v1_recv_report(struct igmp_sock *igmp, struct in_addr from,
 		return -1;
 	}
 
+	/* Collecting IGMP Rx stats */
+	igmp->rx_stats.report_v1++;
+
 	if (PIM_DEBUG_IGMP_TRACE) {
-		zlog_warn("%s %s: FIXME WRITEME", __FILE__,
-			  __PRETTY_FUNCTION__);
+		zlog_warn("%s %s: FIXME WRITEME", __FILE__, __func__);
 	}
 
 	memcpy(&group_addr, igmp_msg + 4, sizeof(struct in_addr));
@@ -444,30 +475,31 @@ int pim_igmp_packet(struct igmp_sock *igmp, char *buf, size_t len)
 
 	ip_hlen = ip_hdr->ip_hl << 2; /* ip_hl gives length in 4-byte words */
 
-	if (PIM_DEBUG_IGMP_PACKETS) {
-		zlog_debug(
-			"Recv IP packet from %s to %s on %s: size=%zu ip_header_size=%zu ip_proto=%d",
-			from_str, to_str, igmp->interface->name, len, ip_hlen,
-			ip_hdr->ip_p);
+	if (ip_hlen > len) {
+		zlog_warn(
+			"IGMP packet header claims size %zu, but we only have %zu bytes",
+			ip_hlen, len);
+		return -1;
 	}
 
 	igmp_msg = buf + ip_hlen;
-	msg_type = *igmp_msg;
 	igmp_msg_len = len - ip_hlen;
 
-	if (PIM_DEBUG_IGMP_PACKETS) {
-		zlog_debug(
-			"Recv IGMP packet from %s to %s on %s: ttl=%d msg_type=%d msg_size=%d",
-			from_str, to_str, igmp->interface->name, ip_hdr->ip_ttl,
-			msg_type, igmp_msg_len);
-	}
-
 	if (igmp_msg_len < PIM_IGMP_MIN_LEN) {
 		zlog_warn("IGMP message size=%d shorter than minimum=%d",
 			  igmp_msg_len, PIM_IGMP_MIN_LEN);
 		return -1;
 	}
 
+	msg_type = *igmp_msg;
+
+	if (PIM_DEBUG_IGMP_PACKETS) {
+		zlog_debug(
+			"Recv IGMP packet from %s to %s on %s: size=%zu ttl=%d msg_type=%d msg_size=%d",
+			from_str, to_str, igmp->interface->name, len, ip_hdr->ip_ttl,
+			msg_type, igmp_msg_len);
+	}
+
 	switch (msg_type) {
 	case PIM_IGMP_MEMBERSHIP_QUERY: {
 		int max_resp_code = igmp_msg[1];
@@ -524,6 +556,9 @@ int pim_igmp_packet(struct igmp_sock *igmp, char *buf, size_t len)
 
 	zlog_warn("Ignoring unsupported IGMP message type: %d", msg_type);
 
+	/* Collecting IGMP Rx stats */
+	igmp->rx_stats.unsupported++;
+
 	return -1;
 }
 
@@ -580,8 +615,8 @@ void pim_igmp_general_query_on(struct igmp_sock *igmp)
 			startup_mode ? "startup" : "non-startup", igmp->fd);
 	}
 	igmp->t_igmp_query_timer = NULL;
-	thread_add_timer(master, pim_igmp_general_query, igmp, query_interval,
-			 &igmp->t_igmp_query_timer);
+	thread_add_timer(router->master, pim_igmp_general_query, igmp,
+			 query_interval, &igmp->t_igmp_query_timer);
 }
 
 void pim_igmp_general_query_off(struct igmp_sock *igmp)
@@ -676,7 +711,8 @@ static void sock_close(struct igmp_sock *igmp)
 	THREAD_OFF(igmp->t_igmp_read);
 
 	if (close(igmp->fd)) {
-		zlog_err(
+		flog_err(
+			EC_LIB_SOCKET,
 			"Failure closing IGMP socket %s fd=%d on interface %s: errno=%d: %s",
 			inet_ntoa(igmp->ifaddr), igmp->fd,
 			igmp->interface->name, errno, safe_strerror(errno));
@@ -712,12 +748,45 @@ void igmp_startup_mode_on(struct igmp_sock *igmp)
 
 static void igmp_group_free(struct igmp_group *group)
 {
-	list_delete_and_null(&group->group_source_list);
+	list_delete(&group->group_source_list);
 
 	XFREE(MTYPE_PIM_IGMP_GROUP, group);
 }
 
-static void igmp_group_delete(struct igmp_group *group)
+static void igmp_group_count_incr(struct igmp_sock *igmp)
+{
+	struct pim_interface *pim_ifp = igmp->interface->info;
+
+	if (!pim_ifp)
+		return;
+
+	++pim_ifp->pim->igmp_group_count;
+	if (pim_ifp->pim->igmp_group_count
+	    == pim_ifp->pim->igmp_watermark_limit) {
+		zlog_warn(
+			"IGMP group count reached watermark limit: %u(vrf: %s)",
+			pim_ifp->pim->igmp_group_count,
+			VRF_LOGNAME(pim_ifp->pim->vrf));
+	}
+}
+
+static void igmp_group_count_decr(struct igmp_sock *igmp)
+{
+	struct pim_interface *pim_ifp = igmp->interface->info;
+
+	if (!pim_ifp)
+		return;
+
+	if (pim_ifp->pim->igmp_group_count == 0) {
+		zlog_warn("Cannot decrement igmp group count below 0(vrf: %s)",
+			  VRF_LOGNAME(pim_ifp->pim->vrf));
+		return;
+	}
+
+	--pim_ifp->pim->igmp_group_count;
+}
+
+void igmp_group_delete(struct igmp_group *group)
 {
 	struct listnode *src_node;
 	struct listnode *src_nextnode;
@@ -742,6 +811,7 @@ static void igmp_group_delete(struct igmp_group *group)
 	}
 
 	group_timer_off(group);
+	igmp_group_count_decr(group->group_igmp_sock);
 	listnode_delete(group->group_igmp_sock->igmp_group_list, group);
 	hash_release(group->group_igmp_sock->igmp_group_hash, group);
 
@@ -764,7 +834,7 @@ void igmp_sock_free(struct igmp_sock *igmp)
 	zassert(igmp->igmp_group_list);
 	zassert(!listcount(igmp->igmp_group_list));
 
-	list_delete_and_null(&igmp->igmp_group_list);
+	list_delete(&igmp->igmp_group_list);
 	hash_free(igmp->igmp_group_hash);
 
 	XFREE(MTYPE_PIM_IGMP_SOCKET, igmp);
@@ -805,22 +875,22 @@ void igmp_sock_delete_all(struct interface *ifp)
 	}
 }
 
-static unsigned int igmp_group_hash_key(void *arg)
+static unsigned int igmp_group_hash_key(const void *arg)
 {
-	struct igmp_group *group = (struct igmp_group *)arg;
+	const struct igmp_group *group = arg;
 
 	return jhash_1word(group->group_addr.s_addr, 0);
 }
 
-static int igmp_group_hash_equal(const void *arg1, const void *arg2)
+static bool igmp_group_hash_equal(const void *arg1, const void *arg2)
 {
 	const struct igmp_group *g1 = (const struct igmp_group *)arg1;
 	const struct igmp_group *g2 = (const struct igmp_group *)arg2;
 
 	if (g1->group_addr.s_addr == g2->group_addr.s_addr)
-		return 1;
+		return true;
 
-	return 0;
+	return false;
 }
 
 static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
@@ -839,21 +909,11 @@ static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
 	}
 
 	igmp = XCALLOC(MTYPE_PIM_IGMP_SOCKET, sizeof(*igmp));
-	if (!igmp) {
-		zlog_warn("%s %s: XCALLOC() failure", __FILE__,
-			  __PRETTY_FUNCTION__);
-		return 0;
-	}
 
 	igmp->igmp_group_list = list_new();
-	if (!igmp->igmp_group_list) {
-		zlog_err("%s %s: failure: igmp_group_list = list_new()",
-			 __FILE__, __PRETTY_FUNCTION__);
-		return 0;
-	}
 	igmp->igmp_group_list->del = (void (*)(void *))igmp_group_free;
 
-	snprintf(hash_name, 64, "IGMP %s hash", ifp->name);
+	snprintf(hash_name, sizeof(hash_name), "IGMP %s hash", ifp->name);
 	igmp->igmp_group_hash = hash_create(igmp_group_hash_key,
 					    igmp_group_hash_equal, hash_name);
 
@@ -867,6 +927,8 @@ static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
 		pim_ifp->igmp_default_robustness_variable;
 	igmp->sock_creation = pim_time_monotonic_sec();
 
+	igmp_stats_init(&igmp->rx_stats);
+
 	if (mtrace_only) {
 		igmp->mtrace_only = mtrace_only;
 		return igmp;
@@ -896,10 +958,9 @@ static int pim_igmp_read(struct thread *t)
 	socklen_t fromlen = sizeof(from);
 	socklen_t tolen = sizeof(to);
 	ifindex_t ifindex = -1;
-	int cont = 1;
 	int len;
 
-	while (cont) {
+	while (1) {
 		len = pim_socket_recvfromto(igmp->fd, buf, sizeof(buf), &from,
 					    &fromlen, &to, &tolen, &ifindex);
 		if (len < 0) {
@@ -925,7 +986,7 @@ static void igmp_read_on(struct igmp_sock *igmp)
 			   igmp->fd);
 	}
 	igmp->t_igmp_read = NULL;
-	thread_add_read(master, pim_igmp_read, igmp, igmp->fd,
+	thread_add_read(router->master, pim_igmp_read, igmp, igmp->fd,
 			&igmp->t_igmp_read);
 }
 
@@ -948,12 +1009,6 @@ struct igmp_sock *pim_igmp_sock_add(struct list *igmp_sock_list,
 	}
 
 	igmp = igmp_sock_new(fd, ifaddr, ifp, mtrace_only);
-	if (!igmp) {
-		zlog_err("%s %s: igmp_sock_new() failure", __FILE__,
-			 __PRETTY_FUNCTION__);
-		close(fd);
-		return 0;
-	}
 
 	igmp_read_on(igmp);
 
@@ -990,9 +1045,8 @@ static int igmp_group_timer(struct thread *t)
 		char group_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<group?>", group->group_addr, group_str,
 			       sizeof(group_str));
-		zlog_debug("%s: Timer for group %s on interface %s",
-			   __PRETTY_FUNCTION__, group_str,
-			   group->group_igmp_sock->interface->name);
+		zlog_debug("%s: Timer for group %s on interface %s", __func__,
+			   group_str, group->group_igmp_sock->interface->name);
 	}
 
 	zassert(group->group_filtermode_isexcl);
@@ -1058,8 +1112,8 @@ void igmp_group_timer_on(struct igmp_group *group, long interval_msec,
 	*/
 	zassert(group->group_filtermode_isexcl);
 
-	thread_add_timer_msec(master, igmp_group_timer, group, interval_msec,
-			      &group->t_group_timer);
+	thread_add_timer_msec(router->master, igmp_group_timer, group,
+			      interval_msec, &group->t_group_timer);
 }
 
 struct igmp_group *find_group_by_addr(struct igmp_sock *igmp,
@@ -1084,13 +1138,15 @@ struct igmp_group *igmp_add_group_by_addr(struct igmp_sock *igmp,
 
 	if (!pim_is_group_224_4(group_addr)) {
 		zlog_warn("%s: Group Specified is not part of 224.0.0.0/4",
-			  __PRETTY_FUNCTION__);
+			  __func__);
 		return NULL;
 	}
 
 	if (pim_is_group_224_0_0_0_24(group_addr)) {
-		zlog_warn("%s: Group specified is part of 224.0.0.0/24",
-			  __PRETTY_FUNCTION__);
+		if (PIM_DEBUG_IGMP_TRACE)
+			zlog_debug(
+				"%s: Group specified %s is part of 224.0.0.0/24",
+				__func__, inet_ntoa(group_addr));
 		return NULL;
 	}
 	/*
@@ -1107,19 +1163,8 @@ struct igmp_group *igmp_add_group_by_addr(struct igmp_sock *igmp,
 	*/
 
 	group = XCALLOC(MTYPE_PIM_IGMP_GROUP, sizeof(*group));
-	if (!group) {
-		zlog_warn("%s %s: XCALLOC() failure", __FILE__,
-			  __PRETTY_FUNCTION__);
-		return NULL; /* error, not found, could not create */
-	}
 
 	group->group_source_list = list_new();
-	if (!group->group_source_list) {
-		zlog_warn("%s %s: list_new() failure", __FILE__,
-			  __PRETTY_FUNCTION__);
-		XFREE(MTYPE_PIM_IGMP_GROUP, group); /* discard group */
-		return NULL; /* error, not found, could not initialize */
-	}
 	group->group_source_list->del = (void (*)(void *))igmp_source_free;
 
 	group->t_group_timer = NULL;
@@ -1147,6 +1192,8 @@ struct igmp_group *igmp_add_group_by_addr(struct igmp_sock *igmp,
 			group_str, igmp->fd, igmp->interface->name);
 	}
 
+	igmp_group_count_incr(igmp);
+
 	/*
 	  RFC 3376: 6.2.2. Definition of Group Timers
 
@@ -1182,3 +1229,42 @@ void igmp_send_query(int igmp_version, struct igmp_group *group, int fd,
 				   group_addr, query_max_response_time_dsec);
 	}
 }
+
+void igmp_send_query_on_intf(struct interface *ifp, int igmp_ver)
+{
+	struct pim_interface *pim_ifp = ifp->info;
+	struct listnode *sock_node = NULL;
+	struct igmp_sock *igmp = NULL;
+	struct in_addr dst_addr;
+	struct in_addr group_addr;
+	int query_buf_size;
+
+	if (!igmp_ver)
+		igmp_ver = 2;
+
+	if (igmp_ver == 3)
+		query_buf_size = PIM_IGMP_BUFSIZE_WRITE;
+	else
+		query_buf_size = IGMP_V12_MSG_SIZE;
+
+	dst_addr.s_addr = htonl(INADDR_ALLHOSTS_GROUP);
+	group_addr.s_addr = PIM_NET_INADDR_ANY;
+
+	if (PIM_DEBUG_IGMP_TRACE)
+		zlog_debug("Issuing general query on request on %s",
+				ifp->name);
+
+	for (ALL_LIST_ELEMENTS_RO(pim_ifp->igmp_socket_list, sock_node, igmp)) {
+
+		char query_buf[query_buf_size];
+
+		igmp_send_query(igmp_ver, 0 /* igmp_group */, igmp->fd,
+				igmp->interface->name, query_buf,
+				sizeof(query_buf), 0 /* num_sources */,
+				dst_addr, group_addr,
+				pim_ifp->igmp_query_max_response_time_dsec,
+				1 /* s_flag: always set for general queries */,
+				igmp->querier_robustness_variable,
+				igmp->querier_query_interval);
+	}
+}
