@@ -66,16 +66,13 @@ static bool mtrace_fwd_info_weak(struct pim_instance *pim,
 	struct pim_nexthop nexthop;
 	struct interface *ifp_in;
 	struct in_addr nh_addr;
-	int ret;
 	char nexthop_str[INET_ADDRSTRLEN];
 
-	nh_addr.s_addr = 0;
+	nh_addr.s_addr = INADDR_ANY;
 
 	memset(&nexthop, 0, sizeof(nexthop));
 
-	ret = pim_nexthop_lookup(pim, &nexthop, mtracep->src_addr, 1);
-
-	if (ret != 0) {
+	if (!pim_nexthop_lookup(pim, &nexthop, mtracep->src_addr, 1)) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_debug("mtrace not found neighbor");
 		return false;
@@ -126,13 +123,20 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	up = pim_upstream_find(pim, &sg);
 
 	if (!up) {
-		sg.src.s_addr = 0;
+		sg.src.s_addr = INADDR_ANY;
 		up = pim_upstream_find(pim, &sg);
 	}
 
 	if (!up)
 		return false;
 
+	if (!up->rpf.source_nexthop.interface) {
+		if (PIM_DEBUG_TRACE)
+			zlog_debug("%s: up %s RPF is not present", __func__,
+				   up->sg_str);
+		return false;
+	}
+
 	ifp_in = up->rpf.source_nexthop.interface;
 	nh_addr = up->rpf.source_nexthop.mrib_nexthop_addr.u.prefix4;
 	total = htonl(MTRACE_UNKNOWN_COUNT);
@@ -156,7 +160,7 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	rspp->rtg_proto = MTRACE_RTG_PROTO_PIM;
 
 	/* 6.2.2. 4. Fill in ... S, and Src Mask */
-	if (sg.src.s_addr) {
+	if (sg.src.s_addr != INADDR_ANY) {
 		rspp->s = 1;
 		rspp->src_mask = MTRACE_SRC_MASK_SOURCE;
 	} else {
@@ -177,9 +181,9 @@ static void mtrace_rsp_set_fwd_code(struct igmp_mtrace_rsp *mtrace_rspp,
 static void mtrace_rsp_init(struct igmp_mtrace_rsp *mtrace_rspp)
 {
 	mtrace_rspp->arrival = 0;
-	mtrace_rspp->incoming.s_addr = 0;
-	mtrace_rspp->outgoing.s_addr = 0;
-	mtrace_rspp->prev_hop.s_addr = 0;
+	mtrace_rspp->incoming.s_addr = INADDR_ANY;
+	mtrace_rspp->outgoing.s_addr = INADDR_ANY;
+	mtrace_rspp->prev_hop.s_addr = INADDR_ANY;
 	mtrace_rspp->in_count = htonl(MTRACE_UNKNOWN_COUNT);
 	mtrace_rspp->out_count = htonl(MTRACE_UNKNOWN_COUNT);
 	mtrace_rspp->total = htonl(MTRACE_UNKNOWN_COUNT);
@@ -267,11 +271,10 @@ static uint32_t query_arrival_time(void)
 	struct timeval tv;
 	uint32_t qat;
 
-	char m_qat[] = "Query arrival time lookup failed: errno=%d: %s";
-
 	if (gettimeofday(&tv, NULL) < 0) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(m_qat, errno, safe_strerror(errno));
+			zlog_warn("Query arrival time lookup failed: errno=%d: %s",
+				  errno, safe_strerror(errno));
 		return 0;
 	}
 	/* not sure second offset correct, as I get different value */
@@ -411,9 +414,7 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 
 	if (interface == NULL) {
 		memset(&nexthop, 0, sizeof(nexthop));
-		ret = pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0);
-
-		if (ret != 0) {
+		if (!pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0)) {
 			close(fd);
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
@@ -561,7 +562,6 @@ static int mtrace_send_response(struct pim_instance *pim,
 				struct igmp_mtrace *mtracep, size_t mtrace_len)
 {
 	struct pim_nexthop nexthop;
-	int ret;
 
 	mtracep->type = PIM_IGMP_MTRACE_RESPONSE;
 
@@ -592,9 +592,7 @@ static int mtrace_send_response(struct pim_instance *pim,
 	} else {
 		memset(&nexthop, 0, sizeof(nexthop));
 		/* TODO: should use unicast rib lookup */
-		ret = pim_nexthop_lookup(pim, &nexthop, mtracep->rsp_addr, 1);
-
-		if (ret != 0) {
+		if (!pim_nexthop_lookup(pim, &nexthop, mtracep->rsp_addr, 1)) {
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
 					"Dropped response qid=%ud, no route to "
@@ -615,7 +613,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	static uint32_t qry_id, qry_src;
 	char mtrace_buf[MTRACE_HDR_SIZE + MTRACE_MAX_HOPS * MTRACE_RSP_SIZE];
 	struct interface *ifp;
-	struct interface *out_ifp;
+	struct interface *out_ifp = NULL;
 	struct pim_interface *pim_ifp;
 	struct pim_instance *pim;
 	struct igmp_mtrace *mtracep;
@@ -671,6 +669,9 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 		return -1;
 	}
 
+	/* Collecting IGMP Rx stats */
+	igmp->rx_stats.mtrace_req++;
+
 	if (PIM_DEBUG_MTRACE)
 		mtrace_debug(pim_ifp, mtracep, igmp_msg_len);
 
@@ -777,7 +778,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	/* 6.2.2. 2. Attempt to determine the forwarding information... */
 
-	if (mtracep->grp_addr.s_addr)
+	if (mtracep->grp_addr.s_addr != INADDR_ANY)
 		fwd_info = mtrace_fwd_info(pim, mtracep, rspp, &out_ifp);
 	else
 		fwd_info = mtrace_fwd_info_weak(pim, mtracep, rspp, &out_ifp);
@@ -795,7 +796,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	reached_source = false;
 
-	if (nh_addr.s_addr == 0) {
+	if (nh_addr.s_addr == INADDR_ANY) {
 		/* no pim? i.e. 7.5.3. No Previous Hop */
 		if (!out_ifp->info) {
 			if (PIM_DEBUG_MTRACE)
@@ -814,7 +815,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 		 * Previous-hop router not known,
 		 * packet is sent to an appropriate multicast address
 		 */
-		inet_aton(MCAST_ALL_ROUTERS, &nh_addr);
+		(void)inet_aton(MCAST_ALL_ROUTERS, &nh_addr);
 	}
 
 	/* 6.2.2 8. If this router is the Rendez-vous Point */
@@ -862,6 +863,16 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 	pim_ifp = ifp->info;
 	pim = pim_ifp->pim;
 
+	if (igmp_msg_len < (int)sizeof(struct igmp_mtrace)) {
+		if (PIM_DEBUG_MTRACE)
+			zlog_warn(
+				"Recv mtrace packet from %s on %s: too short,"
+				" len=%d, min=%zu",
+				from_str, ifp->name, igmp_msg_len,
+				sizeof(struct igmp_mtrace));
+		return -1;
+	}
+
 	mtracep = (struct igmp_mtrace *)igmp_msg;
 
 	recv_checksum = mtracep->checksum;
@@ -881,6 +892,9 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	mtracep->checksum = checksum;
 
+	/* Collecting IGMP Rx stats */
+	igmp->rx_stats.mtrace_rsp++;
+
 	if (PIM_DEBUG_MTRACE)
 		mtrace_debug(pim_ifp, mtracep, igmp_msg_len);
 
