@@ -25,6 +25,7 @@
 #include "vty.h"
 #include "plist.h"
 #include "sockopt.h"
+#include "lib_errors.h"
 
 #include "pimd.h"
 #include "pim_rpf.h"
@@ -41,60 +42,63 @@
 #include "pim_zlookup.h"
 #include "pim_ssm.h"
 #include "pim_sock.h"
+#include "pim_vxlan.h"
 
 static void mroute_read_on(struct pim_instance *pim);
 
 static int pim_mroute_set(struct pim_instance *pim, int enable)
 {
 	int err;
-	int opt;
-	socklen_t opt_len = sizeof(opt);
+	int opt, data;
+	socklen_t data_len = sizeof(data);
 	long flags;
 
 	/*
 	 * We need to create the VRF table for the pim mroute_socket
 	 */
 	if (pim->vrf_id != VRF_DEFAULT) {
-		if (pimd_privs.change(ZPRIVS_RAISE))
-			zlog_err(
-				"pim_mroute_socket_enable: could not raise privs, %s",
-				safe_strerror(errno));
+		frr_with_privs(&pimd_privs) {
 
-		opt = pim->vrf->data.l.table_id;
-		err = setsockopt(pim->mroute_socket, IPPROTO_IP, MRT_TABLE,
-				 &opt, opt_len);
+			data = pim->vrf->data.l.table_id;
+			err = setsockopt(pim->mroute_socket, IPPROTO_IP,
+					 MRT_TABLE,
+					 &data, data_len);
+			if (err) {
+				zlog_warn(
+					"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP, MRT_TABLE=%d): errno=%d: %s",
+					__FILE__, __func__, pim->mroute_socket,
+					data, errno, safe_strerror(errno));
+				return -1;
+			}
+
+		}
+	}
+
+	frr_with_privs(&pimd_privs) {
+		opt = enable ? MRT_INIT : MRT_DONE;
+		/*
+		 * *BSD *cares* about what value we pass down
+		 * here
+		 */
+		data = 1;
+		err = setsockopt(pim->mroute_socket, IPPROTO_IP,
+				 opt, &data, data_len);
 		if (err) {
 			zlog_warn(
-				"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP, MRT_TABLE=%d): errno=%d: %s",
-				__FILE__, __PRETTY_FUNCTION__,
-				pim->mroute_socket, opt, errno,
+				"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP,%s=%d): errno=%d: %s",
+				__FILE__, __func__, pim->mroute_socket,
+				enable ? "MRT_INIT" : "MRT_DONE", data, errno,
 				safe_strerror(errno));
 			return -1;
 		}
-
-		if (pimd_privs.change(ZPRIVS_LOWER))
-			zlog_err(
-				"pim_mroute_socket_enable: could not lower privs, %s",
-				safe_strerror(errno));
-	}
-
-	opt = enable ? MRT_INIT : MRT_DONE;
-	err = setsockopt(pim->mroute_socket, IPPROTO_IP, opt, &opt, opt_len);
-	if (err) {
-		zlog_warn(
-			"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP,%s=%d): errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__, pim->mroute_socket,
-			enable ? "MRT_INIT" : "MRT_DONE", opt, errno,
-			safe_strerror(errno));
-		return -1;
 	}
 
 #if defined(HAVE_IP_PKTINFO)
 	if (enable) {
 		/* Linux and Solaris IP_PKTINFO */
-		opt = 1;
-		if (setsockopt(pim->mroute_socket, IPPROTO_IP, IP_PKTINFO, &opt,
-			       sizeof(opt))) {
+		data = 1;
+		if (setsockopt(pim->mroute_socket, IPPROTO_IP, IP_PKTINFO,
+			       &data, data_len)) {
 			zlog_warn(
 				"Could not set IP_PKTINFO on socket fd=%d: errno=%d: %s",
 				pim->mroute_socket, errno,
@@ -141,7 +145,7 @@ static int pim_mroute_set(struct pim_instance *pim, int enable)
 	return 0;
 }
 
-static const char *igmpmsgtype2str[IGMPMSG_WRVIFWHOLE + 1] = {
+static const char *const igmpmsgtype2str[IGMPMSG_WRVIFWHOLE + 1] = {
 	"<unknown_upcall?>", "NOCACHE", "WRONGVIF", "WHOLEPKT", "WRVIFWHOLE"};
 
 static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
@@ -152,18 +156,18 @@ static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
 	struct pim_rpf *rpg;
 	struct prefix_sg sg;
 
-	rpg = RP(pim_ifp->pim, msg->im_dst);
+	rpg = pim_ifp ? RP(pim_ifp->pim, msg->im_dst) : NULL;
 	/*
 	 * If the incoming interface is unknown OR
 	 * the Interface type is SSM we don't need to
 	 * do anything here
 	 */
-	if ((pim_rpf_addr_is_inaddr_none(rpg)) || (!pim_ifp)
-	    || (!(PIM_I_am_DR(pim_ifp)))) {
+	if (!rpg || pim_rpf_addr_is_inaddr_none(rpg)) {
 		if (PIM_DEBUG_MROUTE_DETAIL)
 			zlog_debug(
-				"%s: Interface is not configured correctly to handle incoming packet: Could be !DR, !pim_ifp, !SM, !RP",
-				__PRETTY_FUNCTION__);
+				"%s: Interface is not configured correctly to handle incoming packet: Could be !pim_ifp, !SM, !RP",
+				__func__);
+
 		return 0;
 	}
 
@@ -175,7 +179,7 @@ static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE_DETAIL)
 			zlog_debug(
 				"%s: Received incoming packet that doesn't originate on our seg",
-				__PRETTY_FUNCTION__);
+				__func__);
 		return 0;
 	}
 
@@ -183,40 +187,53 @@ static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
 	sg.src = msg->im_src;
 	sg.grp = msg->im_dst;
 
-	up = pim_upstream_find_or_add(&sg, ifp, PIM_UPSTREAM_FLAG_MASK_FHR,
-				      __PRETTY_FUNCTION__);
-	if (!up) {
-		if (PIM_DEBUG_MROUTE) {
+	if (!(PIM_I_am_DR(pim_ifp))) {
+		if (PIM_DEBUG_MROUTE_DETAIL)
 			zlog_debug(
-				"%s: Failure to add upstream information for %s",
-				__PRETTY_FUNCTION__, pim_str_sg_dump(&sg));
-		}
+				"%s: Interface is not the DR blackholing incoming traffic for %s",
+				__func__, pim_str_sg_dump(&sg));
+
+		/*
+		 * We are not the DR, but we are still receiving packets
+		 * Let's blackhole those packets for the moment
+		 * As that they will be coming up to the cpu
+		 * and causing us to consider them.
+		 *
+		 * This *will* create a dangling channel_oil
+		 * that I see no way to get rid of.  Just noting
+		 * this for future reference.
+		 */
+		up = pim_upstream_find_or_add(
+			&sg, ifp, PIM_UPSTREAM_FLAG_MASK_SRC_NOCACHE, __func__);
+		pim_upstream_mroute_add(up->channel_oil, __func__);
+
 		return 0;
 	}
 
+	up = pim_upstream_find_or_add(&sg, ifp, PIM_UPSTREAM_FLAG_MASK_FHR,
+				      __func__);
+
 	/*
 	 * I moved this debug till after the actual add because
 	 * I want to take advantage of the up->sg_str being filled in.
 	 */
 	if (PIM_DEBUG_MROUTE) {
 		zlog_debug("%s: Adding a Route %s for WHOLEPKT consumption",
-			   __PRETTY_FUNCTION__, up->sg_str);
+			   __func__, up->sg_str);
 	}
 
 	PIM_UPSTREAM_FLAG_SET_SRC_STREAM(up->flags);
 	pim_upstream_keep_alive_timer_start(up, pim_ifp->pim->keep_alive_time);
 
 	up->channel_oil->cc.pktcnt++;
-	PIM_UPSTREAM_FLAG_SET_FHR(up->flags);
 	// resolve mfcc_parent prior to mroute_add in channel_add_oif
-	if (up->channel_oil->oil.mfcc_parent >= MAXVIFS) {
-		int vif_index = 0;
-		vif_index = pim_if_find_vifindex_by_ifindex(
-			pim_ifp->pim,
-			up->rpf.source_nexthop.interface->ifindex);
-		up->channel_oil->oil.mfcc_parent = vif_index;
+	if (up->rpf.source_nexthop.interface &&
+	    up->channel_oil->oil.mfcc_parent >= MAXVIFS) {
+		pim_upstream_mroute_iif_update(up->channel_oil, __func__);
 	}
 	pim_register_join(up);
+	/* if we have receiver, inherit from parent */
+	pim_upstream_inherited_olist_decide(pim_ifp->pim, up);
 
 	return 0;
 }
@@ -245,46 +262,50 @@ static int pim_mroute_msg_wholepkt(int fd, struct interface *ifp,
 
 		up = pim_upstream_find(pim_ifp->pim, &star);
 
-		if (up && PIM_UPSTREAM_FLAG_TEST_SRC_IGMP(up->flags)) {
+		if (up && PIM_UPSTREAM_FLAG_TEST_CAN_BE_LHR(up->flags)) {
 			up = pim_upstream_add(pim_ifp->pim, &sg, ifp,
 					      PIM_UPSTREAM_FLAG_MASK_SRC_LHR,
-					      __PRETTY_FUNCTION__, NULL);
+					      __func__, NULL);
 			if (!up) {
 				if (PIM_DEBUG_MROUTE)
 					zlog_debug(
 						"%s: Unable to create upstream information for %s",
-						__PRETTY_FUNCTION__,
-						pim_str_sg_dump(&sg));
+						__func__, pim_str_sg_dump(&sg));
 				return 0;
 			}
 			pim_upstream_keep_alive_timer_start(
 				up, pim_ifp->pim->keep_alive_time);
 			pim_upstream_inherited_olist(pim_ifp->pim, up);
-			pim_upstream_switch(pim_ifp->pim, up,
-					    PIM_UPSTREAM_JOINED);
+			pim_upstream_update_join_desired(pim_ifp->pim, up);
 
 			if (PIM_DEBUG_MROUTE)
 				zlog_debug("%s: Creating %s upstream on LHR",
-					   __PRETTY_FUNCTION__, up->sg_str);
+					   __func__, up->sg_str);
 			return 0;
 		}
 		if (PIM_DEBUG_MROUTE_DETAIL) {
 			zlog_debug(
 				"%s: Unable to find upstream channel WHOLEPKT%s",
-				__PRETTY_FUNCTION__, pim_str_sg_dump(&sg));
+				__func__, pim_str_sg_dump(&sg));
 		}
 		return 0;
 	}
 
+	if (!up->rpf.source_nexthop.interface) {
+		if (PIM_DEBUG_PIM_TRACE)
+			zlog_debug("%s: up %s RPF is not present", __func__,
+				   up->sg_str);
+		return 0;
+	}
+
 	pim_ifp = up->rpf.source_nexthop.interface->info;
 
-	rpg = RP(pim_ifp->pim, sg.grp);
+	rpg = pim_ifp ? RP(pim_ifp->pim, sg.grp) : NULL;
 
 	if ((pim_rpf_addr_is_inaddr_none(rpg)) || (!pim_ifp)
 	    || (!(PIM_I_am_DR(pim_ifp)))) {
 		if (PIM_DEBUG_MROUTE) {
-			zlog_debug("%s: Failed Check send packet",
-				   __PRETTY_FUNCTION__);
+			zlog_debug("%s: Failed Check send packet", __func__);
 		}
 		return 0;
 	}
@@ -300,6 +321,15 @@ static int pim_mroute_msg_wholepkt(int fd, struct interface *ifp,
 					pim_str_sg_dump(&sg));
 			return 0;
 		}
+
+		if (!PIM_UPSTREAM_FLAG_TEST_FHR(up->flags)) {
+			if (PIM_DEBUG_PIM_REG)
+				zlog_debug(
+					"%s register forward skipped, not FHR",
+					up->sg_str);
+			return 0;
+		}
+
 		pim_register_send((uint8_t *)buf + sizeof(struct ip),
 				  ntohs(ip_hdr->ip_len) - sizeof(struct ip),
 				  pim_ifp->primary_address, rpg, 0, up);
@@ -333,8 +363,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s could not find input interface for input_vif_index=%d",
-				__PRETTY_FUNCTION__, pim_str_sg_dump(&sg),
-				msg->im_vif);
+				__func__, pim_str_sg_dump(&sg), msg->im_vif);
 		return -1;
 	}
 
@@ -343,8 +372,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s multicast not enabled on interface %s",
-				__PRETTY_FUNCTION__, pim_str_sg_dump(&sg),
-				ifp->name);
+				__func__, pim_str_sg_dump(&sg), ifp->name);
 		return -2;
 	}
 
@@ -354,8 +382,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s could not find channel on interface %s",
-				__PRETTY_FUNCTION__, pim_str_sg_dump(&sg),
-				ifp->name);
+				__func__, pim_str_sg_dump(&sg), ifp->name);
 
 		star_g.src.s_addr = INADDR_ANY;
 		ch = pim_ifchannel_find(ifp, &star_g);
@@ -363,8 +390,8 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 			if (PIM_DEBUG_MROUTE)
 				zlog_debug(
 					"%s: WRONGVIF (*,G)=%s could not find channel on interface %s",
-					__PRETTY_FUNCTION__,
-					pim_str_sg_dump(&star_g), ifp->name);
+					__func__, pim_str_sg_dump(&star_g),
+					ifp->name);
 			return -3;
 		}
 	}
@@ -387,7 +414,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE) {
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s channel is not on Assert NoInfo state for interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+				__func__, ch->sg_str, ifp->name);
 		}
 		return -4;
 	}
@@ -396,7 +423,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE) {
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s interface %s is not downstream for channel",
-				__PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+				__func__, ch->sg_str, ifp->name);
 		}
 		return -5;
 	}
@@ -405,7 +432,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 		if (PIM_DEBUG_MROUTE) {
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s assert_action_a1 failure on interface %s",
-				__PRETTY_FUNCTION__, ch->sg_str, ifp->name);
+				__func__, ch->sg_str, ifp->name);
 		}
 		return -6;
 	}
@@ -418,11 +445,11 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 {
 	const struct ip *ip_hdr = (const struct ip *)buf;
 	struct pim_interface *pim_ifp;
+	struct pim_instance *pim;
 	struct pim_ifchannel *ch;
 	struct pim_upstream *up;
 	struct prefix_sg star_g;
 	struct prefix_sg sg;
-	struct channel_oil *oil;
 
 	pim_ifp = ifp->info;
 
@@ -441,23 +468,28 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 
 	star_g = sg;
 	star_g.src.s_addr = INADDR_ANY;
-#if 0
-  ch = pim_ifchannel_find(ifp, &star_g);
-  if (ch)
-    {
-      if (PIM_DEBUG_MROUTE)
-	zlog_debug ("WRVIFWHOLE (*,G)=%s found ifchannel on interface %s",
-		    pim_str_sg_dump (&star_g), ifp->name);
-      return -1;
-    }
-#endif
+
+	pim = pim_ifp->pim;
+	/*
+	 * If the incoming interface is the pimreg, then
+	 * we know the callback is associated with a pim register
+	 * packet and there is nothing to do here as that
+	 * normal pim processing will see the packet and allow
+	 * us to do the right thing.
+	 */
+	if (ifp == pim->regiface) {
+		return 0;
+	}
 
 	up = pim_upstream_find(pim_ifp->pim, &sg);
 	if (up) {
 		struct pim_upstream *parent;
 		struct pim_nexthop source;
 		struct pim_rpf *rpf = RP(pim_ifp->pim, sg.grp);
-		if (!rpf || !rpf->source_nexthop.interface)
+
+		/* No RPF or No RPF interface or No mcast on RPF interface */
+		if (!rpf || !rpf->source_nexthop.interface
+		    || !rpf->source_nexthop.interface->info)
 			return 0;
 
 		/*
@@ -478,28 +510,33 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 		 * the pimreg period, so I believe we can ignore this packet
 		 */
 		if (!PIM_UPSTREAM_FLAG_TEST_FHR(up->flags)) {
-			// No if channel, but upstream we are at the RP.
-			if (pim_nexthop_lookup(pim_ifp->pim, &source,
-					       up->upstream_register, 0)
-			    == 0) {
+			/*
+			 * No if channel, but upstream we are at the RP.
+			 *
+			 * This could be a anycast RP too and we may
+			 * not have received a register packet from
+			 * the source here at all.  So gracefully
+			 * bow out of doing a nexthop lookup and
+			 * setting the SPTBIT to true
+			 */
+			if (up->upstream_register.s_addr != INADDR_ANY &&
+			    pim_nexthop_lookup(pim_ifp->pim, &source,
+					       up->upstream_register, 0)) {
 				pim_register_stop_send(source.interface, &sg,
 						       pim_ifp->primary_address,
 						       up->upstream_register);
 				up->sptbit = PIM_UPSTREAM_SPTBIT_TRUE;
 			}
-			if (!up->channel_oil)
-				up->channel_oil = pim_channel_oil_add(
-					pim_ifp->pim, &sg,
-					pim_ifp->mroute_vif_index);
+
 			pim_upstream_inherited_olist(pim_ifp->pim, up);
 			if (!up->channel_oil->installed)
-				pim_mroute_add(up->channel_oil,
-					       __PRETTY_FUNCTION__);
+				pim_upstream_mroute_add(up->channel_oil,
+							__func__);
 		} else {
 			if (I_am_RP(pim_ifp->pim, up->sg.grp)) {
 				if (pim_nexthop_lookup(pim_ifp->pim, &source,
-						       up->upstream_register, 0)
-				    == 0)
+						       up->upstream_register,
+						       0))
 					pim_register_stop_send(
 						source.interface, &sg,
 						pim_ifp->primary_address,
@@ -515,13 +552,10 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 	}
 
 	pim_ifp = ifp->info;
-	oil = pim_channel_oil_add(pim_ifp->pim, &sg, pim_ifp->mroute_vif_index);
-	if (!oil->installed)
-		pim_mroute_add(oil, __PRETTY_FUNCTION__);
 	if (pim_if_connected_to_source(ifp, sg.src)) {
 		up = pim_upstream_add(pim_ifp->pim, &sg, ifp,
-				      PIM_UPSTREAM_FLAG_MASK_FHR,
-				      __PRETTY_FUNCTION__, NULL);
+				      PIM_UPSTREAM_FLAG_MASK_FHR, __func__,
+				      NULL);
 		if (!up) {
 			if (PIM_DEBUG_MROUTE)
 				zlog_debug(
@@ -532,13 +566,20 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 		PIM_UPSTREAM_FLAG_SET_SRC_STREAM(up->flags);
 		pim_upstream_keep_alive_timer_start(
 			up, pim_ifp->pim->keep_alive_time);
-		up->channel_oil = oil;
 		up->channel_oil->cc.pktcnt++;
 		pim_register_join(up);
 		pim_upstream_inherited_olist(pim_ifp->pim, up);
+		if (!up->channel_oil->installed)
+			pim_upstream_mroute_add(up->channel_oil, __func__);
 
 		// Send the packet to the RP
 		pim_mroute_msg_wholepkt(fd, ifp, buf);
+	} else {
+		up = pim_upstream_add(pim_ifp->pim, &sg, ifp,
+				      PIM_UPSTREAM_FLAG_MASK_SRC_NOCACHE,
+				      __func__, NULL);
+		if (!up->channel_oil->installed)
+			pim_upstream_mroute_add(up->channel_oil, __func__);
 	}
 
 	return 0;
@@ -558,6 +599,9 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 	struct in_addr ifaddr;
 	struct igmp_sock *igmp;
 
+	if (buf_size < (int)sizeof(struct ip))
+		return 0;
+
 	ip_hdr = (const struct ip *)buf;
 
 	if (ip_hdr->ip_p == IPPROTO_IGMP) {
@@ -584,10 +628,10 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 			pim_inet4_dump("<dst?>", ip_hdr->ip_dst, ip_dst_str,
 				       sizeof(ip_dst_str));
 
-			zlog_warn(
+			zlog_debug(
 				"%s(%s): igmp kernel upcall on %s(%p) for %s -> %s",
-				__PRETTY_FUNCTION__, pim->vrf->name, ifp->name,
-				igmp, ip_src_str, ip_dst_str);
+				__func__, pim->vrf->name, ifp->name, igmp,
+				ip_src_str, ip_dst_str);
 		}
 		if (igmp)
 			pim_igmp_packet(igmp, (char *)buf, buf_size);
@@ -600,8 +644,8 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 				       sizeof(grp_str));
 			zlog_debug(
 				"%s: no kernel upcall proto=%d src: %s dst: %s msg_size=%d",
-				__PRETTY_FUNCTION__, ip_hdr->ip_p, src_str,
-				grp_str, buf_size);
+				__func__, ip_hdr->ip_p, src_str, grp_str,
+				buf_size);
 		}
 
 	} else {
@@ -616,10 +660,9 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 				       sizeof(src_str));
 			pim_inet4_dump("<grp?>", msg->im_dst, grp_str,
 				       sizeof(grp_str));
-			zlog_warn(
+			zlog_debug(
 				"%s: pim kernel upcall %s type=%d ip_p=%d from fd=%d for (S,G)=(%s,%s) on %s vifi=%d  size=%d",
-				__PRETTY_FUNCTION__,
-				igmpmsgtype2str[msg->im_msgtype],
+				__func__, igmpmsgtype2str[msg->im_msgtype],
 				msg->im_msgtype, ip_hdr->ip_p,
 				pim->mroute_socket, src_str, grp_str, ifp->name,
 				msg->im_vif, buf_size);
@@ -671,19 +714,17 @@ static int mroute_read(struct thread *t)
 			if (errno == EWOULDBLOCK || errno == EAGAIN)
 				break;
 
-			if (PIM_DEBUG_MROUTE)
-				zlog_warn(
-					"%s: failure reading rd=%d: fd=%d: errno=%d: %s",
-					__PRETTY_FUNCTION__, rd,
-					pim->mroute_socket, errno,
-					safe_strerror(errno));
+			zlog_warn(
+				"%s: failure reading rd=%d: fd=%d: errno=%d: %s",
+				__func__, rd, pim->mroute_socket, errno,
+				safe_strerror(errno));
 			goto done;
 		}
 
 		result = pim_mroute_msg(pim, buf, rd, ifindex);
 
 		count++;
-		if (count % qpim_packet_process == 0)
+		if (count % router->packet_process == 0)
 			cont = 0;
 	}
 /* Keep reading */
@@ -695,7 +736,7 @@ done:
 
 static void mroute_read_on(struct pim_instance *pim)
 {
-	thread_add_read(master, mroute_read, pim, pim->mroute_socket,
+	thread_add_read(router->master, mroute_read, pim, pim->mroute_socket,
 			&pim->thread);
 }
 
@@ -708,32 +749,29 @@ int pim_mroute_socket_enable(struct pim_instance *pim)
 {
 	int fd;
 
-	if (pimd_privs.change(ZPRIVS_RAISE))
-		zlog_err("pim_mroute_socket_enable: could not raise privs, %s",
-			 safe_strerror(errno));
+	frr_with_privs(&pimd_privs) {
 
-	fd = socket(AF_INET, SOCK_RAW, IPPROTO_IGMP);
+		fd = socket(AF_INET, SOCK_RAW, IPPROTO_IGMP);
 
-	if (fd < 0) {
-		zlog_warn("Could not create mroute socket: errno=%d: %s", errno,
-			  safe_strerror(errno));
-		return -2;
-	}
+		if (fd < 0) {
+			zlog_warn("Could not create mroute socket: errno=%d: %s",
+				  errno,
+				  safe_strerror(errno));
+			return -2;
+		}
 
 #ifdef SO_BINDTODEVICE
-	if (pim->vrf->vrf_id != VRF_DEFAULT
-	    && setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE, pim->vrf->name,
-			  strlen(pim->vrf->name))) {
-		zlog_warn("Could not setsockopt SO_BINDTODEVICE: %s",
-			  safe_strerror(errno));
-		close(fd);
-		return -3;
-	}
+		if (pim->vrf->vrf_id != VRF_DEFAULT
+		    && setsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE,
+				  pim->vrf->name, strlen(pim->vrf->name))) {
+			zlog_warn("Could not setsockopt SO_BINDTODEVICE: %s",
+				  safe_strerror(errno));
+			close(fd);
+			return -3;
+		}
 #endif
 
-	if (pimd_privs.change(ZPRIVS_LOWER))
-		zlog_err("pim_mroute_socket_enable: could not lower privs, %s",
-			 safe_strerror(errno));
+	}
 
 	pim->mroute_socket = fd;
 	if (pim_mroute_set(pim, 1)) {
@@ -786,7 +824,7 @@ int pim_mroute_add_vif(struct interface *ifp, struct in_addr ifaddr,
 	int err;
 
 	if (PIM_DEBUG_MROUTE)
-		zlog_debug("%s: Add Vif %d (%s[%s])", __PRETTY_FUNCTION__,
+		zlog_debug("%s: Add Vif %d (%s[%s])", __func__,
 			   pim_ifp->mroute_vif_index, ifp->name,
 			   pim_ifp->pim->vrf->name);
 
@@ -798,7 +836,7 @@ int pim_mroute_add_vif(struct interface *ifp, struct in_addr ifaddr,
 	if (ifaddr.s_addr == INADDR_ANY) {
 		zlog_warn(
 			"%s: unnumbered interfaces are not supported on this platform",
-			__PRETTY_FUNCTION__);
+			__func__);
 		return -1;
 	}
 	memcpy(&vc.vifc_lcl_addr, &ifaddr, sizeof(vc.vifc_lcl_addr));
@@ -824,9 +862,8 @@ int pim_mroute_add_vif(struct interface *ifp, struct in_addr ifaddr,
 
 		zlog_warn(
 			"%s: failure: setsockopt(fd=%d,IPPROTO_IP,MRT_ADD_VIF,vif_index=%d,ifaddr=%s,flag=%d): errno=%d: %s",
-			__PRETTY_FUNCTION__, pim_ifp->pim->mroute_socket,
-			ifp->ifindex, ifaddr_str, flags, errno,
-			safe_strerror(errno));
+			__func__, pim_ifp->pim->mroute_socket, ifp->ifindex,
+			ifaddr_str, flags, errno, safe_strerror(errno));
 		return -2;
 	}
 
@@ -840,7 +877,7 @@ int pim_mroute_del_vif(struct interface *ifp)
 	int err;
 
 	if (PIM_DEBUG_MROUTE)
-		zlog_debug("%s: Del Vif %d (%s[%s])", __PRETTY_FUNCTION__,
+		zlog_debug("%s: Del Vif %d (%s[%s])", __func__,
 			   pim_ifp->mroute_vif_index, ifp->name,
 			   pim_ifp->pim->vrf->name);
 
@@ -852,44 +889,100 @@ int pim_mroute_del_vif(struct interface *ifp)
 	if (err) {
 		zlog_warn(
 			"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP,MRT_DEL_VIF,vif_index=%d): errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__,
-			pim_ifp->pim->mroute_socket, pim_ifp->mroute_vif_index,
-			errno, safe_strerror(errno));
+			__FILE__, __func__, pim_ifp->pim->mroute_socket,
+			pim_ifp->mroute_vif_index, errno, safe_strerror(errno));
 		return -2;
 	}
 
 	return 0;
 }
 
-int pim_mroute_add(struct channel_oil *c_oil, const char *name)
+/*
+ * Prevent creating MFC entry with OIF=IIF.
+ *
+ * This is a protection against implementation mistakes.
+ *
+ * PIM protocol implicitely ensures loopfree multicast topology.
+ *
+ * IGMP must be protected against adding looped MFC entries created
+ * by both source and receiver attached to the same interface. See
+ * TODO T22.
+ * We shall allow igmp to create upstream when it is DR for the intf.
+ * Assume RP reachable via non DR.
+ */
+bool pim_mroute_allow_iif_in_oil(struct channel_oil *c_oil,
+		int oif_index)
+{
+#ifdef PIM_ENFORCE_LOOPFREE_MFC
+	struct interface *ifp_out;
+	struct pim_interface *pim_ifp;
+
+	if (c_oil->up &&
+		PIM_UPSTREAM_FLAG_TEST_ALLOW_IIF_IN_OIL(c_oil->up->flags))
+		return true;
+
+	ifp_out = pim_if_find_by_vif_index(c_oil->pim, oif_index);
+	if (!ifp_out)
+		return false;
+	pim_ifp = ifp_out->info;
+	if (!pim_ifp)
+		return false;
+	if ((c_oil->oif_flags[oif_index] & PIM_OIF_FLAG_PROTO_IGMP) &&
+			PIM_I_am_DR(pim_ifp))
+		return true;
+
+	return false;
+#else
+	return true;
+#endif
+}
+
+static inline void pim_mroute_copy(struct mfcctl *oil,
+		struct channel_oil *c_oil)
+{
+	int i;
+
+	oil->mfcc_origin = c_oil->oil.mfcc_origin;
+	oil->mfcc_mcastgrp = c_oil->oil.mfcc_mcastgrp;
+	oil->mfcc_parent = c_oil->oil.mfcc_parent;
+
+	for (i = 0; i < MAXVIFS; ++i) {
+		if ((oil->mfcc_parent == i) &&
+				!pim_mroute_allow_iif_in_oil(c_oil, i)) {
+			oil->mfcc_ttls[i] = 0;
+			continue;
+		}
+
+		if (c_oil->oif_flags[i] & PIM_OIF_FLAG_MUTE)
+			oil->mfcc_ttls[i] = 0;
+		else
+			oil->mfcc_ttls[i] = c_oil->oil.mfcc_ttls[i];
+	}
+}
+
+/* This function must not be called directly 0
+ * use pim_upstream_mroute_add or pim_static_mroute_add instead
+ */
+static int pim_mroute_add(struct channel_oil *c_oil, const char *name)
 {
 	struct pim_instance *pim = c_oil->pim;
+	struct mfcctl tmp_oil = { {0} };
 	int err;
-	int orig = 0;
-	int orig_iif_vif = 0;
 
 	pim->mroute_add_last = pim_time_monotonic_sec();
 	++pim->mroute_add_events;
 
-	/* Do not install route if incoming interface is undefined. */
-	if (c_oil->oil.mfcc_parent >= MAXVIFS) {
-		if (PIM_DEBUG_MROUTE) {
-			char buf[1000];
-			zlog_debug(
-				"%s(%s) %s Attempting to add vifi that is invalid to mroute table",
-				__PRETTY_FUNCTION__, name,
-				pim_channel_oil_dump(c_oil, buf, sizeof(buf)));
-		}
-		return -2;
-	}
+	/* Copy the oil to a temporary structure to fixup (without need to
+	 * later restore) before sending the mroute add to the dataplane
+	 */
+	pim_mroute_copy(&tmp_oil, c_oil);
 
 	/* The linux kernel *expects* the incoming
 	 * vif to be part of the outgoing list
 	 * in the case of a (*,G).
 	 */
 	if (c_oil->oil.mfcc_origin.s_addr == INADDR_ANY) {
-		orig = c_oil->oil.mfcc_ttls[c_oil->oil.mfcc_parent];
-		c_oil->oil.mfcc_ttls[c_oil->oil.mfcc_parent] = 1;
+		tmp_oil.mfcc_ttls[c_oil->oil.mfcc_parent] = 1;
 	}
 
 	/*
@@ -901,42 +994,178 @@ int pim_mroute_add(struct channel_oil *c_oil, const char *name)
 	 */
 	if (!c_oil->installed && c_oil->oil.mfcc_origin.s_addr != INADDR_ANY
 	    && c_oil->oil.mfcc_parent != 0) {
-		orig_iif_vif = c_oil->oil.mfcc_parent;
-		c_oil->oil.mfcc_parent = 0;
+		tmp_oil.mfcc_parent = 0;
 	}
 	err = setsockopt(pim->mroute_socket, IPPROTO_IP, MRT_ADD_MFC,
-			 &c_oil->oil, sizeof(c_oil->oil));
+			 &tmp_oil, sizeof(tmp_oil));
 
 	if (!err && !c_oil->installed
 	    && c_oil->oil.mfcc_origin.s_addr != INADDR_ANY
-	    && orig_iif_vif != 0) {
-		c_oil->oil.mfcc_parent = orig_iif_vif;
+	    && c_oil->oil.mfcc_parent != 0) {
+		tmp_oil.mfcc_parent = c_oil->oil.mfcc_parent;
 		err = setsockopt(pim->mroute_socket, IPPROTO_IP, MRT_ADD_MFC,
-				 &c_oil->oil, sizeof(c_oil->oil));
+				 &tmp_oil, sizeof(tmp_oil));
 	}
 
-	if (c_oil->oil.mfcc_origin.s_addr == INADDR_ANY)
-		c_oil->oil.mfcc_ttls[c_oil->oil.mfcc_parent] = orig;
-
 	if (err) {
 		zlog_warn(
 			"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP,MRT_ADD_MFC): errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__, pim->mroute_socket,
-			errno, safe_strerror(errno));
+			__FILE__, __func__, pim->mroute_socket, errno,
+			safe_strerror(errno));
 		return -2;
 	}
 
 	if (PIM_DEBUG_MROUTE) {
 		char buf[1000];
-		zlog_debug("%s(%s), vrf %s Added Route: %s",
-			   __PRETTY_FUNCTION__, name, pim->vrf->name,
+		zlog_debug("%s(%s), vrf %s Added Route: %s", __func__, name,
+			   pim->vrf->name,
 			   pim_channel_oil_dump(c_oil, buf, sizeof(buf)));
 	}
 
-	c_oil->installed = 1;
+	if (!c_oil->installed) {
+		c_oil->installed = 1;
+		c_oil->mroute_creation = pim_time_monotonic_sec();
+	}
+
 	return 0;
 }
 
+static int pim_upstream_get_mroute_iif(struct channel_oil *c_oil,
+		const char *name)
+{
+	vifi_t iif = MAXVIFS;
+	struct interface *ifp = NULL;
+	struct pim_interface *pim_ifp;
+	struct pim_upstream *up = c_oil->up;
+
+	if (up) {
+		if (PIM_UPSTREAM_FLAG_TEST_USE_RPT(up->flags)) {
+			if (up->parent)
+				ifp = up->parent->rpf.source_nexthop.interface;
+		} else {
+			ifp = up->rpf.source_nexthop.interface;
+		}
+		if (ifp) {
+			pim_ifp = (struct pim_interface *)ifp->info;
+			if (pim_ifp)
+				iif = pim_ifp->mroute_vif_index;
+		}
+	}
+	return iif;
+}
+
+static int pim_upstream_mroute_update(struct channel_oil *c_oil,
+		const char *name)
+{
+	char buf[1000];
+
+	if (c_oil->oil.mfcc_parent >= MAXVIFS) {
+		/* the c_oil cannot be installed as a mroute yet */
+		if (PIM_DEBUG_MROUTE)
+			zlog_debug(
+					"%s(%s) %s mroute not ready to be installed; %s",
+					__func__, name,
+					pim_channel_oil_dump(c_oil, buf,
+						sizeof(buf)),
+					c_oil->installed ?
+					"uninstall" : "skip");
+		/* if already installed flush it out as we are going to stop
+		 * updates to it leaving it in a stale state
+		 */
+		if (c_oil->installed)
+			pim_mroute_del(c_oil, name);
+		/* return success (skipped) */
+		return 0;
+	}
+
+	return pim_mroute_add(c_oil, name);
+}
+
+/* IIF associated with SGrpt entries are re-evaluated when the parent
+ * (*,G) entries IIF changes
+ */
+static void pim_upstream_all_sources_iif_update(struct pim_upstream *up)
+{
+	struct listnode *listnode;
+	struct pim_upstream *child;
+
+	for (ALL_LIST_ELEMENTS_RO(up->sources, listnode,
+				child)) {
+		if (PIM_UPSTREAM_FLAG_TEST_USE_RPT(child->flags))
+			pim_upstream_mroute_iif_update(child->channel_oil,
+					__func__);
+	}
+}
+
+/* In the case of "PIM state machine" added mroutes an upstream entry
+ * must be present to decide on the SPT-forwarding vs. RPT-forwarding.
+ */
+int pim_upstream_mroute_add(struct channel_oil *c_oil, const char *name)
+{
+	vifi_t iif;
+
+	iif = pim_upstream_get_mroute_iif(c_oil, name);
+
+	if (c_oil->oil.mfcc_parent != iif) {
+		c_oil->oil.mfcc_parent = iif;
+		if (c_oil->oil.mfcc_origin.s_addr == INADDR_ANY &&
+				c_oil->up)
+			pim_upstream_all_sources_iif_update(c_oil->up);
+	} else {
+		c_oil->oil.mfcc_parent = iif;
+	}
+
+	return pim_upstream_mroute_update(c_oil, name);
+}
+
+/* Look for IIF changes and update the dateplane entry only if the IIF
+ * has changed.
+ */
+int pim_upstream_mroute_iif_update(struct channel_oil *c_oil, const char *name)
+{
+	vifi_t iif;
+	char buf[1000];
+
+	iif = pim_upstream_get_mroute_iif(c_oil, name);
+	if (c_oil->oil.mfcc_parent == iif) {
+		/* no change */
+		return 0;
+	}
+	c_oil->oil.mfcc_parent = iif;
+
+	if (c_oil->oil.mfcc_origin.s_addr == INADDR_ANY &&
+			c_oil->up)
+		pim_upstream_all_sources_iif_update(c_oil->up);
+
+	if (PIM_DEBUG_MROUTE_DETAIL)
+		zlog_debug("%s(%s) %s mroute iif update %d",
+				__func__, name,
+				pim_channel_oil_dump(c_oil, buf,
+					sizeof(buf)), iif);
+	/* XXX: is this hack needed? */
+	c_oil->oil_inherited_rescan = 1;
+	return pim_upstream_mroute_update(c_oil, name);
+}
+
+int pim_static_mroute_add(struct channel_oil *c_oil, const char *name)
+{
+	return pim_mroute_add(c_oil, name);
+}
+
+void pim_static_mroute_iif_update(struct channel_oil *c_oil,
+				int input_vif_index,
+				const char *name)
+{
+	if (c_oil->oil.mfcc_parent == input_vif_index)
+		return;
+
+	c_oil->oil.mfcc_parent = input_vif_index;
+	if (input_vif_index == MAXVIFS)
+		pim_mroute_del(c_oil, name);
+	else
+		pim_static_mroute_add(c_oil, name);
+}
+
 int pim_mroute_del(struct channel_oil *c_oil, const char *name)
 {
 	struct pim_instance *pim = c_oil->pim;
@@ -950,8 +1179,7 @@ int pim_mroute_del(struct channel_oil *c_oil, const char *name)
 			char buf[1000];
 			zlog_debug(
 				"%s %s: vifi %d for route is %s not installed, do not need to send del req. ",
-				__FILE__, __PRETTY_FUNCTION__,
-				c_oil->oil.mfcc_parent,
+				__FILE__, __func__, c_oil->oil.mfcc_parent,
 				pim_channel_oil_dump(c_oil, buf, sizeof(buf)));
 		}
 		return -2;
@@ -963,16 +1191,15 @@ int pim_mroute_del(struct channel_oil *c_oil, const char *name)
 		if (PIM_DEBUG_MROUTE)
 			zlog_warn(
 				"%s %s: failure: setsockopt(fd=%d,IPPROTO_IP,MRT_DEL_MFC): errno=%d: %s",
-				__FILE__, __PRETTY_FUNCTION__,
-				pim->mroute_socket, errno,
+				__FILE__, __func__, pim->mroute_socket, errno,
 				safe_strerror(errno));
 		return -2;
 	}
 
 	if (PIM_DEBUG_MROUTE) {
 		char buf[1000];
-		zlog_debug("%s(%s), vrf %s Deleted Route: %s",
-			   __PRETTY_FUNCTION__, name, pim->vrf->name,
+		zlog_debug("%s(%s), vrf %s Deleted Route: %s", __func__, name,
+			   pim->vrf->name,
 			   pim_channel_oil_dump(c_oil, buf, sizeof(buf)));
 	}
 
@@ -1000,7 +1227,7 @@ void pim_mroute_update_counters(struct channel_oil *c_oil)
 			sg.grp = c_oil->oil.mfcc_mcastgrp;
 			if (PIM_DEBUG_MROUTE)
 				zlog_debug(
-					"Channel(%s) is not installed no need to collect data from kernel",
+					"Channel%s is not installed no need to collect data from kernel",
 					pim_str_sg_dump(&sg));
 		}
 		return;
@@ -1012,18 +1239,14 @@ void pim_mroute_update_counters(struct channel_oil *c_oil)
 
 	pim_zlookup_sg_statistics(c_oil);
 	if (ioctl(pim->mroute_socket, SIOCGETSGCNT, &sgreq)) {
-		if (PIM_DEBUG_MROUTE) {
-			struct prefix_sg sg;
+		struct prefix_sg sg;
 
-			sg.src = c_oil->oil.mfcc_origin;
-			sg.grp = c_oil->oil.mfcc_mcastgrp;
+		sg.src = c_oil->oil.mfcc_origin;
+		sg.grp = c_oil->oil.mfcc_mcastgrp;
 
-			zlog_warn(
-				"ioctl(SIOCGETSGCNT=%lu) failure for (S,G)=(%s): errno=%d: %s",
-				(unsigned long)SIOCGETSGCNT,
-				pim_str_sg_dump(&sg), errno,
-				safe_strerror(errno));
-		}
+		zlog_warn("ioctl(SIOCGETSGCNT=%lu) failure for (S,G)=%s: errno=%d: %s",
+			  (unsigned long)SIOCGETSGCNT, pim_str_sg_dump(&sg),
+			  errno, safe_strerror(errno));
 		return;
 	}
 
