@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -24,6 +23,7 @@
 #include "thread.h"
 #include "memory.h"
 #include "if.h"
+#include "network.h"
 
 #include "pimd.h"
 #include "pim_pim.h"
@@ -39,9 +39,10 @@
 #include "pim_assert.h"
 #include "pim_msg.h"
 #include "pim_register.h"
+#include "pim_errors.h"
+#include "pim_bsm.h"
 
 static int on_pim_hello_send(struct thread *t);
-static int pim_hello_send(struct interface *ifp, uint16_t holdtime);
 
 static const char *pim_pim_msgtype2str(enum pim_msg_type type)
 {
@@ -116,8 +117,9 @@ void pim_sock_delete(struct interface *ifp, const char *delete_message)
 		  delete_message);
 
 	if (!ifp->info) {
-		zlog_err("%s: %s: but PIM not enabled on interface %s (!)",
-			 __PRETTY_FUNCTION__, delete_message, ifp->name);
+		flog_err(EC_PIM_CONFIG,
+			 "%s: %s: but PIM not enabled on interface %s (!)",
+			 __func__, delete_message, ifp->name);
 		return;
 	}
 
@@ -135,6 +137,18 @@ void pim_sock_delete(struct interface *ifp, const char *delete_message)
 	sock_close(ifp);
 }
 
+/* For now check dst address for hello, assrt and join/prune is all pim rtr */
+static bool pim_pkt_dst_addr_ok(enum pim_msg_type type, in_addr_t addr)
+{
+	if ((type == PIM_MSG_TYPE_HELLO) || (type == PIM_MSG_TYPE_ASSERT)
+	    || (type == PIM_MSG_TYPE_JOIN_PRUNE)) {
+		if (addr != qpim_all_pim_routers_addr.s_addr)
+			return false;
+	}
+
+	return true;
+}
+
 int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 {
 	struct ip *ip_hdr;
@@ -147,6 +161,7 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 	uint16_t checksum;     /* computed checksum */
 	struct pim_neighbor *neigh;
 	struct pim_msg_header *header;
+	bool   no_fwd;
 
 	if (len < sizeof(*ip_hdr)) {
 		if (PIM_DEBUG_PIM_PACKETS)
@@ -184,8 +199,15 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 
 	/* for computing checksum */
 	header->checksum = 0;
+	no_fwd = header->Nbit;
 
 	if (header->type == PIM_MSG_TYPE_REGISTER) {
+		if (pim_msg_len < PIM_MSG_REGISTER_LEN) {
+			if (PIM_DEBUG_PIM_PACKETS)
+				zlog_debug("PIM Register Message size=%d shorther than min length %d",
+					   pim_msg_len, PIM_MSG_REGISTER_LEN);
+			return -1;
+		}
 		/* First 8 byte header checksum */
 		checksum = in_cksum(pim_msg, PIM_MSG_REGISTER_LEN);
 		if (checksum != pim_checksum) {
@@ -223,10 +245,25 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 			ifp->name, ip_hdr->ip_ttl, header->ver, pim_msg_len,
 			checksum);
 		if (PIM_DEBUG_PIM_PACKETDUMP_RECV) {
-			pim_pkt_dump(__PRETTY_FUNCTION__, pim_msg, pim_msg_len);
+			pim_pkt_dump(__func__, pim_msg, pim_msg_len);
 		}
 	}
 
+	if (!pim_pkt_dst_addr_ok(header->type, ip_hdr->ip_dst.s_addr)) {
+		char dst_str[INET_ADDRSTRLEN];
+		char src_str[INET_ADDRSTRLEN];
+
+		pim_inet4_dump("<dst?>", ip_hdr->ip_dst, dst_str,
+			       sizeof(dst_str));
+		pim_inet4_dump("<src?>", ip_hdr->ip_src, src_str,
+			       sizeof(src_str));
+		zlog_warn(
+			"%s: Ignoring Pkt. Unexpected IP destination %s for %s (Expected: all_pim_routers_addr) from %s",
+			__func__, dst_str, pim_pim_msgtype2str(header->type),
+			src_str);
+		return -1;
+	}
+
 	switch (header->type) {
 	case PIM_MSG_TYPE_HELLO:
 		return pim_hello_recv(ifp, ip_hdr->ip_src,
@@ -239,7 +276,7 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 					 pim_msg_len - PIM_MSG_HEADER_LEN);
 		break;
 	case PIM_MSG_TYPE_REG_STOP:
-		return pim_register_stop_recv(pim_msg + PIM_MSG_HEADER_LEN,
+		return pim_register_stop_recv(ifp, pim_msg + PIM_MSG_HEADER_LEN,
 					      pim_msg_len - PIM_MSG_HEADER_LEN);
 		break;
 	case PIM_MSG_TYPE_JOIN_PRUNE:
@@ -248,8 +285,8 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 			if (PIM_DEBUG_PIM_PACKETS)
 				zlog_debug(
 					"%s %s: non-hello PIM message type=%d from non-neighbor %s on %s",
-					__FILE__, __PRETTY_FUNCTION__,
-					header->type, src_str, ifp->name);
+					__FILE__, __func__, header->type,
+					src_str, ifp->name);
 			return -1;
 		}
 		pim_neighbor_timer_reset(neigh, neigh->holdtime);
@@ -263,8 +300,8 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 			if (PIM_DEBUG_PIM_PACKETS)
 				zlog_debug(
 					"%s %s: non-hello PIM message type=%d from non-neighbor %s on %s",
-					__FILE__, __PRETTY_FUNCTION__,
-					header->type, src_str, ifp->name);
+					__FILE__, __func__, header->type,
+					src_str, ifp->name);
 			return -1;
 		}
 		pim_neighbor_timer_reset(neigh, neigh->holdtime);
@@ -272,6 +309,11 @@ int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len)
 				       pim_msg + PIM_MSG_HEADER_LEN,
 				       pim_msg_len - PIM_MSG_HEADER_LEN);
 		break;
+	case PIM_MSG_TYPE_BOOTSTRAP:
+		return pim_bsm_process(ifp, ip_hdr, pim_msg, pim_msg_len,
+				       no_fwd);
+		break;
+
 	default:
 		if (PIM_DEBUG_PIM_PACKETS) {
 			zlog_debug(
@@ -287,7 +329,7 @@ static void pim_sock_read_on(struct interface *ifp);
 
 static int pim_sock_read(struct thread *t)
 {
-	struct interface *ifp;
+	struct interface *ifp, *orig_ifp;
 	struct pim_interface *pim_ifp;
 	int fd;
 	struct sockaddr_in from;
@@ -301,7 +343,7 @@ static int pim_sock_read(struct thread *t)
 	static long long count = 0;
 	int cont = 1;
 
-	ifp = THREAD_ARG(t);
+	orig_ifp = ifp = THREAD_ARG(t);
 	fd = THREAD_FD(t);
 
 	pim_ifp = ifp->info;
@@ -321,53 +363,38 @@ static int pim_sock_read(struct thread *t)
 			goto done;
 		}
 
-#ifdef PIM_CHECK_RECV_IFINDEX_SANITY
-		/* ifindex sanity check */
-		if (ifindex != (int)ifp->ifindex) {
-			char from_str[INET_ADDRSTRLEN];
-			char to_str[INET_ADDRSTRLEN];
-			struct interface *recv_ifp;
-
-			if (!inet_ntop(AF_INET, &from.sin_addr, from_str,
-				       sizeof(from_str)))
-				sprintf(from_str, "<from?>");
-			if (!inet_ntop(AF_INET, &to.sin_addr, to_str,
-				       sizeof(to_str)))
-				sprintf(to_str, "<to?>");
-
-			recv_ifp = if_lookup_by_index(ifindex, VRF_DEFAULT);
-			if (recv_ifp) {
-				zassert(ifindex == (int)recv_ifp->ifindex);
-			}
-
-#ifdef PIM_REPORT_RECV_IFINDEX_MISMATCH
-			zlog_warn(
-				"Interface mismatch: recv PIM pkt from %s to %s on fd=%d: recv_ifindex=%d (%s) sock_ifindex=%d (%s)",
-				from_str, to_str, fd, ifindex,
-				recv_ifp ? recv_ifp->name : "<if-notfound>",
-				ifp->ifindex, ifp->name);
-#endif
+		/*
+		 * What?  So with vrf's the incoming packet is received
+		 * on the vrf interface but recvfromto above returns
+		 * the right ifindex, so just use it.  We know
+		 * it's the right interface because we bind to it
+		 */
+		ifp = if_lookup_by_index(ifindex, pim_ifp->pim->vrf_id);
+		if (!ifp || !ifp->info) {
+			if (PIM_DEBUG_PIM_PACKETS)
+				zlog_debug(
+					"%s: Received incoming pim packet on interface(%s:%d) not yet configured for pim",
+					__func__, ifp ? ifp->name : "Unknown",
+					ifindex);
 			goto done;
 		}
-#endif
-
 		int fail = pim_pim_packet(ifp, buf, len);
 		if (fail) {
 			if (PIM_DEBUG_PIM_PACKETS)
 				zlog_debug("%s: pim_pim_packet() return=%d",
-					   __PRETTY_FUNCTION__, fail);
+					   __func__, fail);
 			goto done;
 		}
 
 		count++;
-		if (count % qpim_packet_process == 0)
+		if (count % router->packet_process == 0)
 			cont = 0;
 	}
 
 	result = 0; /* good */
 
 done:
-	pim_sock_read_on(ifp);
+	pim_sock_read_on(orig_ifp);
 
 	if (result) {
 		++pim_ifp->pim_ifstat_hello_recvfail;
@@ -390,8 +417,8 @@ static void pim_sock_read_on(struct interface *ifp)
 			   pim_ifp->pim_sock_fd);
 	}
 	pim_ifp->t_pim_sock_read = NULL;
-	THREAD_READ_ON(master, pim_ifp->t_pim_sock_read, pim_sock_read, ifp,
-		       pim_ifp->pim_sock_fd);
+	thread_add_read(router->master, pim_sock_read, ifp,
+			pim_ifp->pim_sock_fd, &pim_ifp->t_pim_sock_read);
 }
 
 static int pim_sock_open(struct interface *ifp)
@@ -513,18 +540,16 @@ static int pim_msg_send_frame(int fd, char *buf, size_t len,
 		}
 
 			return -1;
-			break;
 		default:
 			if (PIM_DEBUG_PIM_PACKETS) {
 				pim_inet4_dump("<dst?>", ip->ip_dst, dst_str,
 					       sizeof(dst_str));
 				zlog_warn(
 					"%s: sendto() failure to %s: fd=%d msg_size=%zd: errno=%d: %s",
-					__PRETTY_FUNCTION__, dst_str, fd, len,
-					errno, safe_strerror(errno));
+					__func__, dst_str, fd, len, errno,
+					safe_strerror(errno));
 			}
 			return -1;
-			break;
 		}
 	}
 
@@ -538,7 +563,7 @@ int pim_msg_send(int fd, struct in_addr src, struct in_addr dst,
 	socklen_t tolen;
 	unsigned char buffer[10000];
 	unsigned char *msg_start;
-	uint8_t ttl = MAXTTL;
+	uint8_t ttl;
 	struct pim_msg_header *header;
 	struct ip *ip;
 
@@ -580,6 +605,7 @@ int pim_msg_send(int fd, struct in_addr src, struct in_addr dst,
 	ip->ip_id = htons(++ip_id);
 	ip->ip_hl = 5;
 	ip->ip_v = 4;
+	ip->ip_tos = IPTOS_PREC_INTERNETCONTROL;
 	ip->ip_p = PIM_IP_PROTO_PIM;
 	ip->ip_src = src;
 	ip->ip_dst = dst;
@@ -587,13 +613,10 @@ int pim_msg_send(int fd, struct in_addr src, struct in_addr dst,
 	ip->ip_len = htons(sendlen);
 
 	if (PIM_DEBUG_PIM_PACKETS) {
-		struct pim_msg_header *header =
-			(struct pim_msg_header *)pim_msg;
 		char dst_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<dst?>", dst, dst_str, sizeof(dst_str));
-		zlog_debug("%s: to %s on %s: msg_size=%d checksum=%x",
-			   __PRETTY_FUNCTION__, dst_str, ifname, pim_msg_size,
-			   header->checksum);
+		zlog_debug("%s: to %s on %s: msg_size=%d checksum=%x", __func__,
+			   dst_str, ifname, pim_msg_size, header->checksum);
 	}
 
 	memset(&to, 0, sizeof(to));
@@ -602,7 +625,7 @@ int pim_msg_send(int fd, struct in_addr src, struct in_addr dst,
 	tolen = sizeof(to);
 
 	if (PIM_DEBUG_PIM_PACKETDUMP_SEND) {
-		pim_pkt_dump(__PRETTY_FUNCTION__, pim_msg, pim_msg_size);
+		pim_pkt_dump(__func__, pim_msg, pim_msg_size);
 	}
 
 	pim_msg_send_frame(fd, (char *)buffer, sendlen, (struct sockaddr *)&to,
@@ -625,7 +648,7 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 			       sizeof(dst_str));
 		zlog_debug(
 			"%s: to %s on %s: holdt=%u prop_d=%u overr_i=%u dis_join_supp=%d dr_prio=%u gen_id=%08x addrs=%d",
-			__PRETTY_FUNCTION__, dst_str, ifp->name, holdtime,
+			__func__, dst_str, ifp->name, holdtime,
 			pim_ifp->pim_propagation_delay_msec,
 			pim_ifp->pim_override_interval_msec,
 			PIM_IF_TEST_PIM_CAN_DISABLE_JOIN_SUPRESSION(
@@ -635,13 +658,12 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 	}
 
 	pim_tlv_size = pim_hello_build_tlv(
-		ifp->name, pim_msg + PIM_PIM_MIN_LEN,
+		ifp, pim_msg + PIM_PIM_MIN_LEN,
 		sizeof(pim_msg) - PIM_PIM_MIN_LEN, holdtime,
 		pim_ifp->pim_dr_priority, pim_ifp->pim_generation_id,
 		pim_ifp->pim_propagation_delay_msec,
 		pim_ifp->pim_override_interval_msec,
-		PIM_IF_TEST_PIM_CAN_DISABLE_JOIN_SUPRESSION(pim_ifp->options),
-		ifp->connected);
+		PIM_IF_TEST_PIM_CAN_DISABLE_JOIN_SUPRESSION(pim_ifp->options));
 	if (pim_tlv_size < 0) {
 		return -1;
 	}
@@ -651,7 +673,7 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 	zassert(pim_msg_size >= PIM_PIM_MIN_LEN);
 	zassert(pim_msg_size <= PIM_PIM_BUFSIZE_WRITE);
 
-	pim_msg_build_header(pim_msg, pim_msg_size, PIM_MSG_TYPE_HELLO);
+	pim_msg_build_header(pim_msg, pim_msg_size, PIM_MSG_TYPE_HELLO, false);
 
 	if (pim_msg_send(pim_ifp->pim_sock_fd, pim_ifp->primary_address,
 			 qpim_all_pim_routers_addr, pim_msg, pim_msg_size,
@@ -659,7 +681,7 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 		if (PIM_DEBUG_PIM_HELLO) {
 			zlog_debug(
 				"%s: could not send PIM message on interface %s",
-				__PRETTY_FUNCTION__, ifp->name);
+				__func__, ifp->name);
 		}
 		return -2;
 	}
@@ -667,15 +689,11 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 	return 0;
 }
 
-static int pim_hello_send(struct interface *ifp, uint16_t holdtime)
+int pim_hello_send(struct interface *ifp, uint16_t holdtime)
 {
-	struct pim_interface *pim_ifp;
+	struct pim_interface *pim_ifp = ifp->info;
 
-	zassert(ifp);
-	pim_ifp = ifp->info;
-	zassert(pim_ifp);
-
-	if (if_is_loopback(ifp))
+	if (if_is_loopback_or_vrf(ifp))
 		return 0;
 
 	if (hello_send(ifp, holdtime)) {
@@ -697,17 +715,16 @@ static void hello_resched(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
 
-	zassert(ifp);
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
 
 	if (PIM_DEBUG_PIM_HELLO) {
 		zlog_debug("Rescheduling %d sec hello on interface %s",
 			   pim_ifp->pim_hello_period, ifp->name);
 	}
 	THREAD_OFF(pim_ifp->t_pim_hello_timer);
-	THREAD_TIMER_ON(master, pim_ifp->t_pim_hello_timer, on_pim_hello_send,
-			ifp, pim_ifp->pim_hello_period);
+	thread_add_timer(router->master, on_pim_hello_send, ifp,
+			 pim_ifp->pim_hello_period,
+			 &pim_ifp->t_pim_hello_timer);
 }
 
 /*
@@ -719,13 +736,11 @@ static int on_pim_hello_send(struct thread *t)
 	struct interface *ifp;
 
 	ifp = THREAD_ARG(t);
-
 	pim_ifp = ifp->info;
 
 	/*
 	 * Schedule next hello
 	 */
-	pim_ifp->t_pim_hello_timer = NULL;
 	hello_resched(ifp);
 
 	/*
@@ -747,9 +762,7 @@ void pim_hello_restart_now(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
 
-	zassert(ifp);
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
 
 	/*
 	 * Reset next hello timer
@@ -777,9 +790,13 @@ void pim_hello_restart_triggered(struct interface *ifp)
 	int triggered_hello_delay_msec;
 	int random_msec;
 
-	zassert(ifp);
 	pim_ifp = ifp->info;
-	zassert(pim_ifp);
+
+	/*
+	 * No need to ever start loopback or vrf device hello's
+	 */
+	if (if_is_loopback_or_vrf(ifp))
+		return;
 
 	/*
 	 * There exists situations where we have the a RPF out this
@@ -808,7 +825,6 @@ void pim_hello_restart_triggered(struct interface *ifp)
 		}
 
 		THREAD_OFF(pim_ifp->t_pim_hello_timer);
-		pim_ifp->t_pim_hello_timer = NULL;
 	}
 
 	random_msec = triggered_hello_delay_msec;
@@ -819,8 +835,8 @@ void pim_hello_restart_triggered(struct interface *ifp)
 			   random_msec, ifp->name);
 	}
 
-	THREAD_TIMER_MSEC_ON(master, pim_ifp->t_pim_hello_timer,
-			     on_pim_hello_send, ifp, random_msec);
+	thread_add_timer_msec(router->master, on_pim_hello_send, ifp,
+			      random_msec, &pim_ifp->t_pim_hello_timer);
 }
 
 int pim_sock_add(struct interface *ifp)
@@ -863,7 +879,7 @@ int pim_sock_add(struct interface *ifp)
 	old_genid = pim_ifp->pim_generation_id;
 
 	while (old_genid == pim_ifp->pim_generation_id)
-		pim_ifp->pim_generation_id = random();
+		pim_ifp->pim_generation_id = frr_weak_random();
 
 	zlog_info("PIM INTERFACE UP: on interface %s ifindex=%d", ifp->name,
 		  ifp->ifindex);
