@@ -36,20 +36,21 @@
 #include "pim_time.h"
 #include "pim_nht.h"
 #include "pim_oil.h"
+#include "pim_mlag.h"
 
 static struct in_addr pim_rpf_find_rpf_addr(struct pim_upstream *up);
 
 void pim_rpf_set_refresh_time(struct pim_instance *pim)
 {
 	pim->last_route_change_time = pim_time_monotonic_usec();
-	if (PIM_DEBUG_TRACE)
+	if (PIM_DEBUG_PIM_TRACE)
 		zlog_debug("%s: vrf(%s) New last route change time: %" PRId64,
-			   __PRETTY_FUNCTION__, pim->vrf->name,
+			   __func__, pim->vrf->name,
 			   pim->last_route_change_time);
 }
 
-int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
-		       struct in_addr addr, int neighbor_needed)
+bool pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
+			struct in_addr addr, int neighbor_needed)
 {
 	struct pim_zlookup_nexthop nexthop_tab[MULTIPATH_NUM];
 	struct pim_neighbor *nbr = NULL;
@@ -65,11 +66,11 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 	 * it will never work
 	 */
 	if (addr.s_addr == INADDR_NONE)
-		return -1;
+		return false;
 
 	if ((nexthop->last_lookup.s_addr == addr.s_addr)
 	    && (nexthop->last_lookup_time > pim->last_route_change_time)) {
-		if (PIM_DEBUG_TRACE) {
+		if (PIM_DEBUG_PIM_NHT) {
 			char addr_str[INET_ADDRSTRLEN];
 			pim_inet4_dump("<addr?>", addr, addr_str,
 				       sizeof(addr_str));
@@ -77,22 +78,21 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 			pim_addr_dump("<nexthop?>", &nexthop->mrib_nexthop_addr,
 				      nexthop_str, sizeof(nexthop_str));
 			zlog_debug(
-				"%s: Using last lookup for %s at %lld, %" PRId64 " addr%s",
-				__PRETTY_FUNCTION__, addr_str,
-				nexthop->last_lookup_time,
+				"%s: Using last lookup for %s at %lld, %" PRId64
+				" addr %s",
+				__func__, addr_str, nexthop->last_lookup_time,
 				pim->last_route_change_time, nexthop_str);
 		}
 		pim->nexthop_lookups_avoided++;
-		return 0;
+		return true;
 	} else {
-		if (PIM_DEBUG_TRACE) {
+		if (PIM_DEBUG_PIM_NHT) {
 			char addr_str[INET_ADDRSTRLEN];
 			pim_inet4_dump("<addr?>", addr, addr_str,
 				       sizeof(addr_str));
 			zlog_debug(
 				"%s: Looking up: %s, last lookup time: %lld, %" PRId64,
-				__PRETTY_FUNCTION__, addr_str,
-				nexthop->last_lookup_time,
+				__func__, addr_str, nexthop->last_lookup_time,
 				pim->last_route_change_time);
 		}
 	}
@@ -106,8 +106,8 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 		pim_inet4_dump("<addr?>", addr, addr_str, sizeof(addr_str));
 		zlog_warn(
 			"%s %s: could not find nexthop ifindex for address %s",
-			__FILE__, __PRETTY_FUNCTION__, addr_str);
-		return -1;
+			__FILE__, __func__, addr_str);
+		return false;
 	}
 
 	while (!found && (i < num_ifindex)) {
@@ -121,8 +121,8 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 					       sizeof(addr_str));
 				zlog_debug(
 					"%s %s: could not find interface for ifindex %d (address %s)",
-					__FILE__, __PRETTY_FUNCTION__,
-					first_ifindex, addr_str);
+					__FILE__, __func__, first_ifindex,
+					addr_str);
 			}
 			i++;
 			continue;
@@ -135,8 +135,8 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 					       sizeof(addr_str));
 				zlog_debug(
 					"%s: multicast not enabled on input interface %s (ifindex=%d, RPF for source %s)",
-					__PRETTY_FUNCTION__, ifp->name,
-					first_ifindex, addr_str);
+					__func__, ifp->name, first_ifindex,
+					addr_str);
 			}
 			i++;
 		} else if (neighbor_needed
@@ -165,12 +165,12 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 				       sizeof(addr_str));
 			zlog_debug(
 				"%s %s: found nexthop %s for address %s: interface %s ifindex=%d metric=%d pref=%d",
-				__FILE__, __PRETTY_FUNCTION__, nexthop_str,
-				addr_str, ifp->name, first_ifindex,
+				__FILE__, __func__, nexthop_str, addr_str,
+				ifp->name, first_ifindex,
 				nexthop_tab[i].route_metric,
 				nexthop_tab[i].protocol_distance);
 		}
-		/* update nextop data */
+		/* update nexthop data */
 		nexthop->interface = ifp;
 		nexthop->mrib_nexthop_addr = nexthop_tab[i].nexthop_addr;
 		nexthop->mrib_metric_preference =
@@ -179,9 +179,9 @@ int pim_nexthop_lookup(struct pim_instance *pim, struct pim_nexthop *nexthop,
 		nexthop->last_lookup = addr;
 		nexthop->last_lookup_time = pim_time_monotonic_usec();
 		nexthop->nbr = nbr;
-		return 0;
+		return true;
 	} else
-		return -1;
+		return false;
 }
 
 static int nexthop_mismatch(const struct pim_nexthop *nh1,
@@ -194,27 +194,60 @@ static int nexthop_mismatch(const struct pim_nexthop *nh1,
 	       || (nh1->mrib_route_metric != nh2->mrib_route_metric);
 }
 
+static void pim_rpf_cost_change(struct pim_instance *pim,
+		struct pim_upstream *up, uint32_t old_cost)
+{
+	struct pim_rpf *rpf = &up->rpf;
+	uint32_t new_cost;
+
+	new_cost = pim_up_mlag_local_cost(up);
+	if (PIM_DEBUG_MLAG)
+		zlog_debug(
+			"%s: Cost_to_rp of upstream-%s changed to:%u, from:%u",
+			__func__, up->sg_str, new_cost, old_cost);
+
+	if (old_cost == new_cost)
+		return;
+
+	/* Cost changed, it might Impact MLAG DF election, update */
+	if (PIM_DEBUG_MLAG)
+		zlog_debug(
+			"%s: Cost_to_rp of upstream-%s changed to:%u",
+			__func__, up->sg_str,
+			rpf->source_nexthop.mrib_route_metric);
+
+	if (pim_up_mlag_is_local(up))
+		pim_mlag_up_local_add(pim, up);
+}
+
 enum pim_rpf_result pim_rpf_update(struct pim_instance *pim,
-				   struct pim_upstream *up, struct pim_rpf *old,
-				   uint8_t is_new)
+		struct pim_upstream *up, struct pim_rpf *old,
+		const char *caller)
 {
 	struct pim_rpf *rpf = &up->rpf;
 	struct pim_rpf saved;
 	struct prefix nht_p;
-	struct pim_nexthop_cache pnc;
 	struct prefix src, grp;
+	bool neigh_needed = true;
+	uint32_t saved_mrib_route_metric;
+
+	if (PIM_UPSTREAM_FLAG_TEST_STATIC_IIF(up->flags))
+		return PIM_RPF_OK;
+
+	if (up->upstream_addr.s_addr == INADDR_ANY) {
+		zlog_debug("%s(%s): RP is not configured yet for %s",
+			__func__, caller, up->sg_str);
+		return PIM_RPF_OK;
+	}
 
 	saved.source_nexthop = rpf->source_nexthop;
 	saved.rpf_addr = rpf->rpf_addr;
-
-	if (is_new && PIM_DEBUG_ZEBRA) {
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<source?>", up->upstream_addr, source_str,
-			       sizeof(source_str));
-		zlog_debug("%s: NHT Register upstream %s addr %s with Zebra.",
-			   __PRETTY_FUNCTION__, up->sg_str, source_str);
+	saved_mrib_route_metric = pim_up_mlag_local_cost(up);
+	if (old) {
+		old->source_nexthop = saved.source_nexthop;
+		old->rpf_addr = saved.rpf_addr;
 	}
-	/* Register addr with Zebra NHT */
+
 	nht_p.family = AF_INET;
 	nht_p.prefixlen = IPV4_MAX_BITLEN;
 	nht_p.u.prefix4.s_addr = up->upstream_addr.s_addr;
@@ -225,33 +258,25 @@ enum pim_rpf_result pim_rpf_update(struct pim_instance *pim,
 	grp.family = AF_INET;
 	grp.prefixlen = IPV4_MAX_BITLEN;
 	grp.u.prefix4 = up->sg.grp;
-	memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-	if (pim_find_or_track_nexthop(pim, &nht_p, up, NULL, &pnc)) {
-		if (pnc.nexthop_num) {
-			if (!pim_ecmp_nexthop_search(
-				    pim, &pnc, &up->rpf.source_nexthop, &src,
-				    &grp,
-				    !PIM_UPSTREAM_FLAG_TEST_FHR(up->flags)
-					    && !PIM_UPSTREAM_FLAG_TEST_SRC_IGMP(
-						       up->flags)))
-				return PIM_RPF_FAILURE;
-		}
-	} else {
-		if (!pim_ecmp_nexthop_lookup(
-			    pim, &rpf->source_nexthop, up->upstream_addr, &src,
-			    &grp,
-			    !PIM_UPSTREAM_FLAG_TEST_FHR(up->flags)
-				    && !PIM_UPSTREAM_FLAG_TEST_SRC_IGMP(
-					       up->flags)))
-			return PIM_RPF_FAILURE;
+
+	if ((up->sg.src.s_addr == INADDR_ANY && I_am_RP(pim, up->sg.grp)) ||
+	    PIM_UPSTREAM_FLAG_TEST_FHR(up->flags))
+		neigh_needed = false;
+	pim_find_or_track_nexthop(pim, &nht_p, up, NULL, false, NULL);
+	if (!pim_ecmp_nexthop_lookup(pim, &rpf->source_nexthop, &src, &grp,
+				neigh_needed)) {
+		/* Route is Deleted in Zebra, reset the stored NH data */
+		pim_upstream_rpf_clear(pim, up);
+		pim_rpf_cost_change(pim, up, saved_mrib_route_metric);
+		return PIM_RPF_FAILURE;
 	}
 
 	rpf->rpf_addr.family = AF_INET;
 	rpf->rpf_addr.u.prefix4 = pim_rpf_find_rpf_addr(up);
 	if (pim_rpf_addr_is_inaddr_any(rpf) && PIM_DEBUG_ZEBRA) {
 		/* RPF'(S,G) not found */
-		zlog_debug("%s %s: RPF'%s not found: won't send join upstream",
-			   __FILE__, __PRETTY_FUNCTION__, up->sg_str);
+		zlog_debug("%s(%s): RPF'%s not found: won't send join upstream",
+			   __func__, caller, up->sg_str);
 		/* warning only */
 	}
 
@@ -263,8 +288,8 @@ enum pim_rpf_result pim_rpf_update(struct pim_instance *pim,
 			pim_addr_dump("<addr?>",
 				      &rpf->source_nexthop.mrib_nexthop_addr,
 				      nhaddr_str, sizeof(nhaddr_str));
-			zlog_debug("%s %s: (S,G)=%s source nexthop now is: interface=%s address=%s pref=%d metric=%d",
-		 __FILE__, __PRETTY_FUNCTION__,
+			zlog_debug("%s(%s): (S,G)=%s source nexthop now is: interface=%s address=%s pref=%d metric=%d",
+		 __func__, caller,
 		 up->sg_str,
 		 rpf->source_nexthop.interface ? rpf->source_nexthop.interface->name : "<ifname?>",
 		 nhaddr_str,
@@ -281,8 +306,8 @@ enum pim_rpf_result pim_rpf_update(struct pim_instance *pim,
 	if (saved.source_nexthop.interface != rpf->source_nexthop.interface) {
 
 		if (PIM_DEBUG_ZEBRA) {
-			zlog_debug("%s %s: (S,G)=%s RPF_interface(S) changed from %s to %s",
-		 __FILE__, __PRETTY_FUNCTION__,
+			zlog_debug("%s(%s): (S,G)=%s RPF_interface(S) changed from %s to %s",
+		 __func__, caller,
 		 up->sg_str,
 		 saved.source_nexthop.interface ? saved.source_nexthop.interface->name : "<oldif?>",
 		 rpf->source_nexthop.interface ? rpf->source_nexthop.interface->name : "<newif?>");
@@ -297,18 +322,43 @@ enum pim_rpf_result pim_rpf_update(struct pim_instance *pim,
 	if (saved.rpf_addr.u.prefix4.s_addr != rpf->rpf_addr.u.prefix4.s_addr
 	    || saved.source_nexthop
 			       .interface != rpf->source_nexthop.interface) {
-
-		/* return old rpf to caller ? */
-		if (old) {
-			old->source_nexthop = saved.source_nexthop;
-			old->rpf_addr = saved.rpf_addr;
-		}
+		pim_rpf_cost_change(pim, up, saved_mrib_route_metric);
 		return PIM_RPF_CHANGED;
 	}
 
+	if (PIM_DEBUG_MLAG)
+		zlog_debug(
+			"%s(%s): Cost_to_rp of upstream-%s changed to:%u",
+			__func__, caller, up->sg_str,
+			rpf->source_nexthop.mrib_route_metric);
+
+	pim_rpf_cost_change(pim, up, saved_mrib_route_metric);
+
 	return PIM_RPF_OK;
 }
 
+/*
+ * In the case of RP deletion and RP unreachablity,
+ * uninstall the mroute in the kernel and clear the
+ * rpf information in the pim upstream and pim channel
+ * oil data structure.
+ */
+void pim_upstream_rpf_clear(struct pim_instance *pim,
+			    struct pim_upstream *up)
+{
+	if (up->rpf.source_nexthop.interface) {
+		up->rpf.source_nexthop.interface = NULL;
+		up->rpf.source_nexthop.mrib_nexthop_addr.u.prefix4.s_addr =
+			PIM_NET_INADDR_ANY;
+		up->rpf.source_nexthop.mrib_metric_preference =
+			router->infinite_assert_metric.metric_preference;
+		up->rpf.source_nexthop.mrib_route_metric =
+			router->infinite_assert_metric.route_metric;
+		up->rpf.rpf_addr.u.prefix4.s_addr = PIM_NET_INADDR_ANY;
+		pim_upstream_mroute_iif_update(up->channel_oil, __func__);
+	}
+}
+
 /*
   RFC 4601: 4.1.6.  State Summarization Macros
 
@@ -332,7 +382,7 @@ static struct in_addr pim_rpf_find_rpf_addr(struct pim_upstream *up)
 
 	if (!up->rpf.source_nexthop.interface) {
 		zlog_warn("%s: missing RPF interface for upstream (S,G)=%s",
-			  __PRETTY_FUNCTION__, up->sg_str);
+			  __func__, up->sg_str);
 
 		rpf_addr.s_addr = PIM_NET_INADDR_ANY;
 		return rpf_addr;
@@ -363,14 +413,11 @@ int pim_rpf_addr_is_inaddr_none(struct pim_rpf *rpf)
 	switch (rpf->rpf_addr.family) {
 	case AF_INET:
 		return rpf->rpf_addr.u.prefix4.s_addr == INADDR_NONE;
-		break;
 	case AF_INET6:
-		zlog_warn("%s: v6 Unimplmeneted", __PRETTY_FUNCTION__);
+		zlog_warn("%s: v6 Unimplmeneted", __func__);
 		return 1;
-		break;
 	default:
 		return 0;
-		break;
 	}
 
 	return 0;
@@ -381,14 +428,11 @@ int pim_rpf_addr_is_inaddr_any(struct pim_rpf *rpf)
 	switch (rpf->rpf_addr.family) {
 	case AF_INET:
 		return rpf->rpf_addr.u.prefix4.s_addr == INADDR_ANY;
-		break;
 	case AF_INET6:
-		zlog_warn("%s: v6 Unimplmented", __PRETTY_FUNCTION__);
+		zlog_warn("%s: v6 Unimplmented", __func__);
 		return 1;
-		break;
 	default:
 		return 0;
-		break;
 	}
 
 	return 0;
@@ -402,14 +446,14 @@ int pim_rpf_is_same(struct pim_rpf *rpf1, struct pim_rpf *rpf2)
 	return 0;
 }
 
-unsigned int pim_rpf_hash_key(void *arg)
+unsigned int pim_rpf_hash_key(const void *arg)
 {
-	struct pim_nexthop_cache *r = (struct pim_nexthop_cache *)arg;
+	const struct pim_nexthop_cache *r = arg;
 
 	return jhash_1word(r->rpf.rpf_addr.u.prefix4.s_addr, 0);
 }
 
-int pim_rpf_equal(const void *arg1, const void *arg2)
+bool pim_rpf_equal(const void *arg1, const void *arg2)
 {
 	const struct pim_nexthop_cache *r1 =
 		(const struct pim_nexthop_cache *)arg1;
