@@ -28,6 +28,7 @@
 #include "hash.h"
 #include "jhash.h"
 #include "vrf.h"
+#include "lib_errors.h"
 
 #include "pimd.h"
 #include "pim_cmd.h"
@@ -38,6 +39,7 @@
 #include "pim_static.h"
 #include "pim_rp.h"
 #include "pim_ssm.h"
+#include "pim_vxlan.h"
 #include "pim_zlookup.h"
 #include "pim_zebra.h"
 
@@ -46,19 +48,10 @@ const char *const PIM_ALL_ROUTERS = MCAST_ALL_ROUTERS;
 const char *const PIM_ALL_PIM_ROUTERS = MCAST_ALL_PIM_ROUTERS;
 const char *const PIM_ALL_IGMP_ROUTERS = MCAST_ALL_IGMP_ROUTERS;
 
-struct thread_master *master = NULL;
-uint32_t qpim_debugs = 0;
-int qpim_t_periodic =
-	PIM_DEFAULT_T_PERIODIC; /* Period between Join/Prune Messages */
-struct pim_assert_metric qpim_infinite_assert_metric;
-long qpim_rpf_cache_refresh_delay_msec = 50;
-int qpim_packet_process = PIM_DEFAULT_PACKET_PROCESS;
-uint8_t qpim_ecmp_enable = 0;
-uint8_t qpim_ecmp_rebalance_enable = 0;
-struct pim_instance *pimg = NULL;
+DEFINE_MTYPE_STATIC(PIMD, ROUTER, "PIM Router information");
 
-int32_t qpim_register_suppress_time = PIM_REGISTER_SUPPRESSION_TIME_DEFAULT;
-int32_t qpim_register_probe_time = PIM_REGISTER_PROBE_TIME_DEFAULT;
+struct pim_router *router = NULL;
+struct in_addr qpim_all_pim_routers_addr;
 
 void pim_prefix_list_update(struct prefix_list *plist)
 {
@@ -76,25 +69,20 @@ void pim_prefix_list_update(struct prefix_list *plist)
 	}
 }
 
-static void pim_free()
+static void pim_free(void)
 {
 	pim_route_map_terminate();
 
 	zclient_lookup_free();
-
-	zprivs_terminate(&pimd_privs);
 }
 
-void pim_init()
+void pim_router_init(void)
 {
-	if (!inet_aton(PIM_ALL_PIM_ROUTERS, &qpim_all_pim_routers_addr)) {
-		zlog_err(
-			"%s %s: could not solve %s to group address: errno=%d: %s",
-			__FILE__, __PRETTY_FUNCTION__, PIM_ALL_PIM_ROUTERS,
-			errno, safe_strerror(errno));
-		zassert(0);
-		return;
-	}
+	router = XCALLOC(MTYPE_ROUTER, sizeof(*router));
+
+	router->debugs = 0;
+	router->master = frr_init();
+	router->t_periodic = PIM_DEFAULT_T_PERIODIC;
 
 	/*
 	  RFC 4601: 4.6.3.  Assert Metrics
@@ -104,26 +92,51 @@ void pim_init()
 	  return {1,infinity,infinity,0}
 	  }
 	*/
-	qpim_infinite_assert_metric.rpt_bit_flag = 1;
-	qpim_infinite_assert_metric.metric_preference =
+	router->infinite_assert_metric.rpt_bit_flag = 1;
+	router->infinite_assert_metric.metric_preference =
 		PIM_ASSERT_METRIC_PREFERENCE_MAX;
-	qpim_infinite_assert_metric.route_metric = PIM_ASSERT_ROUTE_METRIC_MAX;
-	qpim_infinite_assert_metric.ip_address.s_addr = INADDR_ANY;
+	router->infinite_assert_metric.route_metric =
+		PIM_ASSERT_ROUTE_METRIC_MAX;
+	router->infinite_assert_metric.ip_address.s_addr = INADDR_ANY;
+	router->rpf_cache_refresh_delay_msec = 50;
+	router->register_suppress_time = PIM_REGISTER_SUPPRESSION_TIME_DEFAULT;
+	router->packet_process = PIM_DEFAULT_PACKET_PROCESS;
+	router->register_probe_time = PIM_REGISTER_PROBE_TIME_DEFAULT;
+	router->vrf_id = VRF_DEFAULT;
+	router->pim_mlag_intf_cnt = 0;
+	router->connected_to_mlag = false;
+}
+
+void pim_router_terminate(void)
+{
+	XFREE(MTYPE_ROUTER, router);
+}
+
+void pim_init(void)
+{
+	if (!inet_aton(PIM_ALL_PIM_ROUTERS, &qpim_all_pim_routers_addr)) {
+		flog_err(
+			EC_LIB_SOCKET,
+			"%s %s: could not solve %s to group address: errno=%d: %s",
+			__FILE__, __func__, PIM_ALL_PIM_ROUTERS, errno,
+			safe_strerror(errno));
+		zassert(0);
+		return;
+	}
 
 	pim_cmd_init();
 }
 
-void pim_terminate()
+void pim_terminate(void)
 {
 	struct zclient *zclient;
 
-	pim_free();
-
 	/* reverse prefix_list_init */
 	prefix_list_add_hook(NULL);
 	prefix_list_delete_hook(NULL);
 	prefix_list_reset();
 
+	pim_vxlan_terminate();
 	pim_vrf_terminate();
 
 	zclient = pim_zebra_zclient_get();
@@ -132,5 +145,8 @@ void pim_terminate()
 		zclient_free(zclient);
 	}
 
+	pim_free();
+	pim_router_terminate();
+
 	frr_fini();
 }
