@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -32,33 +31,35 @@
 #include "stream.h"
 #include "zclient.h"
 #include "command.h"
-#include "table.h"
+#include "agg_table.h"
 #include "thread.h"
 #include "privs.h"
 #include "vrf.h"
+#include "lib_errors.h"
+#include "northbound_cli.h"
 
 #include "ripngd/ripngd.h"
 #include "ripngd/ripng_debug.h"
 
 /* If RFC2133 definition is used. */
 #ifndef IPV6_JOIN_GROUP
-#define IPV6_JOIN_GROUP  IPV6_ADD_MEMBERSHIP 
+#define IPV6_JOIN_GROUP  IPV6_ADD_MEMBERSHIP
 #endif
 #ifndef IPV6_LEAVE_GROUP
-#define IPV6_LEAVE_GROUP IPV6_DROP_MEMBERSHIP 
+#define IPV6_LEAVE_GROUP IPV6_DROP_MEMBERSHIP
 #endif
 
-extern struct zebra_privs_t ripngd_privs;
+DEFINE_MTYPE_STATIC(RIPNGD, RIPNG_IF, "ripng interface")
 
 /* Static utility function. */
 static void ripng_enable_apply(struct interface *);
 static void ripng_passive_interface_apply(struct interface *);
-static int ripng_enable_if_lookup(const char *);
+static int ripng_enable_if_lookup(struct ripng *ripng, const char *ifname);
 static int ripng_enable_network_lookup2(struct connected *);
-static void ripng_enable_apply_all(void);
+static void ripng_enable_apply_all(struct ripng *ripng);
 
 /* Join to the all rip routers multicast group. */
-static int ripng_multicast_join(struct interface *ifp)
+static int ripng_multicast_join(struct interface *ifp, int sock)
 {
 	int ret;
 	struct ipv6_mreq mreq;
@@ -74,15 +75,13 @@ static int ripng_multicast_join(struct interface *ifp)
 		 * While this is bogus, privs are available and easy to use
 		 * for this call as a workaround.
 		 */
-		if (ripngd_privs.change(ZPRIVS_RAISE))
-			zlog_err("ripng_multicast_join: could not raise privs");
+		frr_with_privs(&ripngd_privs) {
 
-		ret = setsockopt(ripng->sock, IPPROTO_IPV6, IPV6_JOIN_GROUP,
-				 (char *)&mreq, sizeof(mreq));
-		save_errno = errno;
+			ret = setsockopt(sock, IPPROTO_IPV6, IPV6_JOIN_GROUP,
+					 (char *)&mreq, sizeof(mreq));
+			save_errno = errno;
 
-		if (ripngd_privs.change(ZPRIVS_LOWER))
-			zlog_err("ripng_multicast_join: could not lower privs");
+		}
 
 		if (ret < 0 && save_errno == EADDRINUSE) {
 			/*
@@ -92,7 +91,7 @@ static int ripng_multicast_join(struct interface *ifp)
 			 * group on
 			 * an interface that has just gone down.
 			 */
-			zlog_warn("ripng join on %s EADDRINUSE (ignoring)\n",
+			zlog_warn("ripng join on %s EADDRINUSE (ignoring)",
 				  ifp->name);
 			return 0; /* not an error */
 		}
@@ -113,7 +112,7 @@ static int ripng_multicast_join(struct interface *ifp)
 }
 
 /* Leave from the all rip routers multicast group. */
-static int ripng_multicast_leave(struct interface *ifp)
+static int ripng_multicast_leave(struct interface *ifp, int sock)
 {
 	int ret;
 	struct ipv6_mreq mreq;
@@ -123,10 +122,10 @@ static int ripng_multicast_leave(struct interface *ifp)
 		inet_pton(AF_INET6, RIPNG_GROUP, &mreq.ipv6mr_multiaddr);
 		mreq.ipv6mr_interface = ifp->ifindex;
 
-		ret = setsockopt(ripng->sock, IPPROTO_IPV6, IPV6_LEAVE_GROUP,
+		ret = setsockopt(sock, IPPROTO_IPV6, IPV6_LEAVE_GROUP,
 				 (char *)&mreq, sizeof(mreq));
 		if (ret < 0)
-			zlog_warn("can't setsockopt IPV6_LEAVE_GROUP: %s\n",
+			zlog_warn("can't setsockopt IPV6_LEAVE_GROUP: %s",
 				  safe_strerror(errno));
 
 		if (IS_RIPNG_DEBUG_EVENT)
@@ -162,28 +161,33 @@ static int ripng_if_ipv6_lladdress_check(struct interface *ifp)
 
 static int ripng_if_down(struct interface *ifp)
 {
-	struct route_node *rp;
+	struct agg_node *rp;
 	struct ripng_info *rinfo;
 	struct ripng_interface *ri;
+	struct ripng *ripng;
 	struct list *list = NULL;
 	struct listnode *listnode = NULL, *nextnode = NULL;
 
+	ri = ifp->info;
+	ripng = ri->ripng;
+
 	if (ripng)
-		for (rp = route_top(ripng->table); rp; rp = route_next(rp))
+		for (rp = agg_route_top(ripng->table); rp;
+		     rp = agg_route_next(rp))
 			if ((list = rp->info) != NULL)
 				for (ALL_LIST_ELEMENTS(list, listnode, nextnode,
 						       rinfo))
 					if (rinfo->ifindex == ifp->ifindex)
-						ripng_ecmp_delete(rinfo);
+						ripng_ecmp_delete(ripng, rinfo);
 
-	ri = ifp->info;
 
 	if (ri->running) {
 		if (IS_RIPNG_DEBUG_EVENT)
 			zlog_debug("turn off %s", ifp->name);
 
 		/* Leave from multicast group. */
-		ripng_multicast_leave(ifp);
+		if (ripng)
+			ripng_multicast_leave(ifp, ripng->sock);
 
 		ri->running = 0;
 	}
@@ -192,25 +196,18 @@ static int ripng_if_down(struct interface *ifp)
 }
 
 /* Inteface link up message processing. */
-int ripng_interface_up(int command, struct zclient *zclient,
-		       zebra_size_t length, vrf_id_t vrf_id)
+static int ripng_ifp_up(struct interface *ifp)
 {
-	struct stream *s;
-	struct interface *ifp;
+	if (IS_RIPNG_DEBUG_ZEBRA) {
+		struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
 
-	/* zebra_interface_state_read() updates interface structure in iflist.
-	 */
-	s = zclient->ibuf;
-	ifp = zebra_interface_state_read(s, vrf_id);
-
-	if (ifp == NULL)
-		return 0;
-
-	if (IS_RIPNG_DEBUG_ZEBRA)
 		zlog_debug(
-			"interface up %s index %d flags %llx metric %d mtu %d",
-			ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
-			ifp->metric, ifp->mtu6);
+			"interface up %s vrf %s(%u) index %d flags %llx metric %d mtu %d",
+			ifp->name, VRF_LOGNAME(vrf), ifp->vrf_id, ifp->ifindex,
+			(unsigned long long)ifp->flags, ifp->metric, ifp->mtu6);
+	}
+
+	ripng_interface_sync(ifp);
 
 	/* Check if this interface is RIPng enabled or not. */
 	ripng_enable_apply(ifp);
@@ -225,44 +222,36 @@ int ripng_interface_up(int command, struct zclient *zclient,
 }
 
 /* Inteface link down message processing. */
-int ripng_interface_down(int command, struct zclient *zclient,
-			 zebra_size_t length, vrf_id_t vrf_id)
+static int ripng_ifp_down(struct interface *ifp)
 {
-	struct stream *s;
-	struct interface *ifp;
-
-	/* zebra_interface_state_read() updates interface structure in iflist.
-	 */
-	s = zclient->ibuf;
-	ifp = zebra_interface_state_read(s, vrf_id);
-
-	if (ifp == NULL)
-		return 0;
-
+	ripng_interface_sync(ifp);
 	ripng_if_down(ifp);
 
-	if (IS_RIPNG_DEBUG_ZEBRA)
+	if (IS_RIPNG_DEBUG_ZEBRA) {
+		struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
+
 		zlog_debug(
-			"interface down %s index %d flags %#llx metric %d mtu %d",
-			ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
-			ifp->metric, ifp->mtu6);
+			"interface down %s vrf %s(%u) index %d flags %#llx metric %d mtu %d",
+			ifp->name, VRF_LOGNAME(vrf), ifp->vrf_id, ifp->ifindex,
+			(unsigned long long)ifp->flags, ifp->metric, ifp->mtu6);
+	}
 
 	return 0;
 }
 
 /* Inteface addition message from zebra. */
-int ripng_interface_add(int command, struct zclient *zclient,
-			zebra_size_t length, vrf_id_t vrf_id)
+static int ripng_ifp_create(struct interface *ifp)
 {
-	struct interface *ifp;
+	ripng_interface_sync(ifp);
 
-	ifp = zebra_interface_add_read(zclient->ibuf, vrf_id);
+	if (IS_RIPNG_DEBUG_ZEBRA) {
+		struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
 
-	if (IS_RIPNG_DEBUG_ZEBRA)
 		zlog_debug(
-			"RIPng interface add %s index %d flags %#llx metric %d mtu %d",
-			ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
-			ifp->metric, ifp->mtu6);
+			"RIPng interface add %s vrf %s(%u) index %d flags %#llx metric %d mtu %d",
+			ifp->name, VRF_LOGNAME(vrf), ifp->vrf_id, ifp->ifindex,
+			(unsigned long long)ifp->flags, ifp->metric, ifp->mtu6);
+	}
 
 	/* Check is this interface is RIP enabled or not.*/
 	ripng_enable_apply(ifp);
@@ -276,88 +265,72 @@ int ripng_interface_add(int command, struct zclient *zclient,
 	return 0;
 }
 
-int ripng_interface_delete(int command, struct zclient *zclient,
-			   zebra_size_t length, vrf_id_t vrf_id)
+static int ripng_ifp_destroy(struct interface *ifp)
 {
-	struct interface *ifp;
-	struct stream *s;
-
-	s = zclient->ibuf;
-	/*  zebra_interface_state_read() updates interface structure in iflist
-	 */
-	ifp = zebra_interface_state_read(s, vrf_id);
-
-	if (ifp == NULL)
-		return 0;
+	struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
 
+	ripng_interface_sync(ifp);
 	if (if_is_up(ifp)) {
 		ripng_if_down(ifp);
 	}
 
-	zlog_info("interface delete %s index %d flags %#llx metric %d mtu %d",
-		  ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
-		  ifp->metric, ifp->mtu6);
-
-	/* To support pseudo interface do not free interface structure.  */
-	/* if_delete(ifp); */
-	ifp->ifindex = IFINDEX_DELETED;
+	zlog_info(
+		"interface delete %s vrf %s(%u) index %d flags %#llx metric %d mtu %d",
+		ifp->name, VRF_LOGNAME(vrf), ifp->vrf_id, ifp->ifindex,
+		(unsigned long long)ifp->flags, ifp->metric, ifp->mtu6);
 
 	return 0;
 }
 
-void ripng_interface_clean(void)
+/* VRF update for an interface. */
+int ripng_interface_vrf_update(ZAPI_CALLBACK_ARGS)
 {
-	struct listnode *node, *nnode;
 	struct interface *ifp;
-	struct ripng_interface *ri;
+	vrf_id_t new_vrf_id;
 
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), node, nnode, ifp)) {
-		ri = ifp->info;
+	ifp = zebra_interface_vrf_update_read(zclient->ibuf, vrf_id,
+					      &new_vrf_id);
+	if (!ifp)
+		return 0;
 
-		ri->enable_network = 0;
-		ri->enable_interface = 0;
-		ri->running = 0;
+	if (IS_RIPNG_DEBUG_ZEBRA) {
+		struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
+		struct vrf *nvrf = vrf_lookup_by_id(new_vrf_id);
 
-		if (ri->t_wakeup) {
-			thread_cancel(ri->t_wakeup);
-			ri->t_wakeup = NULL;
-		}
+		zlog_debug("interface %s VRF change vrf %s(%u) new vrf %s(%u)",
+			   ifp->name, VRF_LOGNAME(vrf), vrf_id,
+			   VRF_LOGNAME(nvrf), new_vrf_id);
 	}
+
+	if_update_to_new_vrf(ifp, new_vrf_id);
+	ripng_interface_sync(ifp);
+
+	return 0;
 }
 
-void ripng_interface_reset(void)
+void ripng_interface_clean(struct ripng *ripng)
 {
-	struct listnode *node;
 	struct interface *ifp;
 	struct ripng_interface *ri;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (ripng->vrf, ifp) {
 		ri = ifp->info;
 
 		ri->enable_network = 0;
 		ri->enable_interface = 0;
 		ri->running = 0;
 
-		ri->split_horizon = RIPNG_NO_SPLIT_HORIZON;
-		ri->split_horizon_default = RIPNG_NO_SPLIT_HORIZON;
-
-		ri->list[RIPNG_FILTER_IN] = NULL;
-		ri->list[RIPNG_FILTER_OUT] = NULL;
-
-		ri->prefix[RIPNG_FILTER_IN] = NULL;
-		ri->prefix[RIPNG_FILTER_OUT] = NULL;
-
 		if (ri->t_wakeup) {
 			thread_cancel(ri->t_wakeup);
 			ri->t_wakeup = NULL;
 		}
-
-		ri->passive = 0;
 	}
 }
 
 static void ripng_apply_address_add(struct connected *ifc)
 {
+	struct ripng_interface *ri = ifc->ifp->info;
+	struct ripng *ripng = ri->ripng;
 	struct prefix_ipv6 address;
 	struct prefix *p;
 
@@ -377,15 +350,14 @@ static void ripng_apply_address_add(struct connected *ifc)
 
 	/* Check if this interface is RIP enabled or not
 	   or  Check if this address's prefix is RIP enabled */
-	if ((ripng_enable_if_lookup(ifc->ifp->name) >= 0)
+	if ((ripng_enable_if_lookup(ripng, ifc->ifp->name) >= 0)
 	    || (ripng_enable_network_lookup2(ifc) >= 0))
-		ripng_redistribute_add(ZEBRA_ROUTE_CONNECT,
+		ripng_redistribute_add(ripng, ZEBRA_ROUTE_CONNECT,
 				       RIPNG_ROUTE_INTERFACE, &address,
 				       ifc->ifp->ifindex, NULL, 0);
 }
 
-int ripng_interface_address_add(int command, struct zclient *zclient,
-				zebra_size_t length, vrf_id_t vrf_id)
+int ripng_interface_address_add(ZAPI_CALLBACK_ARGS)
 {
 	struct connected *c;
 	struct prefix *p;
@@ -427,6 +399,8 @@ int ripng_interface_address_add(int command, struct zclient *zclient,
 
 static void ripng_apply_address_del(struct connected *ifc)
 {
+	struct ripng_interface *ri = ifc->ifp->info;
+	struct ripng *ripng = ri->ripng;
 	struct prefix_ipv6 address;
 	struct prefix *p;
 
@@ -444,12 +418,12 @@ static void ripng_apply_address_del(struct connected *ifc)
 	address.prefixlen = p->prefixlen;
 	apply_mask_ipv6(&address);
 
-	ripng_redistribute_delete(ZEBRA_ROUTE_CONNECT, RIPNG_ROUTE_INTERFACE,
-				  &address, ifc->ifp->ifindex);
+	ripng_redistribute_delete(ripng, ZEBRA_ROUTE_CONNECT,
+				  RIPNG_ROUTE_INTERFACE, &address,
+				  ifc->ifp->ifindex);
 }
 
-int ripng_interface_address_delete(int command, struct zclient *zclient,
-				   zebra_size_t length, vrf_id_t vrf_id)
+int ripng_interface_address_delete(ZAPI_CALLBACK_ARGS)
 {
 	struct connected *ifc;
 	struct prefix *p;
@@ -472,30 +446,29 @@ int ripng_interface_address_delete(int command, struct zclient *zclient,
 			/* Check wether this prefix needs to be removed. */
 			ripng_apply_address_del(ifc);
 		}
-		connected_free(ifc);
+		connected_free(&ifc);
 	}
 
 	return 0;
 }
 
-/* RIPng enable interface vector. */
-vector ripng_enable_if;
-
-/* RIPng enable network table. */
-struct route_table *ripng_enable_network;
-
 /* Lookup RIPng enable network. */
 /* Check wether the interface has at least a connected prefix that
- * is within the ripng_enable_network table. */
+ * is within the ripng->enable_network table. */
 static int ripng_enable_network_lookup_if(struct interface *ifp)
 {
+	struct ripng_interface *ri = ifp->info;
+	struct ripng *ripng = ri->ripng;
 	struct listnode *node;
 	struct connected *connected;
 	struct prefix_ipv6 address;
 
+	if (!ripng)
+		return -1;
+
 	for (ALL_LIST_ELEMENTS_RO(ifp->connected, node, connected)) {
 		struct prefix *p;
-		struct route_node *node;
+		struct agg_node *n;
 
 		p = connected->address;
 
@@ -504,10 +477,10 @@ static int ripng_enable_network_lookup_if(struct interface *ifp)
 			address.prefix = p->u.prefix6;
 			address.prefixlen = IPV6_MAX_BITLEN;
 
-			node = route_node_match(ripng_enable_network,
-						(struct prefix *)&address);
-			if (node) {
-				route_unlock_node(node);
+			n = agg_node_match(ripng->enable_network,
+					   (struct prefix *)&address);
+			if (n) {
+				agg_unlock_node(n);
 				return 1;
 			}
 		}
@@ -515,28 +488,33 @@ static int ripng_enable_network_lookup_if(struct interface *ifp)
 	return -1;
 }
 
-/* Check wether connected is within the ripng_enable_network table. */
+/* Check wether connected is within the ripng->enable_network table. */
 static int ripng_enable_network_lookup2(struct connected *connected)
 {
+	struct ripng_interface *ri = connected->ifp->info;
+	struct ripng *ripng = ri->ripng;
 	struct prefix_ipv6 address;
 	struct prefix *p;
 
+	if (!ripng)
+		return -1;
+
 	p = connected->address;
 
 	if (p->family == AF_INET6) {
-		struct route_node *node;
+		struct agg_node *node;
 
 		address.family = p->family;
 		address.prefix = p->u.prefix6;
 		address.prefixlen = IPV6_MAX_BITLEN;
 
 		/* LPM on p->family, p->u.prefix6/IPV6_MAX_BITLEN within
-		 * ripng_enable_network */
-		node = route_node_match(ripng_enable_network,
-					(struct prefix *)&address);
+		 * ripng->enable_network */
+		node = agg_node_match(ripng->enable_network,
+				      (struct prefix *)&address);
 
 		if (node) {
-			route_unlock_node(node);
+			agg_unlock_node(node);
 			return 1;
 		}
 	}
@@ -545,90 +523,92 @@ static int ripng_enable_network_lookup2(struct connected *connected)
 }
 
 /* Add RIPng enable network. */
-static int ripng_enable_network_add(struct prefix *p)
+int ripng_enable_network_add(struct ripng *ripng, struct prefix *p)
 {
-	struct route_node *node;
+	struct agg_node *node;
 
-	node = route_node_get(ripng_enable_network, p);
+	node = agg_node_get(ripng->enable_network, p);
 
 	if (node->info) {
-		route_unlock_node(node);
-		return -1;
+		agg_unlock_node(node);
+		return NB_ERR_INCONSISTENCY;
 	} else
 		node->info = (void *)1;
 
 	/* XXX: One should find a better solution than a generic one */
-	ripng_enable_apply_all();
+	ripng_enable_apply_all(ripng);
 
-	return 1;
+	return NB_OK;
 }
 
 /* Delete RIPng enable network. */
-static int ripng_enable_network_delete(struct prefix *p)
+int ripng_enable_network_delete(struct ripng *ripng, struct prefix *p)
 {
-	struct route_node *node;
+	struct agg_node *node;
 
-	node = route_node_lookup(ripng_enable_network, p);
+	node = agg_node_lookup(ripng->enable_network, p);
 	if (node) {
 		node->info = NULL;
 
 		/* Unlock info lock. */
-		route_unlock_node(node);
+		agg_unlock_node(node);
 
 		/* Unlock lookup lock. */
-		route_unlock_node(node);
+		agg_unlock_node(node);
 
-		return 1;
+		return NB_OK;
 	}
-	return -1;
+
+	return NB_ERR_INCONSISTENCY;
 }
 
 /* Lookup function. */
-static int ripng_enable_if_lookup(const char *ifname)
+static int ripng_enable_if_lookup(struct ripng *ripng, const char *ifname)
 {
 	unsigned int i;
 	char *str;
 
-	for (i = 0; i < vector_active(ripng_enable_if); i++)
-		if ((str = vector_slot(ripng_enable_if, i)) != NULL)
+	if (!ripng)
+		return -1;
+
+	for (i = 0; i < vector_active(ripng->enable_if); i++)
+		if ((str = vector_slot(ripng->enable_if, i)) != NULL)
 			if (strcmp(str, ifname) == 0)
 				return i;
 	return -1;
 }
 
-/* Add interface to ripng_enable_if. */
-static int ripng_enable_if_add(const char *ifname)
+int ripng_enable_if_add(struct ripng *ripng, const char *ifname)
 {
 	int ret;
 
-	ret = ripng_enable_if_lookup(ifname);
+	ret = ripng_enable_if_lookup(ripng, ifname);
 	if (ret >= 0)
-		return -1;
+		return NB_ERR_INCONSISTENCY;
 
-	vector_set(ripng_enable_if, strdup(ifname));
+	vector_set(ripng->enable_if, strdup(ifname));
 
-	ripng_enable_apply_all();
+	ripng_enable_apply_all(ripng);
 
-	return 1;
+	return NB_OK;
 }
 
-/* Delete interface from ripng_enable_if. */
-static int ripng_enable_if_delete(const char *ifname)
+int ripng_enable_if_delete(struct ripng *ripng, const char *ifname)
 {
 	int index;
 	char *str;
 
-	index = ripng_enable_if_lookup(ifname);
+	index = ripng_enable_if_lookup(ripng, ifname);
 	if (index < 0)
-		return -1;
+		return NB_ERR_INCONSISTENCY;
 
-	str = vector_slot(ripng_enable_if, index);
+	str = vector_slot(ripng->enable_if, index);
 	free(str);
-	vector_unset(ripng_enable_if, index);
+	vector_unset(ripng->enable_if, index);
 
-	ripng_enable_apply_all();
+	ripng_enable_apply_all(ripng);
 
-	return 1;
+	return NB_OK;
 }
 
 /* Wake up interface. */
@@ -644,9 +624,10 @@ static int ripng_interface_wakeup(struct thread *t)
 	ri->t_wakeup = NULL;
 
 	/* Join to multicast group. */
-	if (ripng_multicast_join(ifp) < 0) {
-		zlog_err("multicast join failed, interface %s not running",
-			 ifp->name);
+	if (ripng_multicast_join(ifp, ri->ripng->sock) < 0) {
+		flog_err_sys(EC_LIB_SOCKET,
+			     "multicast join failed, interface %s not running",
+			     ifp->name);
 		return 0;
 	}
 
@@ -661,6 +642,8 @@ static int ripng_interface_wakeup(struct thread *t)
 
 static void ripng_connect_set(struct interface *ifp, int set)
 {
+	struct ripng_interface *ri = ifp->info;
+	struct ripng *ripng = ri->ripng;
 	struct listnode *node, *nnode;
 	struct connected *connected;
 	struct prefix_ipv6 address;
@@ -680,19 +663,22 @@ static void ripng_connect_set(struct interface *ifp, int set)
 		if (set) {
 			/* Check once more wether this prefix is within a
 			 * "network IF_OR_PREF" one */
-			if ((ripng_enable_if_lookup(connected->ifp->name) >= 0)
+			if ((ripng_enable_if_lookup(ripng, connected->ifp->name)
+			     >= 0)
 			    || (ripng_enable_network_lookup2(connected) >= 0))
 				ripng_redistribute_add(
-					ZEBRA_ROUTE_CONNECT,
+					ripng, ZEBRA_ROUTE_CONNECT,
 					RIPNG_ROUTE_INTERFACE, &address,
 					connected->ifp->ifindex, NULL, 0);
 		} else {
-			ripng_redistribute_delete(
-				ZEBRA_ROUTE_CONNECT, RIPNG_ROUTE_INTERFACE,
-				&address, connected->ifp->ifindex);
-			if (ripng_redistribute_check(ZEBRA_ROUTE_CONNECT))
+			ripng_redistribute_delete(ripng, ZEBRA_ROUTE_CONNECT,
+						  RIPNG_ROUTE_INTERFACE,
+						  &address,
+						  connected->ifp->ifindex);
+			if (ripng_redistribute_check(ripng,
+						     ZEBRA_ROUTE_CONNECT))
 				ripng_redistribute_add(
-					ZEBRA_ROUTE_CONNECT,
+					ripng, ZEBRA_ROUTE_CONNECT,
 					RIPNG_ROUTE_REDISTRIBUTE, &address,
 					connected->ifp->ifindex, NULL, 0);
 		}
@@ -721,7 +707,7 @@ void ripng_enable_apply(struct interface *ifp)
 		ri->enable_network = 0;
 
 	/* Check interface name configuration. */
-	ret = ripng_enable_if_lookup(ifp->name);
+	ret = ripng_enable_if_lookup(ri->ripng, ifp->name);
 	if (ret >= 0)
 		ri->enable_interface = 1;
 	else
@@ -738,17 +724,13 @@ void ripng_enable_apply(struct interface *ifp)
 
 	/* Update running status of the interface. */
 	if (ri->enable_network || ri->enable_interface) {
-		{
-			if (IS_RIPNG_DEBUG_EVENT)
-				zlog_debug("RIPng turn on %s", ifp->name);
+		zlog_info("RIPng INTERFACE ON %s", ifp->name);
 
-			/* Add interface wake up thread. */
-			if (!ri->t_wakeup)
-				ri->t_wakeup = thread_add_timer(
-					master, ripng_interface_wakeup, ifp, 1);
+		/* Add interface wake up thread. */
+		thread_add_timer(master, ripng_interface_wakeup, ifp, 1,
+				 &ri->t_wakeup);
 
-			ripng_connect_set(ifp, 1);
-		}
+		ripng_connect_set(ifp, 1);
 	} else {
 		if (ri->running) {
 			/* Might as well clean up the route table as well
@@ -763,48 +745,46 @@ void ripng_enable_apply(struct interface *ifp)
 }
 
 /* Set distribute list to all interfaces. */
-static void ripng_enable_apply_all(void)
+static void ripng_enable_apply_all(struct ripng *ripng)
 {
 	struct interface *ifp;
-	struct listnode *node;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+	FOR_ALL_INTERFACES (ripng->vrf, ifp)
 		ripng_enable_apply(ifp);
 }
 
 /* Clear all network and neighbor configuration */
-void ripng_clean_network()
+void ripng_clean_network(struct ripng *ripng)
 {
 	unsigned int i;
 	char *str;
-	struct route_node *rn;
+	struct agg_node *rn;
 
-	/* ripng_enable_network */
-	for (rn = route_top(ripng_enable_network); rn; rn = route_next(rn))
+	/* ripng->enable_network */
+	for (rn = agg_route_top(ripng->enable_network); rn;
+	     rn = agg_route_next(rn))
 		if (rn->info) {
 			rn->info = NULL;
-			route_unlock_node(rn);
+			agg_unlock_node(rn);
 		}
 
-	/* ripng_enable_if */
-	for (i = 0; i < vector_active(ripng_enable_if); i++)
-		if ((str = vector_slot(ripng_enable_if, i)) != NULL) {
+	/* ripng->enable_if */
+	for (i = 0; i < vector_active(ripng->enable_if); i++)
+		if ((str = vector_slot(ripng->enable_if, i)) != NULL) {
 			free(str);
-			vector_slot(ripng_enable_if, i) = NULL;
+			vector_slot(ripng->enable_if, i) = NULL;
 		}
 }
 
-/* Vector to store passive-interface name. */
-vector Vripng_passive_interface;
-
 /* Utility function for looking up passive interface settings. */
-static int ripng_passive_interface_lookup(const char *ifname)
+static int ripng_passive_interface_lookup(struct ripng *ripng,
+					  const char *ifname)
 {
 	unsigned int i;
 	char *str;
 
-	for (i = 0; i < vector_active(Vripng_passive_interface); i++)
-		if ((str = vector_slot(Vripng_passive_interface, i)) != NULL)
+	for (i = 0; i < vector_active(ripng->passive_interface); i++)
+		if ((str = vector_slot(ripng->passive_interface, i)) != NULL)
 			if (strcmp(str, ifname) == 0)
 				return i;
 	return -1;
@@ -814,347 +794,186 @@ void ripng_passive_interface_apply(struct interface *ifp)
 {
 	int ret;
 	struct ripng_interface *ri;
+	struct ripng *ripng;
 
 	ri = ifp->info;
+	ripng = ri->ripng;
+	if (!ripng)
+		return;
 
-	ret = ripng_passive_interface_lookup(ifp->name);
+	ret = ripng_passive_interface_lookup(ripng, ifp->name);
 	if (ret < 0)
 		ri->passive = 0;
 	else
 		ri->passive = 1;
 }
 
-static void ripng_passive_interface_apply_all(void)
+static void ripng_passive_interface_apply_all(struct ripng *ripng)
 {
 	struct interface *ifp;
-	struct listnode *node;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+	FOR_ALL_INTERFACES (ripng->vrf, ifp)
 		ripng_passive_interface_apply(ifp);
 }
 
 /* Passive interface. */
-static int ripng_passive_interface_set(struct vty *vty, const char *ifname)
+int ripng_passive_interface_set(struct ripng *ripng, const char *ifname)
 {
-	if (ripng_passive_interface_lookup(ifname) >= 0)
-		return CMD_WARNING;
+	if (ripng_passive_interface_lookup(ripng, ifname) >= 0)
+		return NB_ERR_INCONSISTENCY;
 
-	vector_set(Vripng_passive_interface, strdup(ifname));
+	vector_set(ripng->passive_interface, strdup(ifname));
 
-	ripng_passive_interface_apply_all();
+	ripng_passive_interface_apply_all(ripng);
 
-	return CMD_SUCCESS;
+	return NB_OK;
 }
 
-static int ripng_passive_interface_unset(struct vty *vty, const char *ifname)
+int ripng_passive_interface_unset(struct ripng *ripng, const char *ifname)
 {
 	int i;
 	char *str;
 
-	i = ripng_passive_interface_lookup(ifname);
+	i = ripng_passive_interface_lookup(ripng, ifname);
 	if (i < 0)
-		return CMD_WARNING;
+		return NB_ERR_INCONSISTENCY;
 
-	str = vector_slot(Vripng_passive_interface, i);
+	str = vector_slot(ripng->passive_interface, i);
 	free(str);
-	vector_unset(Vripng_passive_interface, i);
+	vector_unset(ripng->passive_interface, i);
 
-	ripng_passive_interface_apply_all();
+	ripng_passive_interface_apply_all(ripng);
 
-	return CMD_SUCCESS;
+	return NB_OK;
 }
 
 /* Free all configured RIP passive-interface settings. */
-void ripng_passive_interface_clean(void)
+void ripng_passive_interface_clean(struct ripng *ripng)
 {
 	unsigned int i;
 	char *str;
 
-	for (i = 0; i < vector_active(Vripng_passive_interface); i++)
-		if ((str = vector_slot(Vripng_passive_interface, i)) != NULL) {
+	for (i = 0; i < vector_active(ripng->passive_interface); i++)
+		if ((str = vector_slot(ripng->passive_interface, i)) != NULL) {
 			free(str);
-			vector_slot(Vripng_passive_interface, i) = NULL;
+			vector_slot(ripng->passive_interface, i) = NULL;
 		}
-	ripng_passive_interface_apply_all();
+	ripng_passive_interface_apply_all(ripng);
 }
 
 /* Write RIPng enable network and interface to the vty. */
-int ripng_network_write(struct vty *vty, int config_mode)
+int ripng_network_write(struct vty *vty, struct ripng *ripng)
 {
 	unsigned int i;
 	const char *ifname;
-	struct route_node *node;
-	char buf[BUFSIZ];
+	struct agg_node *node;
 
 	/* Write enable network. */
-	for (node = route_top(ripng_enable_network); node;
-	     node = route_next(node))
-		if (node->info) {
-			struct prefix *p = &node->p;
-			vty_out(vty, "%s%s/%d%s",
-				config_mode ? " network " : "    ",
-				inet_ntop(p->family, &p->u.prefix, buf, BUFSIZ),
-				p->prefixlen, VTY_NEWLINE);
-		}
+	for (node = agg_route_top(ripng->enable_network); node;
+	     node = agg_route_next(node))
+		if (node->info)
+			vty_out(vty, "    %pRN\n", node);
 
 	/* Write enable interface. */
-	for (i = 0; i < vector_active(ripng_enable_if); i++)
-		if ((ifname = vector_slot(ripng_enable_if, i)) != NULL)
-			vty_out(vty, "%s%s%s",
-				config_mode ? " network " : "    ", ifname,
-				VTY_NEWLINE);
-
-	/* Write passive interface. */
-	if (config_mode)
-		for (i = 0; i < vector_active(Vripng_passive_interface); i++)
-			if ((ifname = vector_slot(Vripng_passive_interface, i))
-			    != NULL)
-				vty_out(vty, " passive-interface %s%s", ifname,
-					VTY_NEWLINE);
+	for (i = 0; i < vector_active(ripng->enable_if); i++)
+		if ((ifname = vector_slot(ripng->enable_if, i)) != NULL)
+			vty_out(vty, "    %s\n", ifname);
 
 	return 0;
 }
 
-/* RIPng enable on specified interface or matched network. */
-DEFUN (ripng_network,
-       ripng_network_cmd,
-       "network IF_OR_ADDR",
-       "RIPng enable on specified interface or network.\n"
-       "Interface or address")
-{
-	int idx_if_or_addr = 1;
-	int ret;
-	struct prefix p;
-
-	ret = str2prefix(argv[idx_if_or_addr]->arg, &p);
-
-	/* Given string is IPv6 network or interface name. */
-	if (ret)
-		ret = ripng_enable_network_add(&p);
-	else
-		ret = ripng_enable_if_add(argv[idx_if_or_addr]->arg);
-
-	if (ret < 0) {
-		vty_out(vty, "There is same network configuration %s%s",
-			argv[idx_if_or_addr]->arg, VTY_NEWLINE);
-		return CMD_WARNING;
-	}
-
-	return CMD_SUCCESS;
-}
-
-/* RIPng enable on specified interface or matched network. */
-DEFUN (no_ripng_network,
-       no_ripng_network_cmd,
-       "no network IF_OR_ADDR",
-       NO_STR
-       "RIPng enable on specified interface or network.\n"
-       "Interface or address")
-{
-	int idx_if_or_addr = 2;
-	int ret;
-	struct prefix p;
-
-	ret = str2prefix(argv[idx_if_or_addr]->arg, &p);
-
-	/* Given string is interface name. */
-	if (ret)
-		ret = ripng_enable_network_delete(&p);
-	else
-		ret = ripng_enable_if_delete(argv[idx_if_or_addr]->arg);
-
-	if (ret < 0) {
-		vty_out(vty, "can't find network %s%s",
-			argv[idx_if_or_addr]->arg, VTY_NEWLINE);
-		return CMD_WARNING;
-	}
-
-	return CMD_SUCCESS;
-}
-
-DEFUN (ipv6_ripng_split_horizon,
-       ipv6_ripng_split_horizon_cmd,
-       "ipv6 ripng split-horizon",
-       IPV6_STR
-       "Routing Information Protocol\n"
-       "Perform split horizon\n")
-{
-	VTY_DECLVAR_CONTEXT(interface, ifp);
-	struct ripng_interface *ri;
-
-	ri = ifp->info;
-
-	ri->split_horizon = RIPNG_SPLIT_HORIZON;
-	return CMD_SUCCESS;
-}
-
-DEFUN (ipv6_ripng_split_horizon_poisoned_reverse,
-       ipv6_ripng_split_horizon_poisoned_reverse_cmd,
-       "ipv6 ripng split-horizon poisoned-reverse",
-       IPV6_STR
-       "Routing Information Protocol\n"
-       "Perform split horizon\n"
-       "With poisoned-reverse\n")
-{
-	VTY_DECLVAR_CONTEXT(interface, ifp);
-	struct ripng_interface *ri;
-
-	ri = ifp->info;
-
-	ri->split_horizon = RIPNG_SPLIT_HORIZON_POISONED_REVERSE;
-	return CMD_SUCCESS;
-}
-
-DEFUN (no_ipv6_ripng_split_horizon,
-       no_ipv6_ripng_split_horizon_cmd,
-       "no ipv6 ripng split-horizon [poisoned-reverse]",
-       NO_STR
-       IPV6_STR
-       "Routing Information Protocol\n"
-       "Perform split horizon\n"
-       "With poisoned-reverse\n")
-{
-	VTY_DECLVAR_CONTEXT(interface, ifp);
-	struct ripng_interface *ri;
-
-	ri = ifp->info;
-
-	ri->split_horizon = RIPNG_NO_SPLIT_HORIZON;
-	return CMD_SUCCESS;
-}
-
-DEFUN (ripng_passive_interface,
-       ripng_passive_interface_cmd,
-       "passive-interface IFNAME",
-       "Suppress routing updates on an interface\n"
-       "Interface name\n")
-{
-	int idx_ifname = 1;
-	return ripng_passive_interface_set(vty, argv[idx_ifname]->arg);
-}
-
-DEFUN (no_ripng_passive_interface,
-       no_ripng_passive_interface_cmd,
-       "no passive-interface IFNAME",
-       NO_STR
-       "Suppress routing updates on an interface\n"
-       "Interface name\n")
-{
-	int idx_ifname = 2;
-	return ripng_passive_interface_unset(vty, argv[idx_ifname]->arg);
-}
-
 static struct ripng_interface *ri_new(void)
 {
 	struct ripng_interface *ri;
-	ri = XCALLOC(MTYPE_IF, sizeof(struct ripng_interface));
+
+	ri = XCALLOC(MTYPE_RIPNG_IF, sizeof(struct ripng_interface));
 
 	/* Set default split-horizon behavior.  If the interface is Frame
 	   Relay or SMDS is enabled, the default value for split-horizon is
 	   off.  But currently Zebra does detect Frame Relay or SMDS
 	   interface.  So all interface is set to split horizon.  */
-	ri->split_horizon_default = RIPNG_SPLIT_HORIZON;
-	ri->split_horizon = ri->split_horizon_default;
+	ri->split_horizon =
+		yang_get_default_enum("%s/split-horizon", RIPNG_IFACE);
 
 	return ri;
 }
 
+void ripng_interface_sync(struct interface *ifp)
+{
+	struct vrf *vrf;
+
+	vrf = vrf_lookup_by_id(ifp->vrf_id);
+	if (vrf) {
+		struct ripng_interface *ri;
+
+		ri = ifp->info;
+		if (ri)
+			ri->ripng = vrf->info;
+	}
+}
+
 static int ripng_if_new_hook(struct interface *ifp)
 {
 	ifp->info = ri_new();
+	ripng_interface_sync(ifp);
+
 	return 0;
 }
 
 /* Called when interface structure deleted. */
 static int ripng_if_delete_hook(struct interface *ifp)
 {
-	XFREE(MTYPE_IF, ifp->info);
-	ifp->info = NULL;
+	XFREE(MTYPE_RIPNG_IF, ifp->info);
 	return 0;
 }
 
 /* Configuration write function for ripngd. */
 static int interface_config_write(struct vty *vty)
 {
-	struct listnode *node;
-	struct interface *ifp;
-	struct ripng_interface *ri;
+	struct vrf *vrf;
 	int write = 0;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
-		ri = ifp->info;
+	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+		struct interface *ifp;
 
-		/* Do not display the interface if there is no
-		 * configuration about it.
-		 **/
-		if ((!ifp->desc)
-		    && (ri->split_horizon == ri->split_horizon_default))
-			continue;
+		FOR_ALL_INTERFACES (vrf, ifp) {
+			struct lyd_node *dnode;
 
-		vty_out(vty, "interface %s%s", ifp->name, VTY_NEWLINE);
-		if (ifp->desc)
-			vty_out(vty, " description %s%s", ifp->desc,
-				VTY_NEWLINE);
+			dnode = yang_dnode_get(
+				running_config->dnode,
+				"/frr-interface:lib/interface[name='%s'][vrf='%s']",
+				ifp->name, vrf->name);
+			if (dnode == NULL)
+				continue;
 
-		/* Split horizon. */
-		if (ri->split_horizon != ri->split_horizon_default) {
-			switch (ri->split_horizon) {
-			case RIPNG_SPLIT_HORIZON:
-				vty_out(vty, " ipv6 ripng split-horizon%s",
-					VTY_NEWLINE);
-				break;
-			case RIPNG_SPLIT_HORIZON_POISONED_REVERSE:
-				vty_out(vty,
-					" ipv6 ripng split-horizon poisoned-reverse%s",
-					VTY_NEWLINE);
-				break;
-			case RIPNG_NO_SPLIT_HORIZON:
-			default:
-				vty_out(vty, " no ipv6 ripng split-horizon%s",
-					VTY_NEWLINE);
-				break;
-			}
+			write = 1;
+			nb_cli_show_dnode_cmds(vty, dnode, false);
 		}
-
-		vty_out(vty, "!%s", VTY_NEWLINE);
-
-		write++;
 	}
+
 	return write;
 }
 
+static int interface_config_write(struct vty *vty);
 /* ripngd's interface node. */
 static struct cmd_node interface_node = {
-	INTERFACE_NODE, "%s(config-if)# ", 1 /* VTYSH */
+	.name = "interface",
+	.node = INTERFACE_NODE,
+	.parent_node = CONFIG_NODE,
+	.prompt = "%s(config-if)# ",
+	.config_write = interface_config_write,
 };
 
 /* Initialization of interface. */
-void ripng_if_init()
+void ripng_if_init(void)
 {
 	/* Interface initialize. */
-	if_add_hook(IF_NEW_HOOK, ripng_if_new_hook);
-	if_add_hook(IF_DELETE_HOOK, ripng_if_delete_hook);
-
-	/* RIPng enable network init. */
-	ripng_enable_network = route_table_init();
-
-	/* RIPng enable interface init. */
-	ripng_enable_if = vector_init(1);
-
-	/* RIPng passive interface. */
-	Vripng_passive_interface = vector_init(1);
+	hook_register_prio(if_add, 0, ripng_if_new_hook);
+	hook_register_prio(if_del, 0, ripng_if_delete_hook);
 
 	/* Install interface node. */
-	install_node(&interface_node, interface_config_write);
+	install_node(&interface_node);
 	if_cmd_init();
-
-	install_element(RIPNG_NODE, &ripng_network_cmd);
-	install_element(RIPNG_NODE, &no_ripng_network_cmd);
-	install_element(RIPNG_NODE, &ripng_passive_interface_cmd);
-	install_element(RIPNG_NODE, &no_ripng_passive_interface_cmd);
-
-	install_element(INTERFACE_NODE, &ipv6_ripng_split_horizon_cmd);
-	install_element(INTERFACE_NODE,
-			&ipv6_ripng_split_horizon_poisoned_reverse_cmd);
-	install_element(INTERFACE_NODE, &no_ipv6_ripng_split_horizon_cmd);
+	if_zapi_callbacks(ripng_ifp_create, ripng_ifp_up,
+			  ripng_ifp_down, ripng_ifp_destroy);
 }
