@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Quagga; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -26,10 +25,12 @@
 #include "privs.h"
 #include "queue.h"
 #include "filter.h"
+#include "frr_pthread.h"
 
-#include "bgpd/bgpd.h"
+#include "bgpd/bgpd.c"
 #include "bgpd/bgp_aspath.h"
 #include "bgpd/bgp_attr.h"
+#include "bgpd/bgp_packet.h"
 
 #define VT100_RESET "\x1b[0m"
 #define VT100_RED "\x1b[31m"
@@ -66,7 +67,7 @@ struct test_spec {
 static struct test_segment {
 	const char *name;
 	const char *desc;
-	const u_char asdata[1024];
+	const uint8_t asdata[1024];
 	int len;
 	struct test_spec sp;
 } test_segments[] = {
@@ -473,6 +474,20 @@ static struct test_segment {
 		14,
 		{NULL, NULL, 0, 0, 0, 0, 0, 0},
 	},
+	{
+		/* 28 */
+		"BGP_AS_ZERO",
+		"seq(8466,3,52737,0,4096)",
+		{0x2, 0x5,
+		0x21, 0x12,
+		0x00, 0x03,
+		0xce, 0x01,
+		0x00, 0x00,
+		0x10, 0x00},
+		12,
+		{"8466 3 52737 0 4096", "8466 3 52737 0 4096", 5, 0,
+		 NOT_ALL_PRIVATE, 4096, 4, 8466},
+	},
 	{NULL, NULL, {0}, 0, {NULL, 0, 0}}};
 
 #define COMMON_ATTRS                                                           \
@@ -652,7 +667,7 @@ static struct aspath_tests {
 		&test_segments[6],
 		NULL,
 		AS4_DATA,
-		-1,
+		-2,
 		PEER_CAP_AS4_ADV,
 		{
 			COMMON_ATTRS,
@@ -677,6 +692,21 @@ static struct aspath_tests {
 		COMMON_ATTR_SIZE + 3,
 		&test_segments[0],
 	},
+	/* 13 */
+	{
+		"4b AS4_PATH: BGP_AS_ZERO",
+		&test_segments[28],
+		"8466 3 52737 0 4096",
+		AS4_DATA,
+		-1,
+		PEER_CAP_AS4_RCV | PEER_CAP_AS4_ADV,
+		{
+			COMMON_ATTRS,
+			BGP_ATTR_FLAG_TRANS | BGP_ATTR_FLAG_OPTIONAL,
+			BGP_ATTR_AS4_PATH, 22,
+		},
+		COMMON_ATTR_SIZE + 3,
+	},
 	{NULL, NULL, NULL, 0, 0, 0, {0}, 0},
 };
 
@@ -916,7 +946,7 @@ struct compare_tests {
 };
 
 /* make an aspath from a data stream */
-static struct aspath *make_aspath(const u_char *data, size_t len, int use32bit)
+static struct aspath *make_aspath(const uint8_t *data, size_t len, int use32bit)
 {
 	struct stream *s = NULL;
 	struct aspath *as;
@@ -933,7 +963,7 @@ static struct aspath *make_aspath(const u_char *data, size_t len, int use32bit)
 	return as;
 }
 
-static void printbytes(const u_char *bytes, int len)
+static void printbytes(const uint8_t *bytes, int len)
 {
 	int i = 0;
 	while (i < len) {
@@ -951,7 +981,7 @@ static int validate(struct aspath *as, const struct test_spec *sp)
 {
 	size_t bytes, bytes4;
 	int fails = 0;
-	const u_char *out;
+	const uint8_t *out;
 	static struct stream *s;
 	struct aspath *asinout, *asconfeddel, *asstr, *as4;
 
@@ -1073,7 +1103,7 @@ static int validate(struct aspath *as, const struct test_spec *sp)
 	return fails;
 }
 
-static void empty_get_test()
+static void empty_get_test(void)
 {
 	struct aspath *as = aspath_empty_get();
 	struct test_spec sp = {"", "", 0, 0, 0, 0, 0, 0};
@@ -1218,7 +1248,7 @@ static void aggregate_test(struct tests *t)
 }
 
 /* cmp_left tests  */
-static void cmp_test()
+static void cmp_test(void)
 {
 	unsigned int i;
 #define CMP_TESTS_MAX (sizeof(left_compare) / sizeof(struct compare_tests))
@@ -1274,20 +1304,20 @@ static int handle_attr_test(struct aspath_tests *t)
 
 	asp = make_aspath(t->segment->asdata, t->segment->len, 0);
 
-	peer.ibuf = stream_new(BGP_MAX_PACKET_SIZE);
+	peer.curr = stream_new(BGP_MAX_PACKET_SIZE);
 	peer.obuf = stream_fifo_new();
 	peer.bgp = &bgp;
 	peer.host = (char *)"none";
 	peer.fd = -1;
 	peer.cap = t->cap;
 
-	stream_write(peer.ibuf, t->attrheader, t->len);
-	datalen = aspath_put(peer.ibuf, asp, t->as4 == AS4_DATA);
+	stream_write(peer.curr, t->attrheader, t->len);
+	datalen = aspath_put(peer.curr, asp, t->as4 == AS4_DATA);
 	if (t->old_segment) {
 		char dummyaspath[] = {BGP_ATTR_FLAG_TRANS, BGP_ATTR_AS_PATH,
 				      t->old_segment->len};
-		stream_write(peer.ibuf, dummyaspath, sizeof(dummyaspath));
-		stream_write(peer.ibuf, t->old_segment->asdata,
+		stream_write(peer.curr, dummyaspath, sizeof(dummyaspath));
+		stream_write(peer.curr, t->old_segment->asdata,
 			     t->old_segment->len);
 		datalen += sizeof(dummyaspath) + t->old_segment->len;
 	}
@@ -1338,7 +1368,7 @@ int main(void)
 {
 	int i = 0;
 	qobj_init();
-	bgp_master_init(thread_master_create());
+	bgp_master_init(thread_master_create(NULL), BGP_SOCKET_SNDBUF_SIZE);
 	master = bm->master;
 	bgp_option_set(BGP_OPT_NO_LISTEN);
 	bgp_attr_init();
@@ -1378,6 +1408,10 @@ int main(void)
 
 	i = 0;
 
+	frr_pthread_init();
+	bgp_pthreads_init();
+	bgp_pth_ka->running = true;
+
 	while (aspath_tests[i].desc) {
 		printf("aspath_attr test %d\n", i);
 		attr_test(&aspath_tests[i++]);
