@@ -23,6 +23,7 @@
 #include <time.h>
 
 #include "checksum.h"
+#include "network.h"
 
 struct thread_master *master;
 
@@ -477,7 +478,7 @@ int main(int argc, char **argv)
 		exercise %= MAXDATALEN;
 
 		for (i = 0; i < exercise; i += sizeof(long int)) {
-			long int rand = random();
+			long int rand = frr_weak_random();
 
 			for (j = sizeof(long int); j > 0; j--)
 				buffer[i + (sizeof(long int) - j)] =
@@ -506,7 +507,7 @@ int main(int argc, char **argv)
 			printf("verify: lib failed\n");
 
 		if (ospfd != lib) {
-			printf("Mismatch in values at size %u\n"
+			printf("Mismatch in values at size %d\n"
 			       "ospfd: 0x%04x\tc0: %d\tc1: %d\tx: %d\ty: %d\n"
 			       "isisd: 0x%04x\tc0: %d\tc1: %d\tx: %d\ty: %d\n"
 			       "lib: 0x%04x\n",
