@@ -40,6 +40,7 @@
 #include "zebra/rtadv.h"
 #include "zebra/zebra_mpls.h"
 #include "zebra/debug.h"
+#include "zebra/zebra_errors.h"
 
 /* communicate the withdrawal of a connected address */
 static void connected_withdraw(struct connected *ifc)
@@ -64,7 +65,7 @@ static void connected_withdraw(struct connected *ifc)
 
 	if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_CONFIGURED)) {
 		listnode_delete(ifc->ifp->connected, ifc);
-		connected_free(ifc);
+		connected_free(&ifc);
 	}
 }
 
@@ -119,10 +120,6 @@ struct connected *connected_check_ptp(struct interface *ifp,
 	struct connected *ifc;
 	struct listnode *node;
 
-	/* ignore broadcast addresses */
-	if (p->prefixlen != IPV4_MAX_PREFIXLEN)
-		d = NULL;
-
 	for (ALL_LIST_ELEMENTS_RO(ifp->connected, node, ifc)) {
 		if (!prefix_same(ifc->address, p))
 			continue;
@@ -142,6 +139,12 @@ static int connected_same(struct connected *ifc1, struct connected *ifc2)
 	if (ifc1->ifp != ifc2->ifp)
 		return 0;
 
+	if (ifc1->flags != ifc2->flags)
+		return 0;
+
+	if (ifc1->conf != ifc2->conf)
+		return 0;
+
 	if (ifc1->destination)
 		if (!ifc2->destination)
 			return 0;
@@ -153,12 +156,6 @@ static int connected_same(struct connected *ifc1, struct connected *ifc2)
 		if (!prefix_same(ifc1->destination, ifc2->destination))
 			return 0;
 
-	if (ifc1->flags != ifc2->flags)
-		return 0;
-
-	if (ifc1->conf != ifc2->conf)
-		return 0;
-
 	return 1;
 }
 
@@ -180,7 +177,7 @@ static void connected_update(struct interface *ifp, struct connected *ifc)
 		 */
 		if (connected_same(current, ifc)) {
 			/* nothing to do */
-			connected_free(ifc);
+			connected_free(&ifc);
 			return;
 		}
 
@@ -202,13 +199,23 @@ static void connected_update(struct interface *ifp, struct connected *ifc)
 void connected_up(struct interface *ifp, struct connected *ifc)
 {
 	afi_t afi;
-	struct prefix p;
+	struct prefix p = {0};
 	struct nexthop nh = {
 		.type = NEXTHOP_TYPE_IFINDEX,
 		.ifindex = ifp->ifindex,
 		.vrf_id = ifp->vrf_id,
 	};
+	struct zebra_vrf *zvrf;
+	uint32_t metric;
 
+	zvrf = zebra_vrf_lookup_by_id(ifp->vrf_id);
+	if (!zvrf) {
+		flog_err(
+			EC_ZEBRA_VRF_NOT_FOUND,
+			"%s: Received Up for interface but no associated zvrf: %d",
+			__func__, ifp->vrf_id);
+		return;
+	}
 	if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL))
 		return;
 
@@ -229,52 +236,45 @@ void connected_up(struct interface *ifp, struct connected *ifc)
 			return;
 		break;
 	case AFI_IP6:
-#ifndef LINUX
+#ifndef GNU_LINUX
 		/* XXX: It is already done by rib_bogus_ipv6 within rib_add */
 		if (IN6_IS_ADDR_UNSPECIFIED(&p.u.prefix6))
 			return;
 #endif
 		break;
 	default:
-		zlog_warn("Received unknown AFI: %s", afi2str(afi));
+		flog_warn(EC_ZEBRA_CONNECTED_AFI_UNKNOWN,
+			  "Received unknown AFI: %s", afi2str(afi));
 		return;
 		break;
 	}
 
-	rib_add(afi, SAFI_UNICAST, ifp->vrf_id, ZEBRA_ROUTE_CONNECT, 0, 0, &p,
-		NULL, &nh, RT_TABLE_MAIN, ifp->metric, 0, 0, 0);
+	metric = (ifc->metric < (uint32_t)METRIC_MAX) ?
+				ifc->metric : ifp->metric;
+	rib_add(afi, SAFI_UNICAST, zvrf->vrf->vrf_id, ZEBRA_ROUTE_CONNECT,
+		0, 0, &p, NULL, &nh, 0, zvrf->table_id, metric, 0, 0, 0);
 
-	rib_add(afi, SAFI_MULTICAST, ifp->vrf_id, ZEBRA_ROUTE_CONNECT, 0, 0, &p,
-		NULL, &nh, RT_TABLE_MAIN, ifp->metric, 0, 0, 0);
-
-	if (IS_ZEBRA_DEBUG_RIB_DETAILED) {
-		char buf[PREFIX_STRLEN];
-
-		zlog_debug(
-			"%u: IF %s address %s add/up, scheduling RIB processing",
-			ifp->vrf_id, ifp->name,
-			prefix2str(&p, buf, sizeof(buf)));
-	}
-	rib_update(ifp->vrf_id, RIB_UPDATE_IF_CHANGE);
+	rib_add(afi, SAFI_MULTICAST, zvrf->vrf->vrf_id, ZEBRA_ROUTE_CONNECT,
+		0, 0, &p, NULL, &nh, 0, zvrf->table_id, metric, 0, 0, 0);
 
 	/* Schedule LSP forwarding entries for processing, if appropriate. */
-	if (ifp->vrf_id == VRF_DEFAULT) {
+	if (zvrf->vrf->vrf_id == VRF_DEFAULT) {
 		if (IS_ZEBRA_DEBUG_MPLS) {
 			char buf[PREFIX_STRLEN];
 
 			zlog_debug(
 				"%u: IF %s IP %s address add/up, scheduling MPLS processing",
-				ifp->vrf_id, ifp->name,
+				zvrf->vrf->vrf_id, ifp->name,
 				prefix2str(&p, buf, sizeof(buf)));
 		}
-		mpls_mark_lsps_for_processing(vrf_info_lookup(ifp->vrf_id));
+		mpls_mark_lsps_for_processing(zvrf, &p);
 	}
 }
 
 /* Add connected IPv4 route to the interface. */
 void connected_add_ipv4(struct interface *ifp, int flags, struct in_addr *addr,
-			uint8_t prefixlen, struct in_addr *broad,
-			const char *label)
+			uint16_t prefixlen, struct in_addr *dest,
+			const char *label, uint32_t metric)
 {
 	struct prefix_ipv4 *p;
 	struct connected *ifc;
@@ -286,6 +286,7 @@ void connected_add_ipv4(struct interface *ifp, int flags, struct in_addr *addr,
 	ifc = connected_new();
 	ifc->ifp = ifp;
 	ifc->flags = flags;
+	ifc->metric = metric;
 	/* If we get a notification from the kernel,
 	 * we can safely assume the address is known to the kernel */
 	SET_FLAG(ifc->conf, ZEBRA_IFC_QUEUED);
@@ -298,57 +299,39 @@ void connected_add_ipv4(struct interface *ifp, int flags, struct in_addr *addr,
 							 : prefixlen;
 	ifc->address = (struct prefix *)p;
 
-	/* If there is broadcast or peer address. */
-	if (broad) {
-		p = prefix_ipv4_new();
-		p->family = AF_INET;
-		p->prefix = *broad;
-		p->prefixlen = prefixlen;
-		ifc->destination = (struct prefix *)p;
-
+	/* If there is a peer address. */
+	if (CONNECTED_PEER(ifc)) {
 		/* validate the destination address */
-		if (CONNECTED_PEER(ifc)) {
-			if (IPV4_ADDR_SAME(addr, broad))
-				zlog_warn(
+		if (dest) {
+			p = prefix_ipv4_new();
+			p->family = AF_INET;
+			p->prefix = *dest;
+			p->prefixlen = prefixlen;
+			ifc->destination = (struct prefix *)p;
+
+			if (IPV4_ADDR_SAME(addr, dest))
+				flog_warn(
+					EC_ZEBRA_IFACE_SAME_LOCAL_AS_PEER,
 					"warning: interface %s has same local and peer "
 					"address %s, routing protocols may malfunction",
 					ifp->name, inet_ntoa(*addr));
 		} else {
-			if (broad->s_addr
-			    != ipv4_broadcast_addr(addr->s_addr, prefixlen)) {
-				char buf[2][INET_ADDRSTRLEN];
-				struct in_addr bcalc;
-				bcalc.s_addr = ipv4_broadcast_addr(addr->s_addr,
-								   prefixlen);
-				zlog_warn(
-					"warning: interface %s broadcast addr %s/%d != "
-					"calculated %s, routing protocols may malfunction",
-					ifp->name,
-					inet_ntop(AF_INET, broad, buf[0],
-						  sizeof(buf[0])),
-					prefixlen,
-					inet_ntop(AF_INET, &bcalc, buf[1],
-						  sizeof(buf[1])));
-			}
-		}
-
-	} else {
-		if (CHECK_FLAG(ifc->flags, ZEBRA_IFA_PEER)) {
-			zlog_warn(
+			zlog_debug(
 				"warning: %s called for interface %s "
 				"with peer flag set, but no peer address supplied",
 				__func__, ifp->name);
 			UNSET_FLAG(ifc->flags, ZEBRA_IFA_PEER);
 		}
-
-		/* no broadcast or destination address was supplied */
-		if ((prefixlen == IPV4_MAX_PREFIXLEN) && if_is_pointopoint(ifp))
-			zlog_warn(
-				"warning: PtP interface %s with addr %s/%d needs a "
-				"peer address",
-				ifp->name, inet_ntoa(*addr), prefixlen);
 	}
 
+	/* no destination address was supplied */
+	if (!dest && (prefixlen == IPV4_MAX_PREFIXLEN)
+		&& if_is_pointopoint(ifp))
+		zlog_debug(
+			"warning: PtP interface %s with addr %s/%d needs a "
+			"peer address",
+			ifp->name, inet_ntoa(*addr), prefixlen);
+
 	/* Label of this address. */
 	if (label)
 		ifc->label = XSTRDUP(MTYPE_CONNECTED_LABEL, label);
@@ -369,6 +352,16 @@ void connected_down(struct interface *ifp, struct connected *ifc)
 		.ifindex = ifp->ifindex,
 		.vrf_id = ifp->vrf_id,
 	};
+	struct zebra_vrf *zvrf;
+
+	zvrf = zebra_vrf_lookup_by_id(ifp->vrf_id);
+	if (!zvrf) {
+		flog_err(
+			EC_ZEBRA_VRF_NOT_FOUND,
+			"%s: Received Up for interface but no associated zvrf: %d",
+			__func__, ifp->vrf_id);
+		return;
+	}
 
 	if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL))
 		return;
@@ -394,7 +387,7 @@ void connected_down(struct interface *ifp, struct connected *ifc)
 			return;
 		break;
 	default:
-		zlog_info("Unknown AFI: %s", afi2str(afi));
+		zlog_warn("Unknown AFI: %s", afi2str(afi));
 		break;
 	}
 
@@ -402,34 +395,23 @@ void connected_down(struct interface *ifp, struct connected *ifc)
 	 * Same logic as for connected_up(): push the changes into the
 	 * head.
 	 */
-	rib_delete(afi, SAFI_UNICAST, ifp->vrf_id, ZEBRA_ROUTE_CONNECT, 0, 0,
-		   &p, NULL, &nh, 0, 0, false, NULL);
+	rib_delete(afi, SAFI_UNICAST, zvrf->vrf->vrf_id, ZEBRA_ROUTE_CONNECT, 0,
+		   0, &p, NULL, &nh, 0, zvrf->table_id, 0, 0, false);
 
-	rib_delete(afi, SAFI_MULTICAST, ifp->vrf_id, ZEBRA_ROUTE_CONNECT, 0, 0,
-		   &p, NULL, &nh, 0, 0, false, NULL);
-
-	if (IS_ZEBRA_DEBUG_RIB_DETAILED) {
-		char buf[PREFIX_STRLEN];
-
-		zlog_debug(
-			"%u: IF %s IP %s address down, scheduling RIB processing",
-			ifp->vrf_id, ifp->name,
-			prefix2str(&p, buf, sizeof(buf)));
-	}
-
-	rib_update(ifp->vrf_id, RIB_UPDATE_IF_CHANGE);
+	rib_delete(afi, SAFI_MULTICAST, zvrf->vrf->vrf_id, ZEBRA_ROUTE_CONNECT,
+		   0, 0, &p, NULL, &nh, 0, zvrf->table_id, 0, 0, false);
 
 	/* Schedule LSP forwarding entries for processing, if appropriate. */
-	if (ifp->vrf_id == VRF_DEFAULT) {
+	if (zvrf->vrf->vrf_id == VRF_DEFAULT) {
 		if (IS_ZEBRA_DEBUG_MPLS) {
 			char buf[PREFIX_STRLEN];
 
 			zlog_debug(
 				"%u: IF %s IP %s address down, scheduling MPLS processing",
-				ifp->vrf_id, ifp->name,
+				zvrf->vrf->vrf_id, ifp->name,
 				prefix2str(&p, buf, sizeof(buf)));
 		}
-		mpls_mark_lsps_for_processing(vrf_info_lookup(ifp->vrf_id));
+		mpls_mark_lsps_for_processing(zvrf, &p);
 	}
 }
 
@@ -443,16 +425,6 @@ static void connected_delete_helper(struct connected *ifc, struct prefix *p)
 
 	connected_withdraw(ifc);
 
-	if (IS_ZEBRA_DEBUG_RIB_DETAILED) {
-		char buf[PREFIX_STRLEN];
-
-		zlog_debug(
-			"%u: IF %s IP %s address del, scheduling RIB processing",
-			ifp->vrf_id, ifp->name,
-			prefix2str(p, buf, sizeof(buf)));
-	}
-	rib_update(ifp->vrf_id, RIB_UPDATE_IF_CHANGE);
-
 	/* Schedule LSP forwarding entries for processing, if appropriate. */
 	if (ifp->vrf_id == VRF_DEFAULT) {
 		if (IS_ZEBRA_DEBUG_MPLS) {
@@ -463,14 +435,14 @@ static void connected_delete_helper(struct connected *ifc, struct prefix *p)
 				ifp->vrf_id, ifp->name,
 				prefix2str(p, buf, sizeof(buf)));
 		}
-		mpls_mark_lsps_for_processing(vrf_info_lookup(ifp->vrf_id));
+		mpls_mark_lsps_for_processing(vrf_info_lookup(ifp->vrf_id), p);
 	}
 }
 
 /* Delete connected IPv4 route to the interface. */
 void connected_delete_ipv4(struct interface *ifp, int flags,
-			   struct in_addr *addr, uint8_t prefixlen,
-			   struct in_addr *broad)
+			   struct in_addr *addr, uint16_t prefixlen,
+			   struct in_addr *dest)
 {
 	struct prefix p, d;
 	struct connected *ifc;
@@ -481,10 +453,10 @@ void connected_delete_ipv4(struct interface *ifp, int flags,
 	p.prefixlen = CHECK_FLAG(flags, ZEBRA_IFA_PEER) ? IPV4_MAX_PREFIXLEN
 							: prefixlen;
 
-	if (broad) {
+	if (dest) {
 		memset(&d, 0, sizeof(struct prefix));
 		d.family = AF_INET;
-		d.u.prefix4 = *broad;
+		d.u.prefix4 = *dest;
 		d.prefixlen = prefixlen;
 		ifc = connected_check_ptp(ifp, &p, &d);
 	} else
@@ -495,8 +467,8 @@ void connected_delete_ipv4(struct interface *ifp, int flags,
 
 /* Add connected IPv6 route to the interface. */
 void connected_add_ipv6(struct interface *ifp, int flags, struct in6_addr *addr,
-			struct in6_addr *broad, uint8_t prefixlen,
-			const char *label)
+			struct in6_addr *dest, uint16_t prefixlen,
+			const char *label, uint32_t metric)
 {
 	struct prefix_ipv6 *p;
 	struct connected *ifc;
@@ -508,6 +480,7 @@ void connected_add_ipv6(struct interface *ifp, int flags, struct in6_addr *addr,
 	ifc = connected_new();
 	ifc->ifp = ifp;
 	ifc->flags = flags;
+	ifc->metric = metric;
 	/* If we get a notification from the kernel,
 	 * we can safely assume the address is known to the kernel */
 	SET_FLAG(ifc->conf, ZEBRA_IFC_QUEUED);
@@ -519,16 +492,21 @@ void connected_add_ipv6(struct interface *ifp, int flags, struct in6_addr *addr,
 	p->prefixlen = prefixlen;
 	ifc->address = (struct prefix *)p;
 
-	if (broad) {
+	/* Add global ipv6 address to the RA prefix list */
+	if (!IN6_IS_ADDR_LINKLOCAL(&p->prefix))
+		rtadv_add_prefix(ifp->info, p);
+
+	if (dest) {
 		p = prefix_ipv6_new();
 		p->family = AF_INET6;
-		IPV6_ADDR_COPY(&p->prefix, broad);
+		IPV6_ADDR_COPY(&p->prefix, dest);
 		p->prefixlen = prefixlen;
 		ifc->destination = (struct prefix *)p;
 	} else {
 		if (CHECK_FLAG(ifc->flags, ZEBRA_IFA_PEER)) {
-			zlog_warn("warning: %s called for interface %s with peer flag set, but no peer address supplied",
-				  __func__, ifp->name);
+			zlog_debug(
+				"warning: %s called for interface %s with peer flag set, but no peer address supplied",
+				__func__, ifp->name);
 			UNSET_FLAG(ifc->flags, ZEBRA_IFA_PEER);
 		}
 	}
@@ -551,7 +529,7 @@ void connected_add_ipv6(struct interface *ifp, int flags, struct in6_addr *addr,
 }
 
 void connected_delete_ipv6(struct interface *ifp, struct in6_addr *address,
-			   struct in6_addr *broad, uint8_t prefixlen)
+			   struct in6_addr *dest, uint16_t prefixlen)
 {
 	struct prefix p, d;
 	struct connected *ifc;
@@ -561,10 +539,14 @@ void connected_delete_ipv6(struct interface *ifp, struct in6_addr *address,
 	memcpy(&p.u.prefix6, address, sizeof(struct in6_addr));
 	p.prefixlen = prefixlen;
 
-	if (broad) {
+	/* Delete global ipv6 address from RA prefix list */
+	if (!IN6_IS_ADDR_LINKLOCAL(&p.u.prefix6))
+		rtadv_delete_prefix(ifp->info, &p);
+
+	if (dest) {
 		memset(&d, 0, sizeof(struct prefix));
 		d.family = AF_INET6;
-		IPV6_ADDR_COPY(&d.u.prefix, broad);
+		IPV6_ADDR_COPY(&d.u.prefix6, dest);
 		d.prefixlen = prefixlen;
 		ifc = connected_check_ptp(ifp, &p, &d);
 	} else
