@@ -27,17 +27,17 @@
 #include "filter.h"
 #include "memory.h"
 #include "zebra_memory.h"
-#include "memory_vty.h"
 #include "prefix.h"
 #include "log.h"
 #include "plist.h"
 #include "privs.h"
 #include "sigevent.h"
 #include "vrf.h"
-#include "logicalrouter.h"
 #include "libfrr.h"
 #include "routemap.h"
 
+#include "zebra/zebra_router.h"
+#include "zebra/zebra_errors.h"
 #include "zebra/rib.h"
 #include "zebra/zserv.h"
 #include "zebra/debug.h"
@@ -51,15 +51,17 @@
 #include "zebra/label_manager.h"
 #include "zebra/zebra_netns_notify.h"
 #include "zebra/zebra_rnh.h"
+#include "zebra/zebra_pbr.h"
+#include "zebra/zebra_vxlan.h"
+#include "zebra/zebra_routemap.h"
+#include "zebra/zebra_nb.h"
+
+#if defined(HANDLE_NETLINK_FUZZING)
+#include "zebra/kernel_netlink.h"
+#endif /* HANDLE_NETLINK_FUZZING */
 
 #define ZEBRA_PTM_SUPPORT
 
-/* Zebra instance */
-struct zebra_t zebrad = {
-	.rtm_table_default = 0,
-	.packets_to_process = ZEBRA_ZAPI_PACKETS_TO_PROCESS,
-};
-
 /* process id. */
 pid_t pid;
 
@@ -72,27 +74,32 @@ int retain_mode = 0;
 /* Allow non-quagga entities to delete quagga routes */
 int allow_delete = 0;
 
-/* Don't delete kernel route. */
-int keep_kernel_mode = 0;
+int graceful_restart;
+
+bool v6_rr_semantics = false;
 
 #ifdef HAVE_NETLINK
 /* Receive buffer size for netlink socket */
 uint32_t nl_rcvbufsize = 4194304;
 #endif /* HAVE_NETLINK */
 
+#define OPTION_V6_RR_SEMANTICS 2000
 /* Command line options. */
-struct option longopts[] = {{"batch", no_argument, NULL, 'b'},
-			    {"allow_delete", no_argument, NULL, 'a'},
-			    {"keep_kernel", no_argument, NULL, 'k'},
-			    {"socket", required_argument, NULL, 'z'},
-			    {"ecmp", required_argument, NULL, 'e'},
-			    {"label_socket", no_argument, NULL, 'l'},
-			    {"retain", no_argument, NULL, 'r'},
+const struct option longopts[] = {
+	{"batch", no_argument, NULL, 'b'},
+	{"allow_delete", no_argument, NULL, 'a'},
+	{"keep_kernel", no_argument, NULL, 'k'},
+	{"socket", required_argument, NULL, 'z'},
+	{"ecmp", required_argument, NULL, 'e'},
+	{"retain", no_argument, NULL, 'r'},
+	{"vrfdefaultname", required_argument, NULL, 'o'},
+	{"graceful_restart", required_argument, NULL, 'K'},
 #ifdef HAVE_NETLINK
-			    {"vrfwnetns", no_argument, NULL, 'n'},
-			    {"nl-bufsize", required_argument, NULL, 's'},
+	{"vrfwnetns", no_argument, NULL, 'n'},
+	{"nl-bufsize", required_argument, NULL, 's'},
+	{"v6-rr-semantics", no_argument, NULL, OPTION_V6_RR_SEMANTICS},
 #endif /* HAVE_NETLINK */
-			    {0}};
+	{0}};
 
 zebra_capabilities_t _caps_p[] = {
 	ZCAP_NET_ADMIN, ZCAP_SYS_ADMIN, ZCAP_NET_RAW,
@@ -111,8 +118,6 @@ struct zebra_privs_t zserv_privs = {
 	.cap_num_p = array_size(_caps_p),
 	.cap_num_i = 0};
 
-unsigned int multipath_num = MULTIPATH_NUM;
-
 /* SIGHUP handler. */
 static void sighup(void)
 {
@@ -127,12 +132,37 @@ static void sigint(void)
 {
 	struct vrf *vrf;
 	struct zebra_vrf *zvrf;
+	struct listnode *ln, *nn;
+	struct zserv *client;
+	static bool sigint_done;
+
+	if (sigint_done)
+		return;
+
+	sigint_done = true;
 
 	zlog_notice("Terminating on signal");
 
+	atomic_store_explicit(&zrouter.in_shutdown, true,
+			      memory_order_relaxed);
+
+	/* send RA lifetime of 0 before stopping. rfc4861/6.2.5 */
+	rtadv_stop_ra_all();
+
 	frr_early_fini();
 
-	list_delete_all_node(zebrad.client_list);
+	zebra_dplane_pre_finish();
+
+	/* Clean up GR related info. */
+	zebra_gr_stale_client_cleanup(zrouter.stale_client_list);
+	list_delete_all_node(zrouter.stale_client_list);
+
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, ln, nn, client))
+		zserv_close_client(client);
+
+	zserv_close();
+	list_delete_all_node(zrouter.client_list);
+
 	zebra_ptm_finish();
 
 	if (retain_mode)
@@ -141,20 +171,49 @@ static void sigint(void)
 			if (zvrf)
 				SET_FLAG(zvrf->flags, ZEBRA_VRF_RETAIN);
 		}
-	if (zebrad.lsp_process_q)
-		work_queue_free_and_null(&zebrad.lsp_process_q);
-	vrf_terminate();
+	if (zrouter.lsp_process_q)
+		work_queue_free_and_null(&zrouter.lsp_process_q);
 
-	ns_walk_func(zebra_ns_disabled);
+	vrf_terminate();
+	rtadv_terminate();
+
+	ns_walk_func(zebra_ns_early_shutdown);
 	zebra_ns_notify_close();
 
 	access_list_reset();
 	prefix_list_reset();
-	route_map_finish();
+	/*
+	 * zebra_routemap_finish will
+	 * 1 set rmap upd timer to 0 so that rmap update wont be scheduled again
+	 * 2 Put off the rmap update thread
+	 * 3 route_map_finish
+	 */
+	zebra_routemap_finish();
 
-	list_delete_and_null(&zebrad.client_list);
-	work_queue_free_and_null(&zebrad.ribq);
-	meta_queue_free(zebrad.mq);
+	list_delete(&zrouter.client_list);
+
+	/* Indicate that all new dplane work has been enqueued. When that
+	 * work is complete, the dataplane will enqueue an event
+	 * with the 'finalize' function.
+	 */
+	zebra_dplane_finish();
+}
+
+/*
+ * Final shutdown step for the zebra main thread. This is run after all
+ * async update processing has completed.
+ */
+int zebra_finalize(struct thread *dummy)
+{
+	zlog_info("Zebra final shutdown");
+
+	/* Final shutdown of ns resources */
+	ns_walk_func(zebra_ns_final_shutdown);
+
+	/* Stop dplane thread and finish any cleanup */
+	zebra_dplane_shutdown();
+
+	zebra_router_terminate();
 
 	frr_fini();
 	exit(0);
@@ -185,6 +244,13 @@ struct quagga_signal_t zebra_signals[] = {
 	},
 };
 
+static const struct frr_yang_module_info *const zebra_yang_modules[] = {
+	&frr_interface_info,
+	&frr_route_map_info,
+	&frr_zebra_info,
+	&frr_vrf_info,
+};
+
 FRR_DAEMON_INFO(
 	zebra, ZEBRA, .vty_port = ZEBRA_VTY_PORT, .flags = FRR_NO_ZCLIENT,
 
@@ -194,51 +260,63 @@ FRR_DAEMON_INFO(
 
 	.signals = zebra_signals, .n_signals = array_size(zebra_signals),
 
-	.privs = &zserv_privs, )
+	.privs = &zserv_privs,
+
+	.yang_modules = zebra_yang_modules,
+	.n_yang_modules = array_size(zebra_yang_modules), )
 
 /* Main startup routine. */
 int main(int argc, char **argv)
 {
 	// int batch_mode = 0;
 	char *zserv_path = NULL;
-	/* Socket to external label manager */
-	char *lblmgr_path = NULL;
+	char *vrf_default_name_configured = NULL;
 	struct sockaddr_storage dummy;
 	socklen_t dummylen;
 #if defined(HANDLE_ZAPI_FUZZING)
-	char *fuzzing = NULL;
-#endif
+	char *zapi_fuzzing = NULL;
+#endif /* HANDLE_ZAPI_FUZZING */
+#if defined(HANDLE_NETLINK_FUZZING)
+	char *netlink_fuzzing = NULL;
+#endif /* HANDLE_NETLINK_FUZZING */
 
+	graceful_restart = 0;
 	vrf_configure_backend(VRF_BACKEND_VRF_LITE);
-	logicalrouter_configure_backend(LOGICALROUTER_BACKEND_NETNS);
 
 	frr_preinit(&zebra_di, argc, argv);
 
 	frr_opt_add(
-		"bakz:e:l:r"
+		"baz:e:o:rK:"
 #ifdef HAVE_NETLINK
 		"s:n"
 #endif
 #if defined(HANDLE_ZAPI_FUZZING)
 		"c:"
-#endif
+#endif /* HANDLE_ZAPI_FUZZING */
+#if defined(HANDLE_NETLINK_FUZZING)
+		"w:"
+#endif /* HANDLE_NETLINK_FUZZING */
 		,
 		longopts,
-		"  -b, --batch        Runs in batch mode\n"
-		"  -a, --allow_delete Allow other processes to delete zebra routes\n"
-		"  -z, --socket       Set path of zebra socket\n"
-		"  -e, --ecmp         Specify ECMP to use.\n"
-		"  -l, --label_socket Socket to external label manager\n"
-		"  -k, --keep_kernel  Don't delete old routes which installed by zebra.\n"
-		"  -r, --retain       When program terminates, retain added route by zebra.\n"
+		"  -b, --batch              Runs in batch mode\n"
+		"  -a, --allow_delete       Allow other processes to delete zebra routes\n"
+		"  -z, --socket             Set path of zebra socket\n"
+		"  -e, --ecmp               Specify ECMP to use.\n"
+		"  -r, --retain             When program terminates, retain added route by zebra.\n"
+		"  -o, --vrfdefaultname     Set default VRF name.\n"
+		"  -K, --graceful_restart   Graceful restart at the kernel level, timer in seconds for expiration\n"
 #ifdef HAVE_NETLINK
-		"  -n, --vrfwnetns    Set VRF with NetNS\n"
-		"  -s, --nl-bufsize   Set netlink receive buffer size\n"
+		"  -n, --vrfwnetns          Use NetNS as VRF backend\n"
+		"  -s, --nl-bufsize         Set netlink receive buffer size\n"
+		"      --v6-rr-semantics    Use v6 RR semantics\n"
 #endif /* HAVE_NETLINK */
 #if defined(HANDLE_ZAPI_FUZZING)
-		"  -c <file>          Bypass normal startup use this file for tetsting of zapi"
-#endif
-		);
+		"  -c <file>                Bypass normal startup and use this file for testing of zapi\n"
+#endif /* HANDLE_ZAPI_FUZZING */
+#if defined(HANDLE_NETLINK_FUZZING)
+		"  -w <file>                Bypass normal startup and use this file for testing of netlink input\n"
+#endif /* HANDLE_NETLINK_FUZZING */
+	);
 
 	while (1) {
 		int opt = frr_getopt(argc, argv, NULL);
@@ -255,18 +333,23 @@ int main(int argc, char **argv)
 		case 'a':
 			allow_delete = 1;
 			break;
-		case 'k':
-			keep_kernel_mode = 1;
-			break;
-		case 'e':
-			multipath_num = atoi(optarg);
-			if (multipath_num > MULTIPATH_NUM
-			    || multipath_num <= 0) {
-				zlog_err(
-					"Multipath Number specified must be less than %d and greater than 0",
+		case 'e': {
+			unsigned long int parsed_multipath =
+				strtoul(optarg, NULL, 10);
+			if (parsed_multipath == 0
+			    || parsed_multipath > MULTIPATH_NUM
+			    || parsed_multipath > UINT32_MAX) {
+				flog_err(
+					EC_ZEBRA_BAD_MULTIPATH_NUM,
+					"Multipath Number specified must be less than %u and greater than 0",
 					MULTIPATH_NUM);
 				return 1;
 			}
+			zrouter.multipath_num = parsed_multipath;
+			break;
+		}
+		case 'o':
+			vrf_default_name_configured = optarg;
 			break;
 		case 'z':
 			zserv_path = optarg;
@@ -277,37 +360,48 @@ int main(int argc, char **argv)
 				exit(1);
 			}
 			break;
-		case 'l':
-			lblmgr_path = optarg;
-			break;
 		case 'r':
 			retain_mode = 1;
 			break;
+		case 'K':
+			graceful_restart = atoi(optarg);
+			break;
 #ifdef HAVE_NETLINK
 		case 's':
 			nl_rcvbufsize = atoi(optarg);
 			break;
 		case 'n':
 			vrf_configure_backend(VRF_BACKEND_NETNS);
-			logicalrouter_configure_backend(
-				LOGICALROUTER_BACKEND_OFF);
+			break;
+		case OPTION_V6_RR_SEMANTICS:
+			v6_rr_semantics = true;
 			break;
 #endif /* HAVE_NETLINK */
 #if defined(HANDLE_ZAPI_FUZZING)
 		case 'c':
-			fuzzing = optarg;
+			zapi_fuzzing = optarg;
 			break;
-#endif
+#endif /* HANDLE_ZAPI_FUZZING */
+#if defined(HANDLE_NETLINK_FUZZING)
+		case 'w':
+			netlink_fuzzing = optarg;
+			/* This ensures we are aren't writing any of the
+			 * startup netlink messages that happen when we
+			 * just want to read.
+			 */
+			netlink_read = true;
+			break;
+#endif /* HANDLE_NETLINK_FUZZING */
 		default:
 			frr_help_exit(1);
 			break;
 		}
 	}
 
-	vty_config_lockless();
-	zebrad.master = frr_init();
+	zrouter.master = frr_init();
 
 	/* Zebra related initialize. */
+	zebra_router_init();
 	zserv_init();
 	rib_init();
 	zebra_if_init();
@@ -317,8 +411,7 @@ int main(int argc, char **argv)
 	/*
 	 * Initialize NS( and implicitly the VRF module), and make kernel
 	 * routing socket. */
-	zebra_ns_init();
-
+	zebra_ns_init((const char *)vrf_default_name_configured);
 	zebra_vty_init();
 	access_list_init();
 	prefix_list_init();
@@ -333,17 +426,11 @@ int main(int argc, char **argv)
 	zebra_mpls_init();
 	zebra_mpls_vty_init();
 	zebra_pw_vty_init();
+	zebra_pbr_init();
 
 /* For debug purpose. */
 /* SET_FLAG (zebra_debug_event, ZEBRA_DEBUG_EVENT); */
 
-#if defined(HANDLE_ZAPI_FUZZING)
-	if (fuzzing) {
-		zserv_read_file(fuzzing);
-		exit(0);
-	}
-#endif
-
 	/* Process the configuration file. Among other configuration
 	*  directives we can meet those installing static routes. Such
 	*  requests will not be executed immediately, but queued in
@@ -361,22 +448,46 @@ int main(int argc, char **argv)
 	*  will be equal to the current getpid(). To know about such routes,
 	* we have to have route_read() called before.
 	*/
-	if (!keep_kernel_mode)
-		rib_sweep_route();
+	zrouter.startup_time = monotime(NULL);
+	thread_add_timer(zrouter.master, rib_sweep_route,
+			 NULL, graceful_restart, NULL);
 
 	/* Needed for BSD routing socket. */
 	pid = getpid();
 
-	/* This must be done only after locking pidfile (bug #403). */
-	zebra_zserv_socket_init(zserv_path);
+	/* Start dataplane system */
+	zebra_dplane_start();
+
+	/* Start Zebra API server */
+	zserv_start(zserv_path);
 
 	/* Init label manager */
-	label_manager_init(lblmgr_path);
+	label_manager_init();
 
 	/* RNH init */
 	zebra_rnh_init();
 
-	frr_run(zebrad.master);
+	/* Config handler Init */
+	zebra_evpn_init();
+
+	/* Error init */
+	zebra_error_init();
+
+#if defined(HANDLE_ZAPI_FUZZING)
+	if (zapi_fuzzing) {
+		zserv_read_file(zapi_fuzzing);
+		exit(0);
+	}
+#endif /* HANDLE_ZAPI_FUZZING */
+#if defined(HANDLE_NETLINK_FUZZING)
+	if (netlink_fuzzing) {
+		netlink_read_init(netlink_fuzzing);
+		exit(0);
+	}
+#endif /* HANDLE_NETLINK_FUZZING */
+
+
+	frr_run(zrouter.master);
 
 	/* Not reached... */
 	return 0;
