@@ -33,7 +33,7 @@
 #include "srcdest_table.h"
 
 #include "zebra/rib.h"
-#include "zebra/zserv.h"
+#include "zebra/zebra_router.h"
 #include "zebra/zebra_ns.h"
 #include "zebra/zebra_vrf.h"
 #include "zebra/zebra_routemap.h"
@@ -43,6 +43,7 @@
 #include "zebra/zapi_msg.h"
 #include "zebra/zebra_memory.h"
 #include "zebra/zebra_vxlan.h"
+#include "zebra/zebra_errors.h"
 
 #define ZEBRA_PTM_SUPPORT
 
@@ -51,7 +52,7 @@
 static int zebra_import_table_used[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
 static uint32_t zebra_import_table_distance[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
 
-int is_zebra_import_table_enabled(afi_t afi, uint32_t table_id)
+int is_zebra_import_table_enabled(afi_t afi, vrf_id_t vrf_id, uint32_t table_id)
 {
 	/*
 	 * Make sure that what we are called with actualy makes sense
@@ -74,6 +75,10 @@ static void zebra_redistribute_default(struct zserv *client, vrf_id_t vrf_id)
 	struct route_entry *newre;
 
 	for (afi = AFI_IP; afi <= AFI_IP6; afi++) {
+
+		if (!vrf_bitmap_check(client->redist_default[afi], vrf_id))
+			continue;
+
 		/* Lookup table.  */
 		table = zebra_vrf_table(afi, SAFI_UNICAST, vrf_id);
 		if (!table)
@@ -113,12 +118,12 @@ static void zebra_redistribute(struct zserv *client, int type,
 
 	for (rn = route_top(table); rn; rn = srcdest_route_next(rn))
 		RNODE_FOREACH_RE (rn, newre) {
-			struct prefix *dst_p, *src_p;
+			const struct prefix *dst_p, *src_p;
 			char buf[PREFIX_STRLEN];
 
 			srcdest_rnode_prefixes(rn, &dst_p, &src_p);
 
-			if (IS_ZEBRA_DEBUG_EVENT)
+			if (IS_ZEBRA_DEBUG_RIB)
 				zlog_debug(
 					"%s: client %s %s(%u) checking: selected=%d, type=%d, distance=%d, metric=%d zebra_check_addr=%d",
 					__func__,
@@ -145,52 +150,79 @@ static void zebra_redistribute(struct zserv *client, int type,
 		}
 }
 
+/*
+ * Function to check if prefix is candidate for
+ * redistribute.
+ */
+static bool zebra_redistribute_check(const struct route_entry *re,
+				     struct zserv *client,
+				     const struct prefix *p, int afi)
+{
+	/* Process only if there is valid re */
+	if (!re)
+		return false;
+
+	/* If default route and redistributed */
+	if (is_default_prefix(p)
+	    && vrf_bitmap_check(client->redist_default[afi], re->vrf_id))
+		return true;
+
+	/* If redistribute in enabled for zebra route all */
+	if (vrf_bitmap_check(client->redist[afi][ZEBRA_ROUTE_ALL], re->vrf_id))
+		return true;
+
+	/*
+	 * If multi-instance then check for route
+	 * redistribution for given instance.
+	 */
+	if (re->instance
+	    && redist_check_instance(&client->mi_redist[afi][re->type],
+				     re->instance))
+		return true;
+
+	/* If redistribution is enabled for give route type. */
+	if (vrf_bitmap_check(client->redist[afi][re->type], re->vrf_id))
+		return true;
+
+	return false;
+}
+
 /* Either advertise a route for redistribution to registered clients or */
 /* withdraw redistribution if add cannot be done for client */
-void redistribute_update(struct prefix *p, struct prefix *src_p,
-			 struct route_entry *re, struct route_entry *prev_re)
+void redistribute_update(const struct prefix *p, const struct prefix *src_p,
+			 const struct route_entry *re,
+			 const struct route_entry *prev_re)
 {
 	struct listnode *node, *nnode;
 	struct zserv *client;
-	int send_redistribute;
 	int afi;
 	char buf[PREFIX_STRLEN];
 
 	if (IS_ZEBRA_DEBUG_RIB) {
 		zlog_debug(
-			"%u:%s: Redist update re %p (type %d), old %p (type %d)",
+			"%u:%s: Redist update re %p (%s), old %p (%s)",
 			re->vrf_id, prefix2str(p, buf, sizeof(buf)),
-			re, re->type, prev_re,
-			prev_re ? prev_re->type : -1);
+			re, zebra_route_string(re->type), prev_re,
+			prev_re ? zebra_route_string(prev_re->type) : "None");
 	}
 
 	afi = family2afi(p->family);
 	if (!afi) {
-		zlog_warn("%s: Unknown AFI/SAFI prefix received\n",
-			  __FUNCTION__);
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Unknown AFI/SAFI prefix received\n", __func__);
+		return;
+	}
+	if (!zebra_check_addr(p)) {
+		if (IS_ZEBRA_DEBUG_RIB)
+			zlog_debug("Redist update filter prefix %s",
+				   prefix2str(p, buf, sizeof(buf)));
 		return;
 	}
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
-		send_redistribute = 0;
 
-		if (is_default_prefix(p)
-		    && vrf_bitmap_check(client->redist_default, re->vrf_id))
-			send_redistribute = 1;
-		else if (vrf_bitmap_check(client->redist[afi][ZEBRA_ROUTE_ALL],
-					  re->vrf_id))
-			send_redistribute = 1;
-		else if (re->instance
-			 && redist_check_instance(
-				    &client->mi_redist[afi][re->type],
-				    re->instance))
-			send_redistribute = 1;
-		else if (vrf_bitmap_check(client->redist[afi][re->type],
-					  re->vrf_id))
-			send_redistribute = 1;
-
-		if (send_redistribute) {
-			if (IS_ZEBRA_DEBUG_EVENT) {
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		if (zebra_redistribute_check(re, client, p, afi)) {
+			if (IS_ZEBRA_DEBUG_RIB) {
 				zlog_debug(
 					   "%s: client %s %s(%u), type=%d, distance=%d, metric=%d",
 					   __func__,
@@ -201,63 +233,91 @@ void redistribute_update(struct prefix *p, struct prefix *src_p,
 			}
 			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_ADD,
 						 client, p, src_p, re);
-		} else if (prev_re
-			   && ((re->instance
-				&& redist_check_instance(
-					   &client->mi_redist[afi]
-							     [prev_re->type],
-					   re->instance))
-			       || vrf_bitmap_check(
-					  client->redist[afi][prev_re->type],
-					  re->vrf_id))) {
+		} else if (zebra_redistribute_check(prev_re, client, p, afi))
 			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_DEL,
 						 client, p, src_p, prev_re);
-		}
 	}
 }
 
-void redistribute_delete(struct prefix *p, struct prefix *src_p,
-			 struct route_entry *re)
+/*
+ * During a route delete, where 'new_re' is NULL, redist a delete to all
+ * clients registered for the type of 'old_re'.
+ * During a route update, redist a delete to any clients who will not see
+ * an update when the new route is installed. There are cases when a client
+ * may have seen a redist for 'old_re', but will not see
+ * the redist for 'new_re'.
+ */
+void redistribute_delete(const struct prefix *p, const struct prefix *src_p,
+			 const struct route_entry *old_re,
+			 const struct route_entry *new_re)
 {
 	struct listnode *node, *nnode;
 	struct zserv *client;
-	char buf[INET6_ADDRSTRLEN];
 	int afi;
+	char buf[PREFIX_STRLEN];
+	vrf_id_t vrfid;
+
+	if (old_re)
+		vrfid = old_re->vrf_id;
+	else if (new_re)
+		vrfid = new_re->vrf_id;
+	else
+		return;
 
 	if (IS_ZEBRA_DEBUG_RIB) {
-		inet_ntop(p->family, &p->u.prefix, buf, INET6_ADDRSTRLEN);
-		zlog_debug("%u:%s/%d: Redist delete re %p (type %d)",
-			   re->vrf_id, buf, p->prefixlen, re, re->type);
+		zlog_debug(
+			"%u:%s: Redist del: re %p (%s), new re %p (%s)",
+			vrfid, prefix2str(p, buf, sizeof(buf)),
+			old_re,
+			old_re ? zebra_route_string(old_re->type) : "None",
+			new_re,
+			new_re ? zebra_route_string(new_re->type) : "None");
 	}
 
 	/* Add DISTANCE_INFINITY check. */
-	if (re->distance == DISTANCE_INFINITY)
+	if (old_re && (old_re->distance == DISTANCE_INFINITY)) {
+		if (IS_ZEBRA_DEBUG_RIB)
+			zlog_debug("        Skipping due to Infinite Distance");
 		return;
+	}
 
 	afi = family2afi(p->family);
 	if (!afi) {
-		zlog_warn("%s: Unknown AFI/SAFI prefix received\n",
-			  __FUNCTION__);
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Unknown AFI/SAFI prefix received\n",
+			  __func__);
 		return;
 	}
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
-		if ((is_default_prefix(p)
-		     && vrf_bitmap_check(client->redist_default, re->vrf_id))
-		    || vrf_bitmap_check(client->redist[afi][ZEBRA_ROUTE_ALL],
-					re->vrf_id)
-		    || (re->instance
-			&& redist_check_instance(
-				   &client->mi_redist[afi][re->type],
-				   re->instance))
-		    || vrf_bitmap_check(client->redist[afi][re->type],
-					re->vrf_id)) {
-			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_DEL,
-						 client, p, src_p, re);
+	/* Skip invalid (e.g. linklocal) prefix */
+	if (!zebra_check_addr(p)) {
+		if (IS_ZEBRA_DEBUG_RIB) {
+			zlog_debug(
+				"%u:%s: Redist del old: skipping invalid prefix",
+				vrfid, prefix2str(p, buf, sizeof(buf)));
 		}
+		return;
+	}
+
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+		/*
+		 * Skip this client if it will receive an update for the
+		 * 'new' re
+		 */
+		if (zebra_redistribute_check(new_re, client, p, afi))
+			continue;
+
+		/* Send a delete for the 'old' re to any subscribed client. */
+		if (zebra_redistribute_check(old_re, client, p, afi))
+			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_DEL,
+						 client, p, src_p, old_re);
 	}
 }
 
+
 void zebra_redistribute_add(ZAPI_HANDLER_ARGS)
 {
 	afi_t afi = 0;
@@ -270,19 +330,20 @@ void zebra_redistribute_add(ZAPI_HANDLER_ARGS)
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug(
-			"%s: client proto %s afi=%d, wants %s, vrf %u, instance=%d",
+			"%s: client proto %s afi=%d, wants %s, vrf %s(%u), instance=%d",
 			__func__, zebra_route_string(client->proto), afi,
-			zebra_route_string(type), zvrf_id(zvrf), instance);
+			zebra_route_string(type), VRF_LOGNAME(zvrf->vrf),
+			zvrf_id(zvrf), instance);
 
-	if (afi == 0 || afi > AFI_MAX) {
-		zlog_warn("%s: Specified afi %d does not exist",
-			  __PRETTY_FUNCTION__, afi);
+	if (afi == 0 || afi >= AFI_MAX) {
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Specified afi %d does not exist", __func__, afi);
 		return;
 	}
 
 	if (type == 0 || type >= ZEBRA_ROUTE_MAX) {
-		zlog_warn("%s: Specified Route Type %d does not exist",
-			  __PRETTY_FUNCTION__, type);
+		zlog_debug("%s: Specified Route Type %d does not exist",
+			   __func__, type);
 		return;
 	}
 
@@ -298,8 +359,10 @@ void zebra_redistribute_add(ZAPI_HANDLER_ARGS)
 		if (!vrf_bitmap_check(client->redist[afi][type],
 				      zvrf_id(zvrf))) {
 			if (IS_ZEBRA_DEBUG_EVENT)
-				zlog_debug("%s: setting vrf %u redist bitmap",
-					   __func__, zvrf_id(zvrf));
+				zlog_debug(
+					"%s: setting vrf %s(%u) redist bitmap",
+					__func__, VRF_LOGNAME(zvrf->vrf),
+					zvrf_id(zvrf));
 			vrf_bitmap_set(client->redist[afi][type],
 				       zvrf_id(zvrf));
 			zebra_redistribute(client, type, 0, zvrf_id(zvrf), afi);
@@ -320,15 +383,15 @@ void zebra_redistribute_delete(ZAPI_HANDLER_ARGS)
 	STREAM_GETC(msg, type);
 	STREAM_GETW(msg, instance);
 
-	if (afi == 0 || afi > AFI_MAX) {
-		zlog_warn("%s: Specified afi %d does not exist",
-			  __PRETTY_FUNCTION__, afi);
+	if (afi == 0 || afi >= AFI_MAX) {
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Specified afi %d does not exist", __func__, afi);
 		return;
 	}
 
 	if (type == 0 || type >= ZEBRA_ROUTE_MAX) {
-		zlog_warn("%s: Specified Route Type %d does not exist",
-			  __PRETTY_FUNCTION__, type);
+		zlog_debug("%s: Specified Route Type %d does not exist",
+			   __func__, type);
 		return;
 	}
 
@@ -349,13 +412,39 @@ stream_failure:
 
 void zebra_redistribute_default_add(ZAPI_HANDLER_ARGS)
 {
-	vrf_bitmap_set(client->redist_default, zvrf_id(zvrf));
+	afi_t afi = 0;
+
+	STREAM_GETC(msg, afi);
+
+	if (afi == 0 || afi >= AFI_MAX) {
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Specified afi %u does not exist", __func__, afi);
+		return;
+	}
+
+	vrf_bitmap_set(client->redist_default[afi], zvrf_id(zvrf));
 	zebra_redistribute_default(client, zvrf_id(zvrf));
+
+stream_failure:
+	return;
 }
 
 void zebra_redistribute_default_delete(ZAPI_HANDLER_ARGS)
 {
-	vrf_bitmap_unset(client->redist_default, zvrf_id(zvrf));
+	afi_t afi = 0;
+
+	STREAM_GETC(msg, afi);
+
+	if (afi == 0 || afi >= AFI_MAX) {
+		flog_warn(EC_ZEBRA_REDISTRIBUTE_UNKNOWN_AF,
+			  "%s: Specified afi %u does not exist", __func__, afi);
+		return;
+	}
+
+	vrf_bitmap_unset(client->redist_default[afi], zvrf_id(zvrf));
+
+stream_failure:
+	return;
 }
 
 /* Interface up information. */
@@ -369,12 +458,18 @@ void zebra_interface_up_update(struct interface *ifp)
 			   ifp->name, ifp->vrf_id);
 
 	if (ifp->ptm_status || !ifp->ptm_enable) {
-		for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
-			if (client->ifinfo) {
-				zsend_interface_update(ZEBRA_INTERFACE_UP,
-						       client, ifp);
-				zsend_interface_link_params(client, ifp);
-			}
+		for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode,
+				       client)) {
+			/* Do not send unsolicited messages to synchronous
+			 * clients.
+			 */
+			if (client->synchronous)
+				continue;
+
+			zsend_interface_update(ZEBRA_INTERFACE_UP,
+					       client, ifp);
+			zsend_interface_link_params(client, ifp);
+		}
 	}
 }
 
@@ -388,7 +483,11 @@ void zebra_interface_down_update(struct interface *ifp)
 		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DOWN %s(%u)",
 			   ifp->name, ifp->vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		zsend_interface_update(ZEBRA_INTERFACE_DOWN, client, ifp);
 	}
 }
@@ -403,12 +502,15 @@ void zebra_interface_add_update(struct interface *ifp)
 		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADD %s(%u)", ifp->name,
 			   ifp->vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
-		if (client->ifinfo) {
-			client->ifadd_cnt++;
-			zsend_interface_add(client, ifp);
-			zsend_interface_link_params(client, ifp);
-		}
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
+		client->ifadd_cnt++;
+		zsend_interface_add(client, ifp);
+		zsend_interface_link_params(client, ifp);
+	}
 }
 
 void zebra_interface_delete_update(struct interface *ifp)
@@ -420,7 +522,11 @@ void zebra_interface_delete_update(struct interface *ifp)
 		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DELETE %s(%u)",
 			   ifp->name, ifp->vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		client->ifdel_cnt++;
 		zsend_interface_delete(client, ifp);
 	}
@@ -444,19 +550,25 @@ void zebra_interface_address_add_update(struct interface *ifp,
 	}
 
 	if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL))
-		zlog_warn(
+		flog_warn(
+			EC_ZEBRA_ADVERTISING_UNUSABLE_ADDR,
 			"WARNING: advertising address to clients that is not yet usable.");
 
 	zebra_vxlan_add_del_gw_macip(ifp, ifc->address, 1);
 
 	router_id_add_address(ifc);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		if (CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL)) {
 			client->connected_rt_add_cnt++;
 			zsend_interface_address(ZEBRA_INTERFACE_ADDRESS_ADD,
 						client, ifp, ifc);
 		}
+	}
 }
 
 /* Interface address deletion. */
@@ -480,12 +592,17 @@ void zebra_interface_address_delete_update(struct interface *ifp,
 
 	router_id_del_address(ifc);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		if (CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL)) {
 			client->connected_rt_del_cnt++;
 			zsend_interface_address(ZEBRA_INTERFACE_ADDRESS_DELETE,
 						client, ifp, ifc);
 		}
+	}
 }
 
 /* Interface VRF change. May need to delete from clients not interested in
@@ -501,7 +618,11 @@ void zebra_interface_vrf_update_del(struct interface *ifp, vrf_id_t new_vrf_id)
 			"MESSAGE: ZEBRA_INTERFACE_VRF_UPDATE/DEL %s VRF Id %u -> %u",
 			ifp->name, ifp->vrf_id, new_vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		/* Need to delete if the client is not interested in the new
 		 * VRF. */
 		zsend_interface_update(ZEBRA_INTERFACE_DOWN, client, ifp);
@@ -524,7 +645,11 @@ void zebra_interface_vrf_update_add(struct interface *ifp, vrf_id_t old_vrf_id)
 			"MESSAGE: ZEBRA_INTERFACE_VRF_UPDATE/ADD %s VRF Id %u -> %u",
 			ifp->name, old_vrf_id, ifp->vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
 		/* Need to add if the client is interested in the new VRF. */
 		client->ifadd_cnt++;
 		zsend_interface_add(client, ifp);
@@ -532,24 +657,26 @@ void zebra_interface_vrf_update_add(struct interface *ifp, vrf_id_t old_vrf_id)
 	}
 }
 
-int zebra_add_import_table_entry(struct route_node *rn, struct route_entry *re,
-				 const char *rmap_name)
+int zebra_add_import_table_entry(struct zebra_vrf *zvrf, struct route_node *rn,
+				 struct route_entry *re, const char *rmap_name)
 {
 	struct route_entry *newre;
 	struct route_entry *same;
 	struct prefix p;
-	route_map_result_t ret = RMAP_MATCH;
+	struct nexthop_group *ng;
+	route_map_result_t ret = RMAP_PERMITMATCH;
 	afi_t afi;
 
 	afi = family2afi(rn->p.family);
 	if (rmap_name)
 		ret = zebra_import_table_route_map_check(
-			afi, re->type, re->instance, &rn->p, re->ng.nexthop,
-			re->vrf_id, re->tag, rmap_name);
+			afi, re->type, re->instance, &rn->p,
+			re->nhe->nhg.nexthop,
+			zvrf->vrf->vrf_id, re->tag, rmap_name);
 
-	if (ret != RMAP_MATCH) {
+	if (ret != RMAP_PERMITMATCH) {
 		UNSET_FLAG(re->flags, ZEBRA_FLAG_SELECTED);
-		zebra_del_import_table_entry(rn, re);
+		zebra_del_import_table_entry(zvrf, rn, re);
 		return 0;
 	}
 
@@ -567,7 +694,7 @@ int zebra_add_import_table_entry(struct route_node *rn, struct route_entry *re,
 
 	if (same) {
 		UNSET_FLAG(same->flags, ZEBRA_FLAG_SELECTED);
-		zebra_del_import_table_entry(rn, same);
+		zebra_del_import_table_entry(zvrf, rn, same);
 	}
 
 	newre = XCALLOC(MTYPE_RE, sizeof(struct route_entry));
@@ -576,18 +703,20 @@ int zebra_add_import_table_entry(struct route_node *rn, struct route_entry *re,
 	newre->flags = re->flags;
 	newre->metric = re->metric;
 	newre->mtu = re->mtu;
-	newre->table = zebrad.rtm_table_default;
-	newre->nexthop_num = 0;
-	newre->uptime = time(NULL);
+	newre->table = zvrf->table_id;
+	newre->uptime = monotime(NULL);
 	newre->instance = re->table;
-	route_entry_copy_nexthops(newre, re->ng.nexthop);
 
-	rib_add_multipath(afi, SAFI_UNICAST, &p, NULL, newre);
+	ng = nexthop_group_new();
+	copy_nexthops(&ng->nexthop, re->nhe->nhg.nexthop, NULL);
+
+	rib_add_multipath(afi, SAFI_UNICAST, &p, NULL, newre, ng);
 
 	return 0;
 }
 
-int zebra_del_import_table_entry(struct route_node *rn, struct route_entry *re)
+int zebra_del_import_table_entry(struct zebra_vrf *zvrf, struct route_node *rn,
+				 struct route_entry *re)
 {
 	struct prefix p;
 	afi_t afi;
@@ -595,30 +724,32 @@ int zebra_del_import_table_entry(struct route_node *rn, struct route_entry *re)
 	afi = family2afi(rn->p.family);
 	prefix_copy(&p, &rn->p);
 
-	rib_delete(afi, SAFI_UNICAST, re->vrf_id, ZEBRA_ROUTE_TABLE, re->table,
-		   re->flags, &p, NULL, re->ng.nexthop,
-		   zebrad.rtm_table_default, re->metric, false, NULL);
+	rib_delete(afi, SAFI_UNICAST, zvrf->vrf->vrf_id, ZEBRA_ROUTE_TABLE,
+		   re->table, re->flags, &p, NULL, re->nhe->nhg.nexthop,
+		   re->nhe_id, zvrf->table_id, re->metric, re->distance,
+		   false);
 
 	return 0;
 }
 
 /* Assuming no one calls this with the main routing table */
-int zebra_import_table(afi_t afi, uint32_t table_id, uint32_t distance,
-		       const char *rmap_name, int add)
+int zebra_import_table(afi_t afi, vrf_id_t vrf_id, uint32_t table_id,
+		       uint32_t distance, const char *rmap_name, int add)
 {
 	struct route_table *table;
 	struct route_entry *re;
 	struct route_node *rn;
+	struct zebra_vrf *zvrf = zebra_vrf_lookup_by_id(vrf_id);
 
 	if (!is_zebra_valid_kernel_table(table_id)
-	    || ((table_id == RT_TABLE_MAIN)
-		|| (table_id == zebrad.rtm_table_default)))
-		return (-1);
+	    || (table_id == RT_TABLE_MAIN))
+		return -1;
 
 	if (afi >= AFI_MAX)
-		return (-1);
+		return -1;
 
-	table = zebra_vrf_other_route_table(afi, table_id, VRF_DEFAULT);
+	table = zebra_vrf_get_table_with_table_id(afi, SAFI_UNICAST, vrf_id,
+						  table_id);
 	if (table == NULL) {
 		return 0;
 	} else if (IS_ZEBRA_DEBUG_RIB) {
@@ -672,15 +803,16 @@ int zebra_import_table(afi_t afi, uint32_t table_id, uint32_t distance,
 		if (((afi == AFI_IP) && (rn->p.family == AF_INET))
 		    || ((afi == AFI_IP6) && (rn->p.family == AF_INET6))) {
 			if (add)
-				zebra_add_import_table_entry(rn, re, rmap_name);
+				zebra_add_import_table_entry(zvrf, rn, re,
+							     rmap_name);
 			else
-				zebra_del_import_table_entry(rn, re);
+				zebra_del_import_table_entry(zvrf, rn, re);
 		}
 	}
 	return 0;
 }
 
-int zebra_import_table_config(struct vty *vty)
+int zebra_import_table_config(struct vty *vty, vrf_id_t vrf_id)
 {
 	int i;
 	afi_t afi;
@@ -690,7 +822,7 @@ int zebra_import_table_config(struct vty *vty)
 
 	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
 		for (i = 1; i < ZEBRA_KERNEL_TABLE_MAX; i++) {
-			if (!is_zebra_import_table_enabled(afi, i))
+			if (!is_zebra_import_table_enabled(afi, vrf_id, i))
 				continue;
 
 			if (zebra_import_table_distance[afi][i]
@@ -715,57 +847,86 @@ int zebra_import_table_config(struct vty *vty)
 	return write;
 }
 
-void zebra_import_table_rm_update()
+static void zebra_import_table_rm_update_vrf_afi(struct zebra_vrf *zvrf,
+						 afi_t afi, int table_id,
+						 const char *rmap)
 {
-	afi_t afi;
-	int i;
 	struct route_table *table;
 	struct route_entry *re;
 	struct route_node *rn;
 	const char *rmap_name;
 
-	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
-		for (i = 1; i < ZEBRA_KERNEL_TABLE_MAX; i++) {
-			if (!is_zebra_import_table_enabled(afi, i))
+	rmap_name = zebra_get_import_table_route_map(afi, table_id);
+	if ((!rmap_name) || (strcmp(rmap_name, rmap) != 0))
+		return;
+
+	table = zebra_vrf_get_table_with_table_id(afi, SAFI_UNICAST,
+						  zvrf->vrf->vrf_id, table_id);
+	if (!table) {
+		if (IS_ZEBRA_DEBUG_RIB_DETAILED)
+			zlog_debug("%s: Table id=%d not found", __func__,
+				   table_id);
+		return;
+	}
+
+	for (rn = route_top(table); rn; rn = route_next(rn)) {
+		/*
+		 * For each entry in the non-default routing table,
+		 * add the entry in the main table
+		 */
+		if (!rn->info)
+			continue;
+
+		RNODE_FOREACH_RE (rn, re) {
+			if (CHECK_FLAG(re->status, ROUTE_ENTRY_REMOVED))
 				continue;
-
-			rmap_name = zebra_get_import_table_route_map(afi, i);
-			if (!rmap_name)
-				return;
-
-			table = zebra_vrf_other_route_table(afi, i,
-							    VRF_DEFAULT);
-			for (rn = route_top(table); rn; rn = route_next(rn)) {
-				/* For each entry in the non-default
-				 * routing table,
-				 * add the entry in the main table
-				 */
-				if (!rn->info)
-					continue;
-
-				RNODE_FOREACH_RE (rn, re) {
-					if (CHECK_FLAG(re->status,
-						       ROUTE_ENTRY_REMOVED))
-						continue;
-					break;
-				}
-
-				if (!re)
-					continue;
-
-				if (((afi == AFI_IP)
-				     && (rn->p.family == AF_INET))
-				    || ((afi == AFI_IP6)
-					&& (rn->p.family == AF_INET6)))
-					zebra_add_import_table_entry(rn, re,
-								     rmap_name);
-			}
+			break;
 		}
+
+		if (!re)
+			continue;
+
+		if (((afi == AFI_IP) && (rn->p.family == AF_INET))
+		    || ((afi == AFI_IP6) && (rn->p.family == AF_INET6)))
+			zebra_add_import_table_entry(zvrf, rn, re, rmap_name);
 	}
 
 	return;
 }
 
+static void zebra_import_table_rm_update_vrf(struct zebra_vrf *zvrf,
+					     const char *rmap)
+{
+	afi_t afi;
+	int i;
+
+	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
+		for (i = 1; i < ZEBRA_KERNEL_TABLE_MAX; i++) {
+			if (!is_zebra_import_table_enabled(
+				    afi, zvrf->vrf->vrf_id, i))
+				continue;
+
+			zebra_import_table_rm_update_vrf_afi(zvrf, afi, i,
+							     rmap);
+		}
+	}
+}
+
+void zebra_import_table_rm_update(const char *rmap)
+{
+	struct vrf *vrf;
+	struct zebra_vrf *zvrf;
+
+	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+		zvrf = vrf->info;
+
+		if (!zvrf)
+			continue;
+
+		zebra_import_table_rm_update_vrf(zvrf, rmap);
+	}
+}
+
 /* Interface parameters update */
 void zebra_interface_parameters_update(struct interface *ifp)
 {
@@ -776,7 +937,11 @@ void zebra_interface_parameters_update(struct interface *ifp)
 		zlog_debug("MESSAGE: ZEBRA_INTERFACE_LINK_PARAMS %s(%u)",
 			   ifp->name, ifp->vrf_id);
 
-	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
-		if (client->ifinfo)
-			zsend_interface_link_params(client, ifp);
+	for (ALL_LIST_ELEMENTS(zrouter.client_list, node, nnode, client)) {
+		/* Do not send unsolicited messages to synchronous clients. */
+		if (client->synchronous)
+			continue;
+
+		zsend_interface_link_params(client, ifp);
+	}
 }
